@@ -106,9 +106,6 @@ struct commodity_window
 typedef struct select_commodity_window SelectCommodityWindow;
 typedef struct commodity_window CommodityWindow;
 
-static gnc_commodity_help_callback help_callback = NULL;
-
-
 /* The commodity selection window */
 static SelectCommodityWindow *
 gnc_ui_select_commodity_create(const gnc_commodity * orig_sel,
@@ -129,17 +126,6 @@ gboolean gnc_ui_commodity_dialog_to_object(CommodityWindow * w);
 static void gnc_ui_select_commodity_response_cb (GtkDialog * dialog, gint response, gpointer data);
 #endif
 
-
-/********************************************************************
- * gnc_ui_commodity_set_help_callback
- ********************************************************************/
-void
-gnc_ui_commodity_set_help_callback (gnc_commodity_help_callback cb)
-{
-    help_callback = cb;
-}
-
-
 /********************************************************************
  * gnc_ui_select_commodity_modal_full()
  ********************************************************************/
@@ -183,7 +169,8 @@ gnc_ui_select_commodity_modal_full(gnc_commodity * orig_sel,
                         /* Translators: Replace here and later CUSIP by the name of your local
                            National Securities Identifying Number
                            like gb:SEDOL, de:WKN, ch:Valorennummer, fr:SICOVAM ...
-                           See http://en.wikipedia.org/wiki/ISIN for hints. */
+                           See https://en.wikipedia.org/wiki/ISIN and
+                           https://en.wikipedia.org/wiki/National_numbering_agency for hints. */
                         cusip    ? _("\nExchange code (ISIN, CUSIP or similar): ") : "",
                         cusip    ? cusip : "",
                         mnemonic ? _("\nMnemonic (Ticker symbol or similar): ") : "",
@@ -483,8 +470,8 @@ gnc_ui_update_commodity_picker (GtkWidget *cbwe,
     for (iterator = commodities; iterator; iterator = iterator->next)
     {
         commodity_items =
-            g_list_append(commodity_items,
-                          (gpointer) gnc_commodity_get_printname(iterator->data));
+            g_list_prepend (commodity_items,
+                            (gpointer) gnc_commodity_get_printname(iterator->data));
     }
     g_list_free(commodities);
 
@@ -630,7 +617,7 @@ gnc_ui_update_namespace_picker (GtkWidget *cbwe,
     {
         gtk_list_store_append(GTK_LIST_STORE(model), &iter);
         gtk_list_store_set (GTK_LIST_STORE(model), &iter, 0,
-                            GNC_COMMODITY_NS_NONCURRENCY, -1);
+                            GNC_COMMODITY_NS_NONISO_GUI, -1);
     }
 
     /* add all others to the combobox */
@@ -890,7 +877,6 @@ gnc_ui_build_commodity_dialog(const char * selected_namespace,
                               gboolean     edit)
 {
     CommodityWindow * retval = g_new0(CommodityWindow, 1);
-    GtkWidget *help_button;
     GtkWidget *box;
     GtkWidget *menu;
     GtkWidget *widget, *sec_label;
@@ -920,10 +906,6 @@ gnc_ui_build_commodity_dialog(const char * selected_namespace,
 
     retval->edit_commodity = NULL;
 
-    help_button = GTK_WIDGET(gtk_builder_get_object (builder, "help_button"));
-    if (!help_callback)
-        gtk_widget_hide (help_button);
-
     /* Get widget pointers */
     retval->fullname_entry = GTK_WIDGET(gtk_builder_get_object (builder, "fullname_entry"));
     retval->mnemonic_entry = GTK_WIDGET(gtk_builder_get_object (builder, "mnemonic_entry"));
@@ -1158,8 +1140,7 @@ gnc_ui_common_commodity_modal(gnc_commodity *commodity,
             break;
         case GTK_RESPONSE_HELP:
             DEBUG("case HELP");
-            if (help_callback)
-                help_callback ();
+            gnc_gnome_help (GTK_WINDOW(win->dialog), DF_MANUAL, DL_COMMODITY);
             break;
         default:	/* Cancel, Escape, Close, etc. */
             DEBUG("default: %d", value);
@@ -1320,6 +1301,8 @@ gnc_ui_commodity_dialog_to_object(CommodityWindow * w)
             c = gnc_commodity_new(book, fullname, name_space, mnemonic, code, fraction);
             w->edit_commodity = c;
             gnc_commodity_begin_edit(c);
+
+            gnc_commodity_set_user_symbol(c, user_symbol);
         }
         else
         {
