@@ -24,6 +24,7 @@
 
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <math.h>
 
 #include "gnc-gnome-utils.h"
 #include "gnc-splash.h"
@@ -60,7 +61,7 @@ gnc_show_splash_screen (void)
     GtkWidget *hbox;
     GtkWidget *version;
     GtkWidget *separator;
-    gchar *ver_string, *markup, *vcs;
+    gchar *ver_string, *markup;
 
     if (splash) return;
     if (!gnc_prefs_get_bool(GNC_PREFS_GROUP_GENERAL, GNC_PREF_SHOW_SPLASH)) return;
@@ -69,8 +70,8 @@ gnc_show_splash_screen (void)
     gtk_window_set_decorated(GTK_WINDOW (splash), FALSE);
     gtk_window_set_skip_taskbar_hint (GTK_WINDOW (splash), TRUE);
 
-    // Set the style context for this dialog so it can be easily manipulated with css
-    gnc_widget_set_style_context (GTK_WIDGET(splash), "GncSplash");
+    // Set the name for this dialog so it can be easily manipulated with css
+    gtk_widget_set_name (GTK_WIDGET(splash), "gnc-id-splash");
 
     g_signal_connect (splash, "destroy",
                       G_CALLBACK (splash_destroy_cb), NULL);
@@ -94,22 +95,8 @@ gnc_show_splash_screen (void)
     hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 3);
     gtk_box_set_homogeneous (GTK_BOX (hbox), FALSE);
 
-#ifdef GNC_VCS
-    vcs = GNC_VCS " ";
-#else
-    vcs = "";
-#endif
-
-    /* Allow builder to override the build id (eg distributions may want to
-     * print an package source version number (rpm, dpkg,...) instead of our git ref */
-    if (g_strcmp0("", GNUCASH_BUILD_ID) != 0)
-        ver_string = g_strdup_printf("%s: %s, %s: %s", _("Version"),
-                                     VERSION, _("Build ID"), GNUCASH_BUILD_ID);
-    else
-        ver_string = g_strdup_printf("%s: %s, %s: %s%s (%s)", _("Version"),
-                                     VERSION, _("Build ID"), vcs, GNC_VCS_REV,
-                                     GNC_VCS_REV_DATE);
-
+    ver_string = g_strdup_printf ("%s: %s, %s: %s", _("Version"),
+                                  gnc_version(), _("Build ID"), gnc_build_id());
     version = gtk_label_new(NULL);
     markup = g_markup_printf_escaped(MARKUP_STRING, ver_string);
     gtk_label_set_markup(GTK_LABEL(version), markup);
@@ -123,7 +110,7 @@ gnc_show_splash_screen (void)
        presumably it would be better to inhibit size change of the
        top level container, but I don't know how to do this */
     gtk_label_set_max_width_chars(GTK_LABEL(progress), 34);
-    markup = g_markup_printf_escaped(MARKUP_STRING, _("Loading..."));
+    markup = g_markup_printf_escaped(MARKUP_STRING, _("Loading…"));
     gtk_label_set_markup(GTK_LABEL(progress), markup);
     g_free(markup);
 
@@ -182,13 +169,19 @@ gnc_update_splash_screen (const gchar *string, double percentage)
         }
     }
 
-    if (progress_bar)
+    if (progress_bar )
     {
-        if (percentage < 0)
+         double curr_fraction =
+              round(gtk_progress_bar_get_fraction(GTK_PROGRESS_BAR(progress_bar)) * 100.0);
+         if (percentage >= 0 && percentage <= 100.0 &&
+             round(percentage) == curr_fraction)
+              return; // No change so don't wast time running the main loop
+
+        if (percentage <= 0)
         {
             gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress_bar), 0.0);
         }
-        else
+        else 
         {
             if (percentage <= 100)
             {
