@@ -41,6 +41,7 @@
 #include "gnc-gobject-utils.h"
 #include "gnc-ui-balances.h"
 #include "gnc-ui-util.h"
+#include <gnc-locale-tax.h>
 
 #define TREE_MODEL_ACCOUNT_CM_CLASS "tree-model-account"
 
@@ -103,8 +104,14 @@ typedef struct GncTreeModelAccountPrivate
 
 } GncTreeModelAccountPrivate;
 
+G_DEFINE_TYPE_WITH_CODE (GncTreeModelAccount,
+                         gnc_tree_model_account, GNC_TYPE_TREE_MODEL,
+                         G_ADD_PRIVATE (GncTreeModelAccount)
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL,
+                                                gnc_tree_model_account_tree_model_init))
+
 #define GNC_TREE_MODEL_ACCOUNT_GET_PRIVATE(o)  \
-   ((GncTreeModelAccountPrivate*)g_type_instance_get_private ((GTypeInstance*)o, GNC_TYPE_TREE_MODEL_ACCOUNT))
+   ((GncTreeModelAccountPrivate*)gnc_tree_model_account_get_instance_private((GncTreeModelAccount*)o))
 
 
 /************************************************************/
@@ -152,11 +159,6 @@ gnc_tree_model_account_update_color (gpointer gsettings, gchar *key, gpointer us
 /** A pointer to the parent class of an account tree model. */
 static GObjectClass *parent_class = NULL;
 
-G_DEFINE_TYPE_WITH_CODE (GncTreeModelAccount, gnc_tree_model_account, GNC_TYPE_TREE_MODEL,
-                         G_ADD_PRIVATE (GncTreeModelAccount)
-                         G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL,
-                                                gnc_tree_model_account_tree_model_init))
-
 static void
 gnc_tree_model_account_class_init (GncTreeModelAccountClass *klass)
 {
@@ -392,6 +394,7 @@ gnc_tree_model_account_get_column_type (GtkTreeModel *tree_model, int index)
     case GNC_TREE_MODEL_ACCOUNT_COL_BALANCE:
     case GNC_TREE_MODEL_ACCOUNT_COL_BALANCE_REPORT:
     case GNC_TREE_MODEL_ACCOUNT_COL_BALANCE_PERIOD:
+    case GNC_TREE_MODEL_ACCOUNT_COL_BALANCE_LIMIT:
     case GNC_TREE_MODEL_ACCOUNT_COL_CLEARED:
     case GNC_TREE_MODEL_ACCOUNT_COL_CLEARED_REPORT:
     case GNC_TREE_MODEL_ACCOUNT_COL_RECONCILED:
@@ -420,6 +423,7 @@ gnc_tree_model_account_get_column_type (GtkTreeModel *tree_model, int index)
 
     case GNC_TREE_MODEL_ACCOUNT_COL_HIDDEN:
     case GNC_TREE_MODEL_ACCOUNT_COL_PLACEHOLDER:
+    case GNC_TREE_MODEL_ACCOUNT_COL_OPENING_BALANCE:
         return G_TYPE_BOOLEAN;
 
     default:
@@ -559,6 +563,7 @@ gnc_tree_model_account_compute_period_balance (GncTreeModelAccount *model,
                                                gboolean *negative)
 {
     GncTreeModelAccountPrivate *priv;
+    GNCPrintAmountInfo print_info;
     time64 t1, t2;
     gnc_numeric b3;
 
@@ -582,7 +587,9 @@ gnc_tree_model_account_compute_period_balance (GncTreeModelAccount *model,
     if (negative)
         *negative = gnc_numeric_negative_p (b3);
 
-    return g_strdup(xaccPrintAmount (b3, gnc_account_print_info (acct, TRUE)));
+    print_info = gnc_account_print_info (acct, TRUE);
+
+    return g_strdup (gnc_print_amount_with_bidi_ltr_isolate (b3, print_info));
 }
 
 static gboolean
@@ -824,6 +831,12 @@ gnc_tree_model_account_get_value (GtkTreeModel *tree_model,
         g_free (string);
         break;
 
+    case GNC_TREE_MODEL_ACCOUNT_COL_BALANCE_LIMIT:
+        g_value_init (value, G_TYPE_STRING);
+        string = gnc_ui_account_get_balance_limit_icon_name (account);
+        g_value_take_string (value, string);
+        break;
+
     case GNC_TREE_MODEL_ACCOUNT_COL_CLEARED:
         g_value_init (value, G_TYPE_STRING);
         string = gnc_ui_account_get_print_balance (xaccAccountGetClearedBalanceInCurrency,
@@ -956,6 +969,11 @@ gnc_tree_model_account_get_value (GtkTreeModel *tree_model,
         g_value_set_boolean (value, xaccAccountGetPlaceholder (account));
         break;
 
+    case GNC_TREE_MODEL_ACCOUNT_COL_OPENING_BALANCE:
+        g_value_init (value, G_TYPE_BOOLEAN);
+        g_value_set_boolean (value, xaccAccountGetIsOpeningBalance (account));
+        break;
+
     default:
         g_assert_not_reached ();
         break;
