@@ -213,44 +213,25 @@ static void gnc_ui_to_customer (CustomerWindow *cw, GncCustomer *cust)
     if (cw->dialog_type == NEW_CUSTOMER)
         qof_event_gen(QOF_INSTANCE(cust), QOF_EVENT_ADD, NULL);
 
-    gncCustomerSetID (cust, gtk_editable_get_chars
-                      (GTK_EDITABLE (cw->id_entry), 0, -1));
-    gncCustomerSetName (cust, gtk_editable_get_chars
-                        (GTK_EDITABLE (cw->company_entry), 0, -1));
+    gncCustomerSetID (cust, gtk_entry_get_text (GTK_ENTRY (cw->id_entry)));
+    gncCustomerSetName (cust, gtk_entry_get_text (GTK_ENTRY (cw->company_entry)));
 
-    gncAddressSetName (addr, gtk_editable_get_chars
-                       (GTK_EDITABLE (cw->name_entry), 0, -1));
-    gncAddressSetAddr1 (addr, gtk_editable_get_chars
-                        (GTK_EDITABLE (cw->addr1_entry), 0, -1));
-    gncAddressSetAddr2 (addr, gtk_editable_get_chars
-                        (GTK_EDITABLE (cw->addr2_entry), 0, -1));
-    gncAddressSetAddr3 (addr, gtk_editable_get_chars
-                        (GTK_EDITABLE (cw->addr3_entry), 0, -1));
-    gncAddressSetAddr4 (addr, gtk_editable_get_chars
-                        (GTK_EDITABLE (cw->addr4_entry), 0, -1));
-    gncAddressSetPhone (addr, gtk_editable_get_chars
-                        (GTK_EDITABLE (cw->phone_entry), 0, -1));
-    gncAddressSetFax (addr, gtk_editable_get_chars
-                      (GTK_EDITABLE (cw->fax_entry), 0, -1));
-    gncAddressSetEmail (addr, gtk_editable_get_chars
-                        (GTK_EDITABLE (cw->email_entry), 0, -1));
-
-    gncAddressSetName (shipaddr, gtk_editable_get_chars
-                       (GTK_EDITABLE (cw->shipname_entry), 0, -1));
-    gncAddressSetAddr1 (shipaddr, gtk_editable_get_chars
-                        (GTK_EDITABLE (cw->shipaddr1_entry), 0, -1));
-    gncAddressSetAddr2 (shipaddr, gtk_editable_get_chars
-                        (GTK_EDITABLE (cw->shipaddr2_entry), 0, -1));
-    gncAddressSetAddr3 (shipaddr, gtk_editable_get_chars
-                        (GTK_EDITABLE (cw->shipaddr3_entry), 0, -1));
-    gncAddressSetAddr4 (shipaddr, gtk_editable_get_chars
-                        (GTK_EDITABLE (cw->shipaddr4_entry), 0, -1));
-    gncAddressSetPhone (shipaddr, gtk_editable_get_chars
-                        (GTK_EDITABLE (cw->shipphone_entry), 0, -1));
-    gncAddressSetFax (shipaddr, gtk_editable_get_chars
-                      (GTK_EDITABLE (cw->shipfax_entry), 0, -1));
-    gncAddressSetEmail (shipaddr, gtk_editable_get_chars
-                        (GTK_EDITABLE (cw->shipemail_entry), 0, -1));
+    gncAddressSetName (addr, gtk_entry_get_text (GTK_ENTRY (cw->name_entry)));
+    gncAddressSetAddr1 (addr, gtk_entry_get_text (GTK_ENTRY (cw->addr1_entry)));
+    gncAddressSetAddr2 (addr, gtk_entry_get_text (GTK_ENTRY (cw->addr2_entry)));
+    gncAddressSetAddr3 (addr, gtk_entry_get_text (GTK_ENTRY (cw->addr3_entry)));
+    gncAddressSetAddr4 (addr, gtk_entry_get_text (GTK_ENTRY (cw->addr4_entry)));
+    gncAddressSetPhone (addr, gtk_entry_get_text (GTK_ENTRY (cw->phone_entry)));
+    gncAddressSetFax (addr, gtk_entry_get_text (GTK_ENTRY (cw->fax_entry)));
+    gncAddressSetEmail (addr, gtk_entry_get_text (GTK_ENTRY (cw->email_entry)));
+    gncAddressSetName (shipaddr, gtk_entry_get_text (GTK_ENTRY (cw->shipname_entry)));
+    gncAddressSetAddr1 (shipaddr, gtk_entry_get_text (GTK_ENTRY (cw->shipaddr1_entry)));
+    gncAddressSetAddr2 (shipaddr, gtk_entry_get_text (GTK_ENTRY (cw->shipaddr2_entry)));
+    gncAddressSetAddr3 (shipaddr, gtk_entry_get_text (GTK_ENTRY (cw->shipaddr3_entry)));
+    gncAddressSetAddr4 (shipaddr, gtk_entry_get_text (GTK_ENTRY (cw->shipaddr4_entry)));
+    gncAddressSetPhone (shipaddr, gtk_entry_get_text (GTK_ENTRY (cw->shipphone_entry)));
+    gncAddressSetFax (shipaddr, gtk_entry_get_text (GTK_ENTRY (cw->shipfax_entry)));
+    gncAddressSetEmail (shipaddr, gtk_entry_get_text (GTK_ENTRY (cw->shipemail_entry)));
 
     gncCustomerSetActive (cust, gtk_toggle_button_get_active
                           (GTK_TOGGLE_BUTTON (cw->active_check)));
@@ -283,10 +264,11 @@ static gboolean check_edit_amount (GtkWidget *amount,
                                    gnc_numeric *min, gnc_numeric *max,
                                    const char * error_message)
 {
-    if (!gnc_amount_edit_evaluate (GNC_AMOUNT_EDIT (amount)))
+    GError *error = NULL;
+    if (!gnc_amount_edit_evaluate (GNC_AMOUNT_EDIT (amount), &error))
     {
-        if (error_message)
-            gnc_error_dialog (gnc_ui_get_gtk_window (amount), "%s", error_message);
+        gnc_error_dialog (gnc_ui_get_gtk_window (amount), "%s", error->message);
+        g_error_free (error);
         return TRUE;
     }
     /* We've got a valid-looking number; check mix/max */
@@ -323,26 +305,15 @@ gnc_customer_window_ok_cb (GtkWidget *widget, gpointer data)
     CustomerWindow *cw = data;
     gnc_numeric min, max;
     gchar *string;
+    gnc_commodity *currency;
+    GNCPrintAmountInfo print_info;
 
     /* Check for valid company name */
     if (check_entry_nonempty (cw->company_entry,
-                              _("You must enter a company name. "
-                                "If this customer is an individual (and not a company) "
-                                "you should enter the same value for:\nIdentification "
-                                "- Company Name, and\nPayment Address - Name.")))
+                              _("The Company Name field cannot be left blank, please "
+                                "enter a company name or a person's name.")))
         return;
 
-    /* Make sure we have an address */
-    if (check_entry_nonempty (cw->addr1_entry, NULL) &&
-            check_entry_nonempty (cw->addr2_entry, NULL) &&
-            check_entry_nonempty (cw->addr3_entry, NULL) &&
-            check_entry_nonempty (cw->addr4_entry, NULL))
-    {
-        const char *msg = _("You must enter a billing address.");
-        gnc_error_dialog (gnc_ui_get_gtk_window (widget), "%s", msg);
-        return;
-    }
-
     /* Verify terms, discount, and credit are valid (or empty) */
     min = gnc_numeric_zero ();
     max = gnc_numeric_create (100, 1);
@@ -352,6 +323,12 @@ gnc_customer_window_ok_cb (GtkWidget *widget, gpointer data)
                              "or you must leave it blank.")))
         return;
 
+    currency = gnc_currency_edit_get_currency (GNC_CURRENCY_EDIT(cw->currency_edit));
+    print_info = gnc_commodity_print_info (currency, FALSE);
+    gnc_amount_edit_set_print_info (GNC_AMOUNT_EDIT (cw->credit_amount), print_info);
+    gnc_amount_edit_set_fraction (GNC_AMOUNT_EDIT (cw->credit_amount),
+                                  gnc_commodity_get_fraction (currency));
+
     if (check_edit_amount (cw->credit_amount, &min, NULL,
                            _("Credit must be a positive amount or "
                              "you must leave it blank.")))
@@ -390,7 +367,8 @@ gnc_customer_window_cancel_cb (GtkWidget *widget, gpointer data)
 void
 gnc_customer_window_help_cb (GtkWidget *widget, gpointer data)
 {
-    gnc_gnome_help(HF_HELP, HL_USAGE_CUSTOMER);
+    CustomerWindow *cw = data;
+    gnc_gnome_help (GTK_WINDOW(cw->dialog), DF_MANUAL, DL_USAGE_CUSTOMER);
 }
 
 void
@@ -421,28 +399,26 @@ void
 gnc_customer_name_changed_cb (GtkWidget *widget, gpointer data)
 {
     CustomerWindow *cw = data;
-    char *fullname, *title;
-    const char *id,  *name;
+    char *title;
 
     if (!cw)
         return;
 
-    name = gtk_entry_get_text (GTK_ENTRY (cw->company_entry));
+    const char *header = (cw->dialog_type == EDIT_CUSTOMER) ?
+        _("Edit Customer") : _("New Customer");
+
+    const char *name = gtk_entry_get_text (GTK_ENTRY (cw->company_entry));
     if (!name || *name == '\0')
         name = _("<No name>");
 
-    id = gtk_entry_get_text (GTK_ENTRY (cw->id_entry));
-
-    fullname = g_strconcat (name, " (", id, ")", (char *)NULL);
-
-    if (cw->dialog_type == EDIT_CUSTOMER)
-        title = g_strconcat (_("Edit Customer"), " - ", fullname, (char *)NULL);
+    const char *id = gtk_entry_get_text (GTK_ENTRY (cw->id_entry));
+    if (id && *id)
+        title = g_strdup_printf ("%s - %s (%s)", header, name, id);
     else
-        title = g_strconcat (_("New Customer"), " - ", fullname, (char *)NULL);
+        title = g_strdup_printf ("%s - %s", header, name);
 
     gtk_window_set_title (GTK_WINDOW (cw->dialog), title);
 
-    g_free (fullname);
     g_free (title);
 }
 
@@ -933,6 +909,8 @@ gnc_customer_search (GtkWindow *parent, GncCustomer *start, QofBook *book)
     /* Build the column list in reverse order */
     if (columns == NULL)
     {
+        columns = gnc_search_param_prepend (columns, _("Shipping Contact"), NULL, type,
+                                            CUSTOMER_SHIPADDR, ADDRESS_NAME, NULL);
         columns = gnc_search_param_prepend (columns, _("Contact"), NULL, type,
                                             CUSTOMER_ADDR, ADDRESS_NAME, NULL);
         columns = gnc_search_param_prepend (columns, _("Company"), NULL, type,
