@@ -105,12 +105,9 @@ static void gnc_ui_to_job (JobWindow *jw, GncJob *job)
 
     qof_event_gen(QOF_INSTANCE(job), QOF_EVENT_ADD, NULL);
 
-    gncJobSetID (job, gtk_editable_get_chars (GTK_EDITABLE (jw->id_entry),
-                 0, -1));
-    gncJobSetName (job, gtk_editable_get_chars (GTK_EDITABLE (jw->name_entry),
-                   0, -1));
-    gncJobSetReference (job, gtk_editable_get_chars
-                        (GTK_EDITABLE (jw->desc_entry), 0, -1));
+    gncJobSetID (job, gtk_entry_get_text (GTK_ENTRY (jw->id_entry)));
+    gncJobSetName (job, gtk_entry_get_text (GTK_ENTRY (jw->name_entry)));
+    gncJobSetReference (job, gtk_entry_get_text (GTK_ENTRY (jw->desc_entry)));
     gncJobSetRate (job, gnc_amount_edit_get_amount
                         (GNC_AMOUNT_EDIT (jw->rate_entry)));
     gncJobSetActive (job, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
@@ -151,6 +148,14 @@ gnc_job_verify_ok (JobWindow *jw)
         return FALSE;
     }
 
+    /* Check for valid rate */
+    if (!gnc_amount_edit_evaluate (GNC_AMOUNT_EDIT(jw->rate_entry), NULL))
+    {
+        const char *message = _("The rate amount must be valid or you must leave it blank.");
+        gnc_error_dialog (GTK_WINDOW (jw->dialog), "%s", message);
+        return FALSE;
+    }
+
     /* Set a valid id if one was not created */
     res = gtk_entry_get_text (GTK_ENTRY (jw->id_entry));
     if (g_strcmp0 (res, "") == 0)
@@ -202,7 +207,8 @@ gnc_job_window_cancel_cb (GtkWidget *widget, gpointer data)
 void
 gnc_job_window_help_cb (GtkWidget *widget, gpointer data)
 {
-    gnc_gnome_help(HF_HELP, HL_USAGE_BSNSS);
+    JobWindow *jw = data;
+    gnc_gnome_help (GTK_WINDOW(jw->dialog), DF_MANUAL, DL_USAGE_JOB);
 }
 
 
@@ -231,28 +237,26 @@ void
 gnc_job_name_changed_cb (GtkWidget *widget, gpointer data)
 {
     JobWindow *jw = data;
-    char *fullname, *title;
-    const char *name, *id;
+    char *title;
 
     if (!jw)
         return;
 
-    name = gtk_entry_get_text (GTK_ENTRY (jw->name_entry));
+    const char *header = (jw->dialog_type == EDIT_JOB) ?
+        _("Edit Job") : _("New Job");
+
+    const char *name = gtk_entry_get_text (GTK_ENTRY (jw->name_entry));
     if (!name || *name == '\0')
         name = _("<No name>");
 
-    id = gtk_entry_get_text (GTK_ENTRY (jw->id_entry));
-
-    fullname = g_strconcat (name, " (", id, ")", (char *)NULL);
-
-    if (jw->dialog_type == EDIT_JOB)
-        title = g_strconcat (_("Edit Job"), " - ", fullname, (char *)NULL);
+    const char *id = gtk_entry_get_text (GTK_ENTRY (jw->id_entry));
+    if (id && *id)
+        title = g_strdup_printf ("%s - %s (%s)", header, name, id);
     else
-        title = g_strconcat (_("New Job"), " - ", fullname, (char *)NULL);
+        title = g_strdup_printf ("%s - %s", header, name);
 
     gtk_window_set_title (GTK_WINDOW (jw->dialog), title);
 
-    g_free (fullname);
     g_free (title);
 }
 
