@@ -215,7 +215,7 @@ gnc_payment_window_refresh_handler (G_GNUC_UNUSED GHashTable *changes, gpointer
     PaymentWindow *pw = data;
 
     gnc_payment_window_fill_docs_list (pw);
-    pw->post_acct = gnc_account_select_combo_fill (pw->post_combo, pw->book, pw->acct_types, pw->acct_commodities);
+    pw->post_acct = gnc_account_select_combo_fill (pw->post_combo, pw->book, pw->acct_types, NULL);
 }
 
 static gboolean
@@ -339,14 +339,11 @@ calculate_selected_total_helper (GtkTreeModel *model,
 {
     gnc_numeric *subtotal = (gnc_numeric*) data;
     gnc_numeric cur_val;
-    GValue value = { 0 };
     GNCLot *lot;
     Account *acct;
     gnc_commodity *currency;
 
-    gtk_tree_model_get_value (model, iter, 5, &value);
-    lot = (GNCLot *) g_value_get_pointer (&value);
-    g_value_unset (&value);
+    gtk_tree_model_get (model, iter, 5, &lot, -1);
 
     /* Find the amount's currency to determine the required precision */
     acct = gnc_lot_get_account (lot);
@@ -413,13 +410,10 @@ gnc_payment_dialog_highlight_documents (PaymentWindow *pw)
     {
         do
         {
-            GValue value = { 0 };
             GNCLot *lot;
             GList *li_node;
 
-            gtk_tree_model_get_value (model, &iter, 5, &value);
-            lot = (GNCLot *) g_value_get_pointer (&value);
-            g_value_unset (&value);
+            gtk_tree_model_get (model, &iter, 5, &lot, -1);
 
             if (!lot)
                 continue; /* Lot has been deleted behind our back... */
@@ -446,7 +440,6 @@ gnc_payment_window_fill_docs_list (PaymentWindow *pw)
     GtkListStore *store;
     GtkTreeSelection *selection;
     GList *list = NULL, *node;
-    GNCLot *tx_lot = NULL;
 
     g_return_if_fail (pw->docs_list_tree_view && GTK_IS_TREE_VIEW(pw->docs_list_tree_view));
 
@@ -635,7 +628,7 @@ gnc_payment_dialog_owner_changed (PaymentWindow *pw)
     if (gncOwnerIsValid(owner))
         pw->acct_commodities = gncOwnerGetCommoditiesList (owner);
 
-    pw->post_acct = gnc_account_select_combo_fill (pw->post_combo, pw->book, pw->acct_types, pw->acct_commodities);
+    pw->post_acct = gnc_account_select_combo_fill (pw->post_combo, pw->book, pw->acct_types, NULL);
     if (gncOwnerEqual(&pw->owner, &pw->tx_info->owner) && pw->tx_info->post_acct)
     {
         pw->post_acct = pw->tx_info->post_acct;
@@ -737,13 +730,6 @@ gnc_payment_dialog_remember_account (PaymentWindow *pw, Account *acc)
 }
 
 
-static void
-gnc_payment_set_owner (PaymentWindow *pw, GncOwner *owner)
-{
-    gnc_owner_set_owner (pw->owner_choice, owner);
-    gnc_payment_dialog_owner_changed(pw);
-}
-
 static void
 gnc_payment_update_style_classes (PaymentWindow *pw)
 {
@@ -949,11 +935,8 @@ get_selected_lots (GtkTreeModel *model,
 {
     GList **return_list = data;
     GNCLot *lot;
-    GValue value = { 0 };
 
-    gtk_tree_model_get_value (model, iter, 5, &value);
-    lot = (GNCLot *) g_value_get_pointer (&value);
-    g_value_unset (&value);
+    gtk_tree_model_get (model, iter, 5, &lot, -1);
 
     if (lot)
         *return_list = g_list_insert_sorted (*return_list, lot, (GCompareFunc)gncOwnerLotsSortFunc);
@@ -1186,16 +1169,13 @@ static void print_date (G_GNUC_UNUSED GtkTreeViewColumn *tree_column,
                         GtkTreeIter *iter,
                         G_GNUC_UNUSED gpointer data)
 {
-    GValue value = { 0 };
     time64 doc_date_time;
     gchar *doc_date_str;
 
     g_return_if_fail (cell && iter && tree_model);
 
 
-    gtk_tree_model_get_value (tree_model, iter, 0, &value);
-    doc_date_time = (time64) g_value_get_int64 (&value);
-    g_value_unset (&value);
+    gtk_tree_model_get (tree_model, iter, 0, &doc_date_time, -1);
     doc_date_str = qof_print_date (doc_date_time);
     g_object_set (G_OBJECT (cell), "text", doc_date_str, NULL);
     g_free (doc_date_str);
@@ -1580,23 +1560,34 @@ gboolean gnc_ui_payment_is_customer_payment(const Transaction *txn)
 // ///////////////
 static char *gen_split_desc (Transaction *txn, Split *split)
 {
-    gnc_numeric value = xaccSplitGetValue(split);
+    gnc_numeric value = xaccSplitGetAmount(split);
     Account *xfer_acct = xaccSplitGetAccount(split);
     char *acct_name = gnc_account_get_full_name (xfer_acct);
     const char *action = gnc_get_action_num (txn, split);
     const char *memo = xaccSplitGetMemo (split);
+    char rec_state = xaccSplitGetReconcile (split);
     const char *print_amt = xaccPrintAmount(value, gnc_account_print_info (xfer_acct, TRUE));
     char *split_str = NULL;
+    char *rec_str = NULL;
+
+    if (rec_state == CREC)
+        rec_str = g_strdup_printf("[%s] ", _("Cleared"));
+    else if (rec_state == YREC)
+        rec_str = g_strdup_printf("[%s] ", _("Reconciled"));
+    else
+        rec_str = g_strdup("");
 
     if (action && *action && memo && *memo)
-        split_str = g_strdup_printf ("%s: %s (%s, %s)", acct_name, print_amt,
+        split_str = g_strdup_printf ("%s%s: %s (%s, %s)", rec_str, acct_name, print_amt,
                                         action, memo);
     else if((action && *action) || (memo && *memo))
-        split_str = g_strdup_printf ("%s: %s (%s)", acct_name, print_amt,
+        split_str = g_strdup_printf ("%s%s: %s (%s)", rec_str, acct_name, print_amt,
                                         action ? action : memo);
     else
-        split_str = g_strdup_printf ("%s: %s", acct_name, print_amt);
+        split_str = g_strdup_printf ("%s%s: %s", rec_str, acct_name, print_amt);
+
     g_free (acct_name);
+    g_free (rec_str);
 
     return split_str;
 }
@@ -1633,8 +1624,9 @@ static Split *select_payment_split (GtkWindow *parent, Transaction *txn)
         GList *node;
         GtkWidget *first_rb = NULL;
         int answer = GTK_BUTTONS_OK;
-        const char *message = _("While this transaction has multiple splits that can be considered\nas 'the payment split', gnucash only knows how to handle one.\n"
-                                "Please select one, the others will be ignored.\n\n");
+        const char *message = _("While this transaction has multiple splits that can be considered\n"
+                                "as 'the payment split', GnuCash only knows how to handle one.\n"
+                                "Please select one, the others will be discarded.\n\n");
         GtkDialog *dialog = GTK_DIALOG(
                             gtk_dialog_new_with_buttons (_("Warning"),
                                                          parent,
@@ -1695,10 +1687,11 @@ static Split *select_payment_split (GtkWindow *parent, Transaction *txn)
 
 static GList *select_txn_lots (GtkWindow *parent, Transaction *txn, Account **post_acct, gboolean *abort)
 {
-    gboolean has_no_lot_apar_splits = FALSE;
-    SplitList *post_splits = NULL, *no_lot_post_splits = NULL;
+    SplitList *apar_splits = NULL; /* all spits in txn that are APAR type */
+    SplitList *apar_splits_no_lot = NULL; /* all splits in txn that are APAR type, but not tied to a lot */
     SplitList *iter;
     GList *txn_lots = NULL;
+    GList *unique_apar_accts = NULL;
 
     /* There's no use in continuing if I can't set the post_acct or abort variables */
     if (!post_acct || !abort)
@@ -1707,11 +1700,20 @@ static GList *select_txn_lots (GtkWindow *parent, Transaction *txn, Account **po
     *abort = FALSE;
     *post_acct = NULL;
 
-    post_splits = xaccTransGetAPARAcctSplitList (txn, FALSE);
-    for (iter = post_splits; iter; iter = iter->next)
+    /* Start by filtering out all APAR splits that have lots. Those are the ones we can
+       display as invoices and pre-payments in the payment window. */
+    apar_splits = xaccTransGetAPARAcctSplitList (txn, FALSE);
+    for (iter = apar_splits; iter; iter = iter->next)
     {
         GNCLot *postlot = NULL;
         Split *post_split = iter->data;
+        Account *apar_acct = xaccSplitGetAccount (post_split);
+
+        /* Store found apar_acct in our list of unique_apar_accts
+         * for later processing */
+        if (!g_list_find (unique_apar_accts, apar_acct))
+            unique_apar_accts = g_list_prepend (unique_apar_accts, apar_acct);
+
         postlot = xaccSplitGetLot (post_split);
         if (postlot)
         {
@@ -1719,59 +1721,55 @@ static GList *select_txn_lots (GtkWindow *parent, Transaction *txn, Account **po
             lot_info->lot = postlot;
             lot_info->amount = xaccSplitGetValue (post_split);
             txn_lots = g_list_prepend (txn_lots, lot_info);
-            *post_acct = xaccSplitGetAccount (post_split);
+            *post_acct = apar_acct;
         }
         else
-        {
-            /* Make sure not to override post_acct if it was set above from a lot split */
-            if (!*post_acct)
-                *post_acct = xaccSplitGetAccount (post_split);
-            no_lot_post_splits = g_list_prepend (no_lot_post_splits, post_split);
-            has_no_lot_apar_splits = TRUE;
-        }
+            apar_splits_no_lot = g_list_prepend (apar_splits_no_lot, post_split);
     }
+    g_list_free (apar_splits);
 
-    g_list_free (post_splits);
+    /* If no post_acct was selected from the postlots, fall back to the first apar split's
+     * account if there is one. */
+    if (!*post_acct && apar_splits_no_lot)
+        *post_acct = xaccSplitGetAccount (apar_splits_no_lot->data);
+    g_list_free (apar_splits_no_lot);
 
-    /* If the txn has both APAR splits linked to a business lot and
-     * splits that are not, issue a warning some will be discarded.
+    /* Abort if the txn has splits in more than one APAR account
+     * GnuCash can only handle one post account per payment transaction.
      */
-    if (has_no_lot_apar_splits && gnc_list_length_cmp (txn_lots, 0))
+    if (g_list_length (unique_apar_accts) > 1)
     {
         GtkWidget *dialog;
         char *split_str = g_strdup ("");
-        for (iter = no_lot_post_splits; iter; iter = iter->next)
+
+        for (iter = unique_apar_accts; iter; iter = iter->next)
         {
-            Split *post_split = iter->data;
-            char *tmp_str = gen_split_desc (txn, post_split);
-            char *tmp_str2 = g_strconcat(split_str, "• ", tmp_str, "\n", NULL);
-            g_free (tmp_str);
+            Account *acct = iter->data;
+            char *acct_name = gnc_account_get_full_name (acct);
+            char *tmp_str = g_strconcat(split_str, "• ", acct_name, "\n", NULL);
+            g_free (acct_name);
             g_free (split_str);
-            split_str = tmp_str2;
+            split_str = tmp_str;
         }
 
         dialog = gtk_message_dialog_new (parent,
                                          GTK_DIALOG_DESTROY_WITH_PARENT,
-                                         GTK_MESSAGE_WARNING,
-                                         GTK_BUTTONS_CANCEL,
-                                         _("The transaction has at least one split in a business account that is not part of a business transaction.\n"
-                                         "If you continue these splits will be ignored:\n\n%s\n"
-                                         "Do you wish to continue and ignore these splits?"),
+                                         GTK_MESSAGE_INFO,
+                                         GTK_BUTTONS_CLOSE,
+                                         _("This transaction has splits in multiple business accounts:\n\n%s\n"
+                                         "GnuCash can only handle transactions that post to a single account.\n\n"
+                                         "Please correct this manually by editing the transaction directly and then try again."),
                                          split_str);
-        gtk_dialog_add_buttons (GTK_DIALOG(dialog),
-                                _("Continue"), GTK_BUTTONS_OK, NULL);
-        gtk_dialog_set_default_response (GTK_DIALOG(dialog), GTK_BUTTONS_CANCEL);
-        if (gtk_dialog_run (GTK_DIALOG(dialog)) != GTK_BUTTONS_OK)
-        {
-            *abort = TRUE;
-            g_list_free_full (txn_lots, g_free);
-            txn_lots = NULL;
-        }
+        gtk_dialog_run (GTK_DIALOG(dialog));
         gtk_widget_destroy (dialog);
+        PINFO("Multiple asset accounts in splits of txn \"%s\"; cannot use this for assigning a payment.",
+              xaccTransGetDescription(txn));
         g_free (split_str);
-    }
 
-    g_list_free (no_lot_post_splits);
+        *abort = TRUE;
+        g_list_free_full (txn_lots, g_free);
+        txn_lots = NULL;
+    }
 
     return txn_lots;
 }
@@ -1820,7 +1818,8 @@ PaymentWindow * gnc_ui_payment_new_with_txn (GtkWindow* parent, GncOwner *owner,
         GDate txn_date = xaccTransGetDatePostedGDate (txn);
         gnc_ui_payment_window_set_date(pw, &txn_date);
     }
-    gnc_ui_payment_window_set_amount(pw, xaccSplitGetValue(payment_split));
+
+    gnc_ui_payment_window_set_amount(pw, xaccSplitConvertAmount (payment_split, post_acct));
     if (payment_split)
         gnc_ui_payment_window_set_xferaccount(pw, xaccSplitGetAccount(payment_split));
     return pw;
