@@ -146,27 +146,17 @@ static void gnc_ui_to_vendor (VendorWindow *vw, GncVendor *vendor)
     if (vw->dialog_type == NEW_VENDOR)
         qof_event_gen(QOF_INSTANCE(vendor), QOF_EVENT_ADD, NULL);
 
-    gncVendorSetID (vendor, gtk_editable_get_chars
-                    (GTK_EDITABLE (vw->id_entry), 0, -1));
-    gncVendorSetName (vendor, gtk_editable_get_chars
-                      (GTK_EDITABLE (vw->company_entry), 0, -1));
+    gncVendorSetID (vendor, gtk_entry_get_text (GTK_ENTRY (vw->id_entry)));
+    gncVendorSetName (vendor, gtk_entry_get_text (GTK_ENTRY (vw->company_entry)));
 
-    gncAddressSetName (addr, gtk_editable_get_chars
-                       (GTK_EDITABLE (vw->name_entry), 0, -1));
-    gncAddressSetAddr1 (addr, gtk_editable_get_chars
-                        (GTK_EDITABLE (vw->addr1_entry), 0, -1));
-    gncAddressSetAddr2 (addr, gtk_editable_get_chars
-                        (GTK_EDITABLE (vw->addr2_entry), 0, -1));
-    gncAddressSetAddr3 (addr, gtk_editable_get_chars
-                        (GTK_EDITABLE (vw->addr3_entry), 0, -1));
-    gncAddressSetAddr4 (addr, gtk_editable_get_chars
-                        (GTK_EDITABLE (vw->addr4_entry), 0, -1));
-    gncAddressSetPhone (addr, gtk_editable_get_chars
-                        (GTK_EDITABLE (vw->phone_entry), 0, -1));
-    gncAddressSetFax (addr, gtk_editable_get_chars
-                      (GTK_EDITABLE (vw->fax_entry), 0, -1));
-    gncAddressSetEmail (addr, gtk_editable_get_chars
-                        (GTK_EDITABLE (vw->email_entry), 0, -1));
+    gncAddressSetName (addr, gtk_entry_get_text (GTK_ENTRY (vw->name_entry)));
+    gncAddressSetAddr1 (addr, gtk_entry_get_text (GTK_ENTRY (vw->addr1_entry)));
+    gncAddressSetAddr2 (addr, gtk_entry_get_text (GTK_ENTRY (vw->addr2_entry)));
+    gncAddressSetAddr3 (addr, gtk_entry_get_text (GTK_ENTRY (vw->addr3_entry)));
+    gncAddressSetAddr4 (addr, gtk_entry_get_text (GTK_ENTRY (vw->addr4_entry)));
+    gncAddressSetPhone (addr, gtk_entry_get_text (GTK_ENTRY (vw->phone_entry)));
+    gncAddressSetFax (addr, gtk_entry_get_text (GTK_ENTRY (vw->fax_entry)));
+    gncAddressSetEmail (addr, gtk_entry_get_text (GTK_ENTRY (vw->email_entry)));
 
     gncVendorSetActive (vendor, gtk_toggle_button_get_active
                         (GTK_TOGGLE_BUTTON (vw->active_check)));
@@ -211,23 +201,10 @@ gnc_vendor_window_ok_cb (GtkWidget *widget, gpointer data)
 
     /* Check for valid company name */
     if (check_entry_nonempty (vw->company_entry,
-                              _("You must enter a company name. "
-                                "If this vendor is an individual (and not a company) "
-                                "you should enter the same value for:\nIdentification "
-                                "- Company Name, and\nPayment Address - Name.")))
+                              _("The Company Name field cannot be left blank, please "
+                                "enter a company name or a person's name.")))
         return;
 
-    /* Make sure we have an address */
-    if (check_entry_nonempty (vw->addr1_entry, NULL) &&
-            check_entry_nonempty (vw->addr2_entry, NULL) &&
-            check_entry_nonempty (vw->addr3_entry, NULL) &&
-            check_entry_nonempty (vw->addr4_entry, NULL))
-    {
-        const char *msg = _("You must enter a payment address.");
-        gnc_error_dialog (gnc_ui_get_gtk_window (widget), "%s", msg);
-        return;
-    }
-
     /* Check for valid id and set one if necessary */
     if (g_strcmp0 (gtk_entry_get_text (GTK_ENTRY (vw->id_entry)), "") == 0)
     {
@@ -261,7 +238,8 @@ gnc_vendor_window_cancel_cb (GtkWidget *widget, gpointer data)
 void
 gnc_vendor_window_help_cb (GtkWidget *widget, gpointer data)
 {
-    gnc_gnome_help(HF_HELP, HL_USAGE_VENDOR);
+    VendorWindow *vw = data;
+    gnc_gnome_help (GTK_WINDOW(vw->dialog), DF_MANUAL, DL_USAGE_VENDOR);
 }
 
 void
@@ -289,29 +267,26 @@ void
 gnc_vendor_name_changed_cb (GtkWidget *widget, gpointer data)
 {
     VendorWindow *vw = data;
-    char *name, *id, *fullname, *title;
+    char *title;
 
     if (!vw)
         return;
 
-    name = gtk_editable_get_chars (GTK_EDITABLE (vw->company_entry), 0, -1);
+    const char *header = (vw->dialog_type == EDIT_VENDOR) ?
+        _("Edit Vendor") : _("New Vendor");
+
+    const char *name = gtk_entry_get_text (GTK_ENTRY (vw->company_entry));
     if (!name || *name == '\0')
-        name = g_strdup (_("<No name>"));
+        name = _("<No name>");
 
-    id = gtk_editable_get_chars (GTK_EDITABLE (vw->id_entry), 0, -1);
-
-    fullname = g_strconcat (name, " (", id, ")", (char *)NULL);
-
-    if (vw->dialog_type == EDIT_VENDOR)
-        title = g_strconcat (_("Edit Vendor"), " - ", fullname, (char *)NULL);
+    const char *id = gtk_entry_get_text (GTK_ENTRY (vw->id_entry));
+    if (id && *id)
+        title = g_strdup_printf ("%s - %s (%s)", header, name, id);
     else
-        title = g_strconcat (_("New Vendor"), " - ", fullname, (char *)NULL);
+        title = g_strdup_printf ("%s - %s", header, name);
 
     gtk_window_set_title (GTK_WINDOW (vw->dialog), title);
 
-    g_free (name);
-    g_free (id);
-    g_free (fullname);
     g_free (title);
 }
 
