@@ -57,6 +57,7 @@
 #include "gnc-html-history.h"
 #include "print-session.h"
 
+
 G_DEFINE_TYPE(GncHtmlWebkit, gnc_html_webkit, GNC_TYPE_HTML )
 
 static void gnc_html_webkit_dispose( GObject* obj );
@@ -92,21 +93,19 @@ static char error_404_body[] = N_("The specified URL could not be loaded.");
 #define GNC_PREF_RPT_DFLT_ZOOM "default-zoom"
 
 static gboolean webkit_decide_policy_cb (WebKitWebView* web_view,
-					 WebKitPolicyDecision *decision,
-					 WebKitPolicyDecisionType decision_type,
-					 gpointer user_data);
+                     WebKitPolicyDecision *decision,
+                     WebKitPolicyDecisionType decision_type,
+                     gpointer user_data);
 static void webkit_mouse_target_cb (WebKitWebView* web_view,
-				    WebKitHitTestResult *hit,
-				    guint modifiers, gpointer data);
-#if WEBKIT_MINOR_VERSION >= 8
+                     WebKitHitTestResult *hit,
+                     guint modifiers, gpointer data);
 static gboolean webkit_notification_cb (WebKitWebView *web_view,
-					WebKitNotification *note,
-					gpointer user_data);
-#endif
+                     WebKitNotification *note,
+                     gpointer user_data);
 static gboolean webkit_load_failed_cb (WebKitWebView *web_view,
-				       WebKitLoadEvent event,
-				       gchar *uri, GError *error,
-				       gpointer user_data);
+                     WebKitLoadEvent event,
+                     gchar *uri, GError *error,
+                     gpointer user_data);
 static void webkit_resource_load_started_cb (WebKitWebView *web_view,
                                              WebKitWebResource *resource,
                                              WebKitURIRequest *request,
@@ -119,7 +118,7 @@ static void impl_webkit_show_data( GncHtml* self, const gchar* data, int datalen
 static void impl_webkit_reload( GncHtml* self, gboolean force_rebuild );
 static void impl_webkit_copy_to_clipboard( GncHtml* self );
 static gboolean impl_webkit_export_to_file( GncHtml* self, const gchar* filepath );
-static void impl_webkit_print (GncHtml* self);
+static void impl_webkit_print (GncHtml* self,const gchar* jobname);
 static void impl_webkit_cancel( GncHtml* self );
 static void impl_webkit_set_parent( GncHtml* self, GtkWindow* parent );
 static void impl_webkit_default_zoom_changed(gpointer prefs, gchar *pref, gpointer user_data);
@@ -134,25 +133,20 @@ gnc_html_webkit_webview_new (void)
      GValue val = G_VALUE_INIT;
      GtkStateFlags state = gtk_style_context_get_state (style);
      gtk_style_context_get_property (style, GTK_STYLE_PROPERTY_FONT,
-				     state, &val);
+                     state, &val);
 
      if (G_VALUE_HOLDS_BOXED (&val))
      {
-	  const PangoFontDescription *font =
-	       (const PangoFontDescription*)g_value_get_boxed (&val);
-	  default_font_family = pango_font_description_get_family (font);
-	  g_value_unset (&val);
+      const PangoFontDescription *font =
+           (const PangoFontDescription*)g_value_get_boxed (&val);
+      default_font_family = pango_font_description_get_family (font);
      }
 /* Set default webkit settings */
      webkit_settings = webkit_web_view_get_settings (WEBKIT_WEB_VIEW (view));
      g_object_set (G_OBJECT(webkit_settings),
                    "default-charset", "utf-8",
-#if WEBKIT_MINOR_VERSION >= 10
                    "allow-file-access-from-file-urls", TRUE,
-#endif
-#if WEBKIT_MINOR_VERSION >= 14
                    "allow-universal-access-from-file-urls", TRUE,
-#endif
                    "enable-java", FALSE,
                    "enable-page-cache", FALSE,
                    "enable-plugins", FALSE,
@@ -162,10 +156,10 @@ gnc_html_webkit_webview_new (void)
                    NULL);
      if (default_font_family != NULL)
      {
-	  g_object_set (G_OBJECT (webkit_settings),
-			"default-font-family", default_font_family,
-			NULL);
+          g_object_set (G_OBJECT (webkit_settings),
+              "default-font-family", default_font_family, NULL);
      }
+     g_value_unset (&val);
      return view;
 }
 
@@ -186,7 +180,7 @@ gnc_html_webkit_init( GncHtmlWebkit* self )
 
      /* Scale everything up */
      zoom = gnc_prefs_get_float (GNC_PREFS_GROUP_GENERAL_REPORT,
-				 GNC_PREF_RPT_DFLT_ZOOM);
+                 GNC_PREF_RPT_DFLT_ZOOM);
      webkit_web_view_set_zoom_level (priv->web_view, zoom);
 
 
@@ -204,11 +198,9 @@ gnc_html_webkit_init( GncHtmlWebkit* self )
                        G_CALLBACK (webkit_mouse_target_cb),
                        self);
 
-#if WEBKIT_MINOR_VERSION >= 8
      g_signal_connect (priv->web_view, "show-notification",
                        G_CALLBACK (webkit_notification_cb),
                        self);
-#endif
 
      g_signal_connect (priv->web_view, "load-failed",
                        G_CALLBACK (webkit_load_failed_cb),
@@ -251,15 +243,9 @@ gnc_html_webkit_dispose( GObject* obj )
 
      if ( priv->web_view != NULL )
      {
-         // In Gtk Version 3.20 they relaxed the fact that the widget should be a
-         // direct child of the container otherwise it would be a critical error
-#if GTK_CHECK_VERSION(3,20,0)
-          gtk_container_remove( GTK_CONTAINER(priv->base.container),
-                                GTK_WIDGET(priv->web_view) );
-#else
-          GtkWidget *parent = gtk_widget_get_parent(GTK_WIDGET(priv->web_view));
-          gtk_container_remove( GTK_CONTAINER(priv->base.container), parent);
-#endif
+          gtk_container_remove (GTK_CONTAINER(priv->base.container),
+                                GTK_WIDGET(priv->web_view));
+
           priv->web_view = NULL;
      }
 
@@ -297,8 +283,8 @@ extract_base_name(URLType type, const gchar* path)
 {
      gchar       machine_rexp[] = "^(//[^/]*)/*(/.*)?$";
      gchar       path_rexp[] = "^/*(.*)/+([^/]*)$";
-     regex_t    compiled_m, compiled_p;
-     regmatch_t match[4];
+     regex_t     compiled_m, compiled_p;
+     regmatch_t  match[4];
      gchar       * machine = NULL, * location = NULL, * base = NULL;
      gchar       * basename = NULL;
 
@@ -330,7 +316,6 @@ extract_base_name(URLType type, const gchar* path)
                                          match[2].rm_eo - match[2].rm_so);
                }
           }
-
      }
      else
      {
@@ -479,7 +464,7 @@ handle_embedded_object( GncHtmlWebkit* self, gchar* html_str )
  * widget.
  ********************************************************************/
 
-static void
+static gboolean
 load_to_stream( GncHtmlWebkit* self, URLType type,
                 const gchar* location, const gchar* label )
 {
@@ -490,7 +475,7 @@ load_to_stream( GncHtmlWebkit* self, URLType type,
      DEBUG( "type %s, location %s, label %s", type ? type : "(null)",
             location ? location : "(null)", label ? label : "(null)");
 
-     g_return_if_fail( self != NULL );
+     g_return_val_if_fail( self != NULL, FALSE );
 
      if ( gnc_html_stream_handlers != NULL )
      {
@@ -499,7 +484,23 @@ load_to_stream( GncHtmlWebkit* self, URLType type,
           stream_handler = g_hash_table_lookup( gnc_html_stream_handlers, type );
           if ( stream_handler )
           {
-               gboolean ok = stream_handler( location, &fdata, &fdata_len );
+              GncHtml *weak_html = GNC_HTML(self);
+              gboolean ok;
+
+              g_object_add_weak_pointer(G_OBJECT(self),
+                                        (gpointer*)(&weak_html));
+              ok = stream_handler( location, &fdata, &fdata_len );
+
+              if (!weak_html) // will be NULL if self has been destroyed
+              {
+                  g_free (fdata);
+                  return FALSE;
+              }
+              else
+              {
+                  g_object_remove_weak_pointer(G_OBJECT(self),
+                                               (gpointer*)(&weak_html));
+              }
 
                if ( ok )
                {
@@ -533,7 +534,7 @@ load_to_stream( GncHtmlWebkit* self, URLType type,
                          g_strdup_printf( error_404_format,
                                           _(error_404_title), _(error_404_body) );
                     webkit_web_view_load_html (priv->web_view, fdata,
-					       BASE_URI_NAME);
+                           BASE_URI_NAME);
                }
 
                g_free( fdata );
@@ -546,8 +547,7 @@ load_to_stream( GncHtmlWebkit* self, URLType type,
                     }
                     /* No action required: Webkit jumps to the anchor on its own. */
                }
-
-               return;
+               return TRUE;
           }
      }
 
@@ -561,7 +561,7 @@ load_to_stream( GncHtmlWebkit* self, URLType type,
                {
                     if ( !https_allowed() )
                     {
-                         gnc_error_dialog( priv->base.parent, "%s",
+                        gnc_error_dialog (GTK_WINDOW (priv->base.parent), "%s",
                                            _("Secure HTTP access is disabled. "
                                              "You can enable it in the Network section of "
                                              "the Preferences dialog."));
@@ -571,7 +571,7 @@ load_to_stream( GncHtmlWebkit* self, URLType type,
 
                if ( !http_allowed() )
                {
-                    gnc_error_dialog( priv->base.parent, "%s",
+                   gnc_error_dialog (GTK_WINDOW (priv->base.parent), "%s",
                                       _("Network HTTP access is disabled. "
                                         "You can enable it in the Network section of "
                                         "the Preferences dialog."));
@@ -580,7 +580,6 @@ load_to_stream( GncHtmlWebkit* self, URLType type,
                {
                     gnc_build_url( type, location, label );
                }
-
           }
           else
           {
@@ -593,74 +592,72 @@ load_to_stream( GncHtmlWebkit* self, URLType type,
                webkit_web_view_load_html (priv->web_view, fdata, BASE_URI_NAME);
                g_free( fdata );
           }
-
      }
      while ( FALSE );
+     return TRUE;
 }
-#ifdef WEBKIT2_4
+
 static gboolean
 perform_navigation_policy (WebKitWebView *web_view,
-			   WebKitNavigationPolicyDecision *decision,
-			   GncHtml *self)
+               WebKitNavigationPolicyDecision *decision,
+               GncHtml *self)
 {
      WebKitURIRequest *req = NULL;
-     const gchar* uri; // Can't init it here.
-     gchar *scheme = NULL, *location = NULL, *label = NULL;
+     const gchar* uri, *scheme; // Can't init it here.
+     gchar *location = NULL, *label = NULL;
+     gboolean ignore = FALSE;
      WebKitNavigationAction *action =
-	  webkit_navigation_policy_decision_get_navigation_action (decision);
+      webkit_navigation_policy_decision_get_navigation_action (decision);
      if (webkit_navigation_action_get_navigation_type (action) !=
-	 WEBKIT_NAVIGATION_TYPE_LINK_CLICKED)
+         WEBKIT_NAVIGATION_TYPE_LINK_CLICKED)
      {
-	  webkit_policy_decision_use ((WebKitPolicyDecision*)decision);
-	  return TRUE;
+          webkit_policy_decision_use ((WebKitPolicyDecision*)decision);
+          return TRUE;
      }
      req = webkit_navigation_action_get_request (action);
      uri = webkit_uri_request_get_uri (req);
      scheme =  gnc_html_parse_url (self, uri, &location, &label);
-     impl_webkit_show_url (self, scheme, location, label, FALSE);
+     if (strcmp (scheme, URL_TYPE_FILE) != 0)
+     {
+          impl_webkit_show_url (self, scheme, location, label, FALSE);
+          ignore = TRUE;
+     }
      g_free (location);
      g_free (label);
-     webkit_policy_decision_ignore ((WebKitPolicyDecision*)decision);
+     if (ignore)
+          webkit_policy_decision_ignore ((WebKitPolicyDecision*)decision);
+     else
+          webkit_policy_decision_use ((WebKitPolicyDecision*)decision);
      return TRUE;
 }
-#endif
-/********************************************************************
- * webkit_navigation_requested_cb - called when a URL needs to be
- * loaded within the loading of a page (embedded image).
- ********************************************************************/
 
 static gboolean
 webkit_decide_policy_cb (WebKitWebView *web_view,
-			 WebKitPolicyDecision *decision,
-			 WebKitPolicyDecisionType decision_type,
-			 gpointer user_data)
+             WebKitPolicyDecision *decision,
+             WebKitPolicyDecisionType decision_type,
+             gpointer user_data)
 {
 /* This turns out to be the signal to intercept for handling a link-click. */
-#ifdef WEBKIT2_4
      if (decision_type != WEBKIT_POLICY_DECISION_TYPE_NAVIGATION_ACTION)
      {
-	  webkit_policy_decision_use (decision);
-	  return TRUE;
+          webkit_policy_decision_use (decision);
+          return TRUE;
      }
      return perform_navigation_policy (
-	  web_view, (WebKitNavigationPolicyDecision*) decision,
-	  GNC_HTML (user_data));
-#else
-     webkit_policy_decision_use (decision);
-     return TRUE;
-#endif
+      web_view, (WebKitNavigationPolicyDecision*) decision,
+      GNC_HTML (user_data));
 }
 
 static void
 webkit_mouse_target_cb (WebKitWebView *web_view, WebKitHitTestResult *hit,
-			guint modifiers, gpointer user_data)
+            guint modifiers, gpointer user_data)
 {
      GncHtmlWebkitPrivate* priv;
      GncHtmlWebkit *self = (GncHtmlWebkit*)user_data;
      gchar *uri;
 
      if (!webkit_hit_test_result_context_is_link (hit))
-	  return;
+         return;
 
      priv = GNC_HTML_WEBKIT_GET_PRIVATE (self);
      uri = g_strdup (webkit_hit_test_result_get_link_uri (hit));
@@ -669,13 +666,12 @@ webkit_mouse_target_cb (WebKitWebView *web_view, WebKitHitTestResult *hit,
      if (priv->base.flyover_cb)
      {
           (priv->base.flyover_cb) (GNC_HTML (self), uri,
-				   priv->base.flyover_cb_data);
+                   priv->base.flyover_cb_data);
      }
 }
-#if WEBKIT_MINOR_VERSION >= 8
 static gboolean
 webkit_notification_cb (WebKitWebView* web_view, WebKitNotification *note,
-			gpointer user_data)
+            gpointer user_data)
 {
      GtkWindow *top = NULL;
      GtkWidget *dialog = NULL;
@@ -693,7 +689,6 @@ webkit_notification_cb (WebKitWebView* web_view, WebKitNotification *note,
      gtk_widget_destroy (dialog);
      return TRUE;
 }
-#endif
 
 static gboolean
 webkit_load_failed_cb (WebKitWebView *web_view, WebKitLoadEvent event,
@@ -803,6 +798,7 @@ impl_webkit_show_url( GncHtml* self, URLType type,
      GncHTMLUrlCB url_handler;
      gboolean new_window;
      GncHtmlWebkitPrivate* priv;
+     gboolean stream_loaded = FALSE;
 
      g_return_if_fail( self != NULL );
      g_return_if_fail( GNC_IS_HTML_WEBKIT(self) );
@@ -853,18 +849,19 @@ impl_webkit_show_url( GncHtml* self, URLType type,
           result.base_type = URL_TYPE_FILE;
           result.base_location = NULL;
           result.error_message = NULL;
+          result.parent = GTK_WINDOW (priv->base.parent);
 
           ok = url_handler( location, label, new_window, &result );
           if ( !ok )
           {
                if ( result.error_message )
                {
-                    gnc_error_dialog( priv->base.parent, "%s", result.error_message );
+                   gnc_error_dialog (GTK_WINDOW (priv->base.parent), "%s", result.error_message );
                }
                else
                {
                     /* %s is a URL (some location somewhere). */
-                    gnc_error_dialog( priv->base.parent, _("There was an error accessing %s."), location );
+                    gnc_error_dialog (GTK_WINDOW (priv->base.parent), _("There was an error accessing %s."), location );
                }
 
                if ( priv->base.load_cb )
@@ -892,10 +889,11 @@ impl_webkit_show_url( GncHtml* self, URLType type,
                DEBUG( "resetting base location to %s",
                       priv->base.base_location ? priv->base.base_location : "(null)" );
 
-               load_to_stream( GNC_HTML_WEBKIT(self), result.url_type,
-                               new_location, new_label );
+               stream_loaded = load_to_stream( GNC_HTML_WEBKIT(self),
+                                               result.url_type,
+                                               new_location, new_label );
 
-               if ( priv->base.load_cb != NULL )
+               if ( stream_loaded && priv->base.load_cb != NULL )
                {
                     priv->base.load_cb( GNC_HTML(self), result.url_type,
                                         new_location, new_label, priv->base.load_cb_data );
@@ -930,7 +928,7 @@ impl_webkit_show_url( GncHtml* self, URLType type,
                {
                     if ( !https_allowed() )
                     {
-                         gnc_error_dialog( priv->base.parent, "%s",
+                        gnc_error_dialog (GTK_WINDOW (priv->base.parent), "%s",
                                            _("Secure HTTP access is disabled. "
                                              "You can enable it in the Network section of "
                                              "the Preferences dialog.") );
@@ -942,7 +940,7 @@ impl_webkit_show_url( GncHtml* self, URLType type,
                {
                     if ( !http_allowed() )
                     {
-                         gnc_error_dialog( priv->base.parent, "%s",
+                        gnc_error_dialog (GTK_WINDOW (priv->base.parent), "%s",
                                            _("Network HTTP access is disabled. "
                                              "You can enable it in the Network section of "
                                              "the Preferences dialog.") );
@@ -958,18 +956,18 @@ impl_webkit_show_url( GncHtml* self, URLType type,
                /* FIXME : handle new_window = 1 */
                gnc_html_history_append( priv->base.history,
                                         gnc_html_history_node_new( type, location, label ) );
-               load_to_stream( GNC_HTML_WEBKIT(self), type, location, label );
+               stream_loaded = load_to_stream( GNC_HTML_WEBKIT(self),
+                                               type, location, label );
 
           }
           while ( FALSE );
-
      }
      else
      {
           PERR( "URLType %s not supported.", type );
      }
 
-     if ( priv->base.load_cb != NULL )
+     if ( stream_loaded && priv->base.load_cb != NULL )
      {
           (priv->base.load_cb)( GNC_HTML(self), type, location, label, priv->base.load_cb_data );
      }
@@ -1055,7 +1053,7 @@ impl_webkit_copy_to_clipboard( GncHtml* self )
 
      priv = GNC_HTML_WEBKIT_GET_PRIVATE(self);
      webkit_web_view_execute_editing_command (priv->web_view,
-					      WEBKIT_EDITING_COMMAND_COPY);
+                          WEBKIT_EDITING_COMMAND_COPY);
 }
 
 /**************************************************************
@@ -1117,20 +1115,43 @@ impl_webkit_export_to_file( GncHtml* self, const char *filepath )
  * webkit_web_view_get_snapshot for each page.
  */
 static void
-impl_webkit_print (GncHtml* self)
+impl_webkit_print (GncHtml* self,const gchar* jobname)
 {
      WebKitPrintOperation *op = NULL;
      GtkWindow *top = NULL;
      GncHtmlWebkitPrivate *priv;
+     GtkPrintSettings *print_settings = NULL;
+     WebKitPrintOperationResponse print_response;
+     gchar *export_dirname = NULL;
+     gchar *export_filename = NULL;
+     gchar* basename = NULL;
 
      g_return_if_fail (self != NULL);
      g_return_if_fail (GNC_IS_HTML_WEBKIT (self));
-
      priv = GNC_HTML_WEBKIT_GET_PRIVATE (self);
      op = webkit_print_operation_new (priv->web_view);
-     top = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self)));
-     webkit_print_operation_run_dialog (op, top);
+     basename = g_path_get_basename(jobname);
+     print_settings = gtk_print_settings_new();
+     webkit_print_operation_set_print_settings(op, print_settings);
+     export_filename = g_strdup(jobname);
+     g_free(basename);
+     gtk_print_settings_set(print_settings,
+                    GTK_PRINT_SETTINGS_OUTPUT_BASENAME,
+                    export_filename);
+     webkit_print_operation_set_print_settings(op, print_settings);
+     // Open a print dialog
+     top = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (priv->web_view)));
+     print_response = webkit_print_operation_run_dialog (op, top);
+     if (print_response == WEBKIT_PRINT_OPERATION_RESPONSE_PRINT)
+     {
+          // Get the newly updated print settings
+          g_object_unref(print_settings);
+          print_settings = g_object_ref(webkit_print_operation_get_print_settings(op));
+     }
+     g_free(export_dirname);
+     g_free(export_filename);
      g_object_unref (op);
+     g_object_unref (print_settings);
 }
 
 static void
