@@ -29,8 +29,6 @@
 
 #include <guid.hpp>
 
-extern "C"
-{
 #include "config.h"
 
 #include <gtk/gtk.h>
@@ -50,7 +48,13 @@ extern "C"
 
 #include "gnc-csv-gnumeric-popup.h"
 #include "go-charmap-sel.h"
-}
+
+#include <algorithm>
+#include <exception>
+#include <iostream>
+#include <memory>
+#include <string>
+#include <tuple>
 
 #include "gnc-imp-settings-csv-price.hpp"
 #include "gnc-import-price.hpp"
@@ -66,7 +70,7 @@ static QofLogModule log_module = GNC_MOD_ASSISTANT;
 
 /* Note on memory management
  *
- * The same notes as for assistant-csv-trans-import.cpp appy to
+ * The same notes as for assistant-csv-trans-import.cpp apply to
  * this assistant as well. Please read the note at the top of that
  * file to understand important details about the use of several
  * memory management models in one file.
@@ -93,7 +97,8 @@ public:
     void assist_finish ();
     void assist_compmgr_close ();
 
-    void file_confirm_cb ();
+    void file_activated_cb ();
+    void file_selection_changed_cb ();
 
     void preview_settings_delete ();
     void preview_settings_save ();
@@ -133,12 +138,15 @@ private:
     GtkWidget* preview_cbox_factory (GtkTreeModel* model, uint32_t colnum);
     /* helper function to set rendering parameters for preview data columns */
     void preview_style_column (uint32_t col_num, GtkTreeModel* model);
+    /* helper function to check for a valid filename as opposed to a directory */
+    bool check_for_valid_filename ();
 
     GtkAssistant    *csv_imp_asst;
 
     GtkWidget       *file_page;                     /**< Assistant file page widget */
     GtkWidget       *file_chooser;                  /**< The widget for the file chooser */
-    std::string      m_file_name;                   /**< The import file name */
+    std::string      m_fc_file_name;                /**< The file name currently selected in the file chooser */
+    std::string      m_final_file_name;             /**< The name of the import file effectively to use */
 
     GtkWidget       *preview_page;                  /**< Assistant preview page widget */
     GtkComboBox     *settings_combo;                /**< The Settings Combo */
@@ -152,7 +160,7 @@ private:
     GtkWidget       *skip_errors_button;            /**< The widget for Skip error rows*/
     GtkWidget       *csv_button;                    /**< The widget for the CSV button */
     GtkWidget       *fixed_button;                  /**< The widget for the Fixed Width button */
-    GtkWidget       *over_write_cbutton;            /**< The widget for Price Over Write */
+    GtkWidget       *over_write_cbutton;            /**< The widget for Price Overwrite */
     GtkWidget       *commodity_selector;            /**< The widget for commodity combo box */
     GtkWidget       *currency_selector;             /**< The widget for currency combo box */
     GOCharmapSel    *encselector;                   /**< The widget for selecting the encoding */
@@ -190,7 +198,8 @@ extern "C"
 void csv_price_imp_assist_prepare_cb (GtkAssistant  *assistant, GtkWidget *page, CsvImpPriceAssist* info);
 void csv_price_imp_assist_close_cb (GtkAssistant *gtkassistant, CsvImpPriceAssist* info);
 void csv_price_imp_assist_finish_cb (GtkAssistant *gtkassistant, CsvImpPriceAssist* info);
-void csv_price_imp_file_confirm_cb (GtkWidget *button, CsvImpPriceAssist *info);
+void csv_price_imp_file_activated_changed_cb (GtkFileChooser *chooser, CsvImpPriceAssist *info);
+void csv_price_imp_file_selection_changed_cb (GtkFileChooser *chooser, CsvImpPriceAssist *info);
 void csv_price_imp_preview_del_settings_cb (GtkWidget *button, CsvImpPriceAssist *info);
 void csv_price_imp_preview_save_settings_cb (GtkWidget *button, CsvImpPriceAssist *info);
 void csv_price_imp_preview_settings_sel_changed_cb (GtkComboBox *combo, CsvImpPriceAssist *info);
@@ -228,9 +237,14 @@ csv_price_imp_assist_finish_cb (GtkAssistant *assistant, CsvImpPriceAssist* info
     info->assist_finish ();
 }
 
-void csv_price_imp_file_confirm_cb (GtkWidget *button, CsvImpPriceAssist *info)
+void csv_price_imp_file_activated_changed_cb (GtkFileChooser *chooser, CsvImpPriceAssist *info)
 {
-    info->file_confirm_cb();
+    info->file_activated_cb();
+}
+
+void csv_price_imp_file_selection_changed_cb (GtkFileChooser *chooser, CsvImpPriceAssist *info)
+{
+    info->file_selection_changed_cb();
 }
 
 void csv_price_imp_preview_del_settings_cb (GtkWidget *button, CsvImpPriceAssist *info)
@@ -326,11 +340,29 @@ static void csv_price_imp_preview_currency_fmt_sel_cb (GtkComboBox* format_selec
     info->preview_update_currency_format();
 }
 
+enum GncCommColumn {DISPLAYED_COMM, SORT_COMM, COMM_PTR, SEP};
+
 static void csv_price_imp_preview_currency_sel_cb (GtkComboBox* currency_selector, CsvImpPriceAssist* info)
 {
     info->preview_update_currency();
 }
 
+static gboolean separator_row_func (GtkTreeModel *smodel, GtkTreeIter *siter, gpointer data)
+{
+    gboolean      sep_row;
+    GtkTreeModel *store;
+    GtkTreeIter   iter;
+
+    store = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT(smodel));
+
+    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT(smodel),
+                                                    &iter, siter);
+
+    gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, SEP, &sep_row, -1);
+
+    return sep_row;
+}
+
 static void csv_price_imp_preview_commodity_sel_cb (GtkComboBox* commodity_selector, CsvImpPriceAssist* info)
 {
     info->preview_update_commodity();
@@ -366,7 +398,8 @@ gnc_commodity *get_commodity_from_combo (GtkComboBox *combo)
     gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT(sort_model),
                                                     &iter, &siter);
 
-    gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, 0, &string, 2, &comm, -1);
+    gtk_tree_model_get (GTK_TREE_MODEL(model), &iter,
+                        DISPLAYED_COMM, &string, COMM_PTR, &comm, -1);
 
     PINFO("Commodity string is %s", string);
 
@@ -388,7 +421,7 @@ set_commodity_for_combo (GtkComboBox *combo, gnc_commodity *comm)
 
     while (valid)
     {
-        gtk_tree_model_get (model, &iter, 2, &model_comm, -1);
+        gtk_tree_model_get (model, &iter, COMM_PTR, &model_comm, -1);
         if (model_comm == comm)
         {
             if (gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT(sort_model), &siter, &iter))
@@ -401,7 +434,7 @@ set_commodity_for_combo (GtkComboBox *combo, gnc_commodity *comm)
         valid = gtk_tree_model_iter_next (model, &iter);
     }
     // Not found, set it to first iter
-    valid = gtk_tree_model_get_iter_first (model, &iter);
+    gtk_tree_model_get_iter_first (model, &iter);
     if (gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT(sort_model), &siter, &iter))
         gtk_combo_box_set_active_iter (combo, &siter);
 }
@@ -417,13 +450,15 @@ GtkTreeModel *get_model (bool all_commodity)
     GList *namespace_list = gnc_commodity_table_get_namespaces (commodity_table);
     GtkTreeIter iter;
 
-    store = GTK_TREE_MODEL(gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER));
+    store = GTK_TREE_MODEL(gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_STRING,
+                                                  G_TYPE_POINTER, G_TYPE_BOOLEAN));
     model = gtk_tree_model_sort_new_with_model (store);
-    // set sort to sort on second string, first string will be shown
-    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model), 1, GTK_SORT_ASCENDING);
+    // set sort order
+    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(model), SORT_COMM, GTK_SORT_ASCENDING);
 
     gtk_list_store_append (GTK_LIST_STORE(store), &iter);
-    gtk_list_store_set (GTK_LIST_STORE(store), &iter, 0, " ", 1, " ", 2, nullptr, -1);
+    gtk_list_store_set (GTK_LIST_STORE(store), &iter,
+                            DISPLAYED_COMM, " ", SORT_COMM, " ", COMM_PTR, nullptr, SEP, false, -1);
 
     namespace_list = g_list_first (namespace_list);
     while (namespace_list != nullptr)
@@ -438,6 +473,15 @@ GtkTreeModel *get_model (bool all_commodity)
             {
                 commodity_list = gnc_commodity_table_get_commodities (commodity_table, tmp_namespace);
                 commodity_list  = g_list_first (commodity_list);
+
+                // if this is the CURRENCY, add a row to be identified as a separator row
+                if ((g_utf8_collate (tmp_namespace, GNC_COMMODITY_NS_CURRENCY) == 0) && (all_commodity == true))
+                {
+                    gtk_list_store_append (GTK_LIST_STORE(store), &iter);
+                    gtk_list_store_set (GTK_LIST_STORE(store), &iter, DISPLAYED_COMM, " ",
+                                           SORT_COMM, "CURRENCY-", COMM_PTR, nullptr, SEP, true, -1);
+                }
+
                 while (commodity_list != nullptr)
                 {
                     const gchar *name_str;
@@ -447,11 +491,16 @@ GtkTreeModel *get_model (bool all_commodity)
 
                     name_str = gnc_commodity_get_printname (tmp_commodity);
 
-                    sort_str = g_strconcat (tmp_namespace, "::", gnc_commodity_get_mnemonic (tmp_commodity), nullptr);
-                    DEBUG("Name string is %s, Sort string is %s", name_str, sort_str);
+                    if (g_utf8_collate (tmp_namespace, GNC_COMMODITY_NS_CURRENCY) == 0)
+                        sort_str = g_strconcat ("CURRENCY-", name_str, nullptr);
+                    else
+                        sort_str = g_strconcat ("ALL-OTHER-", name_str, nullptr);
+
+                    DEBUG("Name string is '%s', Sort string is '%s'", name_str, sort_str);
 
                     gtk_list_store_append (GTK_LIST_STORE(store), &iter);
-                    gtk_list_store_set (GTK_LIST_STORE(store), &iter, 0, name_str, 1, sort_str, 2, tmp_commodity, -1);
+                    gtk_list_store_set (GTK_LIST_STORE(store), &iter, DISPLAYED_COMM, name_str,
+                                           SORT_COMM, sort_str, COMM_PTR, tmp_commodity, SEP, false, -1);
 
                     g_free (sort_str);
                     commodity_list = g_list_next (commodity_list);
@@ -462,6 +511,7 @@ GtkTreeModel *get_model (bool all_commodity)
     }
     g_list_free (commodity_list);
     g_list_free (namespace_list);
+    g_object_unref (store);
 
     return model;
 }
@@ -480,6 +530,10 @@ CsvImpPriceAssist::CsvImpPriceAssist ()
     gnc_builder_add_from_file  (builder , "assistant-csv-price-import.glade", "CSV Price Assistant");
     csv_imp_asst = GTK_ASSISTANT(gtk_builder_get_object (builder, "CSV Price Assistant"));
 
+    // Set the name for this assistant so it can be easily manipulated with css
+    gtk_widget_set_name (GTK_WIDGET(csv_imp_asst), "gnc-id-assistant-csv-price-import");
+    gnc_widget_style_context_add_class (GTK_WIDGET(csv_imp_asst), "gnc-class-imports");
+
     /* Enable buttons on all page. */
     gtk_assistant_set_page_complete (csv_imp_asst,
                                      GTK_WIDGET(gtk_builder_get_object (builder, "start_page")),
@@ -500,18 +554,11 @@ CsvImpPriceAssist::CsvImpPriceAssist ()
     /* File chooser Page */
     file_page = GTK_WIDGET(gtk_builder_get_object (builder, "file_page"));
     file_chooser = gtk_file_chooser_widget_new (GTK_FILE_CHOOSER_ACTION_OPEN);
+
+    g_signal_connect (G_OBJECT(file_chooser), "selection-changed",
+                      G_CALLBACK(csv_price_imp_file_selection_changed_cb), this);
     g_signal_connect (G_OBJECT(file_chooser), "file-activated",
-                      G_CALLBACK(csv_price_imp_file_confirm_cb), this);
-    auto button = gtk_button_new_with_mnemonic (_("_OK"));
-    gtk_widget_set_size_request (button, 100, -1);
-    gtk_widget_show (button);
-    auto h_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-    gtk_box_set_homogeneous (GTK_BOX (h_box), TRUE);
-    gtk_widget_set_hexpand (GTK_WIDGET(h_box), TRUE);
-    gtk_box_pack_start (GTK_BOX(h_box), button, FALSE, FALSE, 0);
-    gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER(file_chooser), h_box);
-    g_signal_connect (G_OBJECT(button), "clicked",
-                      G_CALLBACK(csv_price_imp_file_confirm_cb), this);
+                      G_CALLBACK(csv_price_imp_file_activated_changed_cb), this);
 
     auto box = GTK_WIDGET(gtk_builder_get_object (builder, "file_page"));
     gtk_box_pack_start (GTK_BOX(box), file_chooser, TRUE, TRUE, 6);
@@ -524,6 +571,8 @@ CsvImpPriceAssist::CsvImpPriceAssist ()
         // Add Settings combo
         auto settings_store = gtk_list_store_new (2, G_TYPE_POINTER, G_TYPE_STRING);
         settings_combo = GTK_COMBO_BOX(gtk_combo_box_new_with_model_and_entry (GTK_TREE_MODEL(settings_store)));
+        g_object_unref (settings_store);
+
         gtk_combo_box_set_entry_text_column (GTK_COMBO_BOX(settings_combo), SET_NAME);
         gtk_combo_box_set_active (GTK_COMBO_BOX(settings_combo), 0);
 
@@ -587,13 +636,16 @@ CsvImpPriceAssist::CsvImpPriceAssist ()
 
         auto encoding_container = GTK_CONTAINER(gtk_builder_get_object (builder, "encoding_container"));
         gtk_container_add (encoding_container, GTK_WIDGET(encselector));
+        gtk_widget_set_hexpand (GTK_WIDGET(encselector), true);
         gtk_widget_show_all (GTK_WIDGET(encoding_container));
 
         /* Add commodity selection widget */
         commodity_selector = GTK_WIDGET(gtk_builder_get_object (builder, "commodity_cbox"));
         gtk_combo_box_set_model (GTK_COMBO_BOX(commodity_selector), get_model (true));
-        g_signal_connect(G_OBJECT(commodity_selector), "changed",
-                         G_CALLBACK(csv_price_imp_preview_commodity_sel_cb), this);
+        gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX(commodity_selector),
+                                              separator_row_func, nullptr, nullptr);
+        g_signal_connect (G_OBJECT(commodity_selector), "changed",
+                          G_CALLBACK(csv_price_imp_preview_commodity_sel_cb), this);
 
         /* Add currency selection widget */
         currency_selector = GTK_WIDGET(gtk_builder_get_object (builder, "currency_cbox"));
@@ -616,6 +668,7 @@ CsvImpPriceAssist::CsvImpPriceAssist ()
         /* Add it to the assistant. */
         auto date_format_container = GTK_CONTAINER(gtk_builder_get_object (builder, "date_format_container"));
         gtk_container_add (date_format_container, GTK_WIDGET(date_format_combo));
+        gtk_widget_set_hexpand (GTK_WIDGET(date_format_combo), true);
         gtk_widget_show_all (GTK_WIDGET(date_format_container));
 
         /* Add in the currency format combo box and hook it up to an event handler. */
@@ -632,6 +685,7 @@ CsvImpPriceAssist::CsvImpPriceAssist ()
         /* Add it to the assistant. */
         auto currency_format_container = GTK_CONTAINER(gtk_builder_get_object (builder, "currency_format_container"));
         gtk_container_add (currency_format_container, GTK_WIDGET(currency_format_combo));
+        gtk_widget_set_hexpand (GTK_WIDGET(currency_format_combo), true);
         gtk_widget_show_all (GTK_WIDGET(currency_format_container));
 
         /* Connect the CSV/Fixed-Width radio button event handler. */
@@ -654,7 +708,8 @@ CsvImpPriceAssist::CsvImpPriceAssist ()
     summary_page  = GTK_WIDGET(gtk_builder_get_object (builder, "summary_page"));
     summary_label = GTK_WIDGET(gtk_builder_get_object (builder, "summary_label"));
 
-    gnc_restore_window_size (GNC_PREFS_GROUP, GTK_WINDOW(csv_imp_asst));
+    gnc_restore_window_size (GNC_PREFS_GROUP,
+                             GTK_WINDOW(csv_imp_asst), gnc_ui_get_main_window(nullptr));
 
     gtk_builder_connect_signals (builder, this);
     g_object_unref (G_OBJECT(builder));
@@ -675,60 +730,61 @@ CsvImpPriceAssist::~CsvImpPriceAssist ()
  * Code related to the file chooser page
  **************************************************/
 
-/* csv_price_imp_file_confirm_cb
- *
- * call back for ok button in file chooser widget
+/* check_for_valid_filename for a valid file to activate the "Next" button
  */
-void
-CsvImpPriceAssist::file_confirm_cb ()
+bool
+CsvImpPriceAssist::check_for_valid_filename ()
 {
     auto file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(file_chooser));
-    if (!file_name)
-        return;
+    if (!file_name || g_file_test (file_name, G_FILE_TEST_IS_DIR))
+    {
+        g_free (file_name);
+        return false;
+    }
 
     auto filepath = gnc_uri_get_path (file_name);
     auto starting_dir = g_path_get_dirname (filepath);
 
-    m_file_name = file_name;
+    m_fc_file_name = file_name;
     gnc_set_default_directory (GNC_PREFS_GROUP, starting_dir);
 
-    DEBUG("file_name selected is %s", m_file_name.c_str());
+    DEBUG("file_name selected is %s", m_fc_file_name.c_str());
     DEBUG("starting directory is %s", starting_dir);
 
     g_free (filepath);
     g_free (file_name);
     g_free (starting_dir);
 
-    /* Load the file into parse_data. */
-    price_imp = std::unique_ptr<GncPriceImport>(new GncPriceImport);
-    /* Assume data is CSV. User can later override to Fixed Width if needed */
-    try
-    {
-        price_imp->file_format (GncImpFileFormat::CSV);
-        price_imp->load_file (m_file_name);
-        price_imp->tokenize (true);
-    }
-    catch (std::ifstream::failure& e)
-    {
-        /* File loading failed ... */
-        gnc_error_dialog (GTK_WINDOW(csv_imp_asst), "%s", e.what());
-        return;
-    }
-    catch (std::range_error &e)
-    {
-        /* Parsing failed ... */
-        gnc_error_dialog (GTK_WINDOW(csv_imp_asst), "%s", _(e.what()));
-        return;
-    }
-    /* Get settings store and populate */
-    preview_populate_settings_combo();
-    gtk_combo_box_set_active (settings_combo, 0);
+    return true;
+}
 
-    // set over_write to false as default
-    price_imp->over_write (false);
+/* csv_price_imp_file_activated_cb
+ *
+ * call back for file chooser widget
+ */
+void
+CsvImpPriceAssist::file_activated_cb ()
+{
+    gtk_assistant_set_page_complete (csv_imp_asst, file_page, false);
 
-    auto num = gtk_assistant_get_current_page (csv_imp_asst);
-    gtk_assistant_set_current_page (csv_imp_asst, num + 1);
+    /* Test for a valid filename and not a directory */
+    if (check_for_valid_filename ())
+    {
+        gtk_assistant_set_page_complete (csv_imp_asst, file_page, true);
+        gtk_assistant_next_page (csv_imp_asst);
+    }
+}
+
+/* csv_price_imp_file_selection_changed_cb
+ *
+ * call back for file chooser widget
+ */
+void
+CsvImpPriceAssist::file_selection_changed_cb ()
+{
+    /* Enable the "Next" button based on a valid filename */
+    gtk_assistant_set_page_complete (csv_imp_asst, file_page,
+        check_for_valid_filename ());
 }
 
 
@@ -746,7 +802,7 @@ void CsvImpPriceAssist::preview_populate_settings_combo()
 
     // Append the default entry
     auto presets = get_import_presets_price ();
-    for (auto preset : presets)
+    for (const auto& preset : presets)
     {
         GtkTreeIter iter;
         gtk_list_store_append (GTK_LIST_STORE(model), &iter);
@@ -864,7 +920,6 @@ CsvImpPriceAssist::preview_settings_delete ()
 void
 CsvImpPriceAssist::preview_settings_save ()
 {
-    auto title = _("Save the Import Settings.");
     auto new_name = price_imp->settings_name();
 
     /* Check if the entry text matches an already existing preset */
@@ -884,7 +939,7 @@ CsvImpPriceAssist::preview_settings_save ()
             {
                 auto response = gnc_ok_cancel_dialog (GTK_WINDOW(csv_imp_asst),
                         GTK_RESPONSE_OK,
-                        "%s", _("Setting name already exists, over write?"));
+                        "%s", _("Setting name already exists, overwrite?"));
                 if (response != GTK_RESPONSE_OK)
                     return;
 
@@ -948,7 +1003,7 @@ void CsvImpPriceAssist::preview_update_skipped_rows ()
     preview_refresh_table ();
 }
 
-/* Callback triggered when user clicks on Over Write option
+/* Callback triggered when user clicks on Overwrite option
  */
 void CsvImpPriceAssist::preview_over_write (bool over)
 {
@@ -990,6 +1045,11 @@ void CsvImpPriceAssist::preview_update_separators (GtkWidget* widget)
     /* Set the parse options using the checked_separators list. */
     price_imp->separators (checked_separators);
 
+    /* if there are no separators, there will only be one column
+     * so make sure column header is NONE */
+    if (checked_separators.empty())
+        price_imp->set_column_type_price (0, GncPricePropType::NONE);
+
     /* Parse the data using the new options. We don't want to reguess
      * the column types because we want to leave the user's
      * configurations intact. */
@@ -1193,11 +1253,13 @@ void CsvImpPriceAssist::preview_update_col_type (GtkComboBox* cbox)
     if (old_col_type == GncPricePropType::TO_CURRENCY)
     {
         // look for a from_commodity column to reparse
-        preview_reparse_col_type (GncPricePropType::FROM_COMMODITY);
+        preview_reparse_col_type (GncPricePropType::FROM_SYMBOL);
+        preview_reparse_col_type (GncPricePropType::FROM_NAMESPACE);
     }
 
-    // if old_col_type is FROM_COMMODITY, force a reparse of currency
-    if (old_col_type == GncPricePropType::FROM_COMMODITY)
+    // if old_col_type is FROM_SYMBOL, or FROM_NAMESPACE force a reparse of currency
+    if ((old_col_type == GncPricePropType::FROM_SYMBOL) ||
+        (old_col_type == GncPricePropType::FROM_NAMESPACE))
     {
         // look for a to_currency column to reparse
         preview_reparse_col_type (GncPricePropType::TO_CURRENCY);
@@ -1590,9 +1652,7 @@ void CsvImpPriceAssist::preview_refresh_table ()
 
     /* Start with counting the current number of columns (ntcols)
      * we have in the treeview */
-    auto columns = gtk_tree_view_get_columns (treeview);
-    auto ntcols = g_list_length(columns);
-    g_list_free (columns);
+    auto ntcols = gtk_tree_view_get_n_columns (treeview);
 
     /* Drop redundant columns if the model has less data columns than the new model
      * ntcols = n° of columns in treeview (1 error column + x data columns)
@@ -1608,10 +1668,9 @@ void CsvImpPriceAssist::preview_refresh_table ()
     /* Insert columns if the model has more data columns than the treeview. */
     while (ntcols < ncols - PREV_N_FIXED_COLS + 1)
     {
-        /* Default cell renderer is text, except for the first (error) column */
-        auto renderer = gtk_cell_renderer_text_new();
-        if (ntcols == 0)
-            renderer = gtk_cell_renderer_pixbuf_new(); // Error column uses an icon
+        /* Default cell renderer is text, except for the first (error)
+           column uses an icon */
+        auto renderer = (ntcols == 0) ? gtk_cell_renderer_pixbuf_new () : gtk_cell_renderer_text_new ();
         auto col = gtk_tree_view_column_new ();
         gtk_tree_view_column_pack_start (col, renderer, false);
         ntcols = gtk_tree_view_append_column (treeview, col);
@@ -1623,11 +1682,24 @@ void CsvImpPriceAssist::preview_refresh_table ()
         preview_style_column (i, combostore);
 
     auto column_types = price_imp->column_types_price();
+    auto any_of_type = [](std::vector<GncPricePropType>& column_types,
+                          GncPricePropType req_column_type) -> bool
+    {
+        return std::any_of (column_types.begin(), column_types.end(),
+                            [&req_column_type](GncPricePropType column_type) -> bool
+                            { return column_type == req_column_type; });
+    };
 
-    // look for a commodity column, clear the commodity combo
-    auto col_type_comm = std::find (column_types.begin(),
-                column_types.end(), GncPricePropType::FROM_COMMODITY);
-    if (col_type_comm != column_types.end())
+    // look for a namespace column, clear the commodity combo
+    if (any_of_type (column_types, GncPricePropType::FROM_NAMESPACE))
+    {
+        g_signal_handlers_block_by_func (commodity_selector, (gpointer) csv_price_imp_preview_commodity_sel_cb, this);
+        set_commodity_for_combo (GTK_COMBO_BOX(commodity_selector), nullptr);
+        g_signal_handlers_unblock_by_func (commodity_selector, (gpointer) csv_price_imp_preview_commodity_sel_cb, this);
+    }
+
+    // look for a symbol column, clear the commodity combo
+    if (any_of_type (column_types, GncPricePropType::FROM_SYMBOL))
     {
         g_signal_handlers_block_by_func (commodity_selector, (gpointer) csv_price_imp_preview_commodity_sel_cb, this);
         set_commodity_for_combo (GTK_COMBO_BOX(commodity_selector), nullptr);
@@ -1635,9 +1707,7 @@ void CsvImpPriceAssist::preview_refresh_table ()
     }
 
     // look for a currency column, clear the currency combo
-    auto col_type_curr = std::find (column_types.begin(),
-                column_types.end(), GncPricePropType::TO_CURRENCY);
-    if (col_type_curr != column_types.end())
+    if (any_of_type (column_types, GncPricePropType::TO_CURRENCY))
     {
         g_signal_handlers_block_by_func (currency_selector, (gpointer) csv_price_imp_preview_currency_sel_cb, this);
         set_commodity_for_combo (GTK_COMBO_BOX(currency_selector), nullptr);
@@ -1658,25 +1728,34 @@ void CsvImpPriceAssist::preview_refresh_table ()
 void
 CsvImpPriceAssist::preview_refresh ()
 {
+    // Cache skip settings. Updating the widgets one by one
+    // triggers a callback that transfers all skip widgets'
+    // values to settings. So by the time the next widget value
+    // is to be set, that widget's 'new' setting has already been
+    // replaced by its old setting preveneting us from using it
+    // here sensibly.
+
+    auto skip_start_lines = price_imp->skip_start_lines();
+    auto skip_end_lines = price_imp->skip_end_lines();
+    auto skip_alt_lines = price_imp->skip_alt_lines();
+
     // Set start row
     auto adj = gtk_spin_button_get_adjustment (start_row_spin);
     gtk_adjustment_set_upper (adj, price_imp->m_parsed_lines.size());
-    gtk_spin_button_set_value (start_row_spin,
-            price_imp->skip_start_lines());
+    gtk_spin_button_set_value (start_row_spin, skip_start_lines);
 
     // Set end row
     adj = gtk_spin_button_get_adjustment (end_row_spin);
     gtk_adjustment_set_upper (adj, price_imp->m_parsed_lines.size());
-    gtk_spin_button_set_value (end_row_spin,
-            price_imp->skip_end_lines());
+    gtk_spin_button_set_value (end_row_spin, skip_end_lines);
 
     // Set Alternate rows
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(skip_alt_rows_button),
-            price_imp->skip_alt_lines());
+                                  skip_alt_lines);
 
     // Set over-write indicator
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(over_write_cbutton),
-            price_imp->over_write());
+                                  price_imp->over_write());
 
     // Set Import Format
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(csv_button),
@@ -1699,13 +1778,21 @@ CsvImpPriceAssist::preview_refresh ()
             price_imp->to_currency());
 
     // Handle separator checkboxes and custom field, only relevant if the file format is csv
+    // Note we defer the change signal until all buttons have been updated
+    // An early update may result in an incomplete tokenize run that would
+    // cause our list of saved column types to be truncated
     if (price_imp->file_format() == GncImpFileFormat::CSV)
     {
         auto separators = price_imp->separators();
         const auto stock_sep_chars = std::string (" \t,:;-");
+
         for (int i = 0; i < SEP_NUM_OF_TYPES; i++)
+        {
+            g_signal_handlers_block_by_func (sep_button[i], (gpointer) csv_price_imp_preview_sep_button_cb, this);
             gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(sep_button[i]),
                 separators.find (stock_sep_chars[i]) != std::string::npos);
+            g_signal_handlers_unblock_by_func (sep_button[i], (gpointer) csv_price_imp_preview_sep_button_cb, this);
+        }
 
         // If there are any other separators in the separators string,
         // add them as custom separators
@@ -1715,9 +1802,16 @@ CsvImpPriceAssist::preview_refresh ()
             separators.erase(pos);
             pos = separators.find_first_of (stock_sep_chars);
         }
+        g_signal_handlers_block_by_func (custom_cbutton, (gpointer) csv_price_imp_preview_sep_button_cb, this);
+        g_signal_handlers_block_by_func (custom_entry, (gpointer) csv_price_imp_preview_sep_button_cb, this);
+
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(custom_cbutton),
-                !separators.empty());
+                                      !separators.empty());
         gtk_entry_set_text (GTK_ENTRY(custom_entry), separators.c_str());
+
+        g_signal_handlers_unblock_by_func (custom_cbutton, (gpointer) csv_price_imp_preview_sep_button_cb, this);
+        g_signal_handlers_unblock_by_func (custom_entry, (gpointer) csv_price_imp_preview_sep_button_cb, this);
+        csv_price_imp_preview_sep_button_cb (GTK_WIDGET(custom_cbutton), this);
     }
     // Repopulate the parsed data table
     g_idle_add ((GSourceFunc)csv_imp_preview_queue_rebuild_table, this);
@@ -1741,23 +1835,76 @@ void CsvImpPriceAssist::preview_validate_settings ()
 void
 CsvImpPriceAssist::assist_file_page_prepare ()
 {
+    /* Disable the "Next" Assistant Button */
+    gtk_assistant_set_page_complete (csv_imp_asst, file_page, false);
+    gtk_assistant_set_page_complete (csv_imp_asst, preview_page, false);
+
     /* Set the default directory */
-    auto starting_dir = gnc_get_default_directory (GNC_PREFS_GROUP);
-    if (starting_dir)
+    if (!m_final_file_name.empty())
+        gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(file_chooser),
+                                       m_final_file_name.c_str());
+    else
     {
-        gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(file_chooser), starting_dir);
-        g_free (starting_dir);
+        auto starting_dir = gnc_get_default_directory (GNC_PREFS_GROUP);
+        if (starting_dir)
+        {
+            gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(file_chooser), starting_dir);
+            g_free (starting_dir);
+        }
     }
 }
 
 void
 CsvImpPriceAssist::assist_preview_page_prepare ()
 {
-    /* Disable the Forward Assistant Button */
-    gtk_assistant_set_page_complete (csv_imp_asst, preview_page, false);
+    auto go_back = false;
 
-    /* Load the data into the treeview. */
-    preview_refresh_table ();
+
+    if (m_final_file_name != m_fc_file_name)
+    {
+        /* Load the file into parse_data. */
+        price_imp = std::unique_ptr<GncPriceImport>(new GncPriceImport);
+        /* Assume data is CSV. User can later override to Fixed Width if needed */
+        try
+        {
+            price_imp->file_format (GncImpFileFormat::CSV);
+            price_imp->load_file (m_fc_file_name);
+            price_imp->tokenize (true);
+
+            /* Get settings store and populate */
+            preview_populate_settings_combo();
+            gtk_combo_box_set_active (settings_combo, 0);
+
+            // set over_write to false as default
+            price_imp->over_write (false);
+
+            /* Disable the "Next" Assistant Button */
+            gtk_assistant_set_page_complete (csv_imp_asst, preview_page, false);
+        }
+        catch (std::ifstream::failure& e)
+        {
+            /* File loading failed ... */
+            gnc_error_dialog (GTK_WINDOW(csv_imp_asst), "%s", e.what());
+            go_back = true;
+        }
+        catch (std::range_error &e)
+        {
+            /* Parsing failed ... */
+            gnc_error_dialog (GTK_WINDOW(csv_imp_asst), "%s", _(e.what()));
+            go_back = true;
+        }
+    }
+
+    if (go_back)
+        gtk_assistant_previous_page (csv_imp_asst);
+    else
+    {
+        m_final_file_name = m_fc_file_name;
+        preview_refresh ();
+
+        /* Load the data into the treeview. */
+        g_idle_add ((GSourceFunc)csv_imp_preview_queue_rebuild_table, this);
+    }
 }
 
 void
@@ -1774,24 +1921,24 @@ CsvImpPriceAssist::assist_summary_page_prepare ()
     auto added_str = g_strdup_printf (ngettext ("%d added price",
                                                 "%d added prices",
                                                 price_imp->m_prices_added),
-                                      price_imp->m_prices_added);
+                                                price_imp->m_prices_added);
     /* Translators: This is a ngettext(3) message, %d is the number of duplicate prices */
     auto dupl_str = g_strdup_printf (ngettext ("%d duplicate price",
                                                "%d duplicate prices",
                                                price_imp->m_prices_duplicated),
-                                     price_imp->m_prices_duplicated);
+                                               price_imp->m_prices_duplicated);
     /* Translators: This is a ngettext(3) message, %d is the number of replaced prices */
     auto repl_str = g_strdup_printf (ngettext ("%d replaced price",
                                                "%d replaced prices",
                                                price_imp->m_prices_replaced),
-                                     price_imp->m_prices_replaced);
+                                               price_imp->m_prices_replaced);
     auto msg = g_strdup_printf (
         _("The prices were imported from file '%s'.\n\n"
           "Import summary:\n"
           "- %s\n"
           "- %s\n"
           "- %s"),
-          m_file_name.c_str(), added_str, dupl_str,repl_str);
+          m_final_file_name.c_str(), added_str, dupl_str,repl_str);
     text += msg;
     text += "</b></span>";
 
