@@ -265,22 +265,6 @@ gnc_entry_ledger_config_cells (GncEntryLedger *ledger)
     ((ComboCell *)
      gnc_table_layout_get_cell (ledger->table->layout, ENTRY_ACTN_CELL), FALSE);
 
-    /* Use 6 decimal places for all prices and quantities */
-    gnc_price_cell_set_fraction
-    ((PriceCell *)
-     gnc_table_layout_get_cell (ledger->table->layout, ENTRY_PRIC_CELL),
-     1000000);
-
-    gnc_price_cell_set_fraction
-    ((PriceCell *)
-     gnc_table_layout_get_cell (ledger->table->layout, ENTRY_DISC_CELL),
-     1000000);
-
-    gnc_price_cell_set_fraction
-    ((PriceCell *) gnc_table_layout_get_cell (ledger->table->layout,
-            ENTRY_QTY_CELL),
-     1000000);
-
     /* add menu items for the action and payment cells */
     gnc_entry_ledger_config_action (ledger);
 }
@@ -567,7 +551,7 @@ void gnc_entry_ledger_set_default_invoice (GncEntryLedger *ledger,
      * adding bills on a day different from the bill's own date.
      * Note this is for bills only, because for (customer's) invoices
      * it makes more sense to use the current date.
-     * Consult https://bugzilla.gnome.org/show_bug.cgi?id=646541
+     * Consult https://bugs.gnucash.org/show_bug.cgi?id=646541
      * to understand why.
      */
     if (gncInvoiceGetOwnerType (invoice) == GNC_OWNER_VENDOR)
@@ -721,7 +705,7 @@ void
 gnc_entry_ledger_compute_value (GncEntryLedger *ledger,
                                 gnc_numeric *value, gnc_numeric *tax_value)
 {
-    gnc_numeric qty, price, discount;
+    gnc_numeric qty = gnc_numeric_zero(), price = gnc_numeric_zero(), discount = gnc_numeric_zero();
     gint disc_type, disc_how;
     gboolean taxable, taxincluded;
     GncTaxTable *table;
@@ -736,11 +720,16 @@ gnc_entry_ledger_compute_value (GncEntryLedger *ledger,
     disc_type = gnc_entry_ledger_get_type (ledger, ENTRY_DISTYPE_CELL);
     disc_how = gnc_entry_ledger_get_type (ledger, ENTRY_DISHOW_CELL);
 
-    /* Bills and exp-vouchers dont have discounts */
+    /* Some ledger types (see full list in gnc_entry_ledger_set_cells)
+       don't have discounts */
     if (ledger->type == GNCENTRY_BILL_ENTRY ||
-            ledger->type == GNCENTRY_BILL_VIEWER ||
-            ledger->type == GNCENTRY_EXPVOUCHER_ENTRY ||
-            ledger->type == GNCENTRY_EXPVOUCHER_VIEWER)
+        ledger->type == GNCENTRY_BILL_VIEWER ||
+        ledger->type == GNCENTRY_VEND_CREDIT_NOTE_ENTRY ||
+        ledger->type == GNCENTRY_VEND_CREDIT_NOTE_VIEWER ||
+        ledger->type == GNCENTRY_EXPVOUCHER_ENTRY ||
+        ledger->type == GNCENTRY_EXPVOUCHER_VIEWER ||
+        ledger->type == GNCENTRY_EMPL_CREDIT_NOTE_ENTRY ||
+        ledger->type == GNCENTRY_EMPL_CREDIT_NOTE_VIEWER)
     {
         g_assert (gnc_numeric_zero_p (discount));
         disc_type = GNC_AMT_TYPE_VALUE;
@@ -788,6 +777,7 @@ gnc_entry_ledger_compute_value (GncEntryLedger *ledger,
 
     /* return the tax value */
     taxes_unrounded = gncAccountValueTotal (taxes);
+    gncAccountValueDestroy (taxes);
     if (tax_value)
         *tax_value = gnc_numeric_convert (taxes_unrounded, denom,
                                           GNC_HOW_RND_ROUND_HALF_UP);
@@ -1036,15 +1026,15 @@ void gnc_entry_ledger_move_current_entry_updown (GncEntryLedger *ledger,
             return;
 
         /* Special treatment if the equality doesn't hold if we access the
-        dates as timespec. See the comment in gncEntrySetDateGDate() for the
-        reason: Some code used the timespec at noon for the EntryDate, other
-        code used the timespec at the start of day. */
+        dates as time64. See the comment in gncEntrySetDateGDate() for the
+        reason: Some code used the time64 at noon for the EntryDate, other
+        code used the time64 at the start of day. */
         t1 = gncEntryGetDate(current);
         t2 = gncEntryGetDate(target);
         if (t1 != t2)
         {
-            /* Timespecs are not equal, even though the GDates were equal? Then
-            we set the GDates again. This will force the timespecs to be equal
+            /* times are not equal, even though the GDates were equal? Then
+            we set the GDates again. This will force the times to be equal
             as well. */
             gncEntrySetDateGDate(current, &d1);
             gncEntrySetDateGDate(target, &d2);
