@@ -25,7 +25,11 @@
 #include <config.h>
 
 #include <glib/gi18n.h>
+#include <stdbool.h>
+#include <stdio.h>
 
+#include "Account.h"
+#include "Transaction.h"
 #include "account-quickfill.h"
 #include "combocell.h"
 #include "gnc-component-manager.h"
@@ -34,7 +38,7 @@
 #include "gnc-gui-query.h"
 #include "numcell.h"
 #include "quickfillcell.h"
-#include "assoccell.h"
+#include "doclinkcell.h"
 #include "recncell.h"
 #include "split-register.h"
 #include "split-register-p.h"
@@ -50,6 +54,7 @@ static QofLogModule log_module = GNC_MOD_LEDGER;
 static void gnc_split_register_load_xfer_cells (SplitRegister* reg,
                                                 Account* base_account);
 
+static void gnc_split_register_load_desc_cells (SplitRegister* reg);
 static void
 gnc_split_register_load_recn_cells (SplitRegister* reg)
 {
@@ -70,24 +75,24 @@ gnc_split_register_load_recn_cells (SplitRegister* reg)
 }
 
 static void
-gnc_split_register_load_associate_cells (SplitRegister* reg)
+gnc_split_register_load_doclink_cells (SplitRegister* reg)
 {
-    AssocCell *cell;
+    Doclinkcell *cell;
     const char * s;
 
     if (!reg) return;
 
-    cell = (AssocCell *)
-           gnc_table_layout_get_cell (reg->table->layout, ASSOC_CELL);
+    cell = (Doclinkcell *)
+           gnc_table_layout_get_cell (reg->table->layout, DOCLINK_CELL);
 
     if (!cell) return;
 
     /* FIXME: These should get moved to an i18n function */
-    s = gnc_get_association_valid_flags ();
-    gnc_assoc_cell_set_valid_flags (cell, s, ' ');
-    gnc_assoc_cell_set_flag_order (cell, gnc_get_association_flag_order ());
-    gnc_assoc_cell_set_string_getter (cell, gnc_get_association_str);
-    gnc_assoc_cell_set_read_only (cell, TRUE);
+    s = gnc_get_doclink_valid_flags ();
+    gnc_doclink_cell_set_valid_flags (cell, s, ' ');
+    gnc_doclink_cell_set_flag_order (cell, gnc_get_doclink_flag_order ());
+    gnc_doclink_cell_set_string_getter (cell, gnc_get_doclink_str);
+    gnc_doclink_cell_set_read_only (cell, TRUE);
 }
 
 static void
@@ -105,6 +110,7 @@ gnc_split_register_load_type_cells (SplitRegister* reg)
     /* FIXME: These should get moved to an i18n function */
     gnc_recn_cell_set_valid_flags (cell, "IP?", 'I');
     gnc_recn_cell_set_flag_order (cell, "IP");
+    gnc_recn_cell_set_read_only (cell, TRUE);
 }
 
 /** Add a transaction to the register.
@@ -238,13 +244,6 @@ _find_split_with_parent_txn (gconstpointer a, gconstpointer b)
 static void add_quickfill_completions (TableLayout* layout, Transaction* trans,
                                        Split* split, gboolean has_last_num)
 {
-    Split* s;
-    int i = 0;
-
-    gnc_quickfill_cell_add_completion (
-        (QuickFillCell*) gnc_table_layout_get_cell (layout, DESC_CELL),
-        xaccTransGetDescription (trans));
-
     gnc_quickfill_cell_add_completion (
         (QuickFillCell*) gnc_table_layout_get_cell (layout, NOTES_CELL),
         xaccTransGetNotes (trans));
@@ -254,12 +253,12 @@ static void add_quickfill_completions (TableLayout* layout, Transaction* trans,
             (NumCell*) gnc_table_layout_get_cell (layout, NUM_CELL),
             gnc_get_num_action (trans, split));
 
-    while ((s = xaccTransGetSplit (trans, i)) != NULL)
+    for (GList *n = xaccTransGetSplitList (trans); n; n = n->next)
     {
+        Split *s = n->data;
         gnc_quickfill_cell_add_completion (
             (QuickFillCell*) gnc_table_layout_get_cell (layout, MEMO_CELL),
             xaccSplitGetMemo (s));
-        i++;
     }
 }
 
@@ -368,6 +367,7 @@ gnc_split_register_load (SplitRegister* reg, GList* slist,
     Split* split;
     Table* table;
     GList* node;
+    gnc_commodity *account_comm = NULL;
 
     gboolean start_primary_color = TRUE;
     gboolean found_pending = FALSE;
@@ -407,17 +407,31 @@ gnc_split_register_load (SplitRegister* reg, GList* slist,
     pending_trans = xaccTransLookup (&info->pending_trans_guid,
                                      gnc_get_current_book());
 
+    if (default_account)
+        account_comm = gnc_account_get_currency_or_parent (default_account);
+
+    if (!account_comm)
+        account_comm = gnc_default_currency ();
+
     /* Bug 742089: Set the debit and credit cells' print_info to the account */
     gnc_price_cell_set_print_info
     ((PriceCell*) gnc_table_layout_get_cell (table->layout, DEBT_CELL),
-     gnc_account_print_info (default_account, FALSE));
+     gnc_commodity_print_info (account_comm, FALSE));
 
     gnc_price_cell_set_print_info
     ((PriceCell*) gnc_table_layout_get_cell (table->layout, CRED_CELL),
-     gnc_account_print_info (default_account, FALSE));
+     gnc_commodity_print_info (account_comm, FALSE));
 
-    gnc_assoc_cell_set_use_glyphs
-    ((AssocCell *) gnc_table_layout_get_cell (table->layout, ASSOC_CELL));
+    gnc_price_cell_set_print_info
+    ((PriceCell*) gnc_table_layout_get_cell (reg->table->layout, PRIC_CELL),
+     gnc_commodity_print_info (account_comm, FALSE));
+
+    /* Only test for linked document glyths once */
+    if (info->first_pass)
+    {
+        gnc_doclink_cell_set_use_glyphs
+        ((Doclinkcell *) gnc_table_layout_get_cell (table->layout, DOCLINK_CELL));
+    }
 
     /* make sure we have a blank split */
     if (blank_split == NULL)
@@ -519,7 +533,8 @@ gnc_split_register_load (SplitRegister* reg, GList* slist,
 
         /* load up account names into the transfer combobox menus */
         gnc_split_register_load_xfer_cells (reg, default_account);
-        gnc_split_register_load_associate_cells (reg);
+        gnc_split_register_load_desc_cells (reg);
+        gnc_split_register_load_doclink_cells (reg);
         gnc_split_register_load_recn_cells (reg);
         gnc_split_register_load_type_cells (reg);
     }
@@ -596,7 +611,8 @@ gnc_split_register_load (SplitRegister* reg, GList* slist,
             use_autoreadonly &&
             !found_divider_upper)
         {
-            if (xaccTransGetDate (trans) >= autoreadonly_time)
+            if (((table->model->reverse_sort && xaccTransGetDate(trans) < autoreadonly_time) ||
+                 (!table->model->reverse_sort && xaccTransGetDate (trans) >= autoreadonly_time)))
             {
                 table->model->dividing_row_upper = vcell_loc.virt_row;
                 found_divider_upper = TRUE;
@@ -607,9 +623,9 @@ gnc_split_register_load (SplitRegister* reg, GList* slist,
             }
         }
 
-        if (info->show_present_divider &&
-            !found_divider &&
-            (xaccTransGetDate (trans) > present))
+        if (info->show_present_divider && !found_divider &&
+            ((table->model->reverse_sort && xaccTransGetDate (trans) < present) ||
+             (!table->model->reverse_sort && xaccTransGetDate (trans) > present)))
         {
             table->model->dividing_row = vcell_loc.virt_row;
             found_divider = TRUE;
@@ -630,6 +646,9 @@ gnc_split_register_load (SplitRegister* reg, GList* slist,
                     save_loc.phys_col_offset = 0;
                 }
 
+                // used in the setting the rows insensitive
+                table->model->blank_trans_row = vcell_loc.virt_row;
+
                 gnc_split_register_add_transaction (reg,
                                                     blank_trans, blank_split,
                                                     lead_cursor, split_cursor,
@@ -640,7 +659,6 @@ gnc_split_register_load (SplitRegister* reg, GList* slist,
                                                     &vcell_loc);
 
                 table->model->dividing_row_lower = vcell_loc.virt_row;
-
                 if (!multi_line)
                     start_primary_color = !start_primary_color;
 
@@ -653,6 +671,10 @@ gnc_split_register_load (SplitRegister* reg, GList* slist,
         if (info->first_pass)
             add_quickfill_completions (reg->table->layout, trans, split, has_last_num);
 
+        gnc_combo_cell_add_menu_item_unique (
+            (ComboCell*) gnc_table_layout_get_cell (reg->table->layout, DESC_CELL),
+            xaccTransGetDescription (trans));
+
         if (trans == find_trans)
             new_trans_row = vcell_loc.virt_row;
 
@@ -713,6 +735,9 @@ gnc_split_register_load (SplitRegister* reg, GList* slist,
             save_loc.phys_col_offset = 0;
         }
 
+        // used in the setting the rows insensitive
+        table->model->blank_trans_row = vcell_loc.virt_row;
+
         gnc_split_register_add_transaction (reg, blank_trans, blank_split,
                                             lead_cursor, split_cursor,
                                             multi_line, start_primary_color,
@@ -722,7 +747,9 @@ gnc_split_register_load (SplitRegister* reg, GList* slist,
                                             &vcell_loc);
 
         if (future_after_blank)
+        {
             table->model->dividing_row_lower = vcell_loc.virt_row;
+        }
     }
 
     /* go to blank on first pass */
@@ -842,4 +869,17 @@ gnc_split_register_load_xfer_cells (SplitRegister* reg, Account* base_account)
     gnc_combo_cell_use_list_store_cache (cell, store);
 }
 
+static void
+gnc_split_register_load_desc_cells (SplitRegister* reg)
+{
+    ComboCell* cell;
+    GtkListStore* store = gtk_list_store_new (1, G_TYPE_STRING);
+
+    cell = (ComboCell*)
+           gnc_table_layout_get_cell (reg->table->layout, DESC_CELL);
+
+    gnc_combo_cell_use_type_ahead_only (cell);
+
+    gnc_combo_cell_use_list_store_cache (cell, store);
+}
 /* ====================== END OF FILE ================================== */
