@@ -49,22 +49,6 @@ enum
     PROP_CURSOR_NAME, /* the name of the current cursor */
 };
 
-static gboolean
-gnc_header_draw (GtkWidget *header, cairo_t *cr)
-{
-    GnucashSheet *sheet = GNC_HEADER(header)->sheet;
-    GdkWindow *sheet_layout_win = gtk_layout_get_bin_window (GTK_LAYOUT(sheet));
-    gint x, y;
-
-    // use this to get the scroll x value to align the header
-    gdk_window_get_position (sheet_layout_win, &x, &y);
-
-    cairo_set_source_surface (cr, GNC_HEADER(header)->surface, x, 0);
-    cairo_paint (cr);
-
-    return TRUE;
-}
-
 static void
 gnc_header_draw_offscreen (GncHeader *header)
 {
@@ -90,13 +74,13 @@ gnc_header_draw_offscreen (GncHeader *header)
 
     // Get the color type and apply the css class
     color_type = gnc_table_get_color (table, virt_loc, NULL);
-    gnucash_get_style_classes (header->sheet, stylectxt, color_type);
+    gnucash_get_style_classes (header->sheet, stylectxt, color_type, FALSE);
 
     if (header->surface)
         cairo_surface_destroy (header->surface);
     header->surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-                                                header->width,
-                                                header->height);
+                                                  header->width,
+                                                  header->height);
 
     cr = cairo_create (header->surface);
 
@@ -137,7 +121,6 @@ gnc_header_draw_offscreen (GncHeader *header)
         for (j = 0; j < style->ncols; j++)
         {
             CellDimensions *cd;
-            double text_x, text_y, text_w, text_h;
             BasicCell *cell;
             const char *text;
             int width;
@@ -149,6 +132,8 @@ gnc_header_draw_offscreen (GncHeader *header)
             virt_loc.phys_col_offset = j;
 
             cd = gnucash_style_get_cell_dimensions (style, i, j);
+            if (!cd) continue;
+
             height = cd->pixel_height;
             if (header->in_resize && (j == header->resize_col))
                 width = header->resize_col_width;
@@ -172,7 +157,7 @@ gnc_header_draw_offscreen (GncHeader *header)
             if (!text)
                 text = "";
 
-            layout = gtk_widget_create_pango_layout (GTK_WIDGET (header->sheet), text);
+            layout = gtk_widget_create_pango_layout (GTK_WIDGET(header->sheet), text);
 
             pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 
@@ -202,6 +187,53 @@ gnc_header_draw_offscreen (GncHeader *header)
 }
 
 
+gint
+gnc_header_get_cell_offset (GncHeader *header, gint col, gint *cell_width)
+{
+    SheetBlockStyle *style = header->style;
+    gint j;
+    gint offset = 0;
+
+    for (j = 0; j < style->ncols; j++)
+    {
+        CellDimensions *cd;
+
+        cd = gnucash_style_get_cell_dimensions (style, 0, j);
+        if (!cd) continue;
+
+        if (j == col)
+        {
+            *cell_width = cd->pixel_width;
+            break;
+        }
+        offset = offset + cd->pixel_width;
+    }
+    return offset;
+}
+
+
+static gboolean
+gnc_header_draw (GtkWidget *header, cairo_t *cr)
+{
+    GnucashSheet *sheet = GNC_HEADER(header)->sheet;
+    GdkWindow *sheet_layout_win = gtk_layout_get_bin_window (GTK_LAYOUT(sheet));
+    gint x, y;
+
+    // use this to get the scroll x value to align the header
+    gdk_window_get_position (sheet_layout_win, &x, &y);
+
+    // if the register page is moved to another window, the surface is
+    // not created so test for a surface and create one if null
+    if (GNC_HEADER(header)->surface == NULL)
+        gnc_header_draw_offscreen (GNC_HEADER(header));
+
+    cairo_set_source_surface (cr, GNC_HEADER(header)->surface, x, 0);
+    cairo_paint (cr);
+
+    return TRUE;
+}
+
+
 void
 gnc_header_request_redraw (GncHeader *header)
 {
@@ -216,7 +248,7 @@ gnc_header_request_redraw (GncHeader *header)
 static void
 gnc_header_unrealize (GtkWidget *widget)
 {
-    GncHeader *header = GNC_HEADER (widget);
+    GncHeader *header = GNC_HEADER(widget);
     if (header->surface)
         cairo_surface_destroy (header->surface);
     header->surface = NULL;
@@ -229,8 +261,8 @@ gnc_header_unrealize (GtkWidget *widget)
         g_object_unref (header->normal_cursor);
     header->normal_cursor = NULL;
 
-    if (GTK_WIDGET_CLASS (parent_class)->unrealize)
-        GTK_WIDGET_CLASS (parent_class)->unrealize (GTK_WIDGET(header));
+    if (GTK_WIDGET_CLASS(parent_class)->unrealize)
+        GTK_WIDGET_CLASS(parent_class)->unrealize (GTK_WIDGET(header));
 }
 
 
@@ -239,12 +271,12 @@ gnc_header_finalize (GObject *object)
 {
     GncHeader *header;
 
-    header = GNC_HEADER (object);
+    header = GNC_HEADER(object);
 
     g_free (header->cursor_name);
     header->cursor_name = NULL;
 
-    G_OBJECT_CLASS (parent_class)->finalize (object);
+    G_OBJECT_CLASS(parent_class)->finalize (object);
 }
 
 
@@ -256,7 +288,7 @@ gnc_header_reconfigure (GncHeader *header)
     int w, h;
 
     g_return_if_fail (header != NULL);
-    g_return_if_fail (GNC_IS_HEADER (header));
+    g_return_if_fail (GNC_IS_HEADER(header));
 
     sheet = GNUCASH_SHEET(header->sheet);
     old_style = header->style;
@@ -281,8 +313,8 @@ gnc_header_reconfigure (GncHeader *header)
     {
         header->height = h;
         header->width = w;
-        gtk_layout_set_size(GTK_LAYOUT(header), w, h);
-        gtk_widget_set_size_request(GTK_WIDGET(header), -1, h);
+        gtk_layout_set_size (GTK_LAYOUT(header), w, h);
+        gtk_widget_set_size_request (GTK_WIDGET(header), -1, h);
         gnc_header_request_redraw (header);
     }
 }
@@ -292,7 +324,7 @@ gnc_header_set_header_rows (GncHeader *header,
                             int num_phys_rows)
 {
     g_return_if_fail (header != NULL);
-    g_return_if_fail (GNC_IS_HEADER (header));
+    g_return_if_fail (GNC_IS_HEADER(header));
 
     header->num_phys_rows = num_phys_rows;
 }
@@ -314,6 +346,8 @@ pointer_on_resize_line (GncHeader *header, int x, G_GNUC_UNUSED int y, int *col)
     for (j = 0; j < style->ncols; j++)
     {
         cd = gnucash_style_get_cell_dimensions (style, 0, j);
+        if (!cd) continue;
+
         pixels += cd->pixel_width;
         if (x >= pixels - 1 && x <= pixels + 1)
             on_the_line = TRUE;
@@ -340,8 +374,8 @@ find_resize_col (GncHeader *header, int col)
     /* skip to the right over zero-width columns */
     while ((col + 1 < style->ncols) &&
             (cd = gnucash_style_get_cell_dimensions (style, 0, col + 1)) &&
-            (cd->pixel_width == 0))
-        col++;
+            cd && (cd->pixel_width == 0))
+        ++col;
 
     /* now go back left till we have a resizable column */
     while (col >= start)
@@ -389,9 +423,14 @@ static gint
 gnc_header_event (GtkWidget *widget, GdkEvent *event)
 {
     GncHeader *header = GNC_HEADER(widget);
+    GdkWindow *window = gtk_widget_get_window (widget);
     int x, y;
     int col;
 
+    if (!header->resize_cursor)
+        header->resize_cursor = gdk_cursor_new_for_display (gdk_window_get_display (window),
+                                                            GDK_SB_H_DOUBLE_ARROW);
+
     switch (event->type)
     {
     case GDK_MOTION_NOTIFY:
@@ -413,13 +452,11 @@ gnc_header_event (GtkWidget *widget, GdkEvent *event)
             break;
         }
 
-        if (pointer_on_resize_line(header, x, y, &col) &&
+        if (pointer_on_resize_line (header, x, y, &col) &&
                 gnucash_style_col_is_resizable (header->style, col))
-            gdk_window_set_cursor (gtk_widget_get_window (widget),
-                                   header->resize_cursor);
+            gdk_window_set_cursor (window, header->resize_cursor);
         else
-            gdk_window_set_cursor (gtk_widget_get_window (widget),
-                                   header->normal_cursor);
+            gdk_window_set_cursor (window, header->normal_cursor);
         break;
 
     case GDK_BUTTON_PRESS:
@@ -443,6 +480,7 @@ gnc_header_event (GtkWidget *widget, GdkEvent *event)
 
             cd = gnucash_style_get_cell_dimensions
                  (header->style, 0, col);
+            if (!cd) break;
 
             header->in_resize = TRUE;
             header->resize_col = col;
@@ -456,11 +494,11 @@ gnc_header_event (GtkWidget *widget, GdkEvent *event)
         if (event->button.button != 1)
             break;
 
-        x = event->button.x;
-        y = event->button.y;
-
         if (header->in_resize)
         {
+            if (header->resize_col_width == 0)
+                header->resize_col_width = 1;
+
             gnc_header_resize_column
                 (header,
                  header->resize_col,
@@ -521,7 +559,7 @@ gnc_header_get_property (GObject *object,
                          GValue *value,
                          GParamSpec *pspec)
 {
-    GncHeader *header = GNC_HEADER (object);
+    GncHeader *header = GNC_HEADER(object);
 
     switch (param_id)
     {
@@ -532,7 +570,7 @@ gnc_header_get_property (GObject *object,
         g_value_set_string (value, header->cursor_name);
         break;
     default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
         break;
     }
 }
@@ -543,15 +581,15 @@ gnc_header_set_property (GObject *object,
                          const GValue *value,
                          GParamSpec *pspec)
 {
-    GncHeader *header = GNC_HEADER (object);
-    GtkLayout *layout = GTK_LAYOUT (header);
+    GncHeader *header = GNC_HEADER(object);
+    GtkLayout *layout = GTK_LAYOUT(header);
     gboolean needs_update = FALSE;
     gchar *old_name;
 
     switch (param_id)
     {
     case PROP_SHEET:
-        header->sheet = GNUCASH_SHEET (g_value_get_object (value));
+        header->sheet = GNUCASH_SHEET(g_value_get_object (value));
         gtk_scrollable_set_hadjustment (GTK_SCROLLABLE(layout), header->sheet->hadj);
         needs_update = TRUE;
         break;
@@ -564,7 +602,7 @@ gnc_header_set_property (GObject *object,
         g_free (old_name);
         break;
     default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
         break;
     }
 
@@ -580,23 +618,21 @@ gnc_header_init (GncHeader *header)
     header->cursor_name = NULL;
     header->in_resize = FALSE;
     header->resize_col = -1;
-    header->resize_cursor = gdk_cursor_new_for_display (gdk_display_get_default (), GDK_SB_H_DOUBLE_ARROW);
+    header->resize_cursor = NULL;
     header->normal_cursor = NULL;
     header->height = 20;
     header->width = 400;
     header->style = NULL;
 
-    // This sets a style class for when Gtk+ version is less than 3.20
-    gnc_widget_set_css_name (GTK_WIDGET(header), "header");
-
-    gtk_widget_add_events(GTK_WIDGET(header), (GDK_EXPOSURE_MASK
+    gtk_widget_add_events (GTK_WIDGET(header),
+                          (GDK_EXPOSURE_MASK
                           | GDK_BUTTON_PRESS_MASK
                           | GDK_BUTTON_RELEASE_MASK
                           | GDK_POINTER_MOTION_MASK
                           | GDK_POINTER_MOTION_HINT_MASK));
 
-    g_signal_connect(G_OBJECT(header), "configure_event",
-                     G_CALLBACK(gnc_header_reconfigure), NULL);
+    g_signal_connect (G_OBJECT(header), "configure_event",
+                      G_CALLBACK(gnc_header_reconfigure), NULL);
     gtk_widget_show_all (GTK_WIDGET(header));
 }
 
@@ -604,12 +640,10 @@ gnc_header_init (GncHeader *header)
 static void
 gnc_header_class_init (GncHeaderClass *header_class)
 {
-    GObjectClass  *object_class = G_OBJECT_CLASS (header_class);
-    GtkWidgetClass *item_class = GTK_WIDGET_CLASS (header_class);
+    GObjectClass  *object_class = G_OBJECT_CLASS(header_class);
+    GtkWidgetClass *item_class = GTK_WIDGET_CLASS(header_class);
 
-#if GTK_CHECK_VERSION(3,20,0)
-    gtk_widget_class_set_css_name (GTK_WIDGET_CLASS(header_class), "header");
-#endif
+    gtk_widget_class_set_css_name (GTK_WIDGET_CLASS(header_class), "gnc-id-header");
 
     parent_class = g_type_class_peek_parent (header_class);
 
