@@ -21,9 +21,65 @@
 ;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define-module (gnucash report html-document))
+
 (use-modules (gnucash html))
-(use-modules (srfi srfi-9))
+(use-modules (gnucash report html-anytag))
+(use-modules (gnucash report html-chart))
+(use-modules (gnucash report html-style-info))
+(use-modules (gnucash report html-style-sheet))
+(use-modules (gnucash report html-table))
+(use-modules (gnucash report html-text))
+(use-modules (gnucash report report-utilities))
+(use-modules (gnucash utilities))
 (use-modules (ice-9 match))
+(use-modules (srfi srfi-1))
+(use-modules (srfi srfi-9))
+
+(export <html-document>)
+(export gnc:html-document?)
+(export gnc:make-html-document-internal)
+(export gnc:make-html-document)
+(export gnc:html-document-set-title!)
+(export gnc:html-document-title)
+(export gnc:html-document-set-headline!)
+(export gnc:html-document-headline)
+(export gnc:html-document-set-style-text!)
+(export gnc:html-document-style-text)
+(export gnc:html-document-set-style-sheet!)
+(export gnc:html-document-style-sheet)
+(export gnc:html-document-set-style-stack!)
+(export gnc:html-document-style-stack)
+(export gnc:html-document-set-style-internal!)
+(export gnc:html-document-style)
+(export gnc:html-document-set-objects!)
+(export gnc:html-document-objects)
+(export gnc:html-document?)
+(export gnc:html-document-set-style!)
+(export gnc:html-document-tree-collapse)
+(export gnc:html-document-render)
+(export gnc:html-document-push-style)
+(export gnc:html-document-pop-style)
+(export gnc:html-document-add-object!)
+(export gnc:html-document-append-objects!)
+(export gnc:html-document-fetch-markup-style)
+(export gnc:html-document-fetch-data-style)
+(export gnc:html-document-markup-start)
+(export gnc:html-document-markup-end)
+(export gnc:html-document-render-data)
+(export gnc:html-document-export-string)
+(export gnc:html-document-set-export-string)
+(export gnc:html-document-export-error)
+(export gnc:html-document-set-export-error)
+(export <html-object>)
+(export gnc:html-object?)
+(export gnc:make-html-object-internal)
+(export gnc:make-html-object)
+(export gnc:html-object-renderer)
+(export gnc:html-object-set-renderer!)
+(export gnc:html-object-data)
+(export gnc:html-object-set-data!)
+(export gnc:html-object-render)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;  <html-document> class
@@ -50,7 +106,6 @@
 (define gnc:html-document-set-headline! html-document-set-headline)
 (define gnc:html-document-headline html-document-headline)
 (define gnc:html-document-set-style-sheet! html-document-set-style-sheet)
-(define gnc:html-document-set-style-sheet! html-document-set-style-sheet)
 (define gnc:html-document-style-sheet html-document-style-sheet)
 (define gnc:html-document-set-style-stack! html-document-set-style-stack)
 (define gnc:html-document-style-stack html-document-style-stack)
@@ -308,18 +363,6 @@
    ((gnc:html-table-cell? obj)
     (gnc:make-html-object-internal gnc:html-table-cell-render obj))
 
-   ((gnc:html-barchart? obj)
-    (gnc:make-html-object-internal gnc:html-barchart-render obj))
-
-   ((gnc:html-piechart? obj)
-    (gnc:make-html-object-internal gnc:html-piechart-render obj))
-
-   ((gnc:html-scatter? obj)
-    (gnc:make-html-object-internal gnc:html-scatter-render obj))
-
-   ((gnc:html-linechart? obj)
-    (gnc:make-html-object-internal gnc:html-linechart-render obj))
-
    ((gnc:html-object? obj)
     obj)
 
