@@ -21,9 +21,52 @@
 ;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define-module (gnucash report html-style-sheet))
+
 (use-modules (srfi srfi-9))
 (use-modules (ice-9 match))
 (use-modules (gnucash core-utils))
+(use-modules (gnucash engine))
+(use-modules (gnucash utilities))
+(use-modules (gnucash app-utils))
+(use-modules (gnucash options))
+(use-modules (gnucash report html-document))
+(use-modules (gnucash report html-style-info))
+
+(export <html-style-sheet-template>)
+(export gnc:html-style-sheet-template?)
+(export gnc:html-style-sheet-template-version)
+(export gnc:html-style-sheet-template-set-version!)
+(export gnc:html-style-sheet-template-name)
+(export gnc:html-style-sheet-template-set-name!)
+(export gnc:html-style-sheet-template-options-generator)
+(export gnc:html-style-sheet-template-set-options-generator!)
+(export gnc:html-style-sheet-template-renderer)
+(export gnc:html-style-sheet-template-set-renderer!)
+(export gnc:html-style-sheet-template-find)
+(export gnc:define-html-style-sheet)
+(export <html-style-sheet>)
+(export gnc:html-style-sheet?)
+(export gnc:html-style-sheet-name)
+(export gnc:html-style-sheet-set-name!)
+(export gnc:html-style-sheet-type)
+(export gnc:html-style-sheet-set-type!)
+(export gnc:html-style-sheet-options)
+(export gnc:html-style-sheet-set-options!)
+(export gnc:html-style-sheet-renderer)
+(export gnc:html-style-sheet-set-renderer!)
+(export gnc:make-html-style-sheet-internal)
+(export gnc:html-style-sheet-style)
+(export gnc:html-style-sheet-set-style!)
+(export gnc:make-html-style-sheet)
+(export gnc:restore-html-style-sheet)
+(export gnc:html-style-sheet-apply-changes)
+(export gnc:html-style-sheet-render)
+(export gnc:get-html-style-sheets)
+(export gnc:get-html-templates)
+(export gnc:html-style-sheet-find)
+(export gnc:save-style-sheet-options)
+(export gnc:html-style-sheet-remove)
 
 (define *gnc:_style-sheet-templates_* (make-hash-table 23))
 (define *gnc:_style-sheets_* (make-hash-table 23))
@@ -201,14 +244,14 @@
 (define (gnc:get-html-style-sheets)
   (sort (map cdr (hash-map->list cons *gnc:_style-sheets_*))
         (lambda (a b)
-          (string<? (gnc:html-style-sheet-name a)
-                    (gnc:html-style-sheet-name b)))))
+          (gnc:string-locale<? (gnc:html-style-sheet-name a)
+                               (gnc:html-style-sheet-name b)))))
 
 (define (gnc:get-html-templates)
   (sort (map cdr (hash-map->list cons *gnc:_style-sheet-templates_*))
         (lambda (a b)
-          (string<? (gnc:html-style-sheet-template-name a)
-                    (gnc:html-style-sheet-template-name b)))))
+          (gnc:string-locale<? (gnc:html-style-sheet-template-name a)
+                               (gnc:html-style-sheet-template-name b)))))
 
 (define (gnc:html-style-sheet-find tname)
   (hash-ref *gnc:_style-sheets_* tname))
