@@ -23,7 +23,81 @@
 ;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define-module (gnucash report html-table))
+
 (use-modules (srfi srfi-2))
+(use-modules (srfi srfi-9))
+(use-modules (gnucash engine))
+(use-modules (gnucash report html-style-info))
+(use-modules (gnucash report html-document))
+(use-modules (gnucash report report-utilities))
+
+(export <html-table>)
+(export gnc:html-table?)
+(export <html-table-cell>)
+(export gnc:make-html-table-cell-internal)
+(export gnc:make-html-table-cell)
+(export gnc:make-html-table-cell/size)
+(export gnc:make-html-table-cell/markup)
+(export gnc:make-html-table-cell/size/markup)
+(export gnc:make-html-table-header-cell)
+(export gnc:make-html-table-header-cell/markup)
+(export gnc:make-html-table-header-cell/size)
+(export gnc:make-html-table-cell/min-width)
+(export gnc:html-table-cell?)
+(export gnc:html-table-cell-rowspan)
+(export gnc:html-table-cell-set-rowspan!)
+(export gnc:html-table-cell-colspan)
+(export gnc:html-table-cell-set-colspan!)
+(export gnc:html-table-cell-tag)
+(export gnc:html-table-cell-set-tag!)
+(export gnc:html-table-cell-data)
+(export gnc:html-table-cell-set-data-internal!)
+(export gnc:html-table-cell-style)
+(export gnc:html-table-cell-set-style-internal!)
+(export gnc:html-table-cell-set-style!)
+(export gnc:html-table-cell-append-objects!)
+(export gnc:html-table-cell-render)
+(export gnc:make-html-table-internal)
+(export gnc:make-html-table)
+(export gnc:html-table-data)
+(export gnc:html-table-set-data!)
+(export gnc:html-table-caption)
+(export gnc:html-table-set-caption!)
+(export gnc:html-table-set-col-headers!)
+(export gnc:html-table-multirow-col-headers)
+(export gnc:html-table-set-multirow-col-headers!)
+(export gnc:html-table-style)
+(export gnc:html-table-set-style-internal!)
+(export gnc:html-table-row-styles)
+(export gnc:html-table-set-row-styles!)
+(export gnc:html-table-row-markup-table)
+(export gnc:html-table-row-markup)
+(export gnc:html-table-set-row-markup-table!)
+(export gnc:html-table-set-row-markup!)
+(export gnc:html-table-col-styles)
+(export gnc:html-table-set-col-styles!)
+(export gnc:html-table-col-headers-style)
+(export gnc:html-table-set-col-headers-style!)
+(export gnc:html-table-row-headers-style)
+(export gnc:html-table-set-row-headers-style!)
+(export gnc:html-table-set-last-row-style!)
+(export gnc:html-table-set-style!)
+(export gnc:html-table-set-col-style!)
+(export gnc:html-table-set-row-style!)
+(export gnc:html-table-row-style)
+(export gnc:html-table-col-style)
+(export gnc:html-table-num-rows)
+(export gnc:html-table-set-num-rows-internal!)
+(export gnc:html-table-num-columns)
+(export gnc:html-table-append-row/markup!)
+(export gnc:html-table-prepend-row/markup!)
+(export gnc:html-table-append-row!)
+(export gnc:html-table-prepend-row!)
+(export gnc:html-table-get-cell)
+(export gnc:html-table-set-cell!)
+(export gnc:html-table-set-cell/tag!)
+(export gnc:html-table-render)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; 
@@ -35,29 +109,66 @@
 ;; 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define <html-table>
-  (make-record-type "<html-table>"
-                    '(col-headers
-                      row-headers
-                      caption 
-                      data
-		      num-rows
-                      style
-                      col-styles
-                      row-styles
-                      row-markup-table
-                      col-headers-style
-                      row-headers-style)))
+(define-record-type <html-table>
+  (make-html-table col-headers row-headers caption data num-rows style
+                   col-styles row-styles row-markup-table col-headers-style
+                   row-headers-style)
+  html-table?
+  (col-headers html-table-col-headers html-table-set-col-headers!)
+  (row-headers html-table-row-headers html-table-set-row-headers!)
+  (caption html-table-caption html-table-set-caption!)
+  (data html-table-data html-table-set-data!)
+  (num-rows html-table-num-rows html-table-set-num-rows!)
+  (style html-table-style html-table-set-style!)
+  (col-styles html-table-col-styles html-table-set-col-styles!)
+  (row-styles html-table-row-styles html-table-set-row-styles!)
+  (row-markup-table html-table-row-markup-table html-table-set-row-markup-table!)
+  (col-headers-style html-table-col-headers-style)
+  (row-headers-style html-table-row-headers-style))
 
-(define gnc:html-table? 
-  (record-predicate <html-table>))
+(define gnc:html-table? html-table?)
+(define gnc:make-html-table-internal make-html-table)
+(define gnc:html-table-data html-table-data)
+(define gnc:html-table-set-data! html-table-set-data!)
+(define gnc:html-table-caption html-table-caption)
+(define gnc:html-table-set-caption! html-table-set-caption!)
+(define gnc:html-table-multirow-col-headers html-table-col-headers)
+(define gnc:html-table-set-multirow-col-headers! html-table-set-col-headers!)
+(define gnc:html-table-style html-table-style)
+(define gnc:html-table-set-style-internal! html-table-set-style!)
+(define gnc:html-table-row-styles html-table-row-styles)
+(define gnc:html-table-set-row-styles! html-table-set-row-styles!)
+(define gnc:html-table-row-markup-table html-table-row-markup-table)
+(define gnc:html-table-set-row-markup-table! html-table-set-row-markup-table!)
+(define gnc:html-table-col-styles html-table-col-styles)
+(define gnc:html-table-set-col-styles! html-table-set-col-styles!)
+(define gnc:html-table-col-headers-style html-table-col-headers-style)
+(define gnc:html-table-row-headers-style html-table-row-headers-style)
+(define gnc:html-table-num-rows html-table-num-rows)
+(define gnc:html-table-set-num-rows-internal! html-table-set-num-rows!)
 
-(define <html-table-cell>
-  (make-record-type "<html-table-cell>"
-                    '(rowspan colspan tag data style)))
 
-(define gnc:make-html-table-cell-internal
-  (record-constructor <html-table-cell>))
+(define-record-type <html-table-cell>
+  (make-html-table-cell rowspan colspan tag data style)
+  html-table-cell?
+  (rowspan html-table-cell-rowspan html-table-cell-set-rowspan!)
+  (colspan html-table-cell-colspan html-table-cell-set-colspan!)
+  (tag html-table-cell-tag html-table-cell-set-tag!)
+  (data html-table-cell-data html-table-cell-set-data!)
+  (style html-table-cell-style html-table-cell-set-style!))
+
+(define gnc:make-html-table-cell-internal make-html-table-cell)
+(define gnc:html-table-cell? html-table-cell?)
+(define gnc:html-table-cell-rowspan html-table-cell-rowspan)
+(define gnc:html-table-cell-set-rowspan! html-table-cell-set-rowspan!)
+(define gnc:html-table-cell-colspan html-table-cell-colspan)
+(define gnc:html-table-cell-set-colspan! html-table-cell-set-colspan!)
+(define gnc:html-table-cell-tag html-table-cell-tag)
+(define gnc:html-table-cell-set-tag! html-table-cell-set-tag!)
+(define gnc:html-table-cell-data html-table-cell-data)
+(define gnc:html-table-cell-set-data-internal! html-table-cell-set-data!)
+(define gnc:html-table-cell-style html-table-cell-style)
+(define gnc:html-table-cell-set-style-internal! html-table-cell-set-style!)
 
 (define (gnc:make-html-table-cell . objects)
   (gnc:make-html-table-cell-internal 1 1 "td" objects 
@@ -93,39 +204,6 @@
   (gnc:make-html-table-cell-internal rowspan colspan "th"
                                      objects (gnc:make-html-style-table)))
 
-(define gnc:html-table-cell? 
-  (record-predicate <html-table-cell>))
-
-(define gnc:html-table-cell-rowspan
-  (record-accessor <html-table-cell> 'rowspan))
-
-(define gnc:html-table-cell-set-rowspan!
-  (record-modifier <html-table-cell> 'rowspan))
-
-(define gnc:html-table-cell-colspan
-  (record-accessor <html-table-cell> 'colspan))
-
-(define gnc:html-table-cell-set-colspan!
-  (record-modifier <html-table-cell> 'colspan))
-
-(define gnc:html-table-cell-tag
-  (record-accessor <html-table-cell> 'tag))
-
-(define gnc:html-table-cell-set-tag!
-  (record-modifier <html-table-cell> 'tag))
-
-(define gnc:html-table-cell-data
-  (record-accessor <html-table-cell> 'data))
-
-(define gnc:html-table-cell-set-data-internal!
-  (record-modifier <html-table-cell> 'data))
-
-(define gnc:html-table-cell-style
-  (record-accessor <html-table-cell> 'style))
-
-(define gnc:html-table-cell-set-style-internal!
-  (record-modifier <html-table-cell> 'style))
-
 (define (gnc:html-table-cell-set-style! cell tag . rest)
   (let ((newstyle (if (and (= (length rest) 2) (procedure? (car rest)))
                       (apply gnc:make-html-data-style-info rest)
@@ -174,9 +252,6 @@
 ;;  wrapper around HTML tables
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define gnc:make-html-table-internal
-  (record-constructor <html-table>))
-
 (define (gnc:make-html-table)
   (gnc:make-html-table-internal 
    #f                    ;; col-headers 
@@ -192,60 +267,16 @@
    (gnc:make-html-style-table) ;; row-headers-style
    ))
 
-(define gnc:html-table-data
-  (record-accessor <html-table> 'data))
-
-(define gnc:html-table-set-data!
-  (record-modifier <html-table> 'data))
-
-(define gnc:html-table-caption
-  (record-accessor <html-table> 'caption))
-
-(define gnc:html-table-set-caption!
-  (record-modifier <html-table> 'caption))
 
 (define (gnc:html-table-set-col-headers! table col-headers)
   (gnc:html-table-set-multirow-col-headers! table (list col-headers)))
 
-(define gnc:html-table-multirow-col-headers
-  (record-accessor <html-table> 'col-headers))
-
-(define gnc:html-table-set-multirow-col-headers!
-  (record-modifier <html-table> 'col-headers))
-
-(define gnc:html-table-style
-  (record-accessor <html-table> 'style))
-
-(define gnc:html-table-set-style-internal!
-  (record-modifier <html-table> 'style))
-
-(define gnc:html-table-row-styles
-  (record-accessor <html-table> 'row-styles))
-
-(define gnc:html-table-set-row-styles!
-  (record-modifier <html-table> 'row-styles))
-
-(define gnc:html-table-row-markup-table
-  (record-accessor <html-table> 'row-markup-table))
-
 (define (gnc:html-table-row-markup table row)
   (hash-ref (gnc:html-table-row-markup-table table) row))
 
-(define gnc:html-table-set-row-markup-table!
-  (record-modifier <html-table> 'row-markup-table))
-
 (define (gnc:html-table-set-row-markup! table row markup)
   (hash-set! (gnc:html-table-row-markup-table table) row markup))
 
-(define gnc:html-table-col-styles
-  (record-accessor <html-table> 'col-styles))
-
-(define gnc:html-table-set-col-styles!
-  (record-modifier <html-table> 'col-styles))
-
-(define gnc:html-table-col-headers-style
-  (record-accessor <html-table> 'col-headers-style))
-
 (define (gnc:html-table-set-col-headers-style! table tag . rest)
   (let ((newstyle (if (and (= (length rest) 2) (procedure? (car rest)))
                       (apply gnc:make-html-data-style-info rest)
@@ -253,8 +284,6 @@
         (style (gnc:html-table-col-headers-style table)))
     (gnc:html-style-table-set! style tag newstyle)))
 
-(define gnc:html-table-row-headers-style
-  (record-accessor <html-table> 'row-headers-style))
 
 (define (gnc:html-table-set-row-headers-style! table tag . rest)
   (let* ((newstyle (if (and (= (length rest) 2) (procedure? (car rest)))
@@ -298,12 +327,6 @@
 (define (gnc:html-table-col-style table col)
   (hash-ref (gnc:html-table-col-styles table) col))
 
-(define gnc:html-table-num-rows
- (record-accessor <html-table> 'num-rows))
-
-(define gnc:html-table-set-num-rows-internal!
-  (record-modifier <html-table> 'num-rows))
-
 (define (gnc:html-table-num-columns table)
   (apply max (cons 0 (map length (gnc:html-table-data table)))))
 
@@ -388,28 +411,6 @@
     (gnc:html-table-cell-set-tag! tc tag)
     (gnc:html-table-set-cell-datum! table row col tc)))
 
-(define (gnc:html-table-append-column! table newcol)
-  (define width (apply max (cons 0 (map length (gnc:html-table-data table)))))
-  (define (add-fn a b) (list-set-safe! b width a))
-  (let lp ((newcol newcol)
-           (olddata (reverse (gnc:html-table-data table)))
-           (res '())
-           (numrows 0))
-    (cond
-     ((null? newcol)
-      (gnc:html-table-set-num-rows-internal! table numrows)
-      (gnc:html-table-set-data! table res))
-     ((null? olddata)
-      (lp (cdr newcol)
-          '()
-          (cons (add-fn (car newcol) '()) res)
-          (1+ numrows)))
-     (else
-      (lp (cdr newcol)
-          (cdr olddata)
-          (cons (add-fn (car newcol) (car olddata)) res)
-          (1+ numrows))))))
-
 (define (gnc:html-table-render table doc)
   (let* ((retval '())
          (push (lambda (l) (set! retval (cons l retval)))))
