@@ -22,9 +22,55 @@
 ;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define-module (gnucash report html-utilities))
+
+(use-modules (gnucash core-utils))
 (use-modules (gnucash engine))
 (use-modules (gnucash utilities))
+(use-modules (gnucash app-utils))
+(use-modules (gnucash html))
+(use-modules (gnucash report report-core))
+(use-modules (gnucash report report-utilities))
+(use-modules (gnucash report html-style-info))
+(use-modules (gnucash report html-text))
+(use-modules (gnucash report html-table))
 (use-modules (ice-9 match))
+(use-modules (web uri))
+
+(export gnc:html-make-empty-cell)
+(export gnc:html-make-empty-cells)
+(export gnc:account-anchor-text)
+(export gnc:split-anchor-text)
+(export gnc:transaction-anchor-text)
+(export gnc:transaction-doclink-anchor-text)
+(export gnc:report-anchor-text)
+(export gnc:make-report-anchor)
+(export gnc:html-account-anchor)
+(export gnc:html-split-anchor)
+(export gnc:html-transaction-anchor)
+(export gnc:html-transaction-doclink-anchor)
+(export gnc:html-invoice-doclink-anchor)
+(export gnc:html-price-anchor)
+(export gnc:customer-anchor-text)
+(export gnc:job-anchor-text)
+(export gnc:vendor-anchor-text)
+(export gnc:invoice-anchor-text)
+(export gnc:owner-anchor-text)
+(export gnc:owner-report-text)
+(export gnc:assign-colors)
+(export gnc:html-table-append-ruler!)
+(export gnc:html-make-exchangerates)
+(export gnc:html-make-rates-table)
+(export gnc:html-render-options-changed)
+(export gnc:html-make-generic-warning)
+(export gnc:html-make-no-account-warning)
+(export gnc:html-make-generic-budget-warning)
+(export gnc:html-make-generic-options-warning)
+(export gnc:html-make-generic-simple-warning)
+(export gnc:html-make-empty-data-warning)
+(export gnc:html-make-options-link)
+(export gnc:html-js-include)
+(export gnc:html-css-include)
 
 ;; returns a list with n #f (empty cell) values 
 (define (gnc:html-make-empty-cell) #f)
@@ -34,10 +80,12 @@
       (list)))
 
 (define (gnc:register-guid type guid)
-  (gnc-build-url URL-TYPE-REGISTER (string-append type guid) ""))
+  (and guid  (gnc-build-url URL-TYPE-REGISTER (string-append type guid) "")))
 
 (define (gnc:account-anchor-text acct)
-  (gnc:register-guid "acct-guid=" (gncAccountGetGUID acct)))
+  (if acct
+      (gnc:register-guid "acct-guid=" (gncAccountGetGUID acct))
+      (format #t "No Account!")))
 
 (define (gnc:split-anchor-text split)
   (gnc:register-guid "split-guid=" (gncSplitGetGUID split)))
@@ -48,6 +96,9 @@
 (define (gnc:transaction-doclink-anchor-text trans)
   (gnc:register-guid "trans-doclink-guid=" (gncTransGetGUID trans)))
 
+(define (gnc:invoice-doclink-anchor-text invoice)
+  (gnc:register-guid "invoice-doclink-guid=" (gncInvoiceReturnGUID invoice)))
+
 (define (gnc:report-anchor-text report-id)
   (gnc-build-url URL-TYPE-REPORT
 		      (string-append "id=" (number->string report-id))
@@ -123,11 +174,7 @@
 	  (gnc:options-copy-values src-options options)
 	  (for-each
 	   (lambda (l)
-	     (let ((o (gnc:lookup-option options (car l) (cadr l))))
-	       (if o
-		   (gnc:option-set-value o (caddr l))
-		   (warn "gnc:make-report-anchor:" reportname
-			 " No such option: " (car l) (cadr l)))))
+             (gnc-set-option (gnc:optiondb options) (car l) (cadr l) (caddr l)))
 	   optionlist)
 	  (let ((id (gnc:make-report reportname options)))
 	    (gnc:report-anchor-text id)))
@@ -159,6 +206,11 @@
                        (gnc:transaction-doclink-anchor-text trans)
                        text)))
 
+(define (gnc:html-invoice-doclink-anchor invoice text)
+  (gnc:make-html-text (gnc:html-markup-anchor
+                       (gnc:invoice-doclink-anchor-text invoice)
+                       text)))
+
 (define (gnc:html-price-anchor price value)
   (gnc:make-html-text (if price
                           (gnc:html-markup-anchor
@@ -194,31 +246,28 @@
    table (list (gnc:make-html-table-cell/size
                 1 colspan (gnc:make-html-text (gnc:html-markup-hr))))))
 
-;; Create a html-table of all exchange rates. The report-commodity is
-;; 'common-commodity', the exchange rates are given through the
-;; function 'exchange-fn' and the 'accounts' determine which
-;; commodities to show. Returns a html-object, a <html-table>.
-(define (gnc:html-make-exchangerates common-commodity exchange-fn accounts)
-  (let ((comm-list (gnc:accounts-get-commodities accounts common-commodity))
-        (markup (lambda (c) (gnc:make-html-table-cell/markup "number-cell" c)))
-        (table (gnc:make-html-table)))
-    (unless (null? comm-list)
-      (for-each
-       (lambda (commodity)
-         (let* ((orig-amt (gnc:make-gnc-monetary commodity 1))
-                (exchanged (exchange-fn orig-amt common-commodity))
-                (conv-amount (gnc:gnc-monetary-amount exchanged)))
-           (gnc:html-table-append-row!
-            table (list (markup orig-amt)
-                        (markup (gnc:default-price-renderer common-commodity
-                                                            conv-amount))))))
-       comm-list)
-      (gnc:html-table-set-col-headers!
-       table (list (gnc:make-html-table-header-cell/size
-                    1 2 (if (null? (cdr comm-list))
-                            (G_ "Exchange rate")
-                            (G_ "Exchange rates"))))))
-    table))
+;; Create a html-table of all prices. The report-currency is
+;; 'currency', The prices are given through the function 'price-fn'
+;; and the 'accounts' determine which commodities to show. Returns a
+;; html-object, a <html-table>. price-fn is easily obtained from
+;; gnc:case-price-fn
+(define (gnc:html-make-rates-table currency price-fn accounts)
+  (define (cell c) (gnc:make-html-table-cell/markup "number-cell" c))
+  (define table (gnc:make-html-table))
+  (let lp ((comm-list (gnc:accounts-get-commodities accounts currency)) (entries 0))
+    (match comm-list
+      (()
+       (unless (zero? entries)
+         (gnc:html-table-set-col-headers!
+          table (list (gnc:make-html-table-header-cell/size
+                       1 2 (NG_ "Exchange rate" "Exchange rates" entries)))))
+       table)
+      ((comm . rest)
+       (gnc:html-table-append-row!
+        table
+        (list (cell (gnc:make-gnc-monetary comm 1))
+              (cell (gnc:default-price-renderer currency (price-fn comm)))))
+       (lp rest (1+ entries))))))
 
 
 (define (gnc:html-make-generic-budget-warning report-title-string)
@@ -270,17 +319,16 @@
         (try gnc-budget-get-name)
         (format #f "~a" d)))
   (let ((render-list '())
-        (report-list (and=> (gnc:lookup-option options "__general" "report-list")
-                            gnc:option-value)))
+        (report-list (and=> (gnc-lookup-option (gnc:optiondb options) "__general" "report-list")
+                            GncOption-get-value)))
     (define (add-option-if-changed option)
-      (let* ((section (gnc:option-section option))
-             (name (gnc:option-name option))
-             (default-value (gnc:option-default-value option))
-             (value (gnc:option-value option))
+      (let* ((section (GncOption-get-section option))
+             (name (GncOption-get-name option))
+             (value (GncOption-get-value option))
              (retval (cons (format #f "~a / ~a" section name)
                            (disp value))))
-        (if (not (or (equal? default-value value)
-                     (char=? (string-ref section 0) #\_)))
+        (if (and (GncOption-is-changed option)
+                 (not (GncOption-is-internal option)))
             (addto! render-list retval))))
     (define (name-fn name) (if plaintext? name (gnc:html-markup-b name)))
     (define br (if plaintext? "\n" (gnc:html-markup-br)))
@@ -289,7 +337,7 @@
        (let ((report (gnc-report-find (car child))))
          (addto! render-list (cons "Embedded Report" (gnc:report-name report)))))
      (or report-list '()))
-    (gnc:options-for-each add-option-if-changed options)
+    (gnc-optiondb-foreach (gnc:optiondb options) add-option-if-changed)
     (let lp ((render-list (reverse render-list)) (acc '()))
       (match render-list
         (() (if plaintext? (string-concatenate acc) (apply gnc:make-html-text acc)))
@@ -328,15 +376,30 @@
     (G_ "No data")
     (G_ "The selected accounts contain no data/transactions (or only zeroes) for the selected time period")))
 
+(define unreserved-chars-rfc3986
+  (char-set-union
+   (string->char-set "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789")
+   (string->char-set ":/?[]@")         ;gen-delims
+   (string->char-set "-._~")))
+
+;;path must be absolute. On Windows an absolute path begins with a
+;;drive letter followed by a colon.
+(define (make-uri path)
+  (let ((uri-path (uri-encode (gnc:substring-replace path "\\" "/")
+               #:unescaped-chars unreserved-chars-rfc3986)))
+  (string-append
+   (if (char=? (string-ref uri-path 0) #\/) "file://" "file:///")
+   uri-path)))
+
 (define (gnc:html-js-include file)
   (format #f
-          "<script language=\"javascript\" type=\"text/javascript\" src=\"file:///~a\"></script>\n"
-          (gnc-path-find-localized-html-file file)))
+          "<script language=\"javascript\" type=\"text/javascript\" src=~s></script>\n"
+          (make-uri (gnc-resolve-file-path file))))
 
 (define (gnc:html-css-include file)
   (format #f
-          "<link rel=\"stylesheet\" type=\"text/css\" href=\"file:///~a\" />\n"
-          (gnc-path-find-localized-html-file file)))
+          "<link rel=\"stylesheet\" type=\"text/css\" href=~s />\n"
+          (make-uri (gnc-resolve-file-path file))))
 
 
 
