@@ -17,11 +17,94 @@
 ;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
 ;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 
-(use-modules (srfi srfi-26))
+(define-module (gnucash report report-utilities))
+
+(use-modules (srfi srfi-1))
 (use-modules (srfi srfi-13))
+(use-modules (srfi srfi-26))
 (use-modules (ice-9 format))
 (use-modules (ice-9 match))
 
+(use-modules (gnucash utilities))
+(use-modules (gnucash core-utils))
+(use-modules (gnucash engine))
+(use-modules (gnucash app-utils))
+(use-modules (gnucash report html-text))
+(use-modules (gnucash report html-acct-table))
+(use-modules (gnucash gnome-utils))
+
+(export list-ref-safe)
+(export list-set-safe!)
+(export gnc:monetary->string)
+(export gnc:not-all-zeros)
+(export gnc:account-has-shares?)
+(export gnc:account-is-stock?)
+(export gnc:account-is-inc-exp?)
+(export gnc:account-full-name<?)
+(export gnc:accounts-get-children-depth)
+(export gnc:filter-accountlist-type)
+(export gnc:decompose-accountlist)
+(export gnc:account-get-type-string-plural)
+(export gnc:accounts-get-commodities)
+(export gnc:get-current-account-tree-depth)
+(export gnc:accounts-and-all-descendants)
+(export gnc:make-value-collector)
+(export gnc:make-commodity-collector)
+(export gnc:collector+)
+(export gnc:collector-)
+(export gnc:commodity-collector-get-negated)
+(export gnc:account-accumulate-at-dates)
+(export gnc:account-get-balance-at-date)
+(export gnc:account-get-balances-at-dates)
+(export gnc:account-get-comm-balance-at-date)
+(export gnc:account-get-comm-value-interval)
+(export gnc:account-get-comm-value-at-date)
+(export gnc:accounts-get-balance-helper)
+(export gnc:accounts-get-comm-total-profit)
+(export gnc:accounts-get-comm-total-income)
+(export gnc:accounts-get-comm-total-expense)
+(export gnc:accounts-get-comm-total-assets)
+(export gnc:account-get-balance-interval)
+(export gnc:account-get-comm-balance-interval)
+(export gnc:accountlist-get-comm-balance-interval)
+(export gnc:accountlist-get-comm-balance-interval-with-closing)
+(export gnc:accountlist-get-comm-balance-at-date)
+(export gnc:accountlist-get-comm-balance-at-date-with-closing)
+(export gnc:query-set-match-non-voids-only!)
+(export gnc:query-set-match-voids-only!)
+(export gnc:split-voided?)
+(export gnc:report-starting)
+(export gnc:report-render-starting)
+(export gnc:report-percent-done)
+(export gnc:report-finished)
+(export gnc:accounts-count-splits)
+(export gnc-commodity-collector-allzero?)
+(export gnc:monetary+)
+(export gnc:monetaries-add)
+(export gnc:account-get-trans-type-balance-interval)
+(export gnc:account-get-trans-type-balance-interval-with-closing)
+(export gnc:account-get-trans-type-splits-interval)
+(export gnc:budget-get-start-date)
+(export gnc:budget-get-end-date)
+(export gnc:budget-account-get-net)
+(export gnc:budget-accountlist-get-net)
+(export gnc:budget-account-get-initial-balance)
+(export gnc:budget-accountlist-get-initial-balance)
+(export budget-account-sum budget)
+(export gnc:get-account-period-rolledup-budget-value)
+(export gnc:budget-account-get-rolledup-net)
+(export gnc:get-assoc-account-balances)
+(export gnc:select-assoc-account-balance)
+(export gnc:get-assoc-account-balances-total)
+(export gnc:multiline-to-html-text)
+(export gnc:payment-txn->payment-info)
+(export make-file-url)
+(export gnc:strify)
+(export gnc:pk)
+(export gnc:dump-book)
+(export gnc:dump-invoices)
+(export gnc:dump-lot)
+
 (define (list-ref-safe list elt)
   (and (> (length list) elt)
        (list-ref list elt)))
@@ -56,6 +139,18 @@
   (let ((type (xaccAccountGetType account)))
     (member type (list ACCT-TYPE-STOCK ACCT-TYPE-MUTUAL))))
 
+;; account related functions
+;; helper for sorting of account list
+(define (gnc:account-full-name<? a b)
+  (gnc:string-locale<? (gnc-account-get-full-name a) (gnc-account-get-full-name b)))
+
+(define (gnc:accounts-get-children-depth accounts)
+  (1- (apply max
+             (map (lambda (acct)
+                    (+ (gnc-account-get-current-depth acct)
+                       (gnc-account-get-tree-depth acct)))
+                  accounts))))
+
 ;; True if the account is of type income or expense
 
 (define (gnc:account-is-inc-exp? account)
@@ -126,8 +221,8 @@
 	  (sort-and-delete-duplicates
            (map xaccAccountGetCommodity accounts)
            (lambda (a b)
-             (string<? (gnc-commodity-get-unique-name a)
-                       (gnc-commodity-get-unique-name b)))
+             (gnc:string-locale<? (gnc-commodity-get-unique-name a)
+                                  (gnc-commodity-get-unique-name b)))
            gnc-commodity-equiv)))
 
 
@@ -137,12 +232,6 @@
   (let ((root (gnc-get-current-root-account)))
     (gnc-account-get-tree-depth root)))
 
-;; Return accountslist *and* their descendant accounts
-(define (gnc:accounts-and-all-descendants accountslist)
-  (sort-and-delete-duplicates
-   (apply append accountslist (map gnc-account-get-descendants accountslist))
-   gnc:account-path-less-p equal?))
-
 ;;; Here's a statistics collector...  Collects max, min, total, and makes
 ;;; it easy to get at the mean.
 
@@ -400,24 +489,24 @@
       ((date . rest)
        (define (before-date? s) (<= (to-date s) date))
        (define (after-date? s) (< date (to-date s)))
-       (match splits
+       (cond
 
-         ;; end of splits, but still has dates. pad with last-result
-         ;; until end of dates.
-         (() (lp '() rest (cons last-result result) last-result))
+        ;; end of splits, but still has dates. pad with last-result
+        ;; until end of dates.
+        ((null? splits) (lp '() rest (cons last-result result) last-result))
 
-         ;; the next split is still before date.
-         ((and (_ (? before-date?) . _) (head . tail))
-          (lp tail dates result (split->elt head)))
+        ;; the next split is still before date.
+        ((and (pair? (cdr splits)) (before-date? (cadr splits)))
+         (lp (cdr splits) dates result (split->elt (car splits))))
 
-         ;; head split after date, accumulate previous result
-         (((? after-date?) . tail)
-          (lp splits rest (cons last-result result) last-result))
+        ;; head split after date, accumulate previous result
+        ((after-date? (car splits))
+         (lp splits rest (cons last-result result) last-result))
 
-         ;; head split before date, next split after date, or end.
-         ((head . tail)
-          (let ((head-result (split->elt head)))
-            (lp tail rest (cons head-result result) head-result))))))))
+        ;; head split before date, next split after date, or end.
+        (else
+         (let ((head-result (split->elt (car splits))))
+           (lp (cdr splits) rest (cons head-result result) head-result))))))))
 
 ;; This works similar as above but returns a commodity-collector, 
 ;; thus takes care of children accounts with different currencies.
@@ -498,39 +587,6 @@
      accounts)
     collector))
 
-;; Adds all accounts' balances, where the balances are determined with
-;; the get-balance-fn. Intended for usage with a profit and loss
-;; report, hence a) only the income/expense accounts are regarded, and
-;; b) the result is sign reversed. Returns a commodity-collector.
-(define (gnc:accounts-get-comm-total-profit accounts 
-					    get-balance-fn)
-  (issue-deprecation-warning "gnc:accounts-get-comm-total-profit deprecated.")
-  (gnc:accounts-get-balance-helper
-   (gnc:filter-accountlist-type (list ACCT-TYPE-INCOME ACCT-TYPE-EXPENSE) accounts)
-   get-balance-fn
-   (lambda(x) #t)))
-
-;; Adds all accounts' balances, where the balances are determined with
-;; the get-balance-fn. Only the income accounts are regarded, and
-;; the result is sign reversed. Returns a commodity-collector.
-(define (gnc:accounts-get-comm-total-income accounts 
-					    get-balance-fn)
-  (issue-deprecation-warning "gnc:accounts-get-comm-total-income deprecated.")
-  (gnc:accounts-get-balance-helper
-   (gnc:filter-accountlist-type (list ACCT-TYPE-INCOME) accounts)
-   get-balance-fn
-   (lambda(x) #t)))
-
-;; Adds all accounts' balances, where the balances are determined with
-;; the get-balance-fn. Only the expense accounts are regarded, and
-;; the result is sign reversed. Returns a commodity-collector.
-(define (gnc:accounts-get-comm-total-expense accounts 
-                                             get-balance-fn)
-  (issue-deprecation-warning "gnc:accounts-get-comm-total-expense deprecated.")
-  (gnc:accounts-get-balance-helper
-   (gnc:filter-accountlist-type (list ACCT-TYPE-EXPENSE) accounts)
-   get-balance-fn
-   (lambda(x) #t)))
 
 ;; Adds all accounts' balances, where the balances are determined with
 ;; the get-balance-fn. Intended for usage with a balance sheet, hence
@@ -574,54 +630,19 @@
 (define (gnc:accountlist-get-comm-balance-at-date-with-closing accountlist date)
   (gnc:account-get-trans-type-balance-interval-with-closing accountlist #f #f date))
 
-;; utility function - ensure that a query matches only non-voids.  Destructive.
-(define (gnc:query-set-match-non-voids-only! query book)
-  (issue-deprecation-warning
-   "gnc:query-set-match-non-voids-only! is deprecated. add query for\
-(logand CLEARED-ALL (lognot CLEARED-VOIDED)) instead.")
-  (let ((temp-query (qof-query-create-for-splits)))
-    (qof-query-set-book temp-query book)
-
-    (xaccQueryAddClearedMatch
-     temp-query
-     CLEARED-VOIDED
-     QOF-QUERY-AND)
-
-    (let ((inv-query (qof-query-invert temp-query)))
-      (qof-query-merge-in-place query inv-query QOF-QUERY-AND)
-      (qof-query-destroy inv-query)
-      (qof-query-destroy temp-query))))
-
-;; utility function - ensure that a query matches only voids.  Destructive
-
-(define (gnc:query-set-match-voids-only! query book)
-  (issue-deprecation-warning
-   "gnc:query-set-match-non-voids-only! is deprecated. add CLEARED-VOIDED \
-query instead.")
-  (let ((temp-query (qof-query-create-for-splits)))
-    (qof-query-set-book temp-query book)
-
-    (xaccQueryAddClearedMatch
-     temp-query
-     CLEARED-VOIDED
-     QOF-QUERY-AND)
-
-    (qof-query-merge-in-place query temp-query QOF-QUERY-AND)
-    (qof-query-destroy temp-query)))
-
 (define (gnc:split-voided? split)
   (let ((trans (xaccSplitGetParent split)))
     (xaccTransGetVoidStatus trans)))
 
 (define (gnc:report-starting report-name)
   (gnc-window-show-progress (format #f
-				     (G_ "Building '~a' report ...")
+				     (G_ "Building '~a' report …")
 				     (G_ report-name))
 			    0))
 
 (define (gnc:report-render-starting report-name)
   (gnc-window-show-progress (format #f
-				     (G_ "Rendering '~a' report ...")
+				     (G_ "Rendering '~a' report …")
 				     (if (string-null? report-name)
 					 (G_ "Untitled")
 					 (G_ report-name)))
@@ -645,7 +666,7 @@ query instead.")
 
 ;; function to count the total number of splits to be iterated
 (define (gnc:accounts-count-splits accounts)
-  (apply + (map length (map xaccAccountGetSplitList accounts))))
+  (fold (lambda (a b) (+ b (length (xaccAccountGetSplitList a)))) 0 accounts))
 
 ;; Sums up any splits of a certain type affecting a set of accounts.
 ;; the type is an alist '((str "match me") (cased #f) (regexp #f))
@@ -845,14 +866,12 @@ query instead.")
 ;;
 ;; Returns a gnc-numeric value
 (define (gnc:budget-account-get-rolledup-net budget account start-period end-period)
-  (let* ((start (or start-period 0))
-         (end (or end-period (gnc-budget-get-num-periods budget)))
-         (numperiods (- end start -1)))
-  (apply +
-         (map
-          (lambda (period)
-            (gnc:get-account-period-rolledup-budget-value budget account period))
-          (iota numperiods start 1)))))
+  (define end (or end-period (1- (gnc-budget-get-num-periods budget))))
+  (let lp ((period (or start-period 0)) (sum 0))
+    (cond
+     ((> period end) sum)
+     (else (lp (1+ period) (+ sum (gnc:get-account-period-rolledup-budget-value
+                                   budget account period)))))))
 
 ;; ***************************************************************************
 ;; The following 3 functions belong together
@@ -889,6 +908,62 @@ query instead.")
 ;; ***************************************************************************
 ;; Business Functions
 
+(define (sign-equal? a b) (or (= 0 a b) (< 0 (* a b))))
+(define (not-APAR? s)
+  (not (xaccAccountIsAPARType (xaccAccountGetType (xaccSplitGetAccount s)))))
+;; analyse a payment transaction and return a 3-element vector:
+;; (vector invoices overpayment opposing-splits)
+;;
+;; invoices: a list of (cons invoice inv-APAR-split)
+;; opposing-splits: a list of (list pmt-APAR-split partial-amount derived?)
+;;                 partial-amount is a number, derived? is #true if the partial
+;;                 amount does not match the transaction amount
+;; overpayment: a number indicating overpayment amount
+(define (gnc:payment-txn->payment-info txn)
+  (let* ((apar-split (xaccTransGetFirstAPARAcctSplit txn #t))
+        (apar-acct (xaccSplitGetAccount apar-split)))
+    (let lp ((splits (xaccTransGetSplitList txn))
+            (invoices '())
+            (overpayment 0)
+            (opposing-splits '()))
+      (match splits
+        (() (vector invoices opposing-splits overpayment))
+        (((? not-APAR? split) . rest)
+        (gnc:msg "next " (gnc:strify split) " overpayment " (+ overpayment (xaccSplitConvertAmount split apar-acct)))
+        (lp rest invoices (+ overpayment (xaccSplitConvertAmount split apar-acct))
+            opposing-splits))
+        ((split . rest)
+        (let* ((lot (xaccSplitGetLot split))
+                (lot-all-splits (gnc-lot-get-split-list lot)))
+          (define split=? (cut equal? <> split))
+          (match (gncInvoiceGetInvoiceFromLot lot)
+            (() (let lp1 ((lot-splits lot-all-splits)
+                          (overpayment overpayment)
+                          (opposing-splits opposing-splits))
+                  (match lot-splits
+                    (() (lp rest invoices overpayment opposing-splits))
+                    (((? split=?) . tail) (lp1 tail overpayment opposing-splits))
+                    ((s . tail)
+                      (let* ((lot-bal (gnc-lot-get-balance lot))
+                            (lot-bal (if (sign-equal? lot-bal (xaccSplitConvertAmount s apar-acct))
+                                          0 lot-bal))
+                            (derived? (not (zero? lot-bal)))
+                            (partial-amount
+                              (fold
+                              (lambda (a b)
+                                (if (equal? s a) b (+ b (xaccSplitConvertAmount a apar-acct))))
+                              (- lot-bal) lot-all-splits)))
+                        (gnc:msg "next " (gnc:strify s) " overpayment " (+ overpayment partial-amount))
+                        (lp1 tail (+ overpayment partial-amount)
+                            (cons (list s partial-amount derived?)
+                                  opposing-splits)))))))
+            (inv
+              (gnc:msg "next " (gnc:strify split) " overpayment " (+ overpayment (xaccSplitConvertAmount split apar-acct)))
+              (lp rest
+                  (cons (cons inv split) invoices)
+                  (+ overpayment (xaccSplitConvertAmount split apar-acct))
+                  opposing-splits)))))))))
+
 ;; create a stepped list, then add a date in the infinite future for
 ;; the "current" bucket
 (define (make-extended-interval-list to-date num-buckets)
@@ -901,7 +976,7 @@ query instead.")
 (define-public (gnc:owner-splits->aging-list splits num-buckets
                                              to-date date-type receivable?)
   (gnc:msg "processing " (qof-print-date to-date) " date-type " date-type
-           "receivable? " receivable?)
+           " receivable? " receivable?)
   (let ((bucket-dates (make-extended-interval-list to-date (- num-buckets 3)))
         (buckets (make-vector num-buckets 0)))
     (define (addbucket! idx amt)
@@ -919,10 +994,11 @@ query instead.")
                          (xaccSplitGetParent (car splits))))
                (lot (gncInvoiceGetPostedLot invoice))
                (lot-splits (gnc-lot-get-split-list lot))
+               (apar-acct (gncInvoiceGetPostedAcc invoice))
                (bal (fold
                      (lambda (a b)
                        (if (<= (xaccTransGetDate (xaccSplitGetParent a)) to-date)
-                           (+ (xaccSplitGetAmount a) b)
+                           (+ (xaccSplitConvertAmount a apar-acct) b)
                            b))
                      0 lot-splits))
                (bal (if receivable? bal (- bal)))
@@ -937,23 +1013,18 @@ query instead.")
                 (loop (1+ idx) (cdr bucket-dates))))
           (lp (cdr splits))))
 
-       ;; next split is a payment. analyse its sister APAR splits. any
-       ;; split whose lot has no invoice is an overpayment.
+       ;; next split is a payment. analyze using
+       ;; gnc:payment-txn->payment-info and use its overpayment
        ((eqv? (xaccTransGetTxnType (xaccSplitGetParent (car splits)))
               TXN-TYPE-PAYMENT)
         (let* ((txn (xaccSplitGetParent (car splits)))
-               (splitlist (xaccTransGetAPARAcctSplitList txn #f))
-               (overpayment
-                (fold
-                 (lambda (a b)
-                   (if (null? (gncInvoiceGetInvoiceFromLot (xaccSplitGetLot a)))
-                       (- b (gnc-lot-get-balance (xaccSplitGetLot a)))
-                       b))
-                 0 splitlist)))
+               (payment-info (gnc:payment-txn->payment-info txn))
+               (overpayment (vector-ref payment-info 2)))
+          (define (not-txn? s) (not (equal? txn (xaccSplitGetParent s))))
           (gnc:msg "next " (gnc:strify (car splits))
                    " overpayment " overpayment)
           (addbucket! (1- num-buckets) (if receivable? (- overpayment) overpayment))
-          (lp (cdr splits))))
+          (lp (filter not-txn? splits))))
 
        ;; not invoice/prepayment. regular or payment split.
        (else
@@ -962,6 +1033,12 @@ query instead.")
 
 ;; ***************************************************************************
 
+(define (gnc:not-all-zeros data)
+  (cond
+   ((number? data) (not (= 0 data)))
+   ((pair? data) (any gnc:not-all-zeros data))
+   (else #f)))
+
 ;; Adds "file:///" to the beginning of a URL if it doesn't already exist
 ;;
 ;; @param url URL
@@ -1106,7 +1183,7 @@ query instead.")
 
 (define-public (gnc:dump-split s show-acc?)
   (define txn (xaccSplitGetParent s))
-  (format #t "~a Split ~a: ~a~a Amt<~a> ~a~a~a\n"
+  (format #t "~a Split ~a: ~a~a Amt<~a> Val<~a> ~a~a~a\n"
           (xaccSplitGetReconcile s)
           (string-take (gncSplitGetGUID s) 8)
           (qof-print-date (xaccTransGetDate txn))
@@ -1120,10 +1197,12 @@ query instead.")
             (xaccTransGetCurrency txn)
             (xaccSplitGetValue s)))
           (maybe-str 'Desc (xaccTransGetDescription txn))
+          (maybe-str 'Action (xaccSplitGetAction s))
           (maybe-str 'Memo (xaccSplitGetMemo s))))
 
 (define-public (gnc:dump-all-transactions)
   (define query (qof-query-create-for-splits))
+  (define (split-has-no-account? split) (null? (xaccSplitGetAccount split)))
   (display "\n(gnc:dump-all-transactions)\n")
   (qof-query-set-book query (gnc-get-current-book))
   (qof-query-set-sort-order query (list SPLIT-TRANS TRANS-DATE-POSTED) '() '())
@@ -1132,6 +1211,7 @@ query instead.")
     (newline)
     (match splits
       (() (qof-query-destroy query))
+      (((? split-has-no-account?) . rest) (lp rest))
       ((split . rest)
        (let ((trans (xaccSplitGetParent split)))
          (format #t "  Trans ~a: ~a Curr ~a ~a~a\n"
@@ -1184,7 +1264,7 @@ query instead.")
     (define (maybe-date time64)         ;handle INT-MAX differently
       (if (= 9223372036854775807 time64) "?" (qof-print-date time64)))
     (define (maybe-trunc str)
-      (if (> (string-length str) 20) (string-append (substring str 0 17) "...") str))
+      (if (> (string-length str) 20) (string-append (substring str 0 17) "…") str))
     (define (inv-amt->string inv amt)
       (gnc:monetary->string
        (gnc:make-gnc-monetary
