@@ -137,55 +137,40 @@
 (define tax-qtr-real-qtr-year 0)
 
 (define (tax-options-generator)
-  (define options (gnc:new-options))
-  (define (gnc:register-tax-option new-option)
-    (gnc:register-option options new-option))
+  (define options (gnc-new-optiondb))
 
   ;; date at which to report 
   (gnc:options-add-date-interval!
    options gnc:pagename-general 
    (N_ "From") (N_ "To") "a")
 
-  (gnc:register-tax-option
-   (gnc:make-multichoice-option
+  (gnc-register-multichoice-option options
     gnc:pagename-general (N_ "Alternate Period")
     "c" (N_ "Override or modify From: & To:.")
-    (if after-tax-day 'from-to 'last-year)
-    (list (vector 'from-to (N_ "Use From - To") (N_ "Use From - To period."))
-          (vector '1st-est (N_ "1st Est Tax Quarter") (N_ "Jan 1 - Mar 31."))
-          (vector '2nd-est (N_ "2nd Est Tax Quarter") (N_ "Apr 1 - May 31."))
-          (vector '3rd-est (N_ "3rd Est Tax Quarter") (N_ "Jun 1 - Aug 31."))
-          (vector '4th-est (N_ "4th Est Tax Quarter") (N_ "Sep 1 - Dec 31."))
-          (vector 'last-year (N_ "Last Year") (N_ "Last Year."))
-          (vector '1st-last
-                  (N_ "Last Yr 1st Est Tax Qtr")
-                  (N_ "Jan 1 - Mar 31, Last year."))
-          (vector '2nd-last
-                  (N_ "Last Yr 2nd Est Tax Qtr")
-                  (N_ "Apr 1 - May 31, Last year."))
-          (vector '3rd-last
-                  (N_ "Last Yr 3rd Est Tax Qtr")
-                  (N_ "Jun 1 - Aug 31, Last year."))
-          (vector '4th-last
-                  (N_ "Last Yr 4th Est Tax Qtr")
-                  (N_ "Sep 1 - Dec 31, Last year.")))))
+    (if after-tax-day "from-to" "last-year")
+    (list (vector 'from-to (N_ "Use From - To"))
+          (vector '1st-est (N_ "1st Est Tax Quarter (Jan 1 - Mar 31)"))
+          (vector '2nd-est (N_ "2nd Est Tax Quarter (Apr 1 - May 31)"))
+          (vector '3rd-est (N_ "3rd Est Tax Quarter (Jun 1 - Aug 31)"))
+          (vector '4th-est (N_ "4th Est Tax Quarter (Sep 1 - Dec 31)"))
+          (vector 'last-year (N_ "Last Year"))
+          (vector '1st-last (N_ "Last Yr 1st Est Tax Qtr (Jan 1 - Mar 31)"))
+          (vector '2nd-last (N_ "Last Yr 2nd Est Tax Qtr (Apr 1 - May 31)"))
+          (vector '3rd-last (N_ "Last Yr 3rd Est Tax Qtr (Jun 1 - Aug 31)"))
+          (vector '4th-last (N_ "Last Yr 4th Est Tax Qtr (Sep 1 - Dec 31)"))))
 
-  (gnc:register-tax-option
-   (gnc:make-account-list-option
+  (gnc-register-account-list-option options
     gnc:pagename-accounts (N_ "Select Accounts (none = all)")
     "d" (N_ "Select accounts.")
-    (lambda () '())
-    #f #t))
+    '())
   
-  (gnc:register-tax-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     gnc:pagename-display (N_ "Suppress $0.00 values")
-    "f" (N_ "$0.00 valued Accounts won't be printed.") #t))
+    "f" (N_ "$0.00 valued Accounts won't be printed.") #t)
 
-  (gnc:register-tax-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     gnc:pagename-display (N_ "Print Full account names")
-    "g" (N_ "Print all Parent account names.") #f))
+    "g" (N_ "Print all Parent account names.") #f)
 
   (gnc:options-set-default-section options gnc:pagename-general)
 
@@ -456,13 +441,11 @@
 (define (generate-tax-or-txf report-name
                              report-description
                              report-obj
-                             tax-mode?
-                             file-name)
+                             tax-mode?)
 
   (define (get-option pagename optname)
-    (gnc:option-value
-     (gnc:lookup-option 
-      (gnc:report-options report-obj) pagename optname)))
+    (gnc-optiondb-lookup-value
+      (gnc:report-options report-obj) pagename optname))
 
   ;; the number of account generations: children, grandchildren etc.
   (define (num-generations account gen)
@@ -471,9 +454,8 @@
 		 (txf-special-split? (gnc:account-get-txf-code account)))
 	    (+ gen 1)		; Est Fed Tax has a extra generation
 	    gen)	       		; no kids, return input
-	(apply max (gnc:account-map-children
-		    (lambda (x) (num-generations x (+ 1 gen)))
-		    account))))
+	(apply max (map (lambda (x) (num-generations x (1+ gen)))
+                        (or (gnc-account-get-children-sorted account) '())))))
 
   (gnc:report-starting reportname)
   (let* ((from-value (gnc:date-option-absolute-time 
@@ -758,7 +740,7 @@
 	  (to-year    (gnc-print-time64 to-value "%Y"))
           (today-date (gnc-print-time64 (time64CanonicalDayTime (current-time))
                                         "%d.%m.%Y"))
-	  (tax-nr (gnc:option-get-value book gnc:*tax-label* gnc:*tax-nr-label*)))
+	  (tax-nr (gnc:book-get-option-value book gnc:*tax-label* gnc:*tax-nr-label*)))
 
       ;; Now, the main body
       ;; Reset all the balance collectors
@@ -862,14 +844,12 @@ Gehen Sie zu Bearbeiten -> Optionen Steuerbericht, um Konten entsprechend einzur
               (G_ "This report shows your Taxable Income and \
 Deductible Expenses.")
               report-obj
-              #t
-              #f))
+              #t))
  'export-types (list (cons (G_ "XML") 'txf))
- 'export-thunk (lambda (report-obj choice file-name)
+ 'export-thunk (lambda (report-obj choice)
                  (generate-tax-or-txf
                   (G_ "Taxable Income / Deductible Expenses")
                   (G_ "This page shows your Taxable Income and \
 Deductible Expenses.")
                   report-obj
-                  #f
-                  file-name)))
+                  #f)))
