@@ -31,6 +31,7 @@
 (use-modules (gnucash app-utils))
 (use-modules (gnucash report))
 (use-modules (srfi srfi-1))
+(use-modules (ice-9 format))
 
 (define menuname-income (N_ "Income Piechart"))
 (define menuname-expense (N_ "Expense Piechart"))
@@ -67,7 +68,7 @@ balance at a given time"))
 (define optname-price-source (N_ "Price Source"))
 
 (define optname-accounts (N_ "Accounts"))
-(define optname-levels (N_ "Show Accounts until level"))
+(define optname-levels (N_ "Levels of Subaccounts"))
 
 (define optname-fullname (N_ "Show long names"))
 (define optname-show-total (N_ "Show Totals"))
@@ -83,14 +84,8 @@ balance at a given time"))
 ;; The option-generator. The only dependence on the type of piechart
 ;; is the list of account types that the account selection option
 ;; accepts.
-(define (options-generator account-types reverse-balance? do-intervals? depth-based?)
-  (let* ((options (gnc:new-options)) 
-         (add-option 
-          (lambda (new-option)
-            (gnc:register-option options new-option))))
-
-    (add-option
-     (gnc:make-internal-option "__report" "reverse-balance?" reverse-balance?))
+(define (options-generator account-types do-intervals? depth-based?)
+  (let* ((options (gnc-new-optiondb)))
 
     (if do-intervals?
         (gnc:options-add-date-interval!
@@ -100,89 +95,66 @@ balance at a given time"))
          options gnc:pagename-general
          optname-to-date "a"))
 
-    (gnc:options-add-currency! 
+    (gnc:options-add-currency!
      options gnc:pagename-general optname-report-currency "b")
-    
-    (gnc:options-add-price-source! 
+
+    (gnc:options-add-price-source!
      options gnc:pagename-general
      optname-price-source "c" 'pricedb-nearest)
 
     (if do-intervals?
-        (add-option
-         (gnc:make-multichoice-option
+        (gnc-register-multichoice-option options
           gnc:pagename-general optname-averaging
           "f" opthelp-averaging
-          'None
-          (list (vector 'None
-                        (N_ "No Averaging")
-                        (N_ "Just show the amounts, without any averaging."))
-                (vector 'YearDelta
-                        (N_ "Yearly")
-                        (N_ "Show the average yearly amount during the reporting period."))
-                (vector 'MonthDelta
-                        (N_ "Monthly")
-                        (N_ "Show the average monthly amount during the reporting period."))
-                (vector 'WeekDelta
-                        (N_ "Weekly")
-                        (N_ "Show the average weekly amount during the reporting period."))
-                )
-          ))
-        )
+          "None"
+          (list (vector 'None (N_ "No Averaging"))
+                (vector 'YearDelta (N_ "Yearly"))
+                (vector 'MonthDelta (N_ "Monthly"))
+                (vector 'WeekDelta (N_ "Weekly")))))
 
-    (add-option
-     (gnc:make-account-list-option
+    (gnc-register-account-list-limited-option options
       gnc:pagename-accounts optname-accounts
       "a"
       (N_ "Report on these accounts, if chosen account level allows.")
-      (lambda ()
-        (gnc:filter-accountlist-type 
-         account-types
-         (gnc-account-get-descendants-sorted (gnc-get-current-root-account))))
-      (lambda (accounts)
-        (list #t
-              (gnc:filter-accountlist-type
-               account-types
-               accounts)))
-      #t))
+      (gnc:filter-accountlist-type
+       account-types
+         (gnc-account-get-descendants-sorted (gnc-get-current-root-account)))
+      account-types)
 
     (if depth-based?
       (gnc:options-add-account-levels!
        options gnc:pagename-accounts optname-levels "b"
-       (N_ "Show accounts to this depth and not further.")
+       (N_ "Maximum number of levels in the account tree displayed.")
        2))
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-fullname
       "a"
       (if depth-based?
         (N_ "Show the full account name in legend?")
         (N_ "Show the full security name in the legend?"))
-      #f))
+      #f)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-show-total
-      "b" (N_ "Show the total balance in legend?") #t))
+      "b" (N_ "Show the total balance in legend?") #t)
 
 
-     (add-option
-      (gnc:make-simple-boolean-option
+     (gnc-register-simple-boolean-option options
        gnc:pagename-display optname-show-percent
-       "b" (N_ "Show the percentage in legend?") #t))
+       "b" (N_ "Show the percentage in legend?") #t)
 
 
-    (add-option
-     (gnc:make-number-range-option
+    (gnc-register-number-range-option options
       gnc:pagename-display optname-slices
       "c" (N_ "Maximum number of slices in pie.") 7
-      2 24 0 1))
+      2 255 1)
 
-    (gnc:options-add-plot-size! 
-     options gnc:pagename-display 
+    (gnc:options-add-plot-size!
+     options gnc:pagename-display
      optname-plot-width optname-plot-height "d" (cons 'percent 100.0) (cons 'percent 100.0))
 
-    (gnc:options-add-sort-method! 
+    (gnc:options-add-sort-method!
      options gnc:pagename-display
      optname-sort-method "e" 'amount)
 
@@ -209,12 +181,12 @@ balance at a given time"))
   (cond
    ((eq? sort-method 'acct-code)
     (lambda (a b)
-      (string<? (xaccAccountGetCode (cadr a))
-                (xaccAccountGetCode (cadr b)))))
+      (gnc:string-locale<? (xaccAccountGetCode (cadr a))
+                           (xaccAccountGetCode (cadr b)))))
    ((eq? sort-method 'alphabetical)
     (lambda (a b)
-      (string<? (display-name-accounts show-fullname? (cadr a))
-                (display-name-accounts show-fullname? (cadr b)))))
+      (gnc:string-locale<? (display-name-accounts show-fullname? (cadr a))
+                           (display-name-accounts show-fullname? (cadr b)))))
    (else
     (lambda (a b) (> (car a) (car b))))))
 
@@ -223,12 +195,12 @@ balance at a given time"))
   (cond
    ((eq? sort-method 'acct-code)
     (lambda (a b)
-      (string<? (gnc-commodity-get-mnemonic (cadr a))
-                (gnc-commodity-get-mnemonic (cadr b)))))
+      (gnc:string-locale<? (gnc-commodity-get-mnemonic (cadr a))
+                           (gnc-commodity-get-mnemonic (cadr b)))))
    ((eq? sort-method 'alphabetical)
     (lambda (a b)
-      (string<? (display-name-security show-fullname? (cadr a))
-                (display-name-security show-fullname? (cadr b)))))
+      (gnc:string-locale<? (display-name-security show-fullname? (cadr a))
+                           (display-name-security show-fullname? (cadr b)))))
    (else
     (lambda (a b) (> (car a) (car b))))))
 
@@ -315,15 +287,14 @@ balance at a given time"))
 ;; account-types to work on and whether this report works on
 ;; intervals as arguments.
 (define (piechart-renderer report-obj reportname report-guid
-                           account-types do-intervals? depth-based?
+                           account-types do-intervals? depth-based? reverse-balance?
                            display-name sort-comparator get-data)
 
   ;; This is a helper function for looking up option values.
   (define (get-option section name)
-    (gnc:option-value 
-     (gnc:lookup-option 
-      (gnc:report-options report-obj) section name)))
-  
+    (gnc-optiondb-lookup-value
+      (gnc:report-options report-obj) section name))
+
   (gnc:report-starting reportname)
 
   ;; Get all options
@@ -360,7 +331,6 @@ balance at a given time"))
         (height (get-option gnc:pagename-display optname-plot-height))
         (width (get-option gnc:pagename-display optname-plot-width))
 	(sort-method (get-option gnc:pagename-display optname-sort-method))
-	(reverse-balance? (get-option "__report" "reverse-balance?"))
 
         (document (gnc:make-html-document))
         (chart (gnc:make-html-chart))
@@ -410,38 +380,34 @@ balance at a given time"))
            ;; accordingly.
            (report-title
             (case averaging-selection
-              ((YearDelta) (string-append report-title " " (_ "Yearly Average")))
-              ((MonthDelta) (string-append report-title " " (_ "Monthly Average")))
-              ((WeekDelta) (string-append report-title " " (_ "Weekly Average")))
+              ((YearDelta) (string-append report-title " " (G_ "Yearly Average")))
+              ((MonthDelta) (string-append report-title " " (G_ "Monthly Average")))
+              ((WeekDelta) (string-append report-title " " (G_ "Weekly Average")))
               (else report-title)))
            (combined '())
-           (other-anchor "")
-           (print-info (gnc-commodity-print-info report-currency #t)))
+           (other-anchor ""))
 
-      ;; Converts a commodity-collector into one single double
+      ;; Converts a commodity-collector into one single inexact
       ;; number, depending on the report's currency and the
       ;; exchange-fn calculated above. Returns the absolute value
-      ;; as double, multiplied by the averaging-multiplies (smaller
-      ;; than one; multiplication instead of division to avoid
-      ;; division-by-zero issues) in case the user wants to see the
-      ;; amounts averaged over some value.
-      (define (collector->double c)
+      ;; multiplied by the averaging-multiplier (smaller than one;
+      ;; multiplication instead of division to avoid division-by-zero
+      ;; issues) in case the user wants to see the amounts averaged
+      ;; over some value.
+      (define (collector->amount c)
         ;; Future improvement: Let the user choose which kind of
         ;; currency combining she want to be done. Right now
         ;; everything foreign gets converted
         ;; (gnc:sum-collector-commodity) based on the average
         ;; cost of all holdings.
-        (*
-         (gnc:gnc-monetary-amount
-          (gnc:sum-collector-commodity
-           c report-currency
-           exchange-fn))
-         averaging-multiplier))
+        (* (gnc:gnc-monetary-amount
+            (gnc:sum-collector-commodity c report-currency exchange-fn))
+           averaging-multiplier))
 
-      ;; Get balance of an account as double number, already converted
-      ;; to the report's currency.
+      ;; Get balance of an account as an inexact number converted to,
+      ;; and using precision of the report's currency.
       (define (account-balance a subaccts?)
-        (collector->double (profit-fn a subaccts?)))
+        (collector->amount (profit-fn a subaccts?)))
 
       (define (count-accounts current-depth accts)
 	(if (< current-depth tree-depth)
@@ -465,7 +431,7 @@ balance at a given time"))
 
       (define (fix-signs combined)
         (map (lambda (pair)
-               (if (reverse-balance? (cadr pair))
+               (if reverse-balance?
                    (cons (- (car pair)) (cdr pair))
                    pair))
 	     combined))
@@ -488,7 +454,7 @@ balance at a given time"))
                        (sum (apply + (unzip1 finish))))
                   (set! combined
                         (append start
-                                (list (list sum (_ "Other")))))
+                                (list (list sum (G_ "Other")))))
                   (if depth-based?
                     (let ((options (gnc:make-report-options report-guid))
                           (id #f))
@@ -496,9 +462,8 @@ balance at a given time"))
                       (gnc:options-copy-values (gnc:report-options report-obj)
                                                options)
                       ;; and set the destination accounts
-                      (gnc:option-set-value
-                       (gnc:lookup-option options gnc:pagename-accounts
-                                          optname-accounts)
+                      (gnc-set-option
+                       options gnc:pagename-accounts optname-accounts
                        (map cadr finish))
                       (set! id (gnc:make-report report-guid options))
                       ;; set the URL.
@@ -522,13 +487,18 @@ balance at a given time"))
                                              (list gnc:pagename-accounts optname-accounts
                                                    (cons acct subaccts))
                                              (list gnc:pagename-accounts optname-levels
-                                                   (+ 1 tree-depth))
+                                                   (min 6 (+ 1 tree-depth)))
                                              (list gnc:pagename-general
                                                    gnc:optname-reportname
                                                    ((if show-fullname?
                                                         gnc-account-get-full-name
                                                         xaccAccountGetName) acct))))))))
-                               combined))))
+                               combined)))
+                   (scu (gnc-commodity-get-fraction report-currency)))
+
+               (define (round-scu amt)
+                 (gnc-numeric-convert amt scu GNC-HOW-RND-ROUND))
+
                (gnc:html-chart-set-type! chart 'pie)
 
                (gnc:html-chart-set-currency-iso!
@@ -541,11 +511,11 @@ balance at a given time"))
                             (string-append
                              (if do-intervals?
                                  (format #f
-                                         (_ "~a to ~a")
+                                         (G_ "~a to ~a")
                                          (qof-print-date from-date)
                                          (qof-print-date to-date))
                                  (format #f
-                                         (_ "Balance at ~a")
+                                         (G_ "Balance at ~a")
                                          (qof-print-date to-date)))
                              (if show-total?
                                  (let ((total (apply + (unzip1 combined))))
@@ -553,13 +523,14 @@ balance at a given time"))
                                     #f ": ~a"
                                     (gnc:monetary->string
                                      (gnc:make-gnc-monetary
-                                      report-currency total))))
+                                      report-currency
+                                      (round-scu total)))))
                                  ""))))
                (gnc:html-chart-set-width! chart width)
                (gnc:html-chart-set-height! chart height)
                (gnc:html-chart-add-data-series! chart
-                                                "Accounts"
-                                                (unzip1 combined)
+                                                (G_ "Accounts")
+                                                (map round-scu (unzip1 combined))
                                                 (gnc:assign-colors (length combined))
                                                 'urls urls)
                (gnc:html-chart-set-axes-display! chart #f)
@@ -578,7 +549,7 @@ balance at a given time"))
                          (gnc:monetary->string
                           (gnc:make-gnc-monetary
                            report-currency
-                           (car series))))
+                           (round-scu (car series)))))
                         "")
                     (if show-percent?
                         (format #f " (~2,1f%)"
@@ -603,8 +574,8 @@ balance at a given time"))
       document)))
 
 (define (build-report!
-          name acct-types income-expense? depth-based? menuname menutip
-          reverse-balance? uuid)
+         name acct-types income-expense? depth-based? reverse-balance?
+         menuname menutip uuid)
   (gnc:define-report
     'version 1
     'name name
@@ -615,12 +586,12 @@ balance at a given time"))
     'menu-name menuname
     'menu-tip menutip
     'options-generator (lambda () (options-generator acct-types
-                                                     reverse-balance?
                                                      income-expense?
                                                      depth-based?))
     'renderer (lambda (report-obj)
                 (piechart-renderer report-obj name uuid
                                    acct-types income-expense? depth-based?
+                                   reverse-balance?
                                    (if depth-based?
                                        display-name-accounts
                                        display-name-security)
@@ -634,17 +605,15 @@ balance at a given time"))
 (build-report!
   reportname-income
   (list ACCT-TYPE-INCOME)
-  #t #t
+  #t #t #t
   menuname-income menutip-income
-  (lambda (x) #t)
   "e1bd09b8a1dd49dd85760db9d82b045c")
 
 (build-report!
   reportname-expense
   (list ACCT-TYPE-EXPENSE)
-  #t #t
+  #t #t #f
   menuname-expense menutip-expense
-  (lambda (x) #f)
   "9bf1892805cb4336be6320fe48ce5446")
 
 (build-report!
@@ -653,9 +622,8 @@ balance at a given time"))
         ACCT-TYPE-SAVINGS ACCT-TYPE-MONEYMRKT
         ACCT-TYPE-RECEIVABLE ACCT-TYPE-STOCK ACCT-TYPE-MUTUAL
         ACCT-TYPE-CURRENCY)
-  #f #t
+  #f #t #f
   menuname-assets menutip-assets
-  (lambda (x) #f)
   "5c7fd8a1fe9a4cd38884ff54214aa88a")
 
 (build-report!
@@ -664,16 +632,14 @@ balance at a given time"))
         ACCT-TYPE-SAVINGS ACCT-TYPE-MONEYMRKT
         ACCT-TYPE-RECEIVABLE ACCT-TYPE-STOCK ACCT-TYPE-MUTUAL
         ACCT-TYPE-CURRENCY)
-  #f #f
+  #f #f #f
   menuname-securities menutip-securities
-  (lambda (x) #f)
   "e9418ff64f2c11e5b61d1c7508d793ed")
 
 (build-report!
   reportname-liabilities
   (list ACCT-TYPE-LIABILITY ACCT-TYPE-PAYABLE ACCT-TYPE-CREDIT
         ACCT-TYPE-CREDITLINE)
-  #f #t
+  #f #t #t
   menuname-liabilities menutip-liabilities
-  (lambda (x) #t)
   "3fe6dce77da24c66bdc8f8efdea7f9ac")
