@@ -21,12 +21,8 @@
 ;;    This code makes the assumption that you want your balance
 ;;    sheet to no more than daily resolution.
 ;;    
-;;    The Company Name field does not currently default to the name
-;;    in (gnc-get-current-book).
-;;    
 ;;    Line & column alignments still do not conform with
 ;;    textbook accounting practice (they're close though!).
-;;    The 'canonically-tabbed option is currently broken.
 ;;    
 ;;    Progress bar functionality is currently mostly broken.
 ;;    
@@ -76,9 +72,6 @@
 (define optname-report-title (N_ "Report Title"))
 (define opthelp-report-title (N_ "Title for this report."))
 
-(define optname-party-name (N_ "Company name"))
-(define opthelp-party-name (N_ "Name of company/individual."))
-
 (define optname-date (N_ "Balance Sheet Date"))
 (define optname-report-form (N_ "Single column Balance Sheet"))
 (define opthelp-report-form
@@ -145,43 +138,30 @@
 
 ;; options generator
 (define (balance-sheet-options-generator)
-  (let* ((options (gnc:new-options))
-         (book (gnc-get-current-book)) ; XXX Find a way to get the book that opened the report
-         (add-option 
-          (lambda (new-option)
-            (gnc:register-option options new-option))))
-    
-    (add-option
-      (gnc:make-string-option
+  (let* ((options (gnc-new-optiondb)))
+
+    (gnc-register-string-option options
       gnc:pagename-general optname-report-title
-      "a" opthelp-report-title (_ reportname)))
-    (add-option
-      (gnc:make-string-option
-      gnc:pagename-general optname-party-name
-      "b" opthelp-party-name (or (gnc:company-info book gnc:*company-name*) "")))
-    
+      "a" opthelp-report-title (G_ reportname))
+
     ;; date at which to report balance
     (gnc:options-add-report-date!
      options gnc:pagename-general optname-date "c")
-    
-    (add-option
-     (gnc:make-simple-boolean-option
-      gnc:pagename-general optname-report-form
-      "d" opthelp-report-form #t))
 
-    (add-option
-      (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
+      gnc:pagename-general optname-report-form
+      "d" opthelp-report-form #t)
+
+    (gnc-register-simple-boolean-option options
        gnc:pagename-general optname-standard-order
-       "dd" opthelp-standard-order #t))
+       "dd" opthelp-standard-order #t)
 
     ;; accounts to work on
-    (add-option
-     (gnc:make-account-list-option
+    (gnc-register-account-list-option options
       gnc:pagename-accounts optname-accounts
       "a"
       opthelp-accounts
-      (lambda ()
-	(gnc:filter-accountlist-type 
+      (gnc:filter-accountlist-type
          (list ACCT-TYPE-BANK ACCT-TYPE-CASH ACCT-TYPE-CREDIT
                ACCT-TYPE-ASSET ACCT-TYPE-LIABILITY
                ACCT-TYPE-STOCK ACCT-TYPE-MUTUAL ACCT-TYPE-CURRENCY
@@ -189,43 +169,37 @@
                ACCT-TYPE-EQUITY ACCT-TYPE-INCOME ACCT-TYPE-EXPENSE
                ACCT-TYPE-TRADING)
 	 (gnc-account-get-descendants-sorted (gnc-get-current-root-account))))
-      #f #t))
     (gnc:options-add-account-levels!
      options gnc:pagename-accounts optname-depth-limit
      "b" opthelp-depth-limit 3)
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-accounts optname-bottom-behavior
-      "c" opthelp-bottom-behavior #f))
-    
+      "c" opthelp-bottom-behavior #f)
+
     ;; all about currencies
     (gnc:options-add-currency!
      options pagename-commodities
      optname-report-commodity "a")
-    
-    (gnc:options-add-price-source! 
+
+    (gnc:options-add-price-source!
      options pagename-commodities
      optname-price-source "b" 'pricedb-nearest)
-    
-    (add-option 
-     (gnc:make-simple-boolean-option
-      pagename-commodities optname-show-foreign 
-      "c" opthelp-show-foreign #t))
-    
-    (add-option 
-     (gnc:make-simple-boolean-option
+
+    (gnc-register-simple-boolean-option options
+      pagename-commodities optname-show-foreign
+      "c" opthelp-show-foreign #t)
+
+    (gnc-register-simple-boolean-option options
       pagename-commodities optname-show-rates
-      "d" opthelp-show-rates #f))
-    
+      "d" opthelp-show-rates #f)
+
     ;; what to show for zero-balance accounts
-    (add-option 
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-show-zb-accts
-      "a" opthelp-show-zb-accts #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "a" opthelp-show-zb-accts #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-omit-zb-bals
-      "b" opthelp-omit-zb-bals #f))
+      "b" opthelp-omit-zb-bals #f)
     ;; what to show for non-leaf accounts
     (gnc:options-add-subtotal-view!
      options gnc:pagename-display
@@ -233,45 +207,37 @@
      "c")
 
     ;; some detailed formatting options
-    (add-option 
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-account-links
-      "e" opthelp-account-links #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "e" opthelp-account-links #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-use-rules
-      "f" opthelp-use-rules #f))
-    
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "f" opthelp-use-rules #f)
+
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-label-assets
-      "g" opthelp-label-assets #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "g" opthelp-label-assets #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-total-assets
-      "h" opthelp-total-assets #t))
-    
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "h" opthelp-total-assets #t)
+
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-label-liabilities
-      "i" opthelp-label-liabilities #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "i" opthelp-label-liabilities #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-total-liabilities
-      "j" opthelp-total-liabilities #t))
-    
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "j" opthelp-total-liabilities #t)
+
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-label-equity
-      "k" opthelp-label-equity #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "k" opthelp-label-equity #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-total-equity
-      "l" opthelp-total-equity #t))
-    
+      "l" opthelp-total-equity #t)
+
     ;; Set the accounts page as default option tab
     (gnc:options-set-default-section options gnc:pagename-accounts)
-    
+
     options))
 
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -283,16 +249,15 @@
 
 (define (balance-sheet-renderer report-obj)
   (define (get-option pagename optname)
-    (gnc:option-value
-     (gnc:lookup-option
-      (gnc:report-options report-obj) pagename optname)))
+    (gnc-optiondb-lookup-value
+      (gnc:report-options report-obj) pagename optname))
 
   (gnc:report-starting reportname)
 
   ;; get all option's values
   (let* (
          (report-title (get-option gnc:pagename-general optname-report-title))
-         (company-name (get-option gnc:pagename-general optname-party-name))
+         (company-name (or (gnc:company-info (gnc-get-current-book) gnc:*company-name*) ""))
          (reportdate (gnc:time64-end-day-time
                       (gnc:date-option-absolute-time
                        (get-option gnc:pagename-general optname-date))))
@@ -309,7 +274,7 @@
          (parent-balance-mode (get-option gnc:pagename-display
                                           optname-parent-balance-mode))
          (parent-total-mode
-          (assq-ref '((t . #t) (f . #f) (canonically-tabbed . canonically-tabbed))
+          (assq-ref '((t . #t) (f . #f))
                     (get-option gnc:pagename-display optname-parent-total-mode)))
          (show-zb-accts? (get-option gnc:pagename-display optname-show-zb-accts))
          (omit-zb-bals? (get-option gnc:pagename-display optname-omit-zb-bals))
@@ -343,6 +308,7 @@
          (tree-depth (if (eq? depth-limit 'all)
                          (gnc:get-current-account-tree-depth)
                          depth-limit))
+         (price-fn (gnc:case-price-fn price-source report-commodity reportdate))
          ;; exchange rates calculation parameters
          (exchange-fn
           (gnc:case-exchange-fn price-source report-commodity reportdate)))
@@ -383,11 +349,11 @@
       (let* ((liability-table
               (gnc:make-html-acct-table/env/accts table-env liability-accounts)))
         (when label-liabilities?
-          (add-subtotal-line  parent-table (_ "Liabilities") #f #f))
+          (add-subtotal-line  parent-table (G_ "Liabilities") #f #f))
         (gnc:html-table-add-account-balances parent-table liability-table params)
         (when total-liabilities?
           (add-subtotal-line
-           parent-table (_ "Total Liabilities") #f liability-balance))
+           parent-table (G_ "Total Liabilities") #f liability-balance))
         (add-rule parent-table)))
 
     (define (get-total-value-fn account)
@@ -482,10 +448,10 @@
           (gnc:report-percent-done 80)
 
           (when label-assets?
-            (add-subtotal-line left-table (_ "Assets") #f #f))
+            (add-subtotal-line left-table (G_ "Assets") #f #f))
           (gnc:html-table-add-account-balances left-table asset-table params)
           (when total-assets?
-            (add-subtotal-line left-table (_ "Total Assets") #f asset-balance))
+            (add-subtotal-line left-table (G_ "Total Assets") #f asset-balance))
 
           (when report-form?
             (add-rule left-table)
@@ -499,29 +465,29 @@
           (gnc:report-percent-done 88)
 
           (when label-equity?
-            (add-subtotal-line right-table (_ "Equity") #f #f))
+            (add-subtotal-line right-table (G_ "Equity") #f #f))
           (gnc:html-table-add-account-balances right-table equity-table params)
           ;; we omit retained earnings & unrealized gains
           ;; from the balance report, if zero, since they
           ;; are not present on normal balance sheets
           (unless (gnc-commodity-collector-allzero? retained-earnings)
             (add-subtotal-line right-table
-                               (_ "Retained Earnings")
-                               (_ "Retained Losses")
+                               (G_ "Retained Earnings")
+                               (G_ "Retained Losses")
                                retained-earnings))
           (unless (gnc-commodity-collector-allzero? trading-balance)
             (add-subtotal-line right-table
-                               (_ "Trading Gains")
-                               (_ "Trading Losses")
+                               (G_ "Trading Gains")
+                               (G_ "Trading Losses")
                                trading-balance))
           (unless (gnc-commodity-collector-allzero? unrealized-gain-collector)
             (add-subtotal-line right-table
-                               (_ "Unrealized Gains")
-                               (_ "Unrealized Losses")
+                               (G_ "Unrealized Gains")
+                               (G_ "Unrealized Losses")
                                unrealized-gain-collector))
           (when total-equity?
             (add-subtotal-line
-             right-table (_ "Total Equity") #f total-equity-balance))
+             right-table (G_ "Total Equity") #f total-equity-balance))
 
           (add-rule right-table)
 
@@ -531,7 +497,7 @@
                                  total-liabilities? liability-balance))
 
           (add-subtotal-line
-           right-table (gnc:html-string-sanitize (_ "Total Liabilities & Equity"))
+           right-table (gnc:html-string-sanitize (G_ "Total Liabilities & Equity"))
            #f liability-plus-equity)
 
           (gnc:html-document-add-object!
@@ -550,7 +516,7 @@
           (gnc:report-percent-done 90)
           (when show-rates?
             (gnc:html-document-add-object!
-             doc (gnc:html-make-exchangerates report-commodity exchange-fn accounts)))
+             doc (gnc:html-make-rates-table report-commodity price-fn accounts)))
 
           (gnc:report-percent-done 100)))
 
