@@ -46,7 +46,6 @@
 (define optname-show-subaccounts (N_ "Always show sub-accounts"))
 (define optname-accounts (N_ "Account"))
 
-(define optname-select-columns (N_ "Select Columns"))
 (define optname-show-budget (N_ "Show Budget"))
 (define opthelp-show-budget (N_ "Display a column for the budget values."))
 (define optname-show-notes (N_ "Show Budget Notes"))
@@ -95,57 +94,38 @@
 
 ;;List of common helper functions, that is not bound only to options generation or report evaluation
 (define (get-option-val options pagename optname)
-  (gnc:option-value
-   (gnc:lookup-option options pagename optname)))
+  (gnc-optiondb-lookup-value options pagename optname))
 
 (define (set-option-enabled options page opt-name enabled)
-  (gnc-option-db-set-option-selectable-by-name
+  (gnc-optiondb-set-option-selectable-by-name
    options page opt-name enabled))
 
 ;; options generator
 (define (budget-report-options-generator)
-  (let* ((options (gnc:new-options))
-         (add-option
-          (lambda (new-option)
-            (gnc:register-option options new-option)))
+  (let* ((options (gnc-new-optiondb))
          (period-options
-          (list (vector 'first
-                        (N_ "First")
-                        (N_ "The first period of the budget"))
-                (vector 'previous
-                        (N_ "Previous")
-                        (N_ "Budget period was before current period, according to report evaluation date"))
-                (vector 'current
-                        (N_ "Current")
-                        (N_ "Current period, according to report evaluation date"))
-                (vector 'next
-                        (N_ "Next")
-                        (N_ "Next period, according to report evaluation date"))
-                (vector 'last
-                        (N_ "Last")
-                        (N_ "Last budget period"))
-                (vector 'manual
-                        (N_ "Manual period selection")
-                        (N_ "Explicitly select period value with spinner below"))))
+          (list (vector 'first (N_ "First budget period"))
+                (vector 'previous (N_ "Previous budget period"))
+                (vector 'current (N_ "Current budget period"))
+                (vector 'next (N_ "Next budget period"))
+                (vector 'last (N_ "Last budget period"))
+                (vector 'manual (N_ "Manual period selection"))))
          (ui-use-periods #f)
          (ui-start-period-type 'current)
          (ui-end-period-type 'next))
 
-    (gnc:register-option
-     options
-     (gnc:make-budget-option
+    (gnc-register-budget-option options
       gnc:pagename-general optname-budget
-      "a" (N_ "Budget to use.")))
+      "a" (N_ "Budget to use.")
+      (gnc-budget-get-default (gnc-get-current-book)))
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-general optname-accumulate
-      "b" opthelp-accumulate #f))
+      "b" opthelp-accumulate #f)
 
-    (add-option
-     (gnc:make-complex-boolean-option
+    (gnc-register-complex-boolean-option options
       gnc:pagename-general optname-use-budget-period-range
-      "f" opthelp-use-budget-period-range #f #f
+      "f" opthelp-use-budget-period-range #f
       (lambda (value)
         (for-each
          (lambda (opt)
@@ -161,56 +141,50 @@
                             optname-budget-period-end-exact
                             (and value (eq? 'manual ui-end-period-type)))
 
-        (set! ui-use-periods value))))
+        (set! ui-use-periods value)))
 
-    (add-option
-     (gnc:make-multichoice-callback-option
+    (gnc-register-multichoice-callback-option options
       gnc:pagename-general optname-budget-period-start
-      "g1.1" opthelp-budget-period-start 'current period-options #f
+      "g1.1" opthelp-budget-period-start "current" period-options
       (lambda (new-val)
         (set-option-enabled options gnc:pagename-general
                             optname-budget-period-start-exact
                             (and ui-use-periods (eq? 'manual new-val)))
-        (set! ui-start-period-type new-val))))
+        (set! ui-start-period-type new-val)))
 
-    (add-option
-     (gnc:make-number-range-option
+    (gnc-register-number-range-option options
       gnc:pagename-general optname-budget-period-start-exact
       "g1.2" opthelp-budget-period-start-exact
       ;; FIXME: It would be nice if the max number of budget periods (60) was
       ;; defined globally somewhere so we could reference it here.  However, it
       ;; only appears to be defined currently in src/gnome/glade/budget.glade.
-      1 1 60 0 1))
+      1 1 60 1)
 
-    (add-option
-     (gnc:make-multichoice-callback-option
+    (gnc-register-multichoice-callback-option options
       gnc:pagename-general optname-budget-period-end
-      "g2.1" opthelp-budget-period-end 'next period-options #f
+      "g2.1" opthelp-budget-period-end "next" period-options
       (lambda (new-val)
         (set-option-enabled options gnc:pagename-general
                             optname-budget-period-end-exact
                             (and ui-use-periods (eq? 'manual new-val)))
-        (set! ui-end-period-type new-val))))
+        (set! ui-end-period-type new-val)))
 
-    (add-option
-     (gnc:make-number-range-option
+    (gnc-register-number-range-option options
       gnc:pagename-general optname-budget-period-end-exact
       "g2.2" opthelp-budget-period-end-exact
       ;; FIXME: It would be nice if the max number of budget periods (60) was
       ;; defined globally somewhere so we could reference it here.  However, it
       ;; only appears to be defined currently in src/gnome/glade/budget.glade.
-      1 1 60 0 1))
+      1 1 60 1)
     ;; accounts to work on
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-general optname-period-collapse-before
-      "g3" opthelp-period-collapse-before #t))
+      "g3" opthelp-period-collapse-before #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-general optname-period-collapse-after
-      "g4" opthelp-period-collapse-after #t))
+      "g4" opthelp-period-collapse-after #t)
 
     (gnc:options-add-account-selection!
      options gnc:pagename-accounts optname-display-depth
@@ -222,38 +196,31 @@
         (gnc-account-get-descendants-sorted (gnc-get-current-root-account))))
      #f)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-accounts optname-bottom-behavior
-      "c" opthelp-bottom-behavior #f))
+      "c" opthelp-bottom-behavior #f)
 
     ;; columns to display
-    (add-option
-     (gnc:make-complex-boolean-option
+    (gnc-register-complex-boolean-option options
       gnc:pagename-display optname-show-budget
-      "s1" opthelp-show-budget #t #f
+      "s1" opthelp-show-budget #t
       (lambda (x)
-        (set-option-enabled options gnc:pagename-display optname-show-notes x))))
-    (add-option
-     (gnc:make-simple-boolean-option
+        (set-option-enabled options gnc:pagename-display optname-show-notes x)))
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-show-notes
-      "s15" opthelp-show-budget #t))
-    (add-option
-     (gnc:make-simple-boolean-option
+      "s15" opthelp-show-notes #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-show-actual
-      "s2" opthelp-show-actual #t))
-    (add-option
-     (gnc:make-simple-boolean-option
+      "s2" opthelp-show-actual #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-show-difference
-      "s3" opthelp-show-difference #f))
-    (add-option
-     (gnc:make-simple-boolean-option
+      "s3" opthelp-show-difference #f)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-show-totalcol
-      "s4" opthelp-show-totalcol #f))
-    (add-option
-     (gnc:make-simple-boolean-option
+      "s4" opthelp-show-totalcol #f)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-show-zb-accounts
-      "s5" opthelp-show-zb-accounts #t))
+      "s5" opthelp-show-zb-accounts #t)
 
     ;; Set the general page as default option tab
     (gnc:options-set-default-section options gnc:pagename-general)
@@ -361,8 +328,6 @@
       (let* ((comm (xaccAccountGetCommodity acct))
              (reverse-balance? (gnc-reverse-balance acct))
              (maybe-negate (lambda (amt) (if reverse-balance? (- amt) amt)))
-             (unreversed? (gnc-using-unreversed-budgets
-                           (gnc-get-current-book))) ;fwd-compatibility
              (allperiods (filter number? (gnc:list-flatten column-list)))
              (total-periods (if (and accumulate? (not (null? allperiods)))
                                 (iota (1+ (apply max allperiods)))
@@ -412,8 +377,7 @@
            ((null? column-list)
             #f)
 
-           ;; fwd-compatibility for unreversed budgets
-           ((and (eq? (car column-list) 'total) unreversed?)
+           ((eq? (car column-list) 'total)
             (let* ((bgt-total (maybe-negate
                                (gnc:get-account-periodlist-budget-value
                                 budget acct total-periods)))
@@ -425,21 +389,7 @@
                     (disp-cols "total-number-cell" current-col
                                bgt-total act-total dif-total #f))))
 
-           ((eq? (car column-list) 'total)
-            (let* ((bgt-total (gnc:get-account-periodlist-budget-value
-                               budget acct total-periods))
-                   (act-total (gnc:get-account-periodlist-actual-value
-                               budget acct total-periods))
-                   (act-total (if reverse-balance? (- act-total) act-total))
-                   (dif-total (if income-acct?
-                                  (- act-total bgt-total)
-                                  (- bgt-total act-total))))
-              (loop (cdr column-list)
-                    (disp-cols "total-number-cell" current-col
-                               bgt-total act-total dif-total #f))))
-
-           ;; fwd-compatibility for unreversed budgets
-           (unreversed?
+           (else
             (let* ((period-list (cond
                                  ((list? (car column-list)) (car column-list))
                                  (accumulate? (iota (1+ (car column-list))))
@@ -454,28 +404,6 @@
                              (gnc:get-account-periodlist-actual-value
                               budget acct period-list)))
                    (dif-val (- bgt-val act-val)))
-              (loop (cdr column-list)
-                    (disp-cols "number-cell" current-col
-                               bgt-val act-val dif-val note))))
-
-           (else
-            (let* ((period-list (cond
-                                 ((list? (car column-list)) (car column-list))
-                                 (accumulate? (iota (1+ (car column-list))))
-                                 (else (list (car column-list)))))
-                   (note (and (= 1 (length period-list))
-                              (gnc-budget-get-account-period-note
-                               budget acct (car period-list))))
-                   (bgt-val (gnc:get-account-periodlist-budget-value
-                             budget acct period-list))
-                   (act-abs (gnc:get-account-periodlist-actual-value
-                             budget acct period-list))
-                   (act-val (if reverse-balance?
-                                (- act-abs)
-                                act-abs))
-                   (dif-val (if income-acct?
-                                (- act-val bgt-val)
-                                (- bgt-val act-val))))
               (loop (cdr column-list)
                     (disp-cols "number-cell" current-col
                                bgt-val act-val dif-val note))))))))
@@ -699,7 +627,7 @@
                                                    optname-period-collapse-after)))
          (doc (gnc:make-html-document))
          (accounts (if show-subaccts?
-                       (gnc:accounts-and-all-descendants accounts)
+                       (gnc-accounts-and-all-descendants accounts)
                        accounts)))
     ;; end of defines
 
@@ -717,7 +645,7 @@
 
      (else
       (let* ((tree-depth (if (eq? display-depth 'all)
-                             (accounts-get-children-depth accounts)
+                             (gnc:accounts-get-children-depth accounts)
                              display-depth))
              (to-period-val (lambda (v)
                               (inexact->exact
@@ -732,7 +660,7 @@
                    (list 'zero-balance-mode
                          (if show-zb-accts? 'show-leaf-acct 'omit-leaf-acct))
                    (list 'report-budget budget)))
-             (accounts (sort accounts account-full-name<?))
+             (accounts (sort accounts gnc:account-full-name<?))
              (accumulate? (get-option gnc:pagename-general optname-accumulate))
              (acct-table (gnc:make-html-acct-table/env/accts env accounts))
              (footnotes (make-footnote-collector))
