@@ -52,6 +52,7 @@
 (use-modules (gnucash app-utils))
 (use-modules (gnucash report))
 
+;; Translators: This statement is about a range of time
 (define reportname (N_ "Equity Statement"))
 
 ;; define all option's names and help text so that they are properly
@@ -59,9 +60,6 @@
 (define optname-report-title (N_ "Report Title"))
 (define opthelp-report-title (N_ "Title for this report."))
 
-(define optname-party-name (N_ "Company name"))
-(define opthelp-party-name (N_ "Name of company/individual."))
-
 (define optname-start-date (N_ "Start Date"))
 (define optname-end-date (N_ "End Date"))
 
@@ -97,20 +95,11 @@
 
 ;; options generator
 (define (equity-statement-options-generator)
-  (let* ((options (gnc:new-options))
-         (book (gnc-get-current-book)) ; XXX Find a way to get the book that opened the report
-         (add-option 
-          (lambda (new-option)
-            (gnc:register-option options new-option))))
+  (let* ((options (gnc-new-optiondb)))
     
-    (add-option
-      (gnc:make-string-option
+    (gnc-register-string-option options
       (N_ "General") optname-report-title
-      "a" opthelp-report-title (_ reportname)))
-    (add-option
-      (gnc:make-string-option
-      (N_ "General") optname-party-name
-      "b" opthelp-party-name (or (gnc:company-info book gnc:*company-name*) "")))
+      "a" opthelp-report-title (G_ reportname))
     
     ;; date at which to report balance
     (gnc:options-add-date-interval!
@@ -118,13 +107,11 @@
      optname-start-date optname-end-date "c")
     
     ;; accounts to work on
-    (add-option
-     (gnc:make-account-list-option
+    (gnc-register-account-list-option options
       gnc:pagename-accounts optname-accounts
       "a"
       opthelp-accounts
-      (lambda ()
-	(gnc:filter-accountlist-type 
+      (gnc:filter-accountlist-type 
          (list ACCT-TYPE-BANK ACCT-TYPE-CASH ACCT-TYPE-CREDIT
                ACCT-TYPE-ASSET ACCT-TYPE-LIABILITY
                ACCT-TYPE-STOCK ACCT-TYPE-MUTUAL ACCT-TYPE-CURRENCY
@@ -132,7 +119,6 @@
                ACCT-TYPE-EQUITY ACCT-TYPE-INCOME ACCT-TYPE-EXPENSE
                ACCT-TYPE-TRADING)
          (gnc-account-get-descendants-sorted (gnc-get-current-root-account))))
-      #f #t))
     
     ;; all about currencies
     (gnc:options-add-currency!
@@ -143,38 +129,32 @@
      options pagename-commodities
      optname-price-source "b" 'pricedb-nearest)
     
-    (add-option 
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       pagename-commodities optname-show-foreign 
-      "c" opthelp-show-foreign #t))
+      "c" opthelp-show-foreign #t)
     
-    (add-option 
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       pagename-commodities optname-show-rates
-      "d" opthelp-show-rates #f))
+      "d" opthelp-show-rates #f)
     
     ;; some detailed formatting options
-    (add-option 
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-use-rules
-      "f" opthelp-use-rules #f))
+      "f" opthelp-use-rules #f)
     
     ;; closing entry match criteria
     ;; 
     ;; N.B.: transactions really should have a field where we can put
     ;; transaction types like "Adjusting/Closing/Correcting Entries"
-    (add-option
-      (gnc:make-string-option
+    (gnc-register-string-option options
       pagename-entries optname-closing-pattern
-      "a" opthelp-closing-pattern (_ "Closing Entries")))
-    (add-option
-     (gnc:make-simple-boolean-option
+      "a" opthelp-closing-pattern (G_ "Closing Entries"))
+    (gnc-register-simple-boolean-option options
       pagename-entries optname-closing-casing
-      "b" opthelp-closing-casing #f))
-    (add-option
-     (gnc:make-simple-boolean-option
+      "b" opthelp-closing-casing #f)
+    (gnc-register-simple-boolean-option options
       pagename-entries optname-closing-regexp
-      "c" opthelp-closing-regexp #f))
+      "c" opthelp-closing-regexp #f)
     
     ;; Set the accounts page as default option tab
     (gnc:options-set-default-section options gnc:pagename-accounts)
@@ -208,16 +188,15 @@
 
 (define (equity-statement-renderer report-obj)
   (define (get-option pagename optname)
-    (gnc:option-value
-     (gnc:lookup-option 
-      (gnc:report-options report-obj) pagename optname)))
+    (gnc-optiondb-lookup-value
+      (gnc:report-options report-obj) pagename optname))
   
   (gnc:report-starting reportname)
   
   ;; get all option's values
   (let* (
 	 (report-title (get-option gnc:pagename-general optname-report-title))
-	 (company-name (get-option gnc:pagename-general optname-party-name))
+	 (company-name (or (gnc:company-info (gnc-get-current-book) gnc:*company-name*) ""))
 	 ;; this code makes the assumption that you want your equity
 	 ;; statement to no more than daily resolution
          (start-date-printable (gnc:date-option-absolute-time
@@ -239,8 +218,6 @@
                                       optname-report-commodity))
          (price-source (get-option pagename-commodities
                                    optname-price-source))
-         (show-fcur? (get-option pagename-commodities
-                                 optname-show-foreign))
          (show-rates? (get-option pagename-commodities
                                   optname-show-rates))
          (use-rules? (get-option gnc:pagename-display
@@ -280,7 +257,9 @@
 	 (end-exchange-fn
 	  (gnc:case-exchange-fn
 	   price-source report-commodity end-date))
-	 )
+
+         (start-price-fn (gnc:case-price-fn price-source report-commodity start-date))
+         (end-price-fn (gnc:case-price-fn price-source report-commodity end-date)))
 
     (define (unrealized-gains-at-date book-balance exchange-fn date)
       (define cost-fn
@@ -297,12 +276,11 @@
       (gnc:account-get-comm-balance-at-date account end-date #f))
 
     (gnc:html-document-set-title! 
-     doc (format #f
-		  (string-append "~a ~a "
-				 (_ "For Period Covering ~a to ~a"))
-		  company-name report-title
-                  (qof-print-date start-date-printable)
-                  (qof-print-date end-date)))
+     doc (gnc:format (G_ "${company-name} ${report-title} For Period Covering ${start} to ${end}")
+                     'company-name company-name
+                     'report-title report-title
+                     'start (qof-print-date start-date-printable)
+                     'end (qof-print-date end-date)))
     
     (if (null? accounts)
 	
@@ -375,9 +353,10 @@
 				          start-date))
 
 	       (net-unrealized-gains
-                (unrealized-gains-at-date end-book-balance
-				          end-exchange-fn
-				          end-date))
+                (gnc:collector- (unrealized-gains-at-date end-book-balance
+                                                          end-exchange-fn
+                                                          end-date)
+                                start-unrealized-gains))
 
 	       (equity-closing
                 (gnc:account-get-trans-type-balance-interval-with-closing
@@ -420,7 +399,7 @@
 	       ;; Create the account table below where its
 	       ;; percentage time can be tracked.
 	       (build-table (gnc:make-html-table)) ;; gnc:html-table
-	       (period-for (string-append " " (_ "for Period"))))
+	       (period-for (string-append " " (G_ "for Period"))))
 
 	  ;; a helper to add a line to our report
 	  (define (add-report-line
@@ -447,43 +426,43 @@
 
           (add-report-line
            build-table
-           (string-append (_ "Capital") ", " (qof-print-date start-date-printable))
+           (string-append (G_ "Capital") ", " (qof-print-date start-date-printable))
            #f start-total-equity 1 start-exchange-fn #f "primary-subheading")
 
           (add-report-line
            build-table
-           (string-append (_ "Net income") period-for)
-           (string-append (_ "Net loss") period-for)
+           (string-append (G_ "Net income") period-for)
+           (string-append (G_ "Net loss") period-for)
            net-income 0 end-exchange-fn #f #f)
 
           (add-report-line
            build-table
-           (string-append (_ "Investments") period-for) #f
+           (string-append (G_ "Investments") period-for) #f
            investments 0 end-exchange-fn #f #f)
 
           (add-report-line
            build-table
-           (string-append (_ "Withdrawals") period-for)
+           (string-append (G_ "Withdrawals") period-for)
            #f withdrawals 0 end-exchange-fn #f #f)
 
           (unless (gnc-commodity-collector-allzero? net-unrealized-gains)
             (add-report-line
              build-table
-             (_ "Unrealized Gains")
-             (_ "Unrealized Losses")
+             (G_ "Unrealized Gains for Period")
+             (G_ "Unrealized Losses for Period")
              net-unrealized-gains
              0 end-exchange-fn #f #f))
 
           (add-report-line
            build-table
-           (_ "Increase in capital")
-           (_ "Decrease in capital")
+           (G_ "Increase in capital")
+           (G_ "Decrease in capital")
            capital-increase
            1 end-exchange-fn use-rules? #f)
 
           (add-report-line
            build-table
-           (string-append (_ "Capital") ", " (qof-print-date end-date)) #f
+           (string-append (G_ "Capital") ", " (qof-print-date end-date)) #f
            end-total-equity
            1 end-exchange-fn #f "primary-subheading")
 
@@ -496,10 +475,10 @@
 		   (headers (list
 			     (qof-print-date start-date-printable)
 			     (qof-print-date end-date)))
-		   (then (gnc:html-make-exchangerates
-			  report-commodity start-exchange-fn accounts))
-		   (now (gnc:html-make-exchangerates
-                         report-commodity end-exchange-fn accounts)))
+		   (then (gnc:html-make-rates-table
+			  report-commodity start-price-fn accounts))
+		   (now (gnc:html-make-rates-table
+                         report-commodity end-price-fn accounts)))
 	      (gnc:html-table-set-col-headers! curr-tbl headers)
 	      (gnc:html-table-set-style!
 	       curr-tbl "table" 'attribute '("border" "1"))
