@@ -9,12 +9,8 @@
 ;;    This code makes the assumption that you want your income
 ;;    statement to no more than daily resolution.
 ;;    
-;;    The Company Name field does not currently default to the name
-;;    in (gnc-get-current-book).
-;;    
 ;;    Line & column alignments may still not conform with
 ;;    textbook accounting practice (they're close though!).
-;;    The 'canonically-tabbed option is currently broken.
 ;;    
 ;;    Progress bar functionality is currently mostly broken.
 ;;    
@@ -53,9 +49,6 @@
 (define optname-report-title (N_ "Report Title"))
 (define opthelp-report-title (N_ "Title for this report."))
 
-(define optname-party-name (N_ "Company name"))
-(define opthelp-party-name (N_ "Name of company/individual."))
-
 (define optname-start-date (N_ "Start Date"))
 (define optname-end-date (N_ "End Date"))
 
@@ -137,20 +130,11 @@
 
 ;; options generator
 (define (income-statement-options-generator-internal reportname)
-  (let* ((options (gnc:new-options))
-         (book (gnc-get-current-book)) ; XXX Find a way to get the book that opened the report
-         (add-option 
-          (lambda (new-option)
-            (gnc:register-option options new-option))))
-    
-    (add-option
-      (gnc:make-string-option
+  (let* ((options (gnc-new-optiondb)))
+
+    (gnc-register-string-option options
       gnc:pagename-general optname-report-title
-      "a" opthelp-report-title (_ reportname)))
-    (add-option
-      (gnc:make-string-option
-      gnc:pagename-general optname-party-name
-      "b" opthelp-party-name (or (gnc:company-info book gnc:*company-name*) "")))
+      "a" opthelp-report-title (G_ reportname))
     
     ;; period over which to report income
     (gnc:options-add-date-interval!
@@ -158,24 +142,21 @@
      optname-start-date optname-end-date "c")
     
     ;; accounts to work on
-    (add-option
-     (gnc:make-account-list-option
+    (gnc-register-account-list-option options
       gnc:pagename-accounts optname-accounts
       "a"
       opthelp-accounts
-      (lambda ()
 	(gnc:filter-accountlist-type
 	 ;; select, by default, only income and expense accounts
 	 (list ACCT-TYPE-INCOME ACCT-TYPE-EXPENSE)
 	 (gnc-account-get-descendants-sorted (gnc-get-current-root-account))))
-      #f #t))
+
     (gnc:options-add-account-levels!
      options gnc:pagename-accounts optname-depth-limit
      "b" opthelp-depth-limit 3)
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-accounts optname-bottom-behavior
-      "c" opthelp-bottom-behavior #f))
+      "c" opthelp-bottom-behavior #f)
     
     ;; all about currencies
     (gnc:options-add-currency!
@@ -185,26 +166,22 @@
     (gnc:options-add-price-source! 
      options pagename-commodities
      optname-price-source "b" 'pricedb-nearest)
-    
-    (add-option 
-     (gnc:make-simple-boolean-option
+
+    (gnc-register-simple-boolean-option options
       pagename-commodities optname-show-foreign 
-      "c" opthelp-show-foreign #t))
+      "c" opthelp-show-foreign #t)
     
-    (add-option 
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       pagename-commodities optname-show-rates
-      "d" opthelp-show-rates #f))
-    
+      "d" opthelp-show-rates #f)
+
     ;; what to show for zero-balance accounts
-    (add-option 
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-show-zb-accts
-      "a" opthelp-show-zb-accts #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "a" opthelp-show-zb-accts #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-omit-zb-bals
-      "b" opthelp-omit-zb-bals #f))
+      "b" opthelp-omit-zb-bals #f)
     ;; what to show for non-leaf accounts
     (gnc:options-add-subtotal-view!
      options gnc:pagename-display
@@ -212,68 +189,55 @@
      "c")
 
     ;; some detailed formatting options
-    (add-option 
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-account-links
-      "e" opthelp-account-links #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "e" opthelp-account-links #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-use-rules
-      "f" opthelp-use-rules #f))
+      "f" opthelp-use-rules #f)
     
-    (add-option 
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-label-revenue
-      "g" opthelp-label-revenue #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "g" opthelp-label-revenue #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-total-revenue
-      "h" opthelp-total-revenue #t))
+      "h" opthelp-total-revenue #t)
     
-    (add-option 
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-label-trading
-      "h1" opthelp-label-trading #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "h1" opthelp-label-trading #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-total-trading
-      "h2" opthelp-total-trading #t))
+      "h2" opthelp-total-trading #t)
     
-    (add-option 
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-label-expense
-      "i" opthelp-label-expense #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "i" opthelp-label-expense #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-total-expense
-      "j" opthelp-total-expense #t))
+      "j" opthelp-total-expense #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-two-column
-      "k" opthelp-two-column #f))
+      "k" opthelp-two-column #f)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-standard-order
-      "l" opthelp-standard-order #t))
+      "l" opthelp-standard-order #t)
     
     ;; closing entry match criteria
     ;; 
     ;; N.B.: transactions really should have a field where we can put
     ;; transaction types like "Adjusting/Closing/Correcting Entries"
-    (add-option
-      (gnc:make-string-option
+    (gnc-register-string-option options
       pagename-entries optname-closing-pattern
-      "a" opthelp-closing-pattern (_ "Closing Entries")))
-    (add-option
-     (gnc:make-simple-boolean-option
+      "a" opthelp-closing-pattern (G_ "Closing Entries"))
+    (gnc-register-simple-boolean-option options
       pagename-entries optname-closing-casing
-      "b" opthelp-closing-casing #f))
-    (add-option
-     (gnc:make-simple-boolean-option
+      "b" opthelp-closing-casing #f)
+    (gnc-register-simple-boolean-option options
       pagename-entries optname-closing-regexp
-      "c" opthelp-closing-regexp #f))
+      "c" opthelp-closing-regexp #f)
     
     ;; Set the accounts page as default option tab
     (gnc:options-set-default-section options gnc:pagename-accounts)
@@ -287,16 +251,15 @@
 
 (define (income-statement-renderer-internal report-obj reportname)
   (define (get-option pagename optname)
-    (gnc:option-value
-     (gnc:lookup-option 
-      (gnc:report-options report-obj) pagename optname)))
+    (gnc-optiondb-lookup-value
+      (gnc:report-options report-obj) pagename optname))
 
   (gnc:report-starting reportname)
   
   ;; get all option's values
   (let* (
 	 (report-title (get-option gnc:pagename-general optname-report-title))
-	 (company-name (get-option gnc:pagename-general optname-party-name))
+	 (company-name (or (gnc:company-info (gnc-get-current-book) gnc:*company-name*) ""))
          (start-date-printable (gnc:date-option-absolute-time
 				(get-option gnc:pagename-general
 					    optname-start-date)))
@@ -325,7 +288,7 @@
          (parent-balance-mode (get-option gnc:pagename-display
                                            optname-parent-balance-mode))
          (parent-total-mode
-	  (assq-ref '((t . #t) (f . #f) (canonically-tabbed . canonically-tabbed))
+	  (assq-ref '((t . #t) (f . #f))
 		    (get-option gnc:pagename-display
 				optname-parent-total-mode)))
          (show-zb-accts? (get-option gnc:pagename-display
@@ -382,7 +345,7 @@
          ;; exchange rates calculation parameters
 	 (exchange-fn
 	  (gnc:case-exchange-fn price-source report-commodity end-date))
-	 )
+         (price-fn (gnc:case-price-fn price-source report-commodity end-date)))
 
     ;; Wrapper to call gnc:html-table-add-labeled-amount-line!
     ;; with the proper arguments.
@@ -404,10 +367,11 @@
       (gnc:html-table-append-ruler! table (* 2 tree-depth)))
 
     (gnc:html-document-set-title!
-     doc (format #f (string-append "~a ~a " (_ "For Period Covering ~a to ~a"))
-                  company-name report-title
-                  (qof-print-date start-date-printable)
-                  (qof-print-date end-date)))
+     doc (gnc:format (G_ "${company-name} ${report-title} For Period Covering ${start} to ${end}")
+                     'company-name company-name
+                     'report-title report-title
+                     'start (qof-print-date start-date-printable)
+                     'end (qof-print-date end-date)))
 
     (if (null? accounts)
 
@@ -442,10 +406,7 @@
                                 trading-total
                                 (gnc:collector- expense-total)))
 
-               (inc-table (gnc:make-html-table))
-               (exp-table (gnc:make-html-table))
-               (tra-table (gnc:make-html-table))
-
+               (build-table (gnc:make-html-table))
                (table-env
                 (list
                  (list 'start-date start-date)
@@ -478,7 +439,7 @@
                (trading-table
                 (gnc:make-html-acct-table/env/accts table-env trading-accounts))
 
-               (period-for (string-append " " (_ "for Period"))))
+               (period-for (string-append " " (G_ "for Period"))))
 
           ;; a helper to add a line to our report
           (define (add-report-line
@@ -495,74 +456,93 @@
                label                0  1 "text-cell"
                bal          (+ col 1)  1 "number-cell")))
 
-          (let ((space (make-list tree-depth (gnc:make-html-table-cell/min-width 60))))
-            (gnc:html-table-append-row! inc-table space)
-            (gnc:html-table-append-row! exp-table space)
-            (gnc:html-table-append-row! tra-table space))
-          (gnc:report-percent-done 80)
+          (define (add-revenue-table table)
+            (when label-revenue?
+              (add-subtotal-line table (G_ "Revenues") #f #f))
+            (gnc:html-table-add-account-balances table revenue-table params)
+            (when total-revenue?
+              (add-subtotal-line table (G_ "Total Revenue") #f revenue-total))
+            table)
 
-          (when label-revenue?
-            (add-subtotal-line inc-table (_ "Revenues") #f #f))
-          (gnc:html-table-add-account-balances inc-table revenue-table params)
-          (when total-revenue?
-            (add-subtotal-line inc-table (_ "Total Revenue") #f revenue-total))
-          (gnc:report-percent-done 85)
+          (define (add-expense-table table)
+            (when label-expense?
+              (add-subtotal-line table (G_ "Expenses") #f #f))
+            (gnc:html-table-add-account-balances table expense-table params)
+            (when total-expense?
+              (add-subtotal-line table (G_ "Total Expenses") #f expense-total))
+            table)
 
-          (when label-expense?
-            (add-subtotal-line exp-table (_ "Expenses") #f #f))
-          (gnc:html-table-add-account-balances exp-table expense-table params)
-          (when total-expense?
-            (add-subtotal-line exp-table (_ "Total Expenses") #f expense-total))
+          (define (add-trading-table table)
+            (when label-trading?
+              (add-subtotal-line table (G_ "Trading") #f #f))
+            (gnc:html-table-add-account-balances table trading-table params)
+            (when total-trading?
+              (add-subtotal-line table (G_ "Total Trading") #f trading-total))
+            table)
 
-          (when label-trading?
-            (add-subtotal-line tra-table (_ "Trading") #f #f))
-          (gnc:html-table-add-account-balances tra-table trading-table params)
-          (when total-trading?
-            (add-subtotal-line tra-table (_ "Total Trading") #f trading-total))
+          (cond
+           (two-column?
+            (let* ((exp-table (add-expense-table (gnc:make-html-table)))
+                   (inc-table (add-revenue-table (gnc:make-html-table)))
+                   (tra-table (add-trading-table (gnc:make-html-table)))
+                   (inc-cell (gnc:make-html-table-cell inc-table))
+                   (tra-cell (if (null? trading-accounts)
+                                 (gnc:html-make-empty-cell)
+                                 (gnc:make-html-table-cell tra-table)))
+                   (exp-cell (gnc:make-html-table-cell exp-table)))
+              (define (add-cells . lst) (gnc:html-table-append-row! build-table lst))
+              (add-rule (if standard-order? exp-table inc-table))
+              (add-report-line
+               (if standard-order? exp-table inc-table)
+               (string-append (G_ "Net income") period-for)
+               (string-append (G_ "Net loss") period-for)
+               net-income (* 2 (1- tree-depth)) exchange-fn #f #f)
+              (if standard-order?
+                  (add-cells inc-cell tra-cell exp-cell)
+                  (add-cells exp-cell inc-cell tra-cell))))
 
-          (add-report-line
-           (if standard-order? exp-table inc-table)
-           (string-append (_ "Net income") period-for)
-           (string-append (_ "Net loss") period-for)
-           net-income (* 2 (1- tree-depth)) exchange-fn #f #f)
+           ;; single-column
+           (standard-order?
+            (add-revenue-table build-table)
+            (add-rule build-table)
+            (unless (null? trading-accounts)
+              (add-trading-table build-table)
+              (add-rule build-table))
+            (add-expense-table build-table)
+            (add-rule build-table)
+            (add-report-line
+             build-table
+             (string-append (G_ "Net income") period-for)
+             (string-append (G_ "Net loss") period-for)
+             net-income (* 2 (1- tree-depth)) exchange-fn #f #f))
 
-          ;; add the sections in the desired order to document
-          (let ((build-table (gnc:make-html-table))
-                (inc-cell (gnc:make-html-table-cell inc-table))
-                (tra-cell (if (null? trading-accounts)
-                              (gnc:html-make-empty-cell)
-                              (gnc:make-html-table-cell tra-table)))
-                (exp-cell (gnc:make-html-table-cell exp-table)))
-            (define (add-cells . lst) (gnc:html-table-append-row! build-table lst))
-            (cond
-             ((and two-column? standard-order?)
-              (add-cells inc-cell tra-cell exp-cell))
+           (else
+            (add-expense-table build-table)
+            (add-rule build-table)
+            (unless (null? trading-accounts)
+              (add-trading-table build-table)
+              (add-rule build-table))
+            (add-revenue-table build-table)
+            (add-rule build-table)
+            (add-report-line
+             build-table
+             (string-append (G_ "Net income") period-for)
+             (string-append (G_ "Net loss") period-for)
+             net-income (* 2 (1- tree-depth)) exchange-fn #f #f)))
 
-             (two-column?
-              (add-cells exp-cell inc-cell tra-cell))
+          (gnc:html-table-set-style!
+           build-table "td"
+           'attribute '("align" "left")
+           'attribute '("valign" "top"))
 
-             (standard-order?
-              (add-cells inc-cell)
-              (unless (null? trading-accounts) (add-cells tra-cell))
-              (add-cells exp-cell))
-
-             (else
-              (add-cells exp-cell)
-              (add-cells inc-cell)
-              (unless (null? trading-accounts) (add-cells tra-cell))))
-
-            (gnc:html-table-set-style!
-             build-table "td"
-             'attribute '("align" "left")
-             'attribute '("valign" "top"))
-            (gnc:html-document-add-object! doc build-table))
+          (gnc:html-document-add-object! doc build-table)
 
           ;; add currency information if requested
           (gnc:report-percent-done 90)
           (when show-rates?
             (gnc:html-document-add-object!
-             doc (gnc:html-make-exchangerates
-                  report-commodity exchange-fn accounts)))
+             doc (gnc:html-make-rates-table
+                  report-commodity price-fn accounts)))
           (gnc:report-percent-done 100)))
 
     (gnc:report-finished)
