@@ -50,17 +50,17 @@
   (gnc:options->sxml uuid options "test-apr" test-title))
 
 (define (set-option! options section name value)
-  (let ((option (gnc:lookup-option options section name)))
-    (if option
-        (gnc:option-set-value option value)
-        (test-assert (format #f "wrong-option ~a ~a" section name) #f))))
+  (if (gnc-lookup-option (gnc:optiondb options) section name)
+      (gnc-set-option (gnc:optiondb options) section name value)
+      (test-assert (format #f "wrong-option ~a ~a" section name) #f)))
 
 (define (teardown)
   (gnc-clear-current-session))
 
 (define (null-test variant uuid)
   ;; This null-test tests for the presence of report.
-  (let ((options (gnc:make-report-options uuid)))
+  (let* ((book (gnc-get-current-book))
+         (options (gnc:make-report-options uuid)))
     (test-assert (format #f "null-test ~a" variant)
       (options->sxml uuid options "null-test"))))
 
@@ -97,8 +97,9 @@
 
 (define (advanced-tests)
   (test-group-with-cleanup "advanced-portfolio-tests"
-    (let ((account-alist (create-stock-test-data))
-          (options (gnc:make-report-options advanced-uuid)))
+;; let* required here to ensure that the book is created before creating the options.
+    (let* ((account-alist (create-stock-test-data))
+           (options (gnc:make-report-options advanced-uuid)))
       (let ((sxml (options->sxml advanced-uuid options "basic average")))
         (test-equal "advanced: average basis"
           '("AAPL" "AAPL" "NASDAQ" "42.00" "$6.0000" "$484.88" "$252.00" "$800.00"
