@@ -67,6 +67,7 @@
   (test-begin "transaction.scm")
   (null-test)
   (trep-tests)
+  (csv-tests)
   (reconcile-tests)
   ;; (test-end) must be run as the last function, it will
   ;; return #f if any of the tests have failed.
@@ -101,15 +102,13 @@
   (sxml->table-row-col sxml 1 row col))
 
 (define (set-option! options section name value)
-  (let ((option (gnc:lookup-option options section name)))
-    (if option
-        (gnc:option-set-value option value)
-        (test-assert (format #f "wrong-option ~a ~a" section name) #f))))
+  (if (gnc-lookup-option options section name)
+      (gnc-set-option options section name value)
+      (test-assert (format #f "wrong-option ~a ~a" section name) #f)))
 (define (opt-val options section name)
-  (let ((option (gnc:lookup-option options section name)))
-    (if option
-        (gnc:option-value option)
-        (test-assert (format #f "wrong-option ~a ~a" section name) #f))))
+  (if (gnc-lookup-option options section name)
+      (gnc-optiondb-lookup-value options section name)
+      (test-assert (format #f "wrong-option ~a ~a" section name) #f)))
 
 (define structure
   (list "Root" (list (cons 'type ACCT-TYPE-ASSET))
@@ -330,6 +329,21 @@
           '("$31.00")
           (get-row-col sxml -1 -1)))
 
+      ;; Filter Account Name Filters
+      (set-option! options "Filter" "Account Name Filter excludes matched strings"
+                   #t)
+      (let ((sxml (options->sxml options "accounts filter exclude expen.es regex")))
+        (test-equal "account name filter to 'expen.es, regex, negated', sum = -$31.00"
+          '("-$31.00")
+          (get-row-col sxml -1 -1)))
+
+      (set-option! options "Filter" "Use regular expressions for account name filter"
+                   #f)
+      (let ((sxml (options->sxml options "accounts filter exclude expen.es")))
+        (test-equal "account name filter to 'expen.es, negated', sum = $0.00"
+          '("$0.00")
+          (get-row-col sxml -1 -1)))
+
       ;; Test Transaction Filters
       (set! options (default-testing-options))
       (set-option! options "General" "Start Date" (cons 'absolute (gnc-dmy2time64 01 01 1969)))
@@ -400,7 +414,7 @@
       (set! options (default-testing-options))
       (set-option! options "General" "Start Date" (cons 'absolute (gnc-dmy2time64 01 01 1969)))
       (set-option! options "General" "End Date" (cons 'absolute (gnc-dmy2time64 31 12 1970)))      
-      (set-option! options "Accounts" "Filter By..." (list income))
+      (set-option! options "Accounts" "Filter By…" (list income))
       (set-option! options "Accounts" "Filter Type" 'include)
       (let ((sxml (options->sxml options "including bank-income accts only")))
         (test-equal "filter includes bank-income, sum = -$29.00"
@@ -563,7 +577,7 @@
           (apply + (map str->num ((sxpath '(// (table 1) // tr // (td -1) // a // *text*)) sxml)))))
 
       ;; Remove expense multisplit, transaction is not shown
-      (set-option! options "Accounts" "Filter By..." (list expense))
+      (set-option! options "Accounts" "Filter By…" (list expense))
       (set-option! options "Accounts" "Filter Type" 'exclude)
       (let* ((sxml (options->sxml options "multiline, filtered out")))
         (test-equal "multi-line has been excluded"
@@ -913,6 +927,30 @@
           (gnc:html-document-export-string document))))
     (test-end "csv-export")))
 
+(define (csv-tests)
+  (test-begin "csv tests")
+  (test-equal "gnc:lists->csv empty"
+    ""
+    (gnc:lists->csv '(())))
+  (test-equal "gnc:lists->csv simple"
+    "\"from\",\"01/01/2010\""
+    (gnc:lists->csv
+     '(("from" "01/01/2010"))))
+  (test-equal "gnc:lists->csv complex"
+    "\"from\",\"01/01/2010\",,,
+\"to\",\"31/12/2010\",,,
+\"total\",23500.0,30000.0,3.5714285714285716,sym"
+    (gnc:lists->csv
+     '(("from" "01/01/2010")
+       ("to" "31/12/2010")
+       ("total" 23500 30000 25/7 sym))))
+  (test-error "gnc:lists->csv improper list"
+    (gnc:lists->csv
+     '(("from" "01/01/2010")
+       ("to" "31/12/2010")
+       ("total" 23500 30000 25/7 . sym))))
+  (test-end "csv tests"))
+
 (define (reconcile-tests)
   (let* ((env (create-test-env))
          (account-alist (env-create-account-structure-alist env structure))
