@@ -38,6 +38,7 @@
 (define (test-account-get-trans-type-splits-interval)
   (test-group-with-cleanup "test-account-get-trans-type-splits-interval"
   (let* ((env (create-test-env))
+         (book (gnc-get-current-book))
          (ts-now (gnc-localtime (current-time)))
          (test-day (tm:mday ts-now))
          (test-month (+ 1 (tm:mon ts-now)))
@@ -249,7 +250,18 @@
       (coll-A 'add USD -25)
       (test-equal "gnc-commodity-collector-allzero? #t"
         #t
-        (gnc-commodity-collector-allzero? coll-A)))
+        (gnc-commodity-collector-allzero? coll-A))
+
+      ;; the following tests whether commodity-collector rounds
+      ;; inappropriately
+      (coll-A 'reset #f #f)
+      (coll-A 'add GBP 1/3)
+      (coll-A 'add GBP 1/3)
+      (coll-A 'add GBP 1/3)
+      (coll-A 'add GBP -1)
+      (test-equal "gnc-commodity-collector does not round inappropriately"
+        '(("GBP" . 0))
+        (collector->list coll-A)))
     (teardown)))
 
 (define (mnemonic->commodity sym)
@@ -276,6 +288,7 @@
 
 (define (create-test-data)
   (let* ((env (create-test-env))
+         (book (gnc-get-current-book))
          (account-alist (env-create-account-structure-alist env (structure)))
          (asset (cdr (assoc "Asset" account-alist)))
          (bank (cdr (assoc "Bank" account-alist)))
@@ -372,30 +385,6 @@
                                              (gnc-dmy2time64 01 01 2001)
                                              #t)))
 
-      (test-equal "gnc:accounts-get-comm-total-profit"
-        '(("GBP" . 612) ("USD" . 2389))
-        (collector->list
-         (gnc:accounts-get-comm-total-profit all-accounts
-                                             (lambda (acct)
-                                               (gnc:account-get-comm-balance-at-date
-                                                acct (gnc-dmy2time64 01 01 2001) #f)))))
-
-      (test-equal "gnc:accounts-get-comm-total-income"
-        '(("GBP" . 612) ("USD" . 2573))
-        (collector->list
-         (gnc:accounts-get-comm-total-income all-accounts
-                                             (lambda (acct)
-                                               (gnc:account-get-comm-balance-at-date
-                                                acct (gnc-dmy2time64 01 01 2001) #f)))))
-
-      (test-equal "gnc:accounts-get-comm-total-expense"
-        '(("USD" . -184))
-        (collector->list
-         (gnc:accounts-get-comm-total-expense all-accounts
-                                              (lambda (acct)
-                                                (gnc:account-get-comm-balance-at-date
-                                                 acct (gnc-dmy2time64 01 01 2001) #f)))))
-
       (test-equal "gnc:accounts-get-comm-total-assets"
         '(("GBP" . 608) ("USD" . 2394))
         (collector->list
@@ -464,6 +453,10 @@
         44
         (gnc:accounts-count-splits (list expense income)))
 
+      (test-equal "gnc:accounts-count-splits null"
+        0
+        (gnc:accounts-count-splits '()))
+
       (let ((account-balances (gnc:get-assoc-account-balances
                                (list bank gbp-bank)
                                (lambda (acct)
@@ -510,14 +503,36 @@
       5
       (gnc:get-current-account-tree-depth))
 
+    (test-equal "gnc:not-all-zeros #t"
+      #t
+      (gnc:not-all-zeros '((((((1 2 3) 3 5 6 ((4 8))) (2 3)) (3 4 (5)))))))
+
+    (test-equal "gnc:not-all-zeros #f"
+      #f
+      (gnc:not-all-zeros '((((((0 0 0) 0 0 0 ((0 0))) (0 0)) (0 0 (0)))))))
+
+    (test-equal "gnc:not-all-zeros #f"
+      #f
+      (gnc:not-all-zeros 'sym))
+
+    (test-equal "gnc:not-all-zeros #f"
+      #f
+      (gnc:not-all-zeros '()))
+
+    (test-equal "gnc:not-all-zeros #t"
+      #t
+      (gnc:not-all-zeros '(1)))
+
     (test-equal "gnc:accounts-and-all-descendants"
       (list (account-lookup "GBP Bank")
             (account-lookup "GBP Savings")
             (account-lookup "Expenses")
             (account-lookup "Fuel"))
-      (gnc:accounts-and-all-descendants
-       (list (account-lookup "Expenses")
-             (account-lookup "GBP Bank"))))
+      (sort
+       (gnc-accounts-and-all-descendants
+        (list (account-lookup "Expenses")
+              (account-lookup "GBP Bank")))
+       gnc:account-full-name<?))
 
     (teardown)))
 
@@ -580,6 +595,7 @@
 (define (test-get-account-at-dates)
   (test-group-with-cleanup "test-get-balance-at-dates"
     (let* ((env (create-test-env))
+           (book (gnc-get-current-book))
            (structure (list "Root" (list (cons 'type ACCT-TYPE-ASSET))
                             (list "Asset"
                                   (list "Bank1")
@@ -602,25 +618,26 @@
         (gnc:account-accumulate-at-dates bank1 dates))
 
       (env-transfer env 15 01 1970 income bank1 10)
-      (env-transfer env 15 02 1970 income bank1 10)
-      (env-transfer env 15 03 1970 income bank1 10)
-      (let ((clos (env-transfer env 18 03 1970 income bank1 10)))
+      (env-transfer env 15 02 1970 income bank1 20)
+      (env-transfer env 15 03 1970 income bank1 40)
+
+      (let ((clos (env-transfer env 18 03 1970 income bank1 80)))
         (xaccTransSetIsClosingTxn clos #t))
 
-      (env-transfer env 15 12 1969 income bank2 10)
-      (env-transfer env 17 12 1969 income bank2 10)
-      (env-transfer env 15 02 1970 income bank2 10)
+      (env-transfer env 15 12 1969 income bank2 11)
+      (env-transfer env 17 12 1969 income bank2 21)
+      (env-transfer env 15 02 1970 income bank2 41)
 
-      (env-transfer env 15 03 1970 income bank3 10)
+      (env-transfer env 15 03 1970 income bank3 14)
 
-      (env-transfer env 15 01 1970 income bank4 10)
+      (env-transfer env 15 01 1970 income bank4 18)
 
       (test-equal "1 txn in each slot"
-        '(("USD" . 0) ("USD" . 10) ("USD" . 20) ("USD" . 40))
+        '(("USD" . 0) ("USD" . 10) ("USD" . 30) ("USD" . 150))
         (map monetary->pair (gnc:account-get-balances-at-dates bank1 dates)))
 
       (test-equal "1 txn in each slot, tests #:split->amount to ignore closing"
-        '(("USD" . 0) ("USD" . 10) ("USD" . 20) ("USD" . 30))
+        '(("USD" . 0) ("USD" . 10) ("USD" . 30) ("USD" . 70))
         (map monetary->pair
              (gnc:account-get-balances-at-dates
               bank1 dates #:split->amount
@@ -629,31 +646,31 @@
                      (xaccSplitGetAmount s))))))
 
       (test-equal "2 txn before start, 1 in middle"
-        '(("USD" . 20) ("USD" . 20) ("USD" . 30) ("USD" . 30))
+        '(("USD" . 32) ("USD" . 32) ("USD" . 73) ("USD" . 73))
         (map monetary->pair (gnc:account-get-balances-at-dates bank2 dates)))
 
       (test-equal "1 txn in late slot"
-        '(("USD" . 0) ("USD" . 0) ("USD" . 0) ("USD" . 10))
+        '(("USD" . 0) ("USD" . 0) ("USD" . 0) ("USD" . 14))
         (map monetary->pair (gnc:account-get-balances-at-dates bank3 dates)))
 
       (test-equal "1 txn in early slot"
-        '(("USD" . 0) ("USD" . 10) ("USD" . 10) ("USD" . 10))
+        '(("USD" . 0) ("USD" . 18) ("USD" . 18) ("USD" . 18))
         (map monetary->pair (gnc:account-get-balances-at-dates bank4 dates)))
 
       (test-equal "1 txn in each slot"
-        '(#f 10 20 40)
+        '(#f 10 30 150)
         (gnc:account-accumulate-at-dates bank1 dates))
 
       (test-equal "2 txn before start, 1 in middle"
-        '(20 20 30 30)
+        '(32 32 73 73)
         (gnc:account-accumulate-at-dates bank2 dates))
 
       (test-equal "1 txn in late slot"
-        '(#f #f #f 10)
+        '(#f #f #f 14)
         (gnc:account-accumulate-at-dates bank3 dates))
 
       (test-equal "1 txn in late slot, tests #:nosplit->elt"
-        '(x x x 10)
+        '(x x x 14)
         (gnc:account-accumulate-at-dates bank3 dates #:nosplit->elt 'x))
 
       (test-equal "1 txn in late slot, tests #:split->elt"
@@ -661,7 +678,7 @@
         (gnc:account-accumulate-at-dates bank3 dates #:split->elt (const 'y)))
 
       (test-equal "1 txn in early slot"
-        '(#f 10 10 10)
+        '(#f 18 18 18)
         (gnc:account-accumulate-at-dates bank4 dates))
 
       ;; Tests split->date sorting. note the 3 txns created below are
