@@ -66,7 +66,7 @@
 
 ;;Accounts
 (define optname-accounts (N_ "Accounts"))
-(define optname-filterby (N_ "Filter By..."))
+(define optname-filterby (N_ "Filter By…"))
 (define optname-filtertype (N_ "Filter Type"))
 
 ;;Display
@@ -110,6 +110,8 @@
 (define optname-account-matcher (N_ "Account Name Filter"))
 (define optname-account-matcher-regex
   (N_ "Use regular expressions for account name filter"))
+(define optname-account-matcher-exclude
+  (N_ "Account Name Filter excludes matched strings"))
 (define optname-transaction-matcher (N_ "Transaction Filter"))
 (define optname-transaction-matcher-regex
   (N_ "Use regular expressions for transaction filter"))
@@ -468,18 +470,28 @@ in the Options panel."))
          str)
         (display #\" port))))
 
-  (define max-items (apply max (map length lst)))
+  (define max-items
+    (let lp ((lst lst) (maximum 0))
+      (cond
+       ((null? lst) maximum)
+       ((pair? lst) (lp (cdr lst) (max maximum (length (car lst)))))
+       (else (error "strify " lst " must be a proper list")))))
 
   (define (strify obj)
     (cond
-     ((not obj) "")
+     ((or (null? obj) (not obj)) "")
      ((string? obj) (string-sanitize-csv obj))
      ((number? obj) (number->string (exact->inexact obj)))
-     ((list? obj) (string-join
-                   (map strify
-                        (append obj
-                                (make-list (- max-items (length obj)) #f)))
-                   ","))
+     ((pair? obj) (let lp ((row obj) (acc '()) (pad max-items))
+                    (cond
+                     ((zero? pad) (string-concatenate-reverse acc))
+                     ((null? row) (lp '() (cons "," acc) (1- pad)))
+                     ((pair? row) (lp (cdr row)
+                                      (cons* (if (pair? (cdr row)) "," "")
+                                             (strify (car row))
+                                             acc)
+                                      (1- pad)))
+                     (else (error "strify " obj " must be a proper list")))))
      ((gnc:gnc-monetary? obj) (strify (gnc:gnc-monetary-amount obj)))
      (else (object->string obj))))
 
@@ -492,11 +504,8 @@ in the Options panel."))
 ;; Default Transaction Report
 ;;
 (define (gnc:trep-options-generator)
-  (define options (gnc:new-options))
   (define BOOK-SPLIT-ACTION
     (qof-book-use-split-action-for-num-field (gnc-get-current-book)))
-  (define (gnc:register-trep-option new-option)
-    (gnc:register-option options new-option))
 
   ;; (Feb 2018) Note to future hackers - this gnc:trep-options-generator
   ;; defines a long set of options to be assigned as an object in
@@ -504,39 +513,38 @@ in the Options panel."))
   ;; may be modified in a derived report (see income-gst-statement.scm)
   ;; via gnc:make-internal! and gnc-unregister-option to hide
   ;; and remove options, respectively. If an option is unregistered,
-  ;; don't forget to re-register them via gnc:register-option, unless
+  ;; don't forget to re-register them via gnc-register-option, unless
   ;; your derived report truly does not require them.
 
+  (let ((options (gnc-new-optiondb)))
+
   ;; General options
 
   (gnc:options-add-date-interval!
    options gnc:pagename-general optname-startdate optname-enddate "a")
 
-  (gnc:register-trep-option
-   (gnc:make-multichoice-option
+  (gnc-register-multichoice-option options
     gnc:pagename-general optname-date-source
-    "a5" (G_ "Specify date to filter by...")
-    'posted
+    "a5" (G_ "Specify date to filter by…")
+    "posted"
     (list (vector 'posted (G_ "Date Posted"))
           (vector 'reconciled (G_ "Reconciled Date"))
-          (vector 'entered (G_ "Date Entered")))))
+          (vector 'entered (G_ "Date Entered"))))
 
-  (gnc:register-trep-option
-   (gnc:make-complex-boolean-option
+  (gnc-register-complex-boolean-option  options
     pagename-currency optname-common-currency
-    "a" (G_ "Convert all transactions into a common currency.") #f #f
+    "a" (G_ "Convert all transactions into a common currency.") #f
     (lambda (x)
-      (gnc-option-db-set-option-selectable-by-name
+      (gnc-optiondb-set-option-selectable-by-name
        options pagename-currency optname-currency x)
-      (gnc-option-db-set-option-selectable-by-name
+      (gnc-optiondb-set-option-selectable-by-name
        options pagename-currency optname-orig-currency x)
-      (gnc-option-db-set-option-selectable-by-name
-       options pagename-currency optname-price-source x))))
+      (gnc-optiondb-set-option-selectable-by-name
+       options pagename-currency optname-price-source x)))
 
-  (gnc:register-trep-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     pagename-currency optname-orig-currency
-    "b" (G_ "Also show original currency amounts") #f))
+    "b" (G_ "Also show original currency amounts") #f)
 
   (gnc:options-add-currency!
    options pagename-currency optname-currency "c")
@@ -544,104 +552,97 @@ in the Options panel."))
   (gnc:options-add-price-source!
    options pagename-currency optname-price-source "d" 'pricedb-nearest)
 
-  (gnc:register-trep-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     gnc:pagename-general optname-table-export
     "g" (G_ "Formats the table suitable for cut & paste exporting with extra cells.")
-    #f))
+    #f)
 
-  (gnc:register-trep-option
-   (gnc:make-multichoice-option
+  (gnc-register-multichoice-option options
     gnc:pagename-general optname-infobox-display
     "h" (G_ "Add summary of options.")
-    'no-match
+    "no-match"
     ;; This is an alist of conditions for displaying the infobox
     ;; 'no-match for empty-report
     ;; 'match for generated report
     (list (vector 'no-match (G_ "If no transactions matched"))
           (vector 'always (G_ "Always"))
-          (vector 'never (G_ "Never")))))
+          (vector 'never (G_ "Never"))))
 
   ;; Filtering Options
 
-  (gnc:register-trep-option
-   (gnc:make-string-option
+  (gnc-register-string-option options
     pagename-filter optname-account-matcher
     "a5" (G_ "Show only accounts whose full name matches this filter e.g. ':Travel' will match \
 Expenses:Travel:Holiday and Expenses:Business:Travel. It can be left blank, which will \
 disable the filter.")
-    ""))
+    "")
 
-  (gnc:register-trep-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     pagename-filter optname-account-matcher-regex
     "a6"
     (G_ "By default the account filter will search substring only. Set this to true to \
 enable full POSIX regular expressions capabilities. 'Car|Flights' will match both \
 Expenses:Car and Expenses:Flights. Use a period (.) to match a single character e.g. \
 '20../.' will match 'Travel 2017/1 London'. ")
-    #f))
+    #f)
 
-  (gnc:register-trep-option
-   (gnc:make-string-option
+  (gnc-register-simple-boolean-option options
+    pagename-filter optname-account-matcher-exclude "a7"
+    (G_ "If this option is selected, accounts matching filter are excluded.")
+    #f)
+
+  (gnc-register-string-option options
     pagename-filter optname-transaction-matcher
     "i1" (G_ "Show only transactions where description, notes, or memo matches this filter.
 e.g. '#gift' will find all transactions with #gift in description, notes or memo. It can be left \
 blank, which will disable the filter.")
-    ""))
+    "")
 
-  (gnc:register-trep-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     pagename-filter optname-transaction-matcher-regex
     "i2"
     (G_ "By default the transaction filter will search substring only. Set this to true to \
 enable full POSIX regular expressions capabilities. '#work|#family' will match both \
 tags within description, notes or memo.")
-    #f))
+    #f)
 
-  (gnc:register-trep-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     pagename-filter optname-transaction-matcher-exclude
     "i3"
     (G_ "If this option is selected, transactions matching filter are excluded.")
-    #f))
+    #f)
 
-  (gnc:register-trep-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     pagename-filter optname-transaction-matcher-caseinsensitive
     "i4"
     (G_ "If this option is selected, transactions matching filter is not case sensitive.")
-    #f))
+    #f)
 
-  (gnc:register-trep-option
-   (gnc:make-multichoice-option
+  (gnc-register-multichoice-option options
     pagename-filter optname-reconcile-status
     "j1" (G_ "Filter by reconcile status.")
-    'all
-    (keylist->vectorlist reconcile-status-list)))
+    "all"
+    (keylist->vectorlist reconcile-status-list))
 
-  (gnc:register-trep-option
-   (gnc:make-multichoice-option
+  (gnc-register-multichoice-option options
     pagename-filter optname-void-transactions
     "k" (N_ "How to handle void transactions.")
-    'non-void-only
-    (keylist->vectorlist show-void-list)))
+    "non-void-only"
+    (keylist->vectorlist show-void-list))
 
-  (gnc:register-trep-option
-   (gnc:make-multichoice-option
+  (gnc-register-multichoice-option options
     pagename-filter optname-closing-transactions
     "l" (G_ "By default most users should not include closing \
 transactions in a transaction report. Closing transactions are \
 transfers from income and expense accounts to equity, and must usually \
 be excluded from periodic reporting.")
-    'exclude-closing
-    (keylist->vectorlist show-closing-list)))
+    "exclude-closing"
+    (keylist->vectorlist show-closing-list))
 
   ;; Accounts options
 
   ;; account to do report on
-  (gnc:register-trep-option
-   (gnc:make-account-list-option
+  (gnc-register-account-list-option options
     gnc:pagename-accounts optname-accounts
     "a" (G_ "Report on these accounts.")
     ;; select, by default, no accounts! Selecting all accounts will
@@ -649,29 +650,22 @@ be excluded from periodic reporting.")
     ;; is almost never useful. So we instead display the normal error
     ;; message saying "Click here", and the user knows how to
     ;; continue.
-    (lambda ()
-      '())
-    #f #t))
+    '())
 
-  (gnc:register-trep-option
-   (gnc:make-account-list-option
+  (gnc-register-account-list-option options
     gnc:pagename-accounts optname-filterby
     "c1" (G_ "Filter on these accounts.")
-    (lambda ()
-      '())
-    #f #t))
+    '())
 
-  (gnc:register-trep-option
-   (gnc:make-multichoice-callback-option
+  (gnc-register-multichoice-callback-option options
     gnc:pagename-accounts optname-filtertype
     "c" (G_ "Filter account.")
-    'none
+    "none"
     (keylist->vectorlist filter-list)
-    #f
     (lambda (x)
-      (gnc-option-db-set-option-selectable-by-name
+      (gnc-optiondb-set-option-selectable-by-name
        options gnc:pagename-accounts optname-filterby
-       (not (eq? x 'none))))))
+       (not (eq? x 'none)))))
 
   ;; Sorting options
 
@@ -695,182 +689,166 @@ be excluded from periodic reporting.")
               (SUBTOTAL-ENABLED? sec-sortkey BOOK-SPLIT-ACTION))
              (sec-date-sortingtype-enabled (memq sec-sortkey DATE-SORTING-TYPES)))
 
-        (gnc-option-db-set-option-selectable-by-name
+        (gnc-optiondb-set-option-selectable-by-name
          options pagename-sorting optname-prime-subtotal
          prime-sortkey-subtotal-enabled)
 
-        (gnc-option-db-set-option-selectable-by-name
+        (gnc-optiondb-set-option-selectable-by-name
          options pagename-sorting optname-prime-sortorder
          prime-sortkey-enabled)
 
-        (gnc-option-db-set-option-selectable-by-name
+        (gnc-optiondb-set-option-selectable-by-name
          options pagename-sorting optname-sec-subtotal
          sec-sortkey-subtotal-enabled)
 
-        (gnc-option-db-set-option-selectable-by-name
+        (gnc-optiondb-set-option-selectable-by-name
          options pagename-sorting optname-sec-sortorder
          sec-sortkey-enabled)
 
-        (gnc-option-db-set-option-selectable-by-name
+        (gnc-optiondb-set-option-selectable-by-name
          options pagename-sorting optname-full-account-name
          (or (and prime-sortkey-subtotal-enabled prime-sortkey-subtotal-true)
              (and sec-sortkey-subtotal-enabled sec-sortkey-subtotal-true)))
 
-        (gnc-option-db-set-option-selectable-by-name
+        (gnc-optiondb-set-option-selectable-by-name
          options pagename-sorting optname-show-account-code
          (or (and prime-sortkey-subtotal-enabled prime-sortkey-subtotal-true)
              (and sec-sortkey-subtotal-enabled sec-sortkey-subtotal-true)))
 
-        (gnc-option-db-set-option-selectable-by-name
+        (gnc-optiondb-set-option-selectable-by-name
          options pagename-sorting optname-show-account-description
          (or (and prime-sortkey-subtotal-enabled prime-sortkey-subtotal-true)
              (and sec-sortkey-subtotal-enabled sec-sortkey-subtotal-true)))
 
-        (gnc-option-db-set-option-selectable-by-name
+        (gnc-optiondb-set-option-selectable-by-name
          options pagename-sorting optname-indenting
          (or (and prime-sortkey-subtotal-enabled prime-sortkey-subtotal-true)
              (and sec-sortkey-subtotal-enabled sec-sortkey-subtotal-true)
              (and prime-date-sortingtype-enabled (not (eq? 'none prime-date-subtotal)))
              (and sec-date-sortingtype-enabled (not (eq? 'none sec-date-subtotal)))))
 
-        (gnc-option-db-set-option-selectable-by-name
+        (gnc-optiondb-set-option-selectable-by-name
          options pagename-sorting optname-show-subtotals-only
          (or (and prime-sortkey-subtotal-enabled prime-sortkey-subtotal-true)
              (and sec-sortkey-subtotal-enabled sec-sortkey-subtotal-true)
              (and prime-date-sortingtype-enabled (not (eq? 'none prime-date-subtotal)))
              (and sec-date-sortingtype-enabled (not (eq? 'none sec-date-subtotal)))))
 
-        (gnc-option-db-set-option-selectable-by-name
+        (gnc-optiondb-set-option-selectable-by-name
          options pagename-sorting optname-show-informal-headers
          (or (memq prime-sortkey (list 'account-name 'account-code))
              (memq sec-sortkey (list 'account-name 'account-code))))
 
-        (gnc-option-db-set-option-selectable-by-name
+        (gnc-optiondb-set-option-selectable-by-name
          options pagename-sorting optname-prime-date-subtotal
          prime-date-sortingtype-enabled)
 
-        (gnc-option-db-set-option-selectable-by-name
+        (gnc-optiondb-set-option-selectable-by-name
          options pagename-sorting optname-sec-date-subtotal
          sec-date-sortingtype-enabled)))
 
     ;; primary sorting criterion
-    (gnc:register-trep-option
-     (gnc:make-multichoice-callback-option
+    (gnc-register-multichoice-callback-option options
       pagename-sorting optname-prime-sortkey
       "a" (G_ "Sort by this criterion first.")
-      prime-sortkey
-      key-choice-list #f
+      (symbol->string prime-sortkey)
+      key-choice-list
       (lambda (x)
         (set! prime-sortkey x)
-        (apply-selectable-by-name-sorting-options))))
+        (apply-selectable-by-name-sorting-options)))
 
-    (gnc:register-trep-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       pagename-sorting optname-full-account-name
       "j1"
       (G_ "Show the full account name for subtotals and subheadings?")
-      #f))
+      #f)
 
-    (gnc:register-trep-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       pagename-sorting optname-show-account-code
       "j2"
       (G_ "Show the account code for subtotals and subheadings?")
-      #f))
+      #f)
 
-    (gnc:register-trep-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       pagename-sorting optname-show-account-description
       "j3"
       (G_ "Show the account description for subheadings?")
-      #f))
+      #f)
 
-    (gnc:register-trep-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       pagename-sorting optname-show-informal-headers
       "j4"
       (G_ "Show the informal headers for debit/credit accounts?")
-      #f))
+      #f)
 
-    (gnc:register-trep-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       pagename-sorting optname-indenting
       "j5"
       (G_ "Add indenting columns with grouping and subtotals?")
-      #t))
+      #t)
 
-    (gnc:register-trep-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       pagename-sorting optname-show-subtotals-only
       "j6"
       (G_ "Show subtotals only, hiding transactional detail?")
-      #f))
+      #f)
 
-    (gnc:register-trep-option
-     (gnc:make-complex-boolean-option
+    (gnc-register-complex-boolean-option options
       pagename-sorting optname-prime-subtotal
       "e5"
-      (G_ "Subtotal according to the primary key?")
-      prime-sortkey-subtotal-true #f
+      (G_ "Subtotal according to the primary key?") prime-sortkey-subtotal-true
       (lambda (x)
         (set! prime-sortkey-subtotal-true x)
-        (apply-selectable-by-name-sorting-options))))
+        (apply-selectable-by-name-sorting-options)))
 
-    (gnc:register-trep-option
-     (gnc:make-multichoice-callback-option
+     (gnc-register-multichoice-callback-option options
       pagename-sorting optname-prime-date-subtotal
       "e2" (G_ "Do a date subtotal.")
-      prime-date-subtotal
-      date-subtotal-choice-list #f
+      (symbol->string prime-date-subtotal)
+      date-subtotal-choice-list
       (lambda (x)
         (set! prime-date-subtotal x)
-        (apply-selectable-by-name-sorting-options))))
+        (apply-selectable-by-name-sorting-options)))
 
-    (gnc:register-trep-option
-     (gnc:make-multichoice-option
+    (gnc-register-multichoice-option options
       pagename-sorting optname-prime-sortorder
       "e" (G_ "Order of primary sorting.")
-      'ascend
-      ascending-choice-list))
+      "ascend"
+      ascending-choice-list)
 
     ;; Secondary sorting criterion
-    (gnc:register-trep-option
-     (gnc:make-multichoice-callback-option
+    (gnc:register-multichoice-callback-option options
       pagename-sorting optname-sec-sortkey
       "f"
       (G_ "Sort by this criterion second.")
-      sec-sortkey
-      key-choice-list #f
+      (symbol->string sec-sortkey)
+      key-choice-list
       (lambda (x)
         (set! sec-sortkey x)
-        (apply-selectable-by-name-sorting-options))))
+        (apply-selectable-by-name-sorting-options)))
 
-    (gnc:register-trep-option
-     (gnc:make-complex-boolean-option
+    (gnc-register-complex-boolean-option options
       pagename-sorting optname-sec-subtotal
       "i5"
-      (G_ "Subtotal according to the secondary key?")
-      sec-sortkey-subtotal-true #f
+      (G_ "Subtotal according to the secondary key?") sec-sortkey-subtotal-true
       (lambda (x)
         (set! sec-sortkey-subtotal-true x)
-        (apply-selectable-by-name-sorting-options))))
+        (apply-selectable-by-name-sorting-options)))
 
-    (gnc:register-trep-option
-     (gnc:make-multichoice-callback-option
+    (gnc-register-multichoice-callback-option options
       pagename-sorting optname-sec-date-subtotal
       "i2" (G_ "Do a date subtotal.")
-      sec-date-subtotal
-      date-subtotal-choice-list #f
+      (symbol->string sec-date-subtotal)
+      date-subtotal-choice-list
       (lambda (x)
         (set! sec-date-subtotal x)
-        (apply-selectable-by-name-sorting-options))))
+        (apply-selectable-by-name-sorting-options)))
 
-    (gnc:register-trep-option
-     (gnc:make-multichoice-option
+    (gnc-register-multichoice-option options
       pagename-sorting optname-sec-sortorder
       "i" (G_ "Order of Secondary sorting.")
-      'ascend
-      ascending-choice-list)))
+      "ascend"
+      ascending-choice-list))
 
   ;; Display options
 
@@ -882,48 +860,48 @@ be excluded from periodic reporting.")
 
     (define (apply-selectable-by-name-display-options)
       (define detail-is-single? (eq? disp-detail-level? 'single))
-      (gnc-option-db-set-option-selectable-by-name
+      (gnc-optiondb-set-option-selectable-by-name
        options gnc:pagename-display (N_ "Use Full Account Name")
        disp-accname?)
 
-      (gnc-option-db-set-option-selectable-by-name
+      (gnc-optiondb-set-option-selectable-by-name
        options gnc:pagename-display (N_ "Other Account Name")
        detail-is-single?)
 
-      (gnc-option-db-set-option-selectable-by-name
+      (gnc-optiondb-set-option-selectable-by-name
        options gnc:pagename-display (N_ "Sign Reverses")
        (eq? amount-value 'single))
 
-      (gnc-option-db-set-option-selectable-by-name
+      (gnc-optiondb-set-option-selectable-by-name
        options gnc:pagename-display optname-grid
        (not (eq? amount-value 'none)))
 
-      (gnc-option-db-set-option-selectable-by-name
+      (gnc-optiondb-set-option-selectable-by-name
        options gnc:pagename-display "Enable Links"
        (not (eq? amount-value 'none)))
 
-      (gnc-option-db-set-option-selectable-by-name
+      (gnc-optiondb-set-option-selectable-by-name
        options gnc:pagename-display (N_ "Use Full Other Account Name")
        (and disp-other-accname? detail-is-single?))
 
-      (gnc-option-db-set-option-selectable-by-name
+      (gnc-optiondb-set-option-selectable-by-name
        options gnc:pagename-display (N_ "Other Account Code")
        detail-is-single?)
 
-      (gnc-option-db-set-option-selectable-by-name
+      (gnc-optiondb-set-option-selectable-by-name
        options gnc:pagename-display (N_ "Notes")
        disp-memo?))
 
     (for-each
      (lambda (l)
-       (gnc:register-trep-option
-        (gnc:make-simple-boolean-option
-         gnc:pagename-display (car l) (cadr l) (caddr l) (cadddr l))))
+       (gnc-register-simple-boolean-option options
+         gnc:pagename-display (car l) (cadr l) (caddr l) (cadddr l)))
      ;; One list per option here with: option-name, sort-tag,
      ;; help-string, default-value
      (list
       (list (N_ "Date")                         "a"  (G_ "Display the date?") #t)
       (list (N_ "Reconciled Date")              "a2" (G_ "Display the reconciled date?") #f)
+      (list (N_ "Date Entered")                 "a3" (G_ "Display the entered date?") #f)
       (if BOOK-SPLIT-ACTION
           (list (N_ "Num/Action")               "b"  (G_ "Display the check number?") #t)
           (list (N_ "Num")                      "b"  (G_ "Display the check number?") #t))
@@ -944,89 +922,75 @@ be excluded from periodic reporting.")
       (list (N_ "Totals")                       "o"  (G_ "Display the totals?") #t)))
 
     (when BOOK-SPLIT-ACTION
-      (gnc:register-trep-option
-       (gnc:make-simple-boolean-option
+      (gnc-register-simple-boolean-option options
         gnc:pagename-display (N_ "Trans Number")
-        "b2" (G_ "Display the trans number?") #f)))
+        "b2" (G_ "Display the trans number?") #f))
 
     ;; Add an option to display the memo, and disable the notes option
     ;; when memos are not included.
-    (gnc:register-trep-option
-     (gnc:make-complex-boolean-option
+    (gnc-register-complex-boolean-option options
       gnc:pagename-display (N_ "Memo")
       "d"  (G_ "Display the memo?") disp-memo?
-      disp-memo?
       (lambda (x)
         (set! disp-memo? x)
-        (apply-selectable-by-name-display-options))))
+        (apply-selectable-by-name-display-options)))
 
     ;; Ditto for Account Name #t -> Use Full Account Name is selectable
-    (gnc:register-trep-option
-     (gnc:make-complex-boolean-option
+    (gnc-register-complex-boolean-option options
       gnc:pagename-display (N_ "Account Name")
       "e"  (G_ "Display the account name?") disp-accname?
-      disp-accname?
       (lambda (x)
         (set! disp-accname? x)
-        (apply-selectable-by-name-display-options))))
+        (apply-selectable-by-name-display-options)))
 
     ;; Ditto for Other Account Name #t -> Use Full Other Account Name is selectable
-    (gnc:register-trep-option
-     (gnc:make-complex-boolean-option
+    (gnc-register-complex-boolean-option options
       gnc:pagename-display (N_ "Other Account Name")
-      "h5"  (G_ "Display the other account name? (if this is a split transaction, this parameter is guessed).") disp-other-accname?
-      disp-other-accname?
+      "h5"  (G_ "Display the other account name? If this is a split transaction, this parameter is guessed.") disp-other-accname?
       (lambda (x)
         (set! disp-other-accname? x)
-        (apply-selectable-by-name-display-options))))
+        (apply-selectable-by-name-display-options)))
 
-    (gnc:register-trep-option
-     (gnc:make-multichoice-callback-option
+    (gnc-register-multichoice-callback-option options
       gnc:pagename-display optname-detail-level
       "h" (G_ "Amount of detail to display per transaction.")
-      disp-detail-level?
+      (symbol->string disp-detail-level?)
       (list (vector 'multi-line (G_ "One split per line"))
             (vector 'single (G_ "One transaction per line")))
-      #f
       (lambda (x)
         (set! disp-detail-level? x)
-        (apply-selectable-by-name-display-options))))
+        (apply-selectable-by-name-display-options)))
 
-    (gnc:register-trep-option
-     (gnc:make-multichoice-callback-option
+    (gnc-register-multichoice-callback-option options
       gnc:pagename-display (N_ "Amount")
       "m" (G_ "Display the amount?")
-      amount-value
+      (symbol->string amount-value)
       (list
        (vector 'none   (G_ "Hide"))
        (vector 'single (G_ "Single Column"))
        (vector 'double (G_ "Two Columns")))
-      #f
       (lambda (x)
         (set! amount-value x)
-        (apply-selectable-by-name-display-options))))
+        (apply-selectable-by-name-display-options)))
 
-    (gnc:register-trep-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display (N_ "Enable Links")
-      "m2" (G_ "Enable hyperlinks in amounts.") #t))
+      "m2" (G_ "Enable hyperlinks in amounts.") #t)
 
-    (gnc:register-trep-option
-     (gnc:make-multichoice-option
+    (gnc-register-multichoice-option options
       gnc:pagename-display (N_ "Sign Reverses")
       "m1" (G_ "Reverse amount display for certain account types.")
-      'global
-      (keylist->vectorlist sign-reverse-list))))
+      "global"
+      (keylist->vectorlist sign-reverse-list)))
 
   ;; this hidden option will toggle whether the default
   ;; qof-query is run, or a different query which ensures
   ;; no transaction is duplicated. It can be enabled in
   ;; a derived report (eg income-gst-statement.scm)
-  (gnc:register-trep-option
-   (gnc:make-internal-option "__trep" "unique-transactions" #f))
+  (gnc-register-internal-option options "__trep" "unique-transactions" #f)
 
-  (gnc:options-set-default-section options gnc:pagename-general)
-  options)
+  (GncOptionDBPtr-set-default-section options gnc:pagename-general)
+    options))
 
 ;; ;;;;;;;;;;;;;;;;;;;;
 ;; Here comes the big function that builds the whole table.
@@ -1035,10 +999,8 @@ be excluded from periodic reporting.")
                           begindate enddate c_account_1)
 
   (define (opt-val section name)
-    (let ((option (gnc:lookup-option options section name)))
-      (if option
-          (gnc:option-value option)
-          (gnc:error "gnc:lookup-option error: " section "/" name))))
+    (gnc-optiondb-lookup-value (gnc:optiondb options) section name))
+
   (define BOOK-SPLIT-ACTION
     (qof-book-use-split-action-for-num-field (gnc-get-current-book)))
 
@@ -1048,6 +1010,7 @@ be excluded from periodic reporting.")
     (define amount-setting (opt-val gnc:pagename-display (N_ "Amount")))
     (list (cons 'date (opt-val gnc:pagename-display (N_ "Date")))
           (cons 'reconciled-date (opt-val gnc:pagename-display (N_ "Reconciled Date")))
+          (cons 'entered (opt-val gnc:pagename-display (N_ "Date Entered")))
           (cons 'num (if BOOK-SPLIT-ACTION
                          (opt-val gnc:pagename-display (N_ "Num/Action"))
                          (opt-val gnc:pagename-display (N_ "Num"))))
@@ -1150,6 +1113,15 @@ be excluded from periodic reporting.")
                                         (xaccTransGetDate
                                          (xaccSplitGetParent split))))))))
 
+               (add-if (column-uses? 'entered)
+                       (vector (G_ "Date Entered")
+                               (lambda (split transaction-row?)
+                                 (and transaction-row?
+                                      (gnc:make-html-table-cell/markup
+                                       "date-cell" (qof-print-date
+                                                    (xaccTransRetDateEntered
+                                                     (xaccSplitGetParent split))))))))
+
                (add-if (column-uses? 'reconciled-date)
                        (vector (G_ "Reconciled Date")
                                (lambda (split transaction-row?)
@@ -1290,17 +1262,13 @@ be excluded from periodic reporting.")
                                              (opt-val pagename-currency
                                                       optname-currency)))
                                     ""))))
-           ;; For conversion to row-currency. Use midday as the
-           ;; transaction time so it matches a price on the same day.
-           ;; Otherwise it uses midnight which will likely match a
-           ;; price on the previous day
+           ;; For conversion to row-currency.
            (converted-amount (lambda (s)
                                (exchange-fn
                                 (gnc:make-gnc-monetary (split-currency s)
                                                        (split-amount s))
                                 (row-currency s)
-                                (time64CanonicalDayTime
-                                 (xaccTransGetDate (xaccSplitGetParent s))))))
+                                (xaccTransGetDate (xaccSplitGetParent s)))))
            (converted-debit-amount (lambda (s) (and (positive? (split-amount s))
                                                     (converted-amount s))))
            (converted-credit-amount (lambda (s)
@@ -1852,25 +1820,22 @@ be excluded from periodic reporting.")
    (lambda (cell)
      (cell-match? cell row col))
    grid))
-(define (grid-del grid row col)
-  ;; grid filter - del all row/col - if #f then delete whole row/col
-  (filter
-   (lambda (cell)
-     (not (cell-match? cell row col)))
-   grid))
 (define (grid-rows grid)
   (delete-duplicates (map (lambda (cell) (vector-ref cell 0)) grid)))
 (define (grid-cols grid)
   (delete-duplicates (map (lambda (cell) (vector-ref cell 1)) grid)))
 (define (grid-add grid row col data)
-  ;;misonomer - we don't 'add' to existing data, we delete old data
-  ;;stored at row/col and add again. this is fine because the grid
-  ;;should never have duplicate data in the trep.
-  (set! grid (grid-del grid row col))
-  (set! grid (cons (vector row col data) grid))
-  grid)
-(define (grid->html-table grid list-of-rows list-of-cols)
-  (define row-average-enabled? (> (length list-of-cols) 1))
+  ;; we don't need to check for duplicate cells in a row/col because
+  ;; in the trep it should never happen.
+  (cons (vector row col data) grid))
+(define (grid->html-table grid)
+  (define (<? a b)
+    (cond ((string? (car a)) (gnc:string-locale<? (car a) (car b)))
+          ((number? (car a)) (< (car a) (car b)))
+          (else (gnc:error "unknown sortvalue"))))
+  (define list-of-rows (sort (delete 'row-total (grid-rows grid)) <?))
+  (define list-of-cols (sort (delete 'col-total (grid-cols grid)) <?))
+  (define row-average-enabled? (and (pair? list-of-cols) (pair? (cdr list-of-cols))))
   (define (monetary-div monetary divisor)
     (and monetary
          (let* ((amount (gnc:gnc-monetary-amount monetary))
@@ -1934,7 +1899,7 @@ be excluded from periodic reporting.")
           report-obj #:key custom-calculated-cells empty-report-message
           custom-split-filter split->date split->date-include-false?
           custom-source-accounts
-          export-type filename)
+          export-type)
   ;; the trep-renderer is a define* function which, at minimum, takes
   ;; the report object
   ;;
@@ -1954,7 +1919,7 @@ be excluded from periodic reporting.")
 
   (define options (gnc:report-options report-obj))
   (define (opt-val section name)
-    (gnc:option-value (gnc:lookup-option options section name)))
+    (gnc-optiondb-lookup-value (gnc:optiondb options) section name))
   (define BOOK-SPLIT-ACTION
     (qof-book-use-split-action-for-num-field (gnc-get-current-book)))
   (define (is-filter-member split account-list)
@@ -1967,15 +1932,11 @@ be excluded from periodic reporting.")
         (((? from-account?) . _) #t)
         ((_ . rest) (lp rest)))))
 
-  (when filename
-    (issue-deprecation-warning "trep-renderer filename is obsolete, and not \
-supported for exports. please set html-document export-string instead. this \
-warning will be removed in GnuCash 5.0"))
-
   (gnc:report-starting (opt-val gnc:pagename-general gnc:optname-reportname))
 
   (let* ((document (gnc:make-html-document))
          (account-matcher (opt-val pagename-filter optname-account-matcher))
+         (account-matcher-neg (opt-val pagename-filter optname-account-matcher-exclude))
          (account-matcher-regexp
           (and (opt-val pagename-filter optname-account-matcher-regex)
                (if (defined? 'make-regexp)
@@ -1985,14 +1946,16 @@ warning will be removed in GnuCash 5.0"))
                    'no-guile-regex-support)))
          (c_account_0 (or custom-source-accounts
                           (opt-val gnc:pagename-accounts optname-accounts)))
-         (c_account_1 (filter
-                       (lambda (acc)
-                         (if (regexp? account-matcher-regexp)
-                             (regexp-exec account-matcher-regexp
-                                          (gnc-account-get-full-name acc))
-                             (string-contains (gnc-account-get-full-name acc)
-                                              account-matcher)))
-                       c_account_0))
+         (acct? (lambda (acc)
+                  (if (regexp? account-matcher-regexp)
+                      (regexp-exec account-matcher-regexp
+                                   (gnc-account-get-full-name acc))
+                      (string-contains (gnc-account-get-full-name acc)
+                                       account-matcher))))
+         (c_account_1 (if (string-null? account-matcher)
+                          c_account_0
+                          (filter (if account-matcher-neg (negate acct?) acct?)
+                                  c_account_0)))
          (c_account_2 (opt-val gnc:pagename-accounts optname-filterby))
          (filter-mode (opt-val gnc:pagename-accounts optname-filtertype))
          (begindate (gnc:time64-start-day-time
@@ -2244,19 +2207,7 @@ warning will be removed in GnuCash 5.0"))
              (gnc:html-render-options-changed options)))
 
           (when subtotal-table?
-            (let* ((generic<?
-                    (lambda (a b)
-                      (cond ((string? (car a)) (gnc:string-locale<? (car a) (car b)))
-                            ((number? (car a)) (< (car a) (car b)))
-                            (else (gnc:error "unknown sortvalue")))))
-                   (list-of-rows
-                    (stable-sort! (delete 'row-total (grid-rows grid))
-                                  generic<?))
-                   (list-of-cols
-                    (stable-sort! (delete 'col-total (grid-cols grid))
-                                  generic<?)))
-              (gnc:html-document-add-object!
-               document (grid->html-table grid list-of-rows list-of-cols))))
+            (gnc:html-document-add-object! document (grid->html-table grid)))
 
           (unless (and subtotal-table?
                        (opt-val pagename-sorting optname-show-subtotals-only))
