@@ -88,40 +88,47 @@ static void _gnc_sx_instance_event_handler(QofInstance *ent, QofEventId event_ty
 static gnc_commodity* get_transaction_currency(SxTxnCreationData *creation_data, SchedXaction *sx, Transaction *template_txn);
 /* ------------------------------------------------------------ */
 
-static gboolean
-scrub_sx_split_numeric (Split* split, const char *debcred)
+typedef struct
 {
-    const gboolean is_credit = g_strcmp0 (debcred, "credit") == 0;
-    const char *formula = is_credit ?
-        "sx-credit-formula" : "sx-debit-formula";
-    const char *numeric = is_credit ?
-        "sx-credit-numeric" : "sx-debit-numeric";
+    const char *name;
+    gnc_numeric amount;
+} ScrubItem;
+
+static void
+scrub_sx_split_numeric (Split* split, gboolean is_credit, GList **changes)
+{
+    const char *formula = is_credit ? "sx-credit-formula" : "sx-debit-formula";
+    const char *numeric = is_credit ? "sx-credit-numeric" : "sx-debit-numeric";
     char *formval;
     gnc_numeric *numval = NULL;
-    GHashTable *parser_vars = g_hash_table_new (g_str_hash, g_str_equal);
+    GHashTable *parser_vars = g_hash_table_new_full
+        (g_str_hash, g_str_equal, g_free, (GDestroyNotify)g_free);
     char *error_loc;
     gnc_numeric amount = gnc_numeric_zero ();
     gboolean parse_result = FALSE;
-    gboolean num_val_changed = FALSE;
+
     qof_instance_get (QOF_INSTANCE (split),
-		  formula, &formval,
-		  numeric, &numval,
-		  NULL);
-    parse_result =
-        gnc_exp_parser_parse_separate_vars (formval, &amount,
-                                            &error_loc, parser_vars);
+                      formula, &formval,
+                      numeric, &numval,
+                      NULL);
+
+    parse_result = gnc_exp_parser_parse_separate_vars (formval, &amount,
+                                                       &error_loc, parser_vars);
+
     if (!parse_result || g_hash_table_size (parser_vars) != 0)
         amount = gnc_numeric_zero ();
-    g_hash_table_unref (parser_vars);
+
     if (!numval || !gnc_numeric_eq (amount, *numval))
     {
-        qof_instance_set (QOF_INSTANCE (split),
-                          numeric, &amount,
-                          NULL);
-        num_val_changed = TRUE;
+        ScrubItem *change = g_new (ScrubItem, 1);
+        change->name = numeric;
+        change->amount = amount;
+        *changes = g_list_prepend (*changes, change);
     }
+
+    g_hash_table_destroy (parser_vars);
+    g_free (formval);
     g_free (numval);
-    return num_val_changed;
 }
 
 /* Fixes error in pre-2.6.16 where the numeric slot wouldn't get changed if the
@@ -132,14 +139,22 @@ gnc_sx_scrub_split_numerics (gpointer psplit, gpointer puser)
 {
     Split *split = GNC_SPLIT (psplit);
     Transaction *trans = xaccSplitGetParent (split);
-    gboolean changed;
+    GList *changes = NULL;
+    scrub_sx_split_numeric (split, TRUE, &changes);
+    scrub_sx_split_numeric (split, FALSE, &changes);
+    if (!changes)
+        return;
+
     xaccTransBeginEdit (trans);
-    changed = scrub_sx_split_numeric (split, "credit") +
-        scrub_sx_split_numeric (split, "debit");
-    if (!changed)
-        xaccTransRollbackEdit (trans);
-    else
-        xaccTransCommitEdit (trans);
+    for (GList *n = changes; n; n = n->next)
+    {
+        ScrubItem *change = n->data;
+        qof_instance_set (QOF_INSTANCE (split),
+                          change->name, &change->amount,
+                          NULL);
+    }
+    xaccTransCommitEdit (trans);
+    g_list_free_full (changes, g_free);
 }
 
 static void
@@ -194,7 +209,8 @@ GHashTable*
 gnc_sx_instance_get_variables_for_parser(GHashTable *instance_var_hash)
 {
     GHashTable *parser_vars;
-    parser_vars = g_hash_table_new(g_str_hash, g_str_equal);
+
+    parser_vars = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
     g_hash_table_foreach(instance_var_hash, (GHFunc)_sx_var_to_raw_numeric, parser_vars);
     return parser_vars;
 }
@@ -435,7 +451,7 @@ static void
 _build_list_from_hash_elts(gpointer key, gpointer value, gpointer user_data)
 {
     GList **list = (GList**)user_data;
-    *list = g_list_insert_sorted(*list, value, _compare_GncSxVariables);
+    *list = g_list_prepend (*list, value);
 }
 
 GList *
@@ -443,13 +459,14 @@ gnc_sx_instance_get_variables(GncSxInstance *inst)
 {
     GList *vars = NULL;
     g_hash_table_foreach(inst->variable_bindings, _build_list_from_hash_elts, &vars);
-    return vars;
+    return g_list_sort (vars, _compare_GncSxVariables);
 }
 
 static GncSxInstances*
 _gnc_sx_gen_instances(gpointer *data, gpointer user_data)
 {
     GncSxInstances *instances = g_new0(GncSxInstances, 1);
+    GList *instlist = NULL;
     SchedXaction *sx = (SchedXaction*)data;
     const GDate *range_end = (const GDate*)user_data;
     GDate creation_end, remind_end;
@@ -477,8 +494,7 @@ _gnc_sx_gen_instances(gpointer *data, gpointer user_data)
             seq_num = gnc_sx_get_instance_count(sx, postponed->data);
             inst = gnc_sx_instance_new(instances, SX_INSTANCE_STATE_POSTPONED,
                                        &inst_date, postponed->data, seq_num);
-            instances->instance_list =
-                g_list_append(instances->instance_list, inst);
+            instlist = g_list_prepend (instlist, inst);
             gnc_sx_destroy_temporal_state(temporal_state);
             temporal_state = gnc_sx_clone_temporal_state(postponed->data);
             gnc_sx_incr_temporal_state(sx, temporal_state);
@@ -496,7 +512,7 @@ _gnc_sx_gen_instances(gpointer *data, gpointer user_data)
         seq_num = gnc_sx_get_instance_count(sx, temporal_state);
         inst = gnc_sx_instance_new(instances, SX_INSTANCE_STATE_TO_CREATE,
                                    &cur_date, temporal_state, seq_num);
-        instances->instance_list = g_list_append(instances->instance_list, inst);
+        instlist = g_list_prepend (instlist, inst);
         gnc_sx_incr_temporal_state(sx, temporal_state);
         cur_date = xaccSchedXactionGetNextInstance(sx, temporal_state);
     }
@@ -510,12 +526,15 @@ _gnc_sx_gen_instances(gpointer *data, gpointer user_data)
         seq_num = gnc_sx_get_instance_count(sx, temporal_state);
         inst = gnc_sx_instance_new(instances, SX_INSTANCE_STATE_REMINDER,
                                    &cur_date, temporal_state, seq_num);
-        instances->instance_list = g_list_append(instances->instance_list,
-                                                 inst);
+        instlist = g_list_prepend (instlist, inst);
         gnc_sx_incr_temporal_state(sx, temporal_state);
         cur_date = xaccSchedXactionGetNextInstance(sx, temporal_state);
     }
 
+    instances->instance_list = g_list_reverse (instlist);
+
+    gnc_sx_destroy_temporal_state (temporal_state);
+
     return instances;
 }
 
@@ -832,7 +851,7 @@ _find_unreferenced_vars(gchar *key,
         !g_hash_table_lookup_extended(cb_pair->hash, key, NULL, NULL))
     {
         DEBUG("variable [%s] not found", key);
-        cb_pair->list = g_list_append(cb_pair->list, key);
+        cb_pair->list = g_list_prepend (cb_pair->list, key);
     }
 }
 
@@ -912,7 +931,7 @@ gnc_sx_instance_model_update_sx_instances(GncSxInstanceModel *model, SchedXactio
             removed_cb_data.hash = new_instances->variable_names;
             removed_cb_data.list = NULL;
             g_hash_table_foreach(existing->variable_names, (GHFunc)_find_unreferenced_vars, &removed_cb_data);
-            removed_var_names = removed_cb_data.list;
+            removed_var_names = g_list_reverse (removed_cb_data.list);
         }
         DEBUG("%d removed variables", g_list_length(removed_var_names));
 
@@ -922,7 +941,7 @@ gnc_sx_instance_model_update_sx_instances(GncSxInstanceModel *model, SchedXactio
             added_cb_data.hash = existing->variable_names;
             added_cb_data.list = NULL;
             g_hash_table_foreach(new_instances->variable_names, (GHFunc)_find_unreferenced_vars, &added_cb_data);
-            added_var_names = added_cb_data.list;
+            added_var_names = g_list_reverse (added_cb_data.list);
         }
         DEBUG("%d added variables", g_list_length(added_var_names));
 
@@ -1053,6 +1072,8 @@ _get_sx_formula_value(const SchedXaction* sx,
          * localization problems with separators. */
 	numeric->num = numeric_val->num;
 	numeric->denom = numeric_val->denom;
+        g_free (formula_str);
+        g_free (numeric_val);
         return;
     }
 
@@ -1082,6 +1103,8 @@ _get_sx_formula_value(const SchedXaction* sx,
             g_hash_table_destroy(parser_vars);
         }
     }
+    g_free (formula_str);
+    g_free (numeric_val);
 }
 
 static void
@@ -1183,7 +1206,8 @@ static gnc_commodity*
 get_transaction_currency(SxTxnCreationData *creation_data,
                          SchedXaction *sx, Transaction *template_txn)
 {
-    gnc_commodity *first_currency = NULL, *first_cmdty = NULL;
+    gnc_commodity *first_currency = NULL, *first_cmdty = NULL,
+        *fallback_cmdty = NULL;
     gboolean err_flag = FALSE, txn_cmdty_in_splits = FALSE;
     gnc_commodity *txn_cmdty = xaccTransGetCurrency (template_txn);
     GList* txn_splits = xaccTransGetSplitList (template_txn);
@@ -1211,6 +1235,9 @@ get_transaction_currency(SxTxnCreationData *creation_data,
         /* Don't consider the commodity of a transaction that has
          * neither a credit nor a debit formula. */
 
+        if (!fallback_cmdty)
+            fallback_cmdty = xaccAccountGetCommodity (split_account);
+
         if (split_is_marker(t_split))
              continue;
 
@@ -1233,16 +1260,18 @@ get_transaction_currency(SxTxnCreationData *creation_data,
     if (first_currency &&
         (!txn_cmdty_in_splits || !gnc_commodity_is_currency (txn_cmdty)))
         return first_currency;
-    if (!txn_cmdty_in_splits)
+    if (!txn_cmdty_in_splits && first_cmdty)
         return first_cmdty;
-    return txn_cmdty;
+    if (txn_cmdty)
+        return txn_cmdty;
+    return fallback_cmdty;
 }
 
 static gboolean
 create_each_transaction_helper(Transaction *template_txn, void *user_data)
 {
     Transaction *new_txn;
-    GList *txn_splits, *template_splits, *node;
+    GList *txn_splits, *template_splits;
     Split *copying_split;
     SxTxnCreationData *creation_data = (SxTxnCreationData*)user_data;
     SchedXaction *sx = creation_data->instance->parent->sx;
@@ -1536,7 +1565,7 @@ gnc_sx_instance_model_set_variable(GncSxInstanceModel *model,
 static void
 _list_from_hash_elts(gpointer key, gpointer value, GList **result_list)
 {
-    *result_list = g_list_append(*result_list, value);
+    *result_list = g_list_prepend (*result_list, value);
 }
 
 GList*
@@ -1564,7 +1593,7 @@ gnc_sx_instance_model_check_variables(GncSxInstanceModel *model)
                     GncSxVariableNeeded *need = g_new0(GncSxVariableNeeded, 1);
                     need->instance = inst;
                     need->variable = var;
-                    rtn = g_list_append(rtn, need);
+                    rtn = g_list_prepend (rtn, need);
                 }
             }
             g_list_free(var_list);
