@@ -28,8 +28,6 @@
  * restoring data to/from an SQL database
  */
 #include <guid.hpp>
-extern "C"
-{
 #include <config.h>
 #include <qof.h>
 #include <glib.h>
@@ -37,7 +35,6 @@ extern "C"
 #include "gncJobP.h"
 #include "gncEmployeeP.h"
 #include "gncVendorP.h"
-}
 #include <cstdlib>
 #include <cstring>
 #include <sstream>
@@ -182,7 +179,7 @@ GncSqlColumnTableEntryImpl<CT_OWNERREF>::add_to_query(QofIdTypeConst obj_name,
     auto owner = (*getter) (pObject);
 
     QofInstance* inst = nullptr;
-    GncOwnerType type;
+    GncOwnerType type = GNC_OWNER_NONE;
 
     auto type_hdr = std::string{m_col_name} + "_type";
     auto guid_hdr = std::string{m_col_name} + "_guid";
