@@ -20,8 +20,6 @@
  * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
  * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
-extern "C"
-{
 #include <config.h>
 #include <gnc-prefs.h>
 #include <gnc-engine.h>
@@ -32,7 +30,6 @@ extern "C"
 #include <gncTaxTable.h>
 #include <gncInvoice.h>
 #include <gnc-pricedb.h>
-}
 
 #include <algorithm>
 #include <cassert>
@@ -72,6 +69,7 @@ static QofLogModule log_module = G_LOG_DOMAIN;
 
 using StrVec = std::vector<std::string>;
 
+static std::string empty_string{};
 static EntryVec version_table
 {
     gnc_sql_make_table_entry<CT_STRING>(
@@ -87,6 +85,11 @@ GncSqlBackend::GncSqlBackend(GncSqlConnection *conn, QofBook* book) :
         connect (conn);
 }
 
+GncSqlBackend::~GncSqlBackend()
+{
+    connect(nullptr);
+}
+
 void
 GncSqlBackend::connect(GncSqlConnection *conn) noexcept
 {
@@ -99,7 +102,7 @@ GncSqlBackend::connect(GncSqlConnection *conn) noexcept
 GncSqlStatementPtr
 GncSqlBackend::create_statement_from_sql(const std::string& str) const noexcept
 {
-    auto stmt = m_conn->create_statement_from_sql(str);
+    auto stmt = m_conn ? m_conn->create_statement_from_sql(str) : nullptr;
     if (stmt == nullptr)
     {
         PERR ("SQL error: %s\n", str.c_str());
@@ -111,7 +114,7 @@ GncSqlBackend::create_statement_from_sql(const std::string& str) const noexcept
 GncSqlResultPtr
 GncSqlBackend::execute_select_statement(const GncSqlStatementPtr& stmt) const noexcept
 {
-    auto result = m_conn->execute_select_statement(stmt);
+    auto result = m_conn ? m_conn->execute_select_statement(stmt) : nullptr;
     if (result == nullptr)
     {
         PERR ("SQL error: %s\n", stmt->to_sql());
@@ -123,7 +126,7 @@ GncSqlBackend::execute_select_statement(const GncSqlStatementPtr& stmt) const no
 int
 GncSqlBackend::execute_nonselect_statement(const GncSqlStatementPtr& stmt) const noexcept
 {
-    auto result = m_conn->execute_nonselect_statement(stmt);
+    int result = m_conn ? m_conn->execute_nonselect_statement(stmt) : -1;
     if (result == -1)
     {
         PERR ("SQL error: %s\n", stmt->to_sql());
@@ -135,6 +138,9 @@ GncSqlBackend::execute_nonselect_statement(const GncSqlStatementPtr& stmt) const
 std::string
 GncSqlBackend::quote_string(const std::string& str) const noexcept
 {
+    g_return_val_if_fail (m_conn != nullptr, empty_string);
+    if (!m_conn)
+        return empty_string;
     return m_conn->quote_string(str);
 }
 
@@ -142,8 +148,9 @@ bool
 GncSqlBackend::create_table(const std::string& table_name,
                             const EntryVec& col_table) const noexcept
 {
+    g_return_val_if_fail (m_conn != nullptr, false);
+
     ColVec info_vec;
-    gboolean ok = FALSE;
 
     for (auto const& table_row : col_table)
     {
@@ -167,6 +174,7 @@ GncSqlBackend::create_index(const std::string& index_name,
                             const std::string& table_name,
                             const EntryVec& col_table) const noexcept
 {
+    g_return_val_if_fail (m_conn != nullptr, false);
     return m_conn->create_index(index_name, table_name, col_table);
 }
 
@@ -174,6 +182,8 @@ bool
 GncSqlBackend::add_columns_to_table(const std::string& table_name,
                                     const EntryVec& col_table) const noexcept
 {
+    g_return_val_if_fail (m_conn != nullptr, false);
+
     ColVec info_vec;
 
     for (auto const& table_row : col_table)
@@ -184,10 +194,10 @@ GncSqlBackend::add_columns_to_table(const std::string& table_name,
 }
 
 void
-GncSqlBackend::update_progress() const noexcept
+GncSqlBackend::update_progress(double pct) const noexcept
 {
     if (m_percentage != nullptr)
-        (m_percentage) (nullptr, 101.0);
+        (m_percentage) (nullptr, pct);
 }
 
 void
@@ -202,14 +212,14 @@ GncSqlBackend::create_tables() noexcept
 {
     for(auto entry : m_backend_registry)
     {
-        update_progress();
+        update_progress(101.0);
         std::get<1>(entry)->create_tables(this);
     }
 }
 
 /* Main object load order */
 static const StrVec fixed_load_order
-{ GNC_ID_BOOK, GNC_ID_COMMODITY, GNC_ID_ACCOUNT, GNC_ID_LOT };
+{ GNC_ID_BOOK, GNC_ID_COMMODITY, GNC_ID_ACCOUNT, GNC_ID_LOT, GNC_ID_TRANS };
 
 /* Order in which business objects need to be loaded */
 static const StrVec business_fixed_load_order =
@@ -219,7 +229,10 @@ void
 GncSqlBackend::ObjectBackendRegistry::load_remaining(GncSqlBackend* sql_be)
 {
 
-    for (auto entry : m_registry)
+    auto num_types = m_registry.size();
+    auto num_done = fixed_load_order.size() + business_fixed_load_order.size();
+
+    for (const auto& entry : m_registry)
     {
         std::string type;
         GncSqlObjectBackendPtr obe = nullptr;
@@ -234,6 +247,8 @@ GncSqlBackend::ObjectBackendRegistry::load_remaining(GncSqlBackend* sql_be)
                       business_fixed_load_order.end(),
                       type) != business_fixed_load_order.end()) continue;
 
+        num_done++;
+        sql_be->update_progress(num_done * 100 / num_types);
         obe->load_all (sql_be);
     }
 }
@@ -273,22 +288,27 @@ GncSqlBackend::load (QofBook* book, QofBackendLoadType loadType)
         assert (m_book == nullptr);
         m_book = book;
 
+        auto num_types = m_backend_registry.size();
+        auto num_done = 0;
+
         /* Load any initial stuff. Some of this needs to happen in a certain order */
-        for (auto type : fixed_load_order)
+        for (const auto& type : fixed_load_order)
         {
+            num_done++;
             auto obe = m_backend_registry.get_object_backend(type);
             if (obe)
             {
-                update_progress();
+                update_progress(num_done * 100 / num_types);
                 obe->load_all(this);
             }
         }
-        for (auto type : business_fixed_load_order)
+        for (const auto& type : business_fixed_load_order)
         {
+            num_done++;
             auto obe = m_backend_registry.get_object_backend(type);
             if (obe)
             {
-                update_progress();
+                update_progress(num_done * 100 / num_types);
                 obe->load_all(this);
             }
         }
@@ -315,7 +335,7 @@ GncSqlBackend::load (QofBook* book, QofBackendLoadType loadType)
                       gnc_commodity_begin_edit(comm);
                       gnc_commodity_commit_edit(comm);
                   });
-    m_postload_commodities.empty();
+    m_postload_commodities.clear();
 
     /* Mark the sessoion as clean -- though it should never be marked
      * dirty with this backend
@@ -349,7 +369,7 @@ GncSqlBackend::write_account_tree(Account* root)
         }
         g_list_free (descendants);
     }
-    update_progress();
+    update_progress(101.0);
 
     return is_ok;
 }
@@ -357,11 +377,11 @@ GncSqlBackend::write_account_tree(Account* root)
 bool
 GncSqlBackend::write_accounts()
 {
-    update_progress();
+    update_progress(101.0);
     auto is_ok = write_account_tree (gnc_book_get_root_account (m_book));
     if (is_ok)
     {
-        update_progress();
+        update_progress(101.0);
         is_ok = write_account_tree (gnc_book_get_template_root(m_book));
     }
 
@@ -384,7 +404,7 @@ write_tx (Transaction* tx, gpointer data)
     {
         s->is_ok = splitbe->commit(s->be, QOF_INSTANCE(split_node->data));
     }
-    s->be->update_progress ();
+    s->be->update_progress (101.0);
     return (s->is_ok ? 0 : 1);
 }
 
@@ -396,7 +416,7 @@ GncSqlBackend::write_transactions()
 
     (void)xaccAccountTreeForEachTransaction (
         gnc_book_get_root_account (m_book), write_tx, &data);
-    update_progress();
+    update_progress(101.0);
     return data.is_ok;
 }
 
@@ -409,7 +429,7 @@ GncSqlBackend::write_template_transactions()
     if (gnc_account_n_descendants (ra) > 0)
     {
         (void)xaccAccountTreeForEachTransaction (ra, write_tx, &data);
-        update_progress();
+        update_progress(101.0);
     }
 
     return data.is_ok;
@@ -430,7 +450,7 @@ GncSqlBackend::write_schedXactions()
         tmpSX = static_cast<decltype (tmpSX)> (schedXactions->data);
         is_ok = obe->commit (this, QOF_INSTANCE (tmpSX));
     }
-    update_progress();
+    update_progress(101.0);
 
     return is_ok;
 }
@@ -441,10 +461,11 @@ void
 GncSqlBackend::sync(QofBook* book)
 {
     g_return_if_fail (book != NULL);
+    g_return_if_fail (m_conn != nullptr);
 
     reset_version_info();
     ENTER ("book=%p, sql_be->book=%p", book, m_book);
-    update_progress();
+    update_progress(101.0);
 
     /* Create new tables */
     m_is_pristine_db = true;
@@ -498,7 +519,7 @@ GncSqlBackend::sync(QofBook* book)
     else
     {
         set_error (ERR_BACKEND_SERVER_ERR);
-        is_ok = m_conn->rollback_transaction ();
+        m_conn->rollback_transaction ();
     }
     finish_progress();
     LEAVE ("book=%p", book);
@@ -544,12 +565,12 @@ GncSqlBackend::get_object_backend(const std::string& type) const noexcept
 void
 GncSqlBackend::commit (QofInstance* inst)
 {
-    sql_backend be_data;
     gboolean is_dirty;
     gboolean is_destroying;
     gboolean is_infant;
 
     g_return_if_fail (inst != NULL);
+    g_return_if_fail (m_conn != nullptr);
 
     if (qof_book_is_readonly(m_book))
     {
@@ -640,7 +661,7 @@ GncSqlBackend::commit (QofInstance* inst)
 void
 GncSqlBackend::init_version_info() noexcept
 {
-
+    g_return_if_fail (m_conn != nullptr);
     if (m_conn->does_table_exist (VERSION_TABLE_NAME))
     {
         std::string sql {"SELECT * FROM "};
@@ -799,7 +820,6 @@ bool
 GncSqlBackend::object_in_db (const char* table_name, QofIdTypeConst obj_name,
                              const gpointer pObject, const EntryVec& table) const noexcept
 {
-    guint count;
     g_return_val_if_fail (table_name != nullptr, false);
     g_return_val_if_fail (obj_name != nullptr, false);
     g_return_val_if_fail (pObject != nullptr, false);
@@ -881,7 +901,7 @@ GncSqlBackend::build_insert_statement (const char* table_name,
     }
 
     sql << ") VALUES(";
-    for (auto col_value : values)
+    for (const auto& col_value : values)
     {
         if (col_value != *values.begin())
             sql << ",";
