@@ -24,13 +24,12 @@
 #ifndef __GNC_SQL_COLUMN_TABLE_ENTRY_HPP__
 #define __GNC_SQL_COLUMN_TABLE_ENTRY_HPP__
 
-extern "C"
-{
 #include <qof.h>
-}
 #include <memory>
 #include <vector>
 #include <iostream>
+#include <iomanip>
+
 #include "gnc-sql-result.hpp"
 
 struct GncSqlColumnInfo;
@@ -69,8 +68,7 @@ enum GncSqlObjectType
     CT_GUID,
     CT_INT,
     CT_INT64,
-    CT_TIMESPEC,
-    CT_TIME64,
+    CT_TIME,
     CT_GDATE,
     CT_NUMERIC,
     CT_DOUBLE,
@@ -138,6 +136,7 @@ public:
         m_flags{static_cast<ColumnFlags>(f)},
         m_gobj_param_name{gobj_name}, m_qof_param_name{qof_name}, m_getter{get},
         m_setter{set} {}
+    virtual ~GncSqlColumnTableEntry() = default;
 
     /**
      * Load a value into an object from the database row.
@@ -185,6 +184,7 @@ public:
                                                  void* pObject, T get_ref)
         const noexcept
         {
+            static QofLogModule log_module = G_LOG_DOMAIN;
             g_return_if_fail (pObject != NULL);
 
             try
@@ -197,9 +197,19 @@ public:
                     if (target != nullptr)
                         set_parameter (pObject, target, get_setter(obj_name),
                                        m_gobj_param_name);
+                    else
+                        DEBUG("GUID %s returned null %s reference.",
+                             val.c_str(), m_gobj_param_name);
+                }
+                else
+                {
+                    if (val.empty()) DEBUG("Can't load empty guid string for column %s", m_col_name);
+                    else DEBUG("Invalid GUID %s for column %s", val.c_str(), m_col_name);
                 }
             }
-            catch (std::invalid_argument&) {}
+            catch (std::invalid_argument& err) {
+                DEBUG("set_parameter threw %s for column %s", err.what(), m_col_name);
+            }
         }
 
 
@@ -228,12 +238,12 @@ protected:
  */
     void add_objectref_guid_to_table (ColVec& vec) const noexcept;
 private:
-    const char* m_col_name;        /**< Column name */
+    const char* m_col_name = nullptr;        /**< Column name */
     const GncSqlObjectType m_col_type;        /**< Column type */
     unsigned int m_size;       /**< Column size in bytes, for string columns */
     ColumnFlags m_flags;           /**< Column flags */
-    const char* m_gobj_param_name; /**< If non-null, g_object param name */
-    const char* m_qof_param_name;  /**< If non-null, qof parameter name */
+    const char* m_gobj_param_name = nullptr; /**< If non-null, g_object param name */
+    const char* m_qof_param_name = nullptr;  /**< If non-null, qof parameter name */
     QofAccessFunc m_getter;        /**< General access function */
     QofSetterFunc m_setter;        /**< General setter function */
     template <typename T> T get_row_value_from_object(QofIdTypeConst obj_name,
@@ -252,7 +262,7 @@ private:
 };
 
 template <GncSqlObjectType Type>
-class GncSqlColumnTableEntryImpl : public GncSqlColumnTableEntry
+class GncSqlColumnTableEntryImpl final : public GncSqlColumnTableEntry
 {
 public:
     GncSqlColumnTableEntryImpl (const char* name, const GncSqlObjectType type,
@@ -263,6 +273,7 @@ public:
                                 QofSetterFunc set = nullptr) :
         GncSqlColumnTableEntry (name, type, s, f, gobj_name,qof_name, get, set)
         {}
+
     void load(const GncSqlBackend* sql_be, GncSqlRow& row,  QofIdTypeConst obj_name,
               void* pObject) const noexcept override;
     void add_to_table(ColVec& vec) const noexcept override;
@@ -384,6 +395,22 @@ GncSqlColumnTableEntry::add_value_to_vec(QofIdTypeConst obj_name,
     }
 }
 
+template <> inline  void
+GncSqlColumnTableEntry::add_value_to_vec<double*>(QofIdTypeConst obj_name,
+                                         const void* pObject,
+                                         PairVec& vec, std::true_type) const
+{
+    double* s = get_row_value_from_object<double*>(obj_name, pObject);
+
+    if (s != nullptr)
+    {
+        std::ostringstream stream;
+        stream << std::setprecision(12) << std::fixed << *s;
+        vec.emplace_back(std::make_pair(std::string{m_col_name}, stream.str()));
+        return;
+    }
+}
+
 template <typename T> void
 GncSqlColumnTableEntry::add_value_to_vec(QofIdTypeConst obj_name,
                                          const void* pObject,
@@ -397,6 +424,19 @@ GncSqlColumnTableEntry::add_value_to_vec(QofIdTypeConst obj_name,
     return;
 }
 
+template <> inline void
+GncSqlColumnTableEntry::add_value_to_vec<double>(QofIdTypeConst obj_name,
+                                         const void* pObject,
+                                         PairVec& vec, std::false_type) const
+{
+    double s = *get_row_value_from_object<double*>(obj_name, pObject);
+
+    std::ostringstream stream;
+    stream << std::setprecision(12) << std::fixed << s;
+    vec.emplace_back(std::make_pair(std::string{m_col_name}, stream.str()));
+    return;
+}
+
 /**
  * Load an arbitrary object from a result row.
  *
@@ -511,9 +551,19 @@ void set_parameter(T object, P item, QofSetterFunc setter)
 template <typename T, typename P>
 void set_parameter(T object, P item, const char* property)
 {
-    qof_instance_increase_editlevel(object);
+    // Properly use qof_begin_edit and qof_commit_edit{_part2}
+    // here. This is needed to reset the infant state of objects
+    // when loading them initially from sql. Failing to do so
+    // could prevent future editing of these objects
+    // Example of this is https://bugs.gnucash.org/show_bug.cgi?id=795944
+    qof_begin_edit(QOF_INSTANCE(object));
     g_object_set(object, property, item, nullptr);
-    qof_instance_decrease_editlevel(object);
+    if (!qof_commit_edit(QOF_INSTANCE(object))) return;
+    // FIXME I can't use object specific callbacks in generic code
+    // so for now these will silently fail. As the GObject based method
+    // of setting qof objects should go away eventually I won't bother
+    // finding a proper solution for this.
+    qof_commit_edit_part2(QOF_INSTANCE(object), nullptr, nullptr, nullptr);
 };
 
 /**
