@@ -20,13 +20,11 @@
  * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
  * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 ********************************************************************/
-extern "C"
-{
+#include <glib.h>
+
 #include <config.h>
 #include <string.h>
-#include <glib.h>
 #include <unittest-support.h>
-}
 /* Add specific headers for this class */
 #include "../gnc-sql-connection.hpp"
 #include "../gnc-sql-backend.hpp"
@@ -40,7 +38,7 @@ class GncMockSqlBackend : public GncSqlBackend
 public:
     GncMockSqlBackend(GncSqlConnection* conn, QofBook* book) :
         GncSqlBackend(conn, book) {}
-    void session_begin(QofSession*, const char*, bool, bool, bool) override {}
+    void session_begin(QofSession*, const char*, SessionOpenMode) override {}
     void session_end() override {}
     void safe_sync(QofBook* book) override { sync(book); }
 };
@@ -50,8 +48,8 @@ class GncMockSqlConnection;
 class GncMockSqlResult : public GncSqlResult
 {
 public:
-    GncMockSqlResult(const GncMockSqlConnection* conn) :
-        m_conn{conn}, m_iter{this}, m_row{&m_iter} {}
+    GncMockSqlResult() :
+        m_iter{this}, m_row{&m_iter} {}
     uint64_t size() const noexcept { return 1; }
     GncSqlRow& begin() { return m_row; }
     GncSqlRow& end() { return m_row; }
@@ -80,7 +78,6 @@ protected:
             GncMockSqlResult* m_inst;
         };
 private:
-    const GncMockSqlConnection* m_conn;
     IteratorImpl m_iter;
     GncSqlRow m_row;
 };
@@ -96,7 +93,6 @@ public:
 class GncMockSqlConnection : public GncSqlConnection
 {
 public:
-    GncMockSqlConnection() : m_result{this} {}
     GncSqlResultPtr execute_select_statement (const GncSqlStatementPtr&)
         noexcept override { return &m_result; }
     int execute_nonselect_statement (const GncSqlStatementPtr&)
@@ -274,7 +270,7 @@ test_gnc_sql_commit_edit (void)
 {
     QofInstance* inst;
     guint dirty_called = 0;
-    GncMockSqlConnection conn;
+    auto conn{new GncMockSqlConnection};
     const char* msg1 =
         "[GncSqlBackend::commit()] Unknown object type 'null'\n";
     GLogLevelFlags loglevel = static_cast<decltype (loglevel)>
@@ -292,12 +288,12 @@ test_gnc_sql_commit_edit (void)
 
     qof_object_initialize ();
     auto book = qof_book_new();
-    auto sql_be = new GncMockSqlBackend
-        (&conn, book);
+    auto sql_be = new GncMockSqlBackend{conn, book};
     inst  = static_cast<decltype (inst)> (g_object_new (QOF_TYPE_INSTANCE, NULL));
     qof_instance_init_data (inst, QOF_ID_NULL, book);
     qof_book_set_dirty_cb (book, test_dirty_cb, &dirty_called);
     qof_instance_set_dirty_flag (inst, TRUE);
+    gnc_account_create_root (book);
     qof_book_mark_session_dirty (book);
 
     g_assert (qof_instance_get_dirty_flag (inst));
@@ -597,30 +593,6 @@ gnc_sql_add_objectref_guid_col_info_to_list (,// 1
 test_gnc_sql_add_objectref_guid_col_info_to_list (Fixture *fixture, gconstpointer pData)
 {
 }*/
-/* load_timespec
-static void
-load_timespec (const GncSqlBackend* sql_be, GncSqlRow& row,// 2
-*/
-/* static void
-test_load_timespec (Fixture *fixture, gconstpointer pData)
-{
-}*/
-/* add_timespec_col_info_to_list
-static void
-add_timespec_col_info_to_list (const GncSqlBackend* sql_be, const GncSqlColumnTableEntry& table_row,// 2
-*/
-/* static void
-test_add_timespec_col_info_to_list (Fixture *fixture, gconstpointer pData)
-{
-}*/
-/* add_value_timespec_to_vec
-static void
-add_gvalue_timespec_to_slist (const GncSqlBackend* sql_be, QofIdTypeConst obj_name,// 2
-*/
-/* static void
-test_add_value_timespec_to_vec (Fixture *fixture, gconstpointer pData)
-{
-}*/
 /* load_date
 static void
 load_date (const GncSqlBackend* sql_be, GncSqlRow& row,// 2
@@ -915,9 +887,6 @@ test_suite_gnc_backend_sql (void)
 // GNC_TEST_ADD (suitename, "add value guid to vec", Fixture, nullptr, test_add_value_guid_to_vec,  teardown);
 // GNC_TEST_ADD (suitename, "gnc sql add gvalue objectref guid to slist", Fixture, nullptr, test_gnc_sql_add_objectref_guid_to_vec,  teardown);
 // GNC_TEST_ADD (suitename, "gnc sql add objectref guid col info to list", Fixture, nullptr, test_gnc_sql_add_objectref_guid_col_info_to_list,  teardown);
-// GNC_TEST_ADD (suitename, "load timespec", Fixture, nullptr, test_load_timespec,  teardown);
-// GNC_TEST_ADD (suitename, "add timespec col info to list", Fixture, nullptr, test_add_timespec_col_info_to_list,  teardown);
-// GNC_TEST_ADD (suitename, "add value timespec to vec", Fixture, nullptr, test_add_value_timespec_to_vec,  teardown);
 // GNC_TEST_ADD (suitename, "load date", Fixture, nullptr, test_load_date,  teardown);
 // GNC_TEST_ADD (suitename, "add date col info to list", Fixture, nullptr, test_add_date_col_info_to_list,  teardown);
 // GNC_TEST_ADD (suitename, "add value date to vec", Fixture, nullptr, test_add_value_date_to_vec,  teardown);
