@@ -25,8 +25,6 @@
  *                                                                  *
  *******************************************************************/
 #include <guid.hpp>
-extern "C"
-{
 #include <config.h>
 
 #include <stdlib.h>
@@ -44,7 +42,6 @@ extern "C"
 #include <TransLog.h>
 #include <gnc-pricedb.h>
 #include <gnc-pricedb-p.h>
-}
 
 #include "io-gncxml.h"
 #include "sixtp.h"
@@ -764,50 +761,41 @@ kvp_frame_slot_end_handler (gpointer data_for_children,
                             gpointer* result, const gchar* tag)
 {
     KvpFrame* f = (KvpFrame*) parent_data;
-    GSList* lp;
-    guint64 key_node_count;
     gchar* key = NULL;
-    sixtp_child_result* value_cr = NULL;
     KvpValue* value = NULL;
     gboolean delete_value = FALSE;
-
+    sixtp_child_result *cr1 = NULL, *cr2 = NULL, *cr = NULL;
     g_return_val_if_fail (f, FALSE);
 
     if (g_slist_length (data_from_children) != 2) return (FALSE);
-
-    /* check to see that we got exactly one <key> node */
-    lp = data_from_children;
-    key_node_count = 0;
-    for (lp = data_from_children; lp; lp = lp->next)
+    cr1 = (sixtp_child_result*)data_from_children->data;
+    cr2 = (sixtp_child_result*)data_from_children->next->data;
+    
+    if (is_child_result_from_node_named(cr1, "k"))
     {
-        sixtp_child_result* cr = (sixtp_child_result*) lp->data;
-
-        if (is_child_result_from_node_named (cr, "k"))
-        {
-            key = (char*) cr->data;
-            key_node_count++;
-        }
-        else
-        {
-            if (is_child_result_from_node_named (cr, "frame"))
-            {
-                KvpFrame* frame = static_cast<KvpFrame*> (cr->data);
-                value = new KvpValue {frame};
-                delete_value = TRUE;
-            }
-            else
-            {
-                value = static_cast<KvpValue*> (cr->data);
-                delete_value = FALSE;
-            }
-
-            value_cr = cr;
-        }
+        key = (char*)cr1->data;
+        cr = cr2;
     }
-
-    if (key_node_count != 1) return (FALSE);
-
-    value_cr->should_cleanup = TRUE;
+    else if (is_child_result_from_node_named(cr2, "k"))
+    {
+        key = (char*)cr2->data;
+        cr = cr1;
+    }
+    else
+        return FALSE;
+    
+    if (is_child_result_from_node_named (cr, "frame"))
+    {
+        KvpFrame* frame = static_cast<KvpFrame*> (cr->data);
+        value = new KvpValue {frame};
+        delete_value = TRUE;
+    }
+    else
+    {
+        value = static_cast<KvpValue*> (cr->data);
+        delete_value = FALSE;
+    }
+    
     f->set ({key}, value);
     if (delete_value)
         delete value;
@@ -913,7 +901,7 @@ kvp_frame_fail_handler (gpointer data_for_children,
 static void
 kvp_frame_result_cleanup (sixtp_child_result* cr)
 {
-    auto f = static_cast<KvpFrame*> (cr->data);;
+    auto f = static_cast<KvpFrame*> (cr->data);
     if (f) delete f;
 }
 
@@ -2943,7 +2931,7 @@ price_parse_xml_sub_node (GNCPrice* p, xmlNodePtr sub_node, QofBook* book)
         GncGUID* c = dom_tree_to_guid (sub_node);
         if (!c) return FALSE;
         gnc_price_set_guid (p, c);
-        g_free (c);
+        guid_free (c);
     }
     else if (g_strcmp0 ("price:commodity", (char*)sub_node->name) == 0)
     {
@@ -2961,8 +2949,7 @@ price_parse_xml_sub_node (GNCPrice* p, xmlNodePtr sub_node, QofBook* book)
     {
         time64 time = dom_tree_to_time64 (sub_node);
         if (!dom_tree_valid_time64 (time, sub_node->name)) time = 0;
-        Timespec ts = {time, 0};
-        gnc_price_set_time (p, ts);
+        gnc_price_set_time64 (p, time);
     }
     else if (g_strcmp0 ("price:source", (char*)sub_node->name) == 0)
     {
