@@ -20,16 +20,14 @@
  * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
  ********************************************************************/
-extern "C"
-{
+#include <glib.h>
+
 #include <config.h>
 
 #include <string.h>
 
 #include <gnc-engine.h>
-}
 
-#include <glib.h>
 #include "gnc-xml-helper.h"
 #include "sixtp-utils.h"
 #include "sixtp-dom-parsers.h"
@@ -151,16 +149,19 @@ dom_tree_to_boolean (xmlNodePtr node, gboolean* b)
     if (g_ascii_strncasecmp (text, "true", 4) == 0)
     {
         *b = TRUE;
+        g_free (text);
         return TRUE;
     }
     else if (g_ascii_strncasecmp (text, "false", 5) == 0)
     {
         *b = FALSE;
+        g_free (text);
         return TRUE;
     }
     else
     {
         *b = FALSE;
+        g_free (text);
         return FALSE;
     }
 }
@@ -317,10 +318,12 @@ dom_tree_to_list_kvp_value (xmlNodePtr node)
         new_val = dom_tree_to_kvp_value (mark);
         if (new_val)
         {
-            list = g_list_append (list, (gpointer)new_val);
+            list = g_list_prepend (list, (gpointer)new_val);
         }
     }
 
+    list = g_list_reverse (list);
+
     ret = new KvpValue {list};
 
     return ret;
