@@ -23,8 +23,6 @@
 
 #define __EXTENSIONS__
 #include <guid.hpp>
-extern "C"
-{
 #include <config.h>
 
 #include <ctype.h>
@@ -36,13 +34,10 @@ extern "C"
 #include <time.h>
 #include <errno.h>
 
-#ifdef GNUCASH_MAJOR_VERSION
 #ifndef HAVE_STRPTIME
 #include "strptime.h"
 #endif
 #include <gnc-date.h>
-#endif
-}
 
 #include "sixtp.h"
 #include "sixtp-utils.h"
@@ -129,7 +124,7 @@ concatenate_child_result_chars (GSList* data_from_children)
         else
         {
             char* temp;
-            temp = g_strconcat (name, (gchar*) cr->data, NULL);
+            temp = g_strconcat (name, (gchar*) cr->data, nullptr);
             g_free (name);
             name = temp;
         }
@@ -346,7 +341,7 @@ simple_chars_only_parser_new (sixtp_end_handler end_handler)
 }
 
 /****************************************************************************/
-/* generic timespec handler.
+/* generic timespec handler for XML Version 1 files.
 
    A collection of node functions intended to parse a sub-node set
    that looks like this:
@@ -356,34 +351,20 @@ simple_chars_only_parser_new (sixtp_end_handler end_handler)
        <ns>658864000</ns>
      </date-posted>
 
-   and produce a Timespec*.  The start handler for the top allocates
-   the Timespec * and passes it to the children.  The <s> block sets
-   the seconds and the <ns> block (if any) sets the nanoseconds.  If
-   all goes well, returns the Timespec* as the result.
+   and produce a time64.  The start handler for the top allocates
+   the time64 and passes it to the children.  The <s> block sets
+   the seconds and the <ns> block is ignored.  If
+   all goes well, returns the time64 as the result.
 */
 
-gboolean
-string_to_timespec_secs (const gchar* str, Timespec* ts)
-{
-    *ts = gnc_iso8601_to_timespec_gmt (str);
-    return (TRUE);
-}
-
-gboolean
-string_to_timespec_nsecs (const gchar* str, Timespec* ts)
-{
-    /* We don't do nanoseconds anymore. */
-    return (TRUE);
-}
-
 /* Top level timespec node:
 
    input: user end handler *
-   returns: Timespec*
+   returns: time64
 
-   start: Allocates TimespecParseInfo* for data_for_children.
+   start: Allocates Time64ParseInfo* for data_for_children.
    characters: none (whitespace only).
-   end: g_free TimespecParseInfo + any other actions
+   end: g_free Time64ParseInfo + any other actions
 
    cleanup-result: NA
    cleanup-chars: NA
@@ -399,7 +380,7 @@ generic_timespec_start_handler (GSList* sibling_data, gpointer parent_data,
                                 gpointer* data_for_children, gpointer* result,
                                 const gchar* tag, gchar** attrs)
 {
-    TimespecParseInfo* tsp = g_new0 (TimespecParseInfo, 1);
+    Time64ParseInfo* tsp = g_new0 (Time64ParseInfo, 1);
     g_return_val_if_fail (tsp, FALSE);
     *data_for_children = tsp;
     return (TRUE);
@@ -410,18 +391,9 @@ generic_timespec_start_handler (GSList* sibling_data, gpointer parent_data,
    new timespec.  Otherwise, you can presume that everything's been
    cleaned up properly and return FALSE.  */
 gboolean
-timespec_parse_ok (TimespecParseInfo* info)
+timespec_parse_ok (Time64ParseInfo* info)
 {
-
-    if ((info->s_block_count > 1) || (info->ns_block_count > 1) ||
-        ((info->s_block_count == 0) && (info->ns_block_count == 0)))
-    {
-        return (FALSE);
-    }
-    else
-    {
-        return (TRUE);
-    }
+    return info->s_block_count != 1;
 }
 
 /* generic_timespec_end_handler - must be customized and provided by
@@ -429,12 +401,12 @@ timespec_parse_ok (TimespecParseInfo* info)
 
 /* <s> (parent timespec-node)
 
-   input: TimespecParseInfo *
+   input: Time64ParseInfo *
    returns: NA
 
    start: NA
    characters: accumulate.
-   end: convert characters to secs part of input Timespec and inc s_block_count.
+   end: convert characters to secs part of input time64 and inc s_block_count.
 
    cleanup-result: NA
    cleanup-chars: g_free data.
@@ -451,18 +423,18 @@ generic_timespec_secs_end_handler (gpointer data_for_children,
                                    gpointer* result, const gchar* tag)
 {
     gchar* txt = NULL;
-    TimespecParseInfo* info = (TimespecParseInfo*) parent_data;
-    gboolean ok;
+    Time64ParseInfo* info = (Time64ParseInfo*) parent_data;
 
     g_return_val_if_fail (parent_data, FALSE);
 
     txt = concatenate_child_result_chars (data_from_children);
     g_return_val_if_fail (txt, FALSE);
 
-    ok = string_to_timespec_secs (txt, & (info->ts));
+    info->time = gnc_iso8601_to_time64_gmt (txt);
     g_free (txt);
 
-    g_return_val_if_fail (ok, FALSE);
+// gnc_iso8601_to_time64_gmt returns INT64_MAX on failure.
+    g_return_val_if_fail (info->time < INT64_MAX, FALSE);
 
     info->s_block_count++;
     return (TRUE);
@@ -470,15 +442,15 @@ generic_timespec_secs_end_handler (gpointer data_for_children,
 
 /* <s> (parent timespec-node)
 
-   input: TimespecParseInfo *
+   input: Time64ParseInfo *
    returns: NA
 
    start: NA
    characters: accumulate.
-   end: convert characters to secs part of input Timespec and inc s_block_count.
+   end: NA
 
    cleanup-result: NA
-   cleanup-chars: g_free data.
+   cleanup-chars: NA.
    fail: NA
    result-fail: NA
    chars-fail: g_free data.
@@ -491,22 +463,7 @@ generic_timespec_nsecs_end_handler (gpointer data_for_children,
                                     gpointer parent_data, gpointer global_data,
                                     gpointer* result, const gchar* tag)
 {
-    gchar* txt = NULL;
-    TimespecParseInfo* info = (TimespecParseInfo*) parent_data;
-    gboolean ok;
-
-    g_return_val_if_fail (parent_data, FALSE);
-
-    txt = concatenate_child_result_chars (data_from_children);
-    g_return_val_if_fail (txt, FALSE);
-
-    ok = string_to_timespec_nsecs (txt, & (info->ts));
-    g_free (txt);
-
-    g_return_val_if_fail (ok, FALSE);
-
-    info->ns_block_count++;
-    return (TRUE);
+    return TRUE;
 }
 
 static sixtp*
