@@ -1,5 +1,5 @@
 /********************************************************************\
- * gnc-filepath-utils.c -- file path resolutin utilitie             *
+ * gnc-filepath-utils.c -- file path resolution utility             *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -26,7 +26,11 @@
  * @author Copyright (c) 2000 Dave Peticolas
  */
 
-extern "C" {
+#include <glib.h>
+#include <glib/gi18n.h>
+#include <glib/gprintf.h>
+#include <glib/gstdio.h>
+
 #include <config.h>
 
 #include <platform.h>
@@ -35,10 +39,6 @@ extern "C" {
 #include <Shlobj.h>
 #endif
 
-#include <glib.h>
-#include <glib/gi18n.h>
-#include <glib/gprintf.h>
-#include <glib/gstdio.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -62,12 +62,12 @@ extern "C" {
 #ifdef MAC_INTEGRATION
 #include <Foundation/Foundation.h>
 #endif
-}
 
 #include "gnc-locale-utils.hpp"
 #include <boost/filesystem.hpp>
 #include <boost/locale.hpp>
 #include <iostream>
+#include <numeric>
 
 /* Below cvt and bfs_locale should be used with boost::filesystem::path (bfs)
  * objects created alter in this source file. The rationale is as follows:
@@ -462,7 +462,7 @@ copy_recursive(const bfs::path& src, const bfs::path& dest)
     }
     catch(const bfs::filesystem_error& ex)
     {
-        g_warning("An error occured while trying to migrate the user configation from\n%s to\n%s"
+        g_warning("An error occurred while trying to migrate the user configation from\n%s to\n%s"
                   "(Error: %s)",
                   src.string().c_str(), gnc_userdata_home_str.c_str(),
                   ex.what());
@@ -500,7 +500,6 @@ get_user_data_dir ()
 static bfs::path
 get_user_data_dir()
 {
-    char *retval = NULL;
     NSFileManager*fm = [NSFileManager defaultManager];
     NSArray* appSupportDir = [fm URLsForDirectory:NSApplicationSupportDirectory
     inDomains:NSUserDomainMask];
@@ -584,7 +583,6 @@ get_userconfig_home(void)
 
 static std::string migrate_gnc_datahome()
 {
-    auto success = false;
     // Specify location of dictionaries
     bfs::path old_dir(g_get_home_dir(), cvt);
     old_dir /= ".gnucash";
@@ -669,59 +667,59 @@ static std::string migrate_gnc_datahome()
 
     /* Step 3: inform the user of additional changes */
     if (full_copy || !succeeded.empty() || !conf_exist_vec.empty() || !failed.empty())
-        migration_msg << bl::translate ("Notice") << std::endl << std::endl;
+        migration_msg << _("Notice") << std::endl << std::endl;
 
     if (full_copy)
     {
         migration_msg
-        << bl::translate ("Your gnucash metadata has been migrated.") << std::endl << std::endl
+        << _("Your gnucash metadata has been migrated.") << std::endl << std::endl
         /* Translators: this refers to a directory name. */
-        << bl::translate ("Old location:") << " " << old_dir.string() << std::endl
+        << _("Old location:") << " " << old_dir.string() << std::endl
         /* Translators: this refers to a directory name. */
-        << bl::translate ("New location:") << " " << gnc_userdata_home.string() << std::endl << std::endl
+        << _("New location:") << " " << gnc_userdata_home.string() << std::endl << std::endl
         // Translators {1} will be replaced with the package name (typically Gnucash) at runtime
-        << bl::format (bl::translate ("If you no longer intend to run {1} 2.6.x or older on this system you can safely remove the old directory."))
+        << bl::format (std::string{_("If you no longer intend to run {1} 2.6.x or older on this system you can safely remove the old directory.")})
         % PACKAGE_NAME;
     }
 
     if (full_copy &&
         (!succeeded.empty() || !conf_exist_vec.empty() || !failed.empty()))
         migration_msg << std::endl << std::endl
-                        << bl::translate ("In addition:");
+                        << _("In addition:");
 
     if (!succeeded.empty())
     {
         migration_msg << std::endl << std::endl;
         if (full_copy)
-            migration_msg << bl::format (bl::translate ("The following file has been copied to {1} instead:",
+            migration_msg << bl::format (std::string{ngettext("The following file has been copied to {1} instead:",
                                                         "The following files have been copied to {1} instead:",
-                                                        succeeded.size())) % gnc_userconfig_home.string().c_str();
+                                                       succeeded.size())}) % gnc_userconfig_home.string().c_str();
         else
-            migration_msg << bl::format (bl::translate ("The following file in {1} has been renamed:"))
+            migration_msg << bl::format (std::string{_("The following file in {1} has been renamed:")})
                                          % gnc_userconfig_home.string().c_str();
 
         migration_msg << std::endl;
-        for (auto success_file : succeeded)
+        for (const auto& success_file : succeeded)
             migration_msg << "- " << success_file << std::endl;
     }
     if (!conf_exist_vec.empty())
     {
-        migration_msg << std::endl << std::endl
-                      << bl::translate ("The following file has become obsolete and will be ignored:",
+        migration_msg << "\n\n" 
+                      << ngettext("The following file has become obsolete and will be ignored:",
                                         "The following files have become obsolete and will be ignored:",
                                         conf_exist_vec.size())
                       << std::endl;
-        for (auto obs_file : conf_exist_vec)
+        for (const auto& obs_file : conf_exist_vec)
             migration_msg << "- " << obs_file << std::endl;
     }
     if (!failed.empty())
     {
         migration_msg << std::endl << std::endl
-                      << bl::format (bl::translate ("The following file could not be moved to {1}:",
+                      << bl::format (std::string{ngettext("The following file could not be moved to {1}:",
                                                     "The following files could not be moved to {1}:",
-                                                    failed.size())) % gnc_userconfig_home.string().c_str()
+                                                          failed.size())}) % gnc_userconfig_home.string().c_str()
                       << std::endl;
-        for (auto failed_file : failed)
+        for (const auto& failed_file : failed)
             migration_msg << "- " << failed_file << std::endl;
     }
 
@@ -1173,7 +1171,9 @@ gnc_build_data_path (const gchar *filename)
 gchar *
 gnc_build_scm_path (const gchar *filename)
 {
-    gchar *result = g_build_filename(gnc_path_get_scmdir(), filename, (gchar *)NULL);
+    gchar *scmdir = gnc_path_get_scmdir ();
+    gchar *result = g_build_filename (scmdir, filename, (gchar *)NULL);
+    g_free (scmdir);
     return result;
 }
 
@@ -1189,7 +1189,9 @@ gnc_build_scm_path (const gchar *filename)
 gchar *
 gnc_build_report_path (const gchar *filename)
 {
-    gchar *result = g_build_filename(gnc_path_get_reportdir(), filename, (gchar *)NULL);
+    gchar *rptdir = gnc_path_get_reportdir ();
+    gchar *result = g_build_filename (rptdir, filename, (gchar *)NULL);
+    g_free (rptdir);
     return result;
 }
 
@@ -1205,7 +1207,9 @@ gnc_build_report_path (const gchar *filename)
 gchar *
 gnc_build_reports_path (const gchar *dirname)
 {
-    gchar *result = g_build_filename(gnc_path_get_reportsdir(), dirname, (gchar *)NULL);
+    gchar *rptsdir = gnc_path_get_reportsdir ();
+    gchar *result = g_build_filename (rptsdir, dirname, (gchar *)NULL);
+    g_free (rptsdir);
     return result;
 }
 
@@ -1221,7 +1225,9 @@ gnc_build_reports_path (const gchar *dirname)
 gchar *
 gnc_build_stdreports_path (const gchar *filename)
 {
-    gchar *result = g_build_filename(gnc_path_get_stdreportsdir(), filename, (gchar *)NULL);
+    gchar *stdrptdir = gnc_path_get_stdreportsdir ();
+    gchar *result = g_build_filename (stdrptdir, filename, (gchar *)NULL);
+    g_free (stdrptdir);
     return result;
 }
 
@@ -1235,7 +1241,7 @@ gnc_filepath_locate_file (const gchar *default_path, const gchar *name)
     if (g_path_is_absolute (name))
         fullname = g_strdup (name);
     else if (default_path)
-        fullname = g_build_filename (default_path, name, NULL);
+        fullname = g_build_filename (default_path, name, nullptr);
     else
         fullname = gnc_resolve_file_path (name);
 
@@ -1252,7 +1258,10 @@ gnc_filepath_locate_file (const gchar *default_path, const gchar *name)
 gchar *
 gnc_filepath_locate_data_file (const gchar *name)
 {
-    return gnc_filepath_locate_file (gnc_path_get_pkgdatadir(), name);
+    gchar *pkgdatadir = gnc_path_get_pkgdatadir ();
+    gchar *result = gnc_filepath_locate_file (pkgdatadir, name);
+    g_free (pkgdatadir);
+    return result;
 }
 
 gchar *
@@ -1262,7 +1271,7 @@ gnc_filepath_locate_pixmap (const gchar *name)
     gchar *fullname;
     gchar* pkgdatadir = gnc_path_get_pkgdatadir ();
 
-    default_path = g_build_filename (pkgdatadir, "pixmaps", NULL);
+    default_path = g_build_filename (pkgdatadir, "pixmaps", nullptr);
     g_free(pkgdatadir);
     fullname = gnc_filepath_locate_file (default_path, name);
     g_free(default_path);
@@ -1277,7 +1286,7 @@ gnc_filepath_locate_ui_file (const gchar *name)
     gchar *fullname;
     gchar* pkgdatadir = gnc_path_get_pkgdatadir ();
 
-    default_path = g_build_filename (pkgdatadir, "ui", NULL);
+    default_path = g_build_filename (pkgdatadir, "ui", nullptr);
     g_free(pkgdatadir);
     fullname = gnc_filepath_locate_file (default_path, name);
     g_free(default_path);
@@ -1288,8 +1297,33 @@ gnc_filepath_locate_ui_file (const gchar *name)
 gchar *
 gnc_filepath_locate_doc_file (const gchar *name)
 {
-    return gnc_filepath_locate_file (gnc_path_get_pkgdocdir(), name);
+    gchar *docdir = gnc_path_get_pkgdocdir ();
+    gchar *result = gnc_filepath_locate_file (docdir, name);
+    g_free (docdir);
+    return result;
 }
 
+GList *
+gnc_list_all_paths (void)
+{
+    if (gnc_userdata_home.empty())
+        gnc_filepath_init ();
+
+    std::vector<EnvPaths> paths
+        { { "GNC_USERDATA_DIR", gnc_userdata_home_str.c_str(), true},
+          { "GNC_USERCONFIG_DIR", gnc_userconfig_home_str.c_str(), true },
+          { "GNC_BIN", g_getenv ("GNC_BIN"), false },
+          { "GNC_LIB", g_getenv ("GNC_LIB"), false },
+          { "GNC_CONF", g_getenv ("GNC_CONF"), false },
+          { "GNC_DATA", g_getenv ("GNC_DATA"), false },
+        };
+    auto accum = [](const auto& a, const auto& b)
+    {
+        EnvPaths *ep = g_new0 (EnvPaths, 1);
+        *ep = b;
+        return g_list_prepend (a, ep);
+    };
+    return std::accumulate (paths.rbegin(), paths.rend(), (GList*) nullptr, accum);
+}
 
 /* =============================== END OF FILE ========================== */
