@@ -29,6 +29,10 @@
 #ifndef GNC_FILEPATH_UTILS_H
 #define GNC_FILEPATH_UTILS_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /** The gnc_resolve_file_path() routine is a utility that will accept
  *    a fragmentary filename as input, and resolve it into a fully
  *    qualified path in the file system, i.e. a path that begins with
@@ -106,10 +110,13 @@ char * gnc_filepath_init (void);
 
 const gchar *gnc_userdata_dir (void);
 gchar *gnc_build_userdata_path (const gchar *filename);
+gchar *gnc_build_userconfig_path (const gchar *filename);
 gchar *gnc_build_book_path (const gchar *filename);
 gchar *gnc_build_translog_path (const gchar *filename);
 gchar *gnc_build_data_path (const gchar *filename);
+gchar *gnc_build_scm_path (const gchar *filename);
 gchar *gnc_build_report_path (const gchar *filename);
+gchar *gnc_build_reports_path (const gchar *dirname);
 gchar *gnc_build_stdreports_path (const gchar *filename);
 
 const gchar *gnc_userconfig_dir (void);
@@ -169,4 +176,26 @@ gchar *gnc_filepath_locate_ui_file (const gchar *name);
  */
 gchar *gnc_filepath_locate_doc_file (const gchar *name);
 
+typedef struct
+{
+    const gchar *env_name;
+    const gchar *env_path;
+    gboolean modifiable;
+} EnvPaths;
+
+
+/** Returns a GList* of the environment variables used by GnuCash.
+ *
+ *  @return a GList* of EnvPaths structs, describing the environment
+ *  variables used by GnuCash.
+ *
+ *  @note It is the caller's responsibility to free the GList with
+ *  g_list_free_full (paths, g_free)
+ */
+GList *gnc_list_all_paths (void);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* GNC_FILEPATH_UTILS_H */
