@@ -490,7 +490,6 @@ get_trading_split (Transaction *trans, Account *base,
     Account *ns_account;
     Account *account;
     Account* root = gnc_book_get_root_account (xaccTransGetBook (trans));
-    gnc_commodity *root_currency = find_root_currency ();
 
     trading_account = xaccScrubUtilityGetOrMakeAccount (root,
                                                         NULL,
@@ -654,7 +653,7 @@ xaccTransClearTradingSplits (Transaction *trans)
 
     xaccTransBeginEdit (trans);
     /* destroy_splits doesn't actually free the splits but this gets
-     * the list ifself freed.
+     * the list itself freed.
      */
     g_list_free_full (trading_splits, destroy_split);
     xaccTransCommitEdit (trans);
