@@ -26,11 +26,9 @@
 \********************************************************************/
 
 #define __EXTENSIONS__
-extern "C"
-{
+#include <glib.h>
 
 #include <config.h>
-#include <glib.h>
 #include <libintl.h>
 #include <stdlib.h>
 #include "platform.h"
@@ -45,7 +43,6 @@ extern "C"
 #ifdef G_OS_WIN32
 #  include <windows.h>
 #endif
-}
 
 #include <cinttypes>
 #include <unicode/calendar.h>
@@ -734,6 +731,8 @@ qof_scan_date_internal (const char *buff, int *day, int *month, int *year,
     /* today's date */
     gnc_time (&secs);
     now = gnc_localtime (&secs);
+    if (!now)
+        return FALSE;
     now_day = now->tm_mday;
     now_month = now->tm_mon + 1;
     now_year = now->tm_year + 1900;
@@ -1116,7 +1115,6 @@ gnc_date_timestamp (void)
 time64
 gnc_iso8601_to_time64_gmt(const char *cstr)
 {
-    time64 time;
     if (!cstr) return INT64_MAX;
     try
     {
@@ -1141,8 +1139,6 @@ gnc_iso8601_to_time64_gmt(const char *cstr)
 char *
 gnc_time64_to_iso8601_buff (time64 time, char * buff)
 {
-    constexpr size_t max_iso_date_length = 32;
-
     if (! buff) return NULL;
     try
     {
@@ -1234,8 +1230,9 @@ GDate* gnc_g_date_new_today ()
     auto result = g_date_new_dmy (ymd.day, month, ymd.year);
     g_assert(g_date_valid (result));
     return result;
-}void
+}
 
+void
 gnc_gdate_set_today (GDate* gd)
 {
     GDate *today = gnc_g_date_new_today ();
@@ -1269,6 +1266,14 @@ gnc_tm_get_day_start (struct tm *tm, time64 time_val)
     gnc_tm_set_day_start(tm);
 }
 
+void
+gnc_tm_set_day_neutral (struct tm *tm)
+{
+    auto time_val{gnc_dmy2time64_internal(tm->tm_mday, tm->tm_mon + 1,
+                                          tm->tm_year + 1900, DayPart::neutral)};
+    gnc_localtime_r(&time_val, tm);
+}
+
 static void
 gnc_tm_get_day_neutral (struct tm *tm, time64 time_val)
 {
@@ -1302,11 +1307,9 @@ time64
 gnc_time64_get_day_neutral (time64 time_val)
 {
     struct tm tm;
-    time64 new_time;
-
-    gnc_tm_get_day_neutral(&tm, time_val);
-    new_time = gnc_mktime(&tm);
-    return new_time;
+    gnc_localtime_r(&time_val, &tm);
+    return gnc_dmy2time64_internal(tm.tm_mday, tm.tm_mon + 1, tm.tm_year + 1900,
+                                   DayPart::neutral);
 }
 
 time64
@@ -1328,6 +1331,12 @@ gnc_tm_get_today_start (struct tm *tm)
     gnc_tm_get_day_start(tm, time(NULL));
 }
 
+void
+gnc_tm_get_today_neutral (struct tm *tm)
+{
+    gnc_tm_get_day_neutral(tm, time(NULL));
+}
+
 void
 gnc_tm_get_today_end (struct tm *tm)
 {
@@ -1526,31 +1535,27 @@ gnc_gdate_set_quarter_start (GDate *date)
 void
 gnc_gdate_set_quarter_end (GDate *date)
 {
-    gint months;
+     const GDateMonth months[] = {G_DATE_MARCH, G_DATE_JUNE,
+                                  G_DATE_SEPTEMBER, G_DATE_DECEMBER};
+     const GDateDay days[] = {31, 30, 30, 31};
+     int quarter = (g_date_get_month (date) - 1) / 3;
 
-    /* Set the date to the first day of the specified month. */
-    g_date_set_day(date, 1);
-
-    /* Add 1-3 months to get the first day of the next quarter.*/
-    months = (g_date_get_month(date) - G_DATE_JANUARY) % 3;
-    g_date_add_months(date, 3 - months);
-
-    /* Now back up one day */
-    g_date_subtract_days(date, 1);
+     g_date_set_month (date, months[quarter]);
+     g_date_set_day (date, days[quarter]);
 }
 
 void
 gnc_gdate_set_prev_quarter_start (GDate *date)
 {
-    gnc_gdate_set_quarter_start(date);
     g_date_subtract_months(date, 3);
+    gnc_gdate_set_quarter_start(date);
 }
 
 void
 gnc_gdate_set_prev_quarter_end (GDate *date)
 {
-    gnc_gdate_set_quarter_end(date);
     g_date_subtract_months(date, 3);
+    gnc_gdate_set_quarter_end(date);
 }
 
 /* ================================================= */
