@@ -22,24 +22,20 @@
  *                                                                  *
  *******************************************************************/
 
-extern "C"
-{
-#include <config.h>
-
-#include <math.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include "qof.h"
-}
-
 #include <glib.h>
-#include <stdint.h>
+
+#include <cmath>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
+#include <cstdint>
+#include <sstream>
 #include <boost/regex.hpp>
 #include <boost/locale/encoding_utf.hpp>
-#include <sstream>
-#include <cstdlib>
+
+#include <config.h>
+#include <stdint.h>
+#include "qof.h"
 
 #include "gnc-numeric.hpp"
 #include "gnc-rational.hpp"
@@ -125,8 +121,8 @@ GncNumeric::GncNumeric(const std::string& str, bool autoround)
     static const std::string hex_frag("(0x[a-f0-9]+)");
     static const std::string slash( "[ \\t]*/[ \\t]*");
     /* The llvm standard C++ library refused to recognize the - in the
-     * numer_frag patter with the default ECMAScript syntax so we use the awk
-     * syntax.
+     * numer_frag pattern with the default ECMAScript syntax so we use the
+     * awk syntax.
      */
     static const regex numeral(numer_frag);
     static const regex hex(hex_frag);
@@ -660,8 +656,6 @@ gnc_numeric_positive_p(gnc_numeric a)
 int
 gnc_numeric_compare(gnc_numeric a, gnc_numeric b)
 {
-    gint64 aa, bb;
-
     if (gnc_numeric_check(a) || gnc_numeric_check(b))
     {
         return 0;
@@ -758,9 +752,9 @@ gnc_numeric_add(gnc_numeric a, gnc_numeric b,
     {
         return gnc_numeric_error(GNC_ERROR_ARG);
     }
-    denom = denom_lcd(a, b, denom, how);
     try
     {
+        denom = denom_lcd(a, b, denom, how);
         if ((how & GNC_NUMERIC_DENOM_MASK) != GNC_HOW_DENOM_EXACT)
         {
             GncNumeric an (a), bn (b);
@@ -807,14 +801,13 @@ gnc_numeric
 gnc_numeric_sub(gnc_numeric a, gnc_numeric b,
                 gint64 denom, gint how)
 {
-    gnc_numeric nb;
     if (gnc_numeric_check(a) || gnc_numeric_check(b))
     {
         return gnc_numeric_error(GNC_ERROR_ARG);
     }
-    denom = denom_lcd(a, b, denom, how);
     try
     {
+        denom = denom_lcd(a, b, denom, how);
         if ((how & GNC_NUMERIC_DENOM_MASK) != GNC_HOW_DENOM_EXACT)
         {
             GncNumeric an (a), bn (b);
@@ -865,9 +858,10 @@ gnc_numeric_mul(gnc_numeric a, gnc_numeric b,
     {
         return gnc_numeric_error(GNC_ERROR_ARG);
     }
-    denom = denom_lcd(a, b, denom, how);
+
     try
     {
+        denom = denom_lcd(a, b, denom, how);
         if ((how & GNC_NUMERIC_DENOM_MASK) != GNC_HOW_DENOM_EXACT)
         {
             GncNumeric an (a), bn (b);
@@ -919,9 +913,9 @@ gnc_numeric_div(gnc_numeric a, gnc_numeric b,
     {
         return gnc_numeric_error(GNC_ERROR_ARG);
     }
-    denom = denom_lcd(a, b, denom, how);
     try
     {
+        denom = denom_lcd(a, b, denom, how);
         if ((how & GNC_NUMERIC_DENOM_MASK) != GNC_HOW_DENOM_EXACT)
         {
             GncNumeric an (a), bn (b);
@@ -1091,7 +1085,7 @@ gnc_numeric_to_decimal(gnc_numeric *a, guint8 *max_decimal_places)
     }
     catch (const std::exception& err)
     {
-        PWARN("%s", err.what());
+        PINFO ("%s", err.what());
         return FALSE;
     }
 }
@@ -1201,11 +1195,11 @@ gnc_numeric_error(GNCNumericErrorCode error_code)
 gchar *
 gnc_numeric_to_string(gnc_numeric n)
 {
-    gchar *result;
-    gint64 tmpnum = n.num;
-    gint64 tmpdenom = n.denom;
+    char *result;
+    int64_t tmpnum = n.num;
+    int64_t tmpdenom = n.denom;
 
-    result = g_strdup_printf("%" G_GINT64_FORMAT "/%" G_GINT64_FORMAT, tmpnum, tmpdenom);
+    result = g_strdup_printf("%" PRId64 "/%" PRId64, tmpnum, tmpdenom);
 
     return result;
 }
@@ -1215,13 +1209,14 @@ gnc_num_dbg_to_string(gnc_numeric n)
 {
     static char buff[1000];
     static char *p = buff;
-    gint64 tmpnum = n.num;
-    gint64 tmpdenom = n.denom;
+    static const uint64_t size = 50;
+    int64_t tmpnum = n.num;
+    int64_t tmpdenom = n.denom;
 
-    p += 100;
+    p += size;
     if (p - buff >= 1000) p = buff;
 
-    sprintf(p, "%" G_GINT64_FORMAT "/%" G_GINT64_FORMAT, tmpnum, tmpdenom);
+    snprintf(p, size, "%" PRId64 "/%" PRId64, tmpnum, tmpdenom);
 
     return p;
 }
