@@ -37,9 +37,7 @@
 #include "gncAddressP.h"
 #include "gncBillTermP.h"
 #include "gncInvoice.h"
-#ifdef GNUCASH_MAJOR_VERSION
 #include "gncBusiness.h"
-#endif
 
 #include "gncCustomer.h"
 #include "gncCustomerP.h"
@@ -55,9 +53,9 @@ struct _gncCustomer
     QofInstance     inst;
 
     /* The following fields are identical to 'vendor' */
-    char *          id;
-    char *          name;
-    char *          notes;
+    const char *    id;
+    const char *    name;
+    const char *    notes;
     GncBillTerm *   terms;
     GncAddress *    addr;
     gnc_commodity * currency;
@@ -86,7 +84,7 @@ static QofLogModule log_module = GNC_MOD_BUSINESS;
 /* ============================================================== */
 /* misc inline funcs */
 
-G_INLINE_FUNC void mark_customer (GncCustomer *customer);
+static inline void mark_customer (GncCustomer *customer);
 void mark_customer (GncCustomer *customer)
 {
     qof_instance_set_dirty(&customer->inst);
@@ -105,7 +103,7 @@ enum
 };
 
 /* GObject Initialization */
-G_DEFINE_TYPE(GncCustomer, gnc_customer, QOF_TYPE_INSTANCE);
+G_DEFINE_TYPE(GncCustomer, gnc_customer, QOF_TYPE_INSTANCE)
 
 static void
 gnc_customer_init(GncCustomer* cust)
@@ -131,7 +129,6 @@ gnc_customer_get_property (GObject         *object,
                            GParamSpec      *pspec)
 {
     GncCustomer *cust;
-    gchar *key;
     g_return_if_fail(GNC_IS_CUSTOMER(object));
 
     cust = GNC_CUSTOMER(object);
@@ -162,7 +159,6 @@ gnc_customer_set_property (GObject         *object,
                            GParamSpec      *pspec)
 {
     GncCustomer *cust;
-    gchar *key;
 
     g_return_if_fail(GNC_IS_CUSTOMER(object));
 
@@ -355,14 +351,17 @@ static void gncCustomerFree (GncCustomer *cust)
     gncAddressDestroy (cust->addr);
     gncAddressBeginEdit (cust->shipaddr);
     gncAddressDestroy (cust->shipaddr);
+
+    gncJobFreeList (cust->jobs);
     g_list_free (cust->jobs);
     g_free (cust->balance);
 
-    if (cust->terms)
-        gncBillTermDecRef (cust->terms);
-    if (cust->taxtable)
+    if (!qof_book_shutting_down (qof_instance_get_book (QOF_INSTANCE(cust))))
     {
-        gncTaxTableDecRef (cust->taxtable);
+        if (cust->terms)
+            gncBillTermDecRef (cust->terms);
+        if (cust->taxtable)
+            gncTaxTableDecRef (cust->taxtable);
     }
 
     /* qof_instance_release (&cust->inst); */
@@ -373,13 +372,9 @@ static void gncCustomerFree (GncCustomer *cust)
 /* Set Functions */
 
 #define SET_STR(obj, member, str) { \
-        char * tmp; \
-        \
         if (!g_strcmp0 (member, str)) return; \
         gncCustomerBeginEdit (obj); \
-        tmp = CACHE_INSERT (str); \
-        CACHE_REMOVE (member); \
-        member = tmp; \
+        CACHE_REPLACE(member, str); \
         }
 
 void gncCustomerSetID (GncCustomer *cust, const char *id)
@@ -707,9 +702,9 @@ GList * gncCustomerGetJoblist (const GncCustomer *cust, gboolean show_all)
         {
             GncJob *j = iterator->data;
             if (gncJobGetActive (j))
-                list = g_list_append (list, j);
+                list = g_list_prepend (list, j);
         }
-        return list;
+        return g_list_reverse (list);
     }
 }
 
@@ -987,10 +982,13 @@ gncCustomerGetCachedBalance (GncCustomer *cust)
 
 void gncCustomerSetCachedBalance (GncCustomer *cust, const gnc_numeric *new_bal)
 {
-    if (!new_bal && cust->balance)
+    if (!new_bal)
     {
-        g_free (cust->balance);
-        cust->balance = NULL;
+        if (cust->balance)
+        {
+            g_free (cust->balance);
+            cust->balance = NULL;
+        }
         return;
     }
 
