@@ -30,11 +30,8 @@
  */
 
 #include "guid.hpp"
-extern "C"
-{
 #include <config.h>
 #include <glib.h>
-}
 
 #include <utility>
 #include "qof.h"
@@ -112,9 +109,9 @@ typedef struct QofInstancePrivate
 }  QofInstancePrivate;
 
 #define GET_PRIVATE(o)  \
-    ((QofInstancePrivate*)g_type_instance_get_private((GTypeInstance*)o, QOF_TYPE_INSTANCE))
+    ((QofInstancePrivate*)qof_instance_get_instance_private((QofInstance*)o))
 
-G_DEFINE_TYPE_WITH_PRIVATE(QofInstance, qof_instance, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE(QofInstance, qof_instance, G_TYPE_OBJECT)
 QOF_GOBJECT_FINALIZE(qof_instance);
 #undef G_PARAM_READWRITE
 #define G_PARAM_READWRITE static_cast<GParamFlags>(G_PARAM_READABLE | G_PARAM_WRITABLE)
@@ -176,7 +173,7 @@ static void qof_instance_class_init(QofInstanceClass *klass)
                            "Object Last Update",
                            "A pointer to the last time this object was "
                            "updated.  This value is present for use by "
-                           "backends and shouldnot be written by other "
+                           "backends and shouldn't be written by other "
                            "code.",
                            G_PARAM_READWRITE));
 
@@ -319,9 +316,8 @@ qof_instance_dispose (GObject *instp)
     QofInstance* inst = QOF_INSTANCE(instp);
 
     priv = GET_PRIVATE(instp);
-    if (!priv->collection)
-        return;
-    qof_collection_remove_entity(inst);
+    if (priv->collection)
+        qof_collection_remove_entity(inst);
 
     CACHE_REMOVE(inst->e_type);
     inst->e_type = NULL;
@@ -694,7 +690,6 @@ gboolean
 qof_instance_get_dirty (QofInstance *inst)
 {
     QofInstancePrivate *priv;
-    QofCollection *coll;
 
     if (!inst)
     {
@@ -709,7 +704,6 @@ void
 qof_instance_set_dirty(QofInstance* inst)
 {
     QofInstancePrivate *priv;
-    QofCollection *coll;
 
     priv = GET_PRIVATE(inst);
     priv->dirty = TRUE;
@@ -1030,10 +1024,9 @@ qof_commit_edit_part2(QofInstance *inst,
                 on_error(inst, errcode);
             return FALSE;
         }
-        /* XXX the backend commit code should clear dirty!! */
-        priv->dirty = FALSE;
+        if (!priv->dirty) //Cleared if the save was successful
+            priv->infant = FALSE;
     }
-    priv->infant = FALSE;
 
     if (priv->do_free)
     {
@@ -1072,15 +1065,7 @@ qof_instance_set_kvp (QofInstance * inst, GValue const * value, unsigned count,
 
 void qof_instance_get_path_kvp (QofInstance * inst, GValue * value, std::vector<std::string> const & path)
 {
-    auto temp = gvalue_from_kvp_value (inst->kvp_data->get_slot (path));
-    if (G_IS_VALUE (temp))
-    {
-        if (G_IS_VALUE (value))
-            g_value_unset (value);
-        g_value_init (value, G_VALUE_TYPE (temp));
-        g_value_copy (temp, value);
-        gnc_gvalue_free (temp);
-    }
+    gvalue_from_kvp_value (inst->kvp_data->get_slot (path), value);
 }
 
 void
@@ -1092,15 +1077,7 @@ qof_instance_get_kvp (QofInstance * inst, GValue * value, unsigned count, ...)
     for (unsigned i{0}; i < count; ++i)
         path.push_back (va_arg (args, char const *));
     va_end (args);
-    auto temp = gvalue_from_kvp_value (inst->kvp_data->get_slot (path));
-    if (G_IS_VALUE (temp))
-    {
-        if (G_IS_VALUE (value))
-            g_value_unset (value);
-        g_value_init (value, G_VALUE_TYPE (temp));
-        g_value_copy (temp, value);
-        gnc_gvalue_free (temp);
-    }
+    gvalue_from_kvp_value (inst->kvp_data->get_slot (path), value);
 }
 
 void
