@@ -31,9 +31,7 @@
  @author Copyright (c) 2005 Neil Williams <linux@codehelp.co.uk>
  @author Copyright (c) 2016 Aaron Laws
    */
-
-extern "C"
-{
+#include <glib.h>
 
 #include <config.h>
 
@@ -52,9 +50,7 @@ extern "C"
 #include "qofobject-p.h"
 
 static QofLogModule log_module = QOF_MOD_SESSION;
-} //extern 'C'
 
-#include <glib.h>
 #include "qofbook-p.h"
 #include "qof-backend.hpp"
 #include "qofsession.hpp"
