@@ -14,18 +14,13 @@
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
  * along with this program; if not, you can retrieve it from        *
- * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html            *
+ * https://www.gnu.org/licenses/old-licenses/gpl-2.0.html            *
  * or contact:                                                      *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
  * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
  * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  ********************************************************************/
-#ifdef __cplusplus
-extern "C"
-{
-#endif
-
 #include <config.h>
 #include "platform.h"
 #include <string.h>
@@ -35,9 +30,6 @@ extern "C"
 #include <unittest-support.h>
 /* Add specific headers for this class */
 
-#ifdef __cplusplus
-}
-#endif
 #include "../gnc-date.h"
 #include "../gnc-date-p.h"
 #include <locale.h>
@@ -48,7 +40,6 @@ extern "C"
 #endif
 
 static const gchar *suitename = "/qof/gnc-date";
-static const time64 secs_per_year = INT64_C(3600) * (INT64_C(24) * INT64_C(365) + 6);
 static const time64 max_secs = (INT64_C(3600) * (INT64_C(24) * INT64_C(365) + 6)) * (INT64_C(9999) - INT64_C(1970));
 
 typedef struct
@@ -70,18 +61,6 @@ typedef struct
     time64 t5;
 } FixtureA;
 
-static int
-offset_secs (TZOffset tz)
-{
-    return 3600 * tz.hours + 60 * tz.minutes;
-}
-
-static char*
-offset_string (TZOffset tz)
-{
-    return g_strdup_printf("%+02d%02d", tz.hours, tz.minutes);
-}
-
 static void setup (FixtureA *f, gconstpointer pData)
 {
     f->t0 = gnc_time(NULL);
@@ -486,10 +465,10 @@ test_gnc_setlocale (int category, gchar *locale)
         if (localeval != NULL)
             return;
     }
-    g_fprintf (stderr, "There are some differences between distros in the way they name"
-              "locales, and this can cause trouble with the locale-based"
-              "formatting. If you get the assert in this function, run locale -a"
-              "and make sure that en_US, en_GB, and fr_FR are installed and that"
+    g_fprintf (stderr, "There are some differences between distros in the way they name "
+              "locales, and this can cause trouble with the locale-based "
+              "formatting. If you get the assert in this function, run locale -a "
+              "and make sure that en_US, en_GB, and fr_FR are installed and that "
               "if a suffix is needed it's in the suffixes array.");
     g_assert_not_reached ();
 }
@@ -642,7 +621,7 @@ static void tm_set_dmy (struct tm *tm, gint year, gint month, gint mday)
 static void
 test_qof_print_date_dmy_buff (void)
 {
-    gchar buff[MAX_DATE_LENGTH], t_buff[MAX_DATE_LENGTH];
+    gchar buff[MAX_DATE_LENGTH + 1], t_buff[MAX_DATE_LENGTH + 1];
     gchar *locale = g_strdup (setlocale (LC_TIME, NULL));
     struct tm tm = { 0, 0, 0, 0, 0, 0, 0, 0, 0
 #ifdef HAVE_STRUCT_TM_GMTOFF
@@ -652,47 +631,47 @@ test_qof_print_date_dmy_buff (void)
 
     qof_date_format_set (QOF_DATE_FORMAT_UK);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), 23, 11, 1974), ==, strlen (buff));
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, 23, 11, 1974), ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "23/11/1974");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), 2, 2, 1961), ==, strlen (buff));
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, 2, 2, 1961), ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "02/02/1961");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), 16, 6, 2045), ==, strlen (buff));
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, 16, 6, 2045), ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "16/06/2045");
 
     qof_date_format_set (QOF_DATE_FORMAT_CE);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), 23, 11, 1974), ==, strlen (buff));
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, 23, 11, 1974), ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "23.11.1974");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), 2, 2, 1961), ==, strlen (buff));
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, 2, 2, 1961), ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "02.02.1961");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), 16, 6, 2045), ==, strlen (buff));
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, 16, 6, 2045), ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "16.06.2045");
 
     qof_date_format_set (QOF_DATE_FORMAT_US);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), 23, 11, 1974), ==, strlen (buff));
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, 23, 11, 1974), ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "11/23/1974");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), 2, 2, 1961), ==, strlen (buff));
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, 2, 2, 1961), ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "02/02/1961");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), 16, 6, 2045), ==, strlen (buff));
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, 16, 6, 2045), ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "06/16/2045");
 
     qof_date_format_set (QOF_DATE_FORMAT_ISO);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), 23, 11, 1974), ==, strlen (buff));
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, 23, 11, 1974), ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "1974-11-23");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), 2, 2, 1961),
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, 2, 2, 1961),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "1961-02-02");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), 16, 6, 2045),
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, 16, 6, 2045),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "2045-06-16");
 
@@ -701,7 +680,7 @@ test_qof_print_date_dmy_buff (void)
     tm_set_dmy (&tm, 1974, 11, 23);
     strftime(t_buff, MAX_DATE_LENGTH, GNC_D_FMT, &tm);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), tm.tm_mday,
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, tm.tm_mday,
                      tm.tm_mon + 1, tm.tm_year + 1900),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, t_buff);
@@ -710,7 +689,7 @@ test_qof_print_date_dmy_buff (void)
     tm_set_dmy (&tm, 1961, 2, 2);
     strftime(t_buff, MAX_DATE_LENGTH, GNC_D_FMT, &tm);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), tm.tm_mday,
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, tm.tm_mday,
                      tm.tm_mon + 1, tm.tm_year + 1900),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, t_buff);
@@ -718,7 +697,7 @@ test_qof_print_date_dmy_buff (void)
     tm_set_dmy (&tm, 2045, 6, 16);
     strftime(t_buff, MAX_DATE_LENGTH, GNC_D_FMT, &tm);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), tm.tm_mday,
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, tm.tm_mday,
                      tm.tm_mon + 1, tm.tm_year + 1900),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, t_buff);
@@ -727,22 +706,22 @@ test_qof_print_date_dmy_buff (void)
     tm_set_dmy (&tm, 1974, 11, 23);
     strftime(t_buff, MAX_DATE_LENGTH, GNC_D_FMT, &tm);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), tm.tm_mday,
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, tm.tm_mday,
                      tm.tm_mon + 1, tm.tm_year + 1900),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, t_buff);
     tm_set_dmy (&tm, 1961, 2, 2);
     strftime(t_buff, MAX_DATE_LENGTH, GNC_D_FMT, &tm);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), tm.tm_mday,
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, tm.tm_mday,
                      tm.tm_mon + 1, tm.tm_year + 1900),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, t_buff);
     memset ((gpointer)buff, 0, sizeof (buff));
     tm_set_dmy (&tm, 2045, 6, 16);
     strftime(t_buff, MAX_DATE_LENGTH, GNC_D_FMT, &tm);
-    memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), tm.tm_mday,
+    memset ((gpointer)buff, 0, MAX_DATE_LENGTH);
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, tm.tm_mday,
                      tm.tm_mon + 1, tm.tm_year + 1900),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, t_buff);
@@ -751,14 +730,14 @@ test_qof_print_date_dmy_buff (void)
     tm_set_dmy (&tm, 1974, 11, 23);
     strftime(t_buff, MAX_DATE_LENGTH, GNC_D_FMT, &tm);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), tm.tm_mday,
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, tm.tm_mday,
                      tm.tm_mon + 1, tm.tm_year + 1900),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, t_buff);
     tm_set_dmy (&tm, 1961, 2, 2);
     strftime(t_buff, MAX_DATE_LENGTH, GNC_D_FMT, &tm);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), tm.tm_mday,
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, tm.tm_mday,
                      tm.tm_mon + 1, tm.tm_year + 1900),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, t_buff);
@@ -766,7 +745,7 @@ test_qof_print_date_dmy_buff (void)
     tm_set_dmy (&tm, 2045, 6, 16);
     strftime(t_buff, MAX_DATE_LENGTH, GNC_D_FMT, &tm);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_dmy_buff (buff, sizeof (buff), tm.tm_mday,
+    g_assert_cmpint (qof_print_date_dmy_buff (buff, MAX_DATE_LENGTH, tm.tm_mday,
                      tm.tm_mon + 1, tm.tm_year + 1900),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, t_buff);
@@ -787,7 +766,7 @@ test_qof_print_date_dmy_buff (void)
 
 #define test_assert_localized_timestring(time, datestr)                 \
     {                                                                   \
-        gchar t_buff[MAX_DATE_LENGTH];                                  \
+        gchar t_buff[MAX_DATE_LENGTH + 1];                                  \
         struct tm *ltime = gnc_localtime ((time64 *)(&time));           \
         strftime (t_buff, sizeof (t_buff), GNC_D_FMT, ltime);           \
         gnc_tm_free (ltime);                                            \
@@ -802,7 +781,7 @@ qof_print_date_buff (char * buff, size_t len, time64 t)// C: 3 in 1  Local: 2:0:
 static void
 test_qof_print_date_buff (void)
 {
-    gchar buff[MAX_DATE_LENGTH], ans[MAX_DATE_LENGTH];
+    gchar buff[MAX_DATE_LENGTH + 1], ans[MAX_DATE_LENGTH + 1];
     gchar *locale = g_strdup (setlocale (LC_TIME, NULL));
 
     struct tm tm1 = {0, 0, 12, 23, 10, 74};
@@ -814,110 +793,110 @@ test_qof_print_date_buff (void)
 
     qof_date_format_set (QOF_DATE_FORMAT_UK);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time1),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time1),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "23/11/1974");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time2),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time2),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "02/02/1961");
 
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time3),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time3),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "16/06/2045");
 
     qof_date_format_set (QOF_DATE_FORMAT_CE);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time1),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time1),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "23.11.1974");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time2),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time2),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "02.02.1961");
 
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time3),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time3),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "16.06.2045");
 
     qof_date_format_set (QOF_DATE_FORMAT_US);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time1),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time1),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "11/23/1974");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time2),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time2),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "02/02/1961");
 
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time3),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time3),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "06/16/2045");
 
     qof_date_format_set (QOF_DATE_FORMAT_ISO);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time1),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time1),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "1974-11-23");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time2),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time2),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "1961-02-02");
 
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time3),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time3),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "2045-06-16");
 
     qof_date_format_set (QOF_DATE_FORMAT_LOCALE);
     test_gnc_setlocale (LC_TIME, "en_US");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time1),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time1),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "11/23/1974");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time2),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time2),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "02/02/1961");
 
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time3),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time3),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "06/16/2045");
 
     test_gnc_setlocale (LC_TIME, "en_GB");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time1),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time1),
                      ==, strlen (buff));
     strftime(ans, MAX_DATE_LENGTH, GNC_D_FMT, &tm1);
     g_assert_cmpstr (buff, ==, ans);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time2),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time2),
                      ==, strlen (buff));
     strftime(ans, MAX_DATE_LENGTH, GNC_D_FMT, &tm2);
     g_assert_cmpstr (buff, ==, ans);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time3),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time3),
                      ==, strlen (buff));
     strftime(ans, MAX_DATE_LENGTH, GNC_D_FMT, &tm3);
     g_assert_cmpstr (buff, ==, ans);
 
     test_gnc_setlocale (LC_TIME, "fr_FR");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time1),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time1),
                      ==, strlen (buff));
     strftime(ans, MAX_DATE_LENGTH, GNC_D_FMT, &tm1);
     g_assert_cmpstr (buff, ==, ans);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time2),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time2),
                      ==, strlen (buff));
     strftime(ans, MAX_DATE_LENGTH, GNC_D_FMT, &tm2);
     g_assert_cmpstr (buff, ==, ans);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_date_buff (buff, sizeof (buff), time3),
+    g_assert_cmpint (qof_print_date_buff (buff, MAX_DATE_LENGTH, time3),
                      ==, strlen (buff));
     strftime(ans, MAX_DATE_LENGTH, GNC_D_FMT, &tm3);
     g_assert_cmpstr (buff, ==, ans);
@@ -932,7 +911,7 @@ qof_print_gdate( char *buf, size_t len, const GDate *gd )// C: 6 in 5  Local: 0:
 static void
 test_qof_print_gdate (void)
 {
-    gchar buff[MAX_DATE_LENGTH], t_buff[MAX_DATE_LENGTH];
+    gchar buff[MAX_DATE_LENGTH + 1], t_buff[MAX_DATE_LENGTH + 1];
     gchar *locale = g_strdup (setlocale (LC_TIME, NULL));
     GDate *gd1 = g_date_new_dmy (23, 11, 1974);
     GDate *gd2 = g_date_new_dmy (2, 2, 1961);
@@ -940,59 +919,59 @@ test_qof_print_gdate (void)
 
     qof_date_format_set (QOF_DATE_FORMAT_UK);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd1),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd1),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "23/11/1974");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd2),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd2),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "02/02/1961");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd3),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd3),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "16/06/2045");
 
     qof_date_format_set (QOF_DATE_FORMAT_CE);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd1),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd1),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "23.11.1974");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd2),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd2),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "02.02.1961");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd3),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd3),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "16.06.2045");
 
 
     qof_date_format_set (QOF_DATE_FORMAT_US);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd1),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd1),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "11/23/1974");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd2),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd2),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "02/02/1961");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd3),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd3),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "06/16/2045");
 
 
     qof_date_format_set (QOF_DATE_FORMAT_ISO);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd1),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd1),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "1974-11-23");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd2),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd2),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "1961-02-02");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd3),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd3),
                      ==, strlen (buff));
     g_assert_cmpstr (buff, ==, "2045-06-16");
 
@@ -1000,34 +979,34 @@ test_qof_print_gdate (void)
     qof_date_format_set (QOF_DATE_FORMAT_LOCALE);
     test_gnc_setlocale (LC_TIME, "en_US");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd1),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd1),
                      ==, strlen (buff));
     g_date_strftime (t_buff, MAX_DATE_LENGTH, GNC_D_FMT, gd1);
     g_assert_cmpstr (buff, ==, t_buff);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd2),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd2),
                      ==, strlen (buff));
     g_date_strftime (t_buff, MAX_DATE_LENGTH, GNC_D_FMT, gd2);
     g_assert_cmpstr (buff, ==, t_buff);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd3),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd3),
                      ==, strlen (buff));
     g_date_strftime (t_buff, MAX_DATE_LENGTH, GNC_D_FMT, gd3);
     g_assert_cmpstr (buff, ==, t_buff);
 
     test_gnc_setlocale (LC_TIME, "en_GB");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd1),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd1),
                      ==, strlen (buff));
     g_date_strftime (t_buff, MAX_DATE_LENGTH, GNC_D_FMT, gd1);
     g_assert_cmpstr (buff, ==, t_buff);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd2),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd2),
                      ==, strlen (buff));
     g_date_strftime (t_buff, MAX_DATE_LENGTH, GNC_D_FMT, gd2);
     g_assert_cmpstr (buff, ==, t_buff);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd3),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd3),
                      ==, strlen (buff));
     g_date_strftime (t_buff, MAX_DATE_LENGTH, GNC_D_FMT, gd3);
     g_assert_cmpstr (buff, ==, t_buff);
@@ -1035,17 +1014,17 @@ test_qof_print_gdate (void)
 
     test_gnc_setlocale (LC_TIME, "fr_FR");
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd1),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd1),
                      ==, strlen (buff));
     g_date_strftime (t_buff, MAX_DATE_LENGTH, GNC_D_FMT, gd1);
     g_assert_cmpstr (buff, ==, t_buff);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd2),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd2),
                      ==, strlen (buff));
     g_date_strftime (t_buff, MAX_DATE_LENGTH, GNC_D_FMT, gd2);
     g_assert_cmpstr (buff, ==, t_buff);
     memset ((gpointer)buff, 0, sizeof (buff));
-    g_assert_cmpint (qof_print_gdate (buff, sizeof (buff), gd3),
+    g_assert_cmpint (qof_print_gdate (buff, MAX_DATE_LENGTH, gd3),
                      ==, strlen (buff));
     g_date_strftime (t_buff, MAX_DATE_LENGTH, GNC_D_FMT, gd3);
     g_assert_cmpstr (buff, ==, t_buff);
@@ -1357,15 +1336,6 @@ test_gnc_date_timestamp (void)
 
     g_free (timestr);
 }
-/* gnc_iso8601_to_time64_gmt
-time64
-gnc_iso8601_to_time64_gmt(const char *str)// C: 6 in 3  Local: 0:0:0
-*/
-static gint
-get_nanoseconds (GDateTime *gdt)
-{
-    return g_date_time_get_microsecond (gdt) * 1000;
-}
 
 static void
 test_gnc_iso8601_to_time64_gmt (FixtureA *f, gconstpointer pData)
@@ -1391,18 +1361,6 @@ test_gnc_iso8601_to_time64_gmt (FixtureA *f, gconstpointer pData)
     t = gnc_iso8601_to_time64_gmt ("2061-01-25 23:21:19.0 -05:00");
     g_assert_cmpint (t, ==, f->t5);
 }
-/* gnc_time64_to_iso8601_buff
-char *
-gnc_time64_to_iso8601_buff (time64 t, char * buff)// C: 18 in 7  Local: 0:0:0
-*/
-static time64
-g_date_time_to_time64 (GDateTime *gdt)
-{
-    time64 t;
-    t = g_date_time_to_unix (gdt);
-    return t;
-}
-
 #define ISO8601_SIZE MAX_DATE_LENGTH + 4
 static gchar*
 format_timestring (time64 t, TZOffset tz)
@@ -1545,22 +1503,6 @@ test_gnc_dmy2time64_end (FixtureB *f, gconstpointer pData)
     g_log_set_default_handler (hdlr, 0);
 }
 
-static GDateTime*
-offset_adjust(GDateTime *gdt)
-{
-     Testfuncs *tf = gnc_date_load_funcs();
-     GTimeZone *zone = tf->timezone_new_local();
-     int interval = g_time_zone_find_interval(zone, G_TIME_TYPE_STANDARD,
-					      g_date_time_to_unix(gdt));
-     int offset = g_time_zone_get_offset(zone, interval) / 60;
-     int off_hr = (offset / 60) + (offset % 60 ? (offset < 0 ? -1 : 1) : 0);
-     int correction = off_hr < -10 ? -10 - off_hr : off_hr > 13 ? 13 - off_hr : 0;
-     GDateTime* new_gdt = g_date_time_add_hours(gdt, correction);
-     g_date_time_unref(gdt);
-     g_slice_free(Testfuncs, tf);
-     return new_gdt;
-}
-
 /*gnc_dmy2time64_neutral*/
 static void
 test_gnc_dmy2time64_neutral (FixtureB *f, gconstpointer pData)
