@@ -23,8 +23,6 @@
  ********************************************************************/
 #include <glib.h>
 
-extern "C"
-{
 #include <config.h>
 #include <string.h>
 #include <unittest-support.h>
@@ -42,8 +40,7 @@ extern "C"
 #endif
 
 static const gchar *suitename = "/engine/Transaction";
-void test_suite_transaction ( void );
-}
+extern "C" void test_suite_transaction ( void );
 
 #include <qof-backend.hpp>
 #include <kvp-frame.hpp>
@@ -137,16 +134,16 @@ setup (Fixture *fixture, gconstpointer pData)
     xaccAccountSetCommodity (fixture->acc2, fixture->curr);
     txn->date_posted = posted;
     txn->date_entered = entered;
-    split1->memo = static_cast<char*>(CACHE_INSERT ("foo"));
-    split1->action = static_cast<char*>(CACHE_INSERT ("bar"));
+    split1->memo = CACHE_INSERT ("foo");
+    split1->action = CACHE_INSERT ("bar");
     split1->amount = gnc_numeric_create (100000, 1000);
     split1->value = gnc_numeric_create (3200, 240);
     split2->amount = gnc_numeric_create (-3200, 240);
     split2->value = gnc_numeric_create (-3200, 240);
     split1->acc = fixture->acc1;
     split2->acc = fixture->acc2;
-    txn->num = static_cast<char*>(CACHE_INSERT ("123"));
-    txn->description = static_cast<char*>(CACHE_INSERT ("Waldo Pepper"));
+    txn->num = CACHE_INSERT ("123");
+    txn->description = CACHE_INSERT ("Waldo Pepper");
     xaccTransBeginEdit (txn);
     {
         xaccTransSetCurrency (txn, fixture->curr);
@@ -289,7 +286,6 @@ void mark_trans (Transaction *trans)// Local: 3:0:0
 static void
 test_mark_trans (Fixture *fixture, gconstpointer pData)
 {
-    gboolean dirty_split = FALSE;
     GList *splits = NULL;
 
     for (splits = (fixture->txn)->splits; splits; splits = splits->next)
@@ -360,7 +356,7 @@ test_gnc_transaction_dispose ()
     auto split = static_cast<Split*>(g_object_new (GNC_TYPE_SPLIT, "book", book, NULL));
     auto s_ref = split;
     gnc_commodity *curr = gnc_commodity_new (book, "Gnu Rand", "CURRENCY",
-                          "GNR", "", 240), *t_curr = NULL;
+                          "GNR", "", 240);
     gnc_commodity *c_ref = curr;
     g_object_add_weak_pointer (G_OBJECT (split), (gpointer*) &s_ref);
     g_object_add_weak_pointer (G_OBJECT (curr), (gpointer*) &c_ref);
@@ -467,7 +463,6 @@ test_xaccMallocTransaction (Fixture *fixture, gconstpointer pData)
 #endif
     auto msg = _func ": assertion 'book' failed";
 #undef _func
-    auto logdomain = "gnc.engine";
     auto loglevel = static_cast<GLogLevelFlags>(G_LOG_LEVEL_CRITICAL | G_LOG_FLAG_FATAL);
     auto check = test_error_struct_new ("gnc.engine", loglevel, msg);
     fixture->hdlrs = test_log_set_fatal_handler (fixture->hdlrs, check,
@@ -600,7 +595,6 @@ test_xaccTransClone (Fixture *fixture, gconstpointer pData)
     Transaction *newtxn = NULL, *oldtxn = fixture->txn;
     QofBook *old_book = qof_instance_get_book (QOF_INSTANCE (oldtxn));
     GList *newnode, *oldnode;
-    int foo, bar;
 
     oldtxn->date_posted = posted;
     oldtxn->date_entered = entered;
@@ -661,8 +655,6 @@ test_xaccTransCopyFromClipBoard (Fixture *fixture, gconstpointer pData)
     Account *acc1 = xaccMallocAccount (book);
     Transaction *to_txn = xaccMallocTransaction (book);
     time64 now = gnc_time (nullptr);
-    time64 never = 0;
-    auto to_frame = to_txn->inst.kvp_data;
 
     xaccAccountSetCommodity (acc1, fixture->comm);
     xaccTransCopyFromClipBoard (txn, to_txn, fixture->acc1, acc1, FALSE);
@@ -722,7 +714,7 @@ test_xaccFreeTransaction (Fixture *fixture, gconstpointer pData)
     /* so the "free" doesn't, leaving the structure for us to test */
     g_object_ref (txn);
     g_object_ref (orig);
-    orig->num = static_cast<char*>(CACHE_INSERT (txn_num));
+    orig->num = CACHE_INSERT (txn_num);
     txn->orig = orig;
 
     fixture->func->xaccFreeTransaction (txn);
@@ -755,7 +747,6 @@ static void
 test_xaccTransEqual (Fixture *fixture, gconstpointer pData)
 {
 
-    QofBook *book = qof_instance_get_book (QOF_INSTANCE (fixture->txn));
     QofBook *book2 = qof_book_new ();
     Transaction *txn0 = fixture->txn;
     Transaction *clone = xaccTransClone (txn0);
@@ -774,6 +765,7 @@ test_xaccTransEqual (Fixture *fixture, gconstpointer pData)
     auto check3 = test_error_struct_new(logdomain, loglevel, "");
     auto cleanup = test_error_struct_new (logdomain, loglevel, "");
     auto split0 = xaccTransGetSplit (txn0, 0);
+    qof_log_set_level (GNC_MOD_ENGINE, QOF_LOG_INFO);
     test_add_error (check);
     test_add_error (check2);
     test_add_error (cleanup);
@@ -849,17 +841,17 @@ test_xaccTransEqual (Fixture *fixture, gconstpointer pData)
     g_assert (xaccTransEqual (txn1, clone, TRUE, FALSE, TRUE, TRUE));
     g_assert_cmpint (check->hits, ==, 5);
 
-    txn1->num = g_strdup("321");
+    txn1->num = CACHE_INSERT("321");
     g_free (check->msg);
     check->msg = g_strdup ("[xaccTransEqual] num differs: 321 vs 123");
     g_assert (!xaccTransEqual (txn1, txn0, TRUE, FALSE, TRUE, TRUE));
     g_assert_cmpint (check->hits, ==, 6);
 
-    g_free(clone->num);
-    clone->num = static_cast<char*>(CACHE_INSERT("123"));
-    g_free(txn1->num);
+    g_free ((char*)clone->num);
+    clone->num = CACHE_INSERT("123");
+    CACHE_REMOVE(txn1->num);
     txn1->num = g_strdup("123");
-    clone->description = g_strdup("salt pork");
+    clone->description = CACHE_INSERT("salt pork");
     g_free (check->msg);
     check->msg = g_strdup ("[xaccTransEqual] descriptions differ: salt pork vs Waldo Pepper");
     g_assert (!xaccTransEqual (clone, txn0, TRUE, FALSE, TRUE, TRUE));
@@ -871,8 +863,8 @@ test_xaccTransEqual (Fixture *fixture, gconstpointer pData)
 
     xaccTransBeginEdit (clone);
     cleanup->msg = g_strdup_printf (cleanup_fmt, clone->orig);
-    g_free(clone->description);
-    clone->description = static_cast<char*>(CACHE_INSERT ("Waldo Pepper"));
+    CACHE_REMOVE(clone->description);
+    clone->description = CACHE_INSERT ("Waldo Pepper");
     auto frame = qof_instance_get_slots (QOF_INSTANCE (clone));
     frame->set({"qux", "quux", "corge"}, new KvpValue(654.321));
     xaccTransCommitEdit (clone);
@@ -885,7 +877,7 @@ test_xaccTransEqual (Fixture *fixture, gconstpointer pData)
     g_assert_cmpint (check->hits, ==, 9);
     xaccTransBeginEdit (clone);
     cleanup->msg = g_strdup_printf (cleanup_fmt, clone->orig);
-    clone->description = static_cast<char*>(CACHE_INSERT ("Waldo Pepper"));
+    clone->description = CACHE_INSERT ("Waldo Pepper");
     frame->set({"qux", "quux", "corge"}, new KvpValue(123.456));
     xaccTransCommitEdit (clone);
     g_free (cleanup->msg);
@@ -975,8 +967,8 @@ test_xaccTransGetImbalanceValue (Fixture *fixture, gconstpointer pData)
     g_assert (gnc_numeric_equal (xaccTransGetImbalanceValue (fixture->txn),
                                  gnc_numeric_zero ()));
     split1->acc = fixture->acc1;
-    split1->memo = static_cast<char*>(CACHE_INSERT ("foo"));
-    split1->action = static_cast<char*>(CACHE_INSERT ("bar"));
+    split1->memo = CACHE_INSERT ("foo");
+    split1->action = CACHE_INSERT ("bar");
     split1->amount = gnc_numeric_create (100000, 1000);
     split1->value = gnc_numeric_create (3200, 240);
     xaccTransBeginEdit (fixture->txn);
@@ -1001,8 +993,8 @@ test_xaccTransGetImbalance (Fixture *fixture, gconstpointer pData)
     g_assert_cmpint (g_list_length (mlist), ==, 0);
 
     split1->acc = fixture->acc1;
-    split1->memo = static_cast<char*>(CACHE_INSERT ("foo"));
-    split1->action = static_cast<char*>(CACHE_INSERT ("bar"));
+    split1->memo = CACHE_INSERT ("foo");
+    split1->action = CACHE_INSERT ("bar");
     split1->amount = gnc_numeric_create (100000, 1000);
     split1->value = gnc_numeric_create (3200, 240);
     xaccTransBeginEdit (fixture->txn);
@@ -1022,7 +1014,6 @@ test_xaccTransGetImbalance_trading (Fixture *fixture,
     auto split2 = xaccMallocSplit (book);
     Account *acc1 = xaccMallocAccount (book);
     Account *acc2 = xaccMallocAccount (book);
-    gnc_numeric value;
     MonetaryList *mlist;
     qof_book_begin_edit (book);
     qof_instance_set (QOF_INSTANCE (book),
@@ -1043,13 +1034,13 @@ test_xaccTransGetImbalance_trading (Fixture *fixture,
     g_assert (!xaccTransIsBalanced (fixture->txn));
     /* Make it look like a proper trading accounts transactionm */
     split1->acc = acc1;
-    split1->memo = static_cast<char*>(CACHE_INSERT ("foo"));
-    split1->action = static_cast<char*>(CACHE_INSERT ("bar"));
+    split1->memo = CACHE_INSERT ("foo");
+    split1->action = CACHE_INSERT ("bar");
     split1->amount = gnc_numeric_create (-10000, 100);
     split1->value = gnc_numeric_create (-3200, 240);
     split2->acc = acc2;
-    split2->memo = static_cast<char*>(CACHE_INSERT ("foo"));
-    split2->action = static_cast<char*>(CACHE_INSERT ("bar"));
+    split2->memo = CACHE_INSERT ("foo");
+    split2->action = CACHE_INSERT ("bar");
     split2->amount = gnc_numeric_create (3000, 240);
     split2->value = gnc_numeric_create (3200, 240);
     xaccTransBeginEdit (fixture->txn);
@@ -1091,8 +1082,8 @@ test_xaccTransIsBalanced (Fixture *fixture, gconstpointer pData)
     g_assert (xaccTransIsBalanced (fixture->txn));
 
     split1->acc = fixture->acc1;
-    split1->memo = static_cast<char*>(CACHE_INSERT ("foo"));
-    split1->action = static_cast<char*>(CACHE_INSERT ("bar"));
+    split1->memo = CACHE_INSERT ("foo");
+    split1->action = CACHE_INSERT ("bar");
     split1->amount = gnc_numeric_create (100000, 1000);
     split1->value = gnc_numeric_create (3200, 240);
     xaccTransBeginEdit (fixture->txn);
@@ -1124,13 +1115,13 @@ test_xaccTransIsBalanced_trading (Fixture *fixture, gconstpointer pData)
     /* The setup transaction is unbalanced in a trading-accounts environment. */
     g_assert (!xaccTransIsBalanced (fixture->txn));
     split1->acc = acc1;
-    split1->memo = static_cast<char*>(CACHE_INSERT ("foo"));
-    split1->action = static_cast<char*>(CACHE_INSERT ("bar"));
+    split1->memo = CACHE_INSERT ("foo");
+    split1->action = CACHE_INSERT ("bar");
     split1->amount = gnc_numeric_create (3200, 240);
     split1->value = gnc_numeric_create (3200, 240);
     split2->acc = acc2;
-    split2->memo = static_cast<char*>(CACHE_INSERT ("foo"));
-    split2->action = static_cast<char*>(CACHE_INSERT ("bar"));
+    split2->memo = CACHE_INSERT ("foo");
+    split2->action = CACHE_INSERT ("bar");
     split2->amount = gnc_numeric_create (-10000, 100);
     split2->value = gnc_numeric_create (-3000, 240);
     xaccTransBeginEdit (fixture->txn);
@@ -1256,7 +1247,6 @@ test_xaccTransGetAccountBalance (Fixture *fixture, gconstpointer pData)
 #undef _func
     auto loglevel = static_cast<GLogLevelFlags>(G_LOG_LEVEL_CRITICAL | G_LOG_FLAG_FATAL);
     auto check = test_error_struct_new ("gnc.engine", loglevel, msg1);
-    auto split1 = xaccTransFindSplitByAccount(fixture->txn, fixture->acc1);
     gnc_numeric rate;
 
     fixture->hdlrs = test_log_set_fatal_handler (fixture->hdlrs, check,
@@ -1322,6 +1312,7 @@ test_xaccTransBeginEdit ()
     auto check2 = test_error_struct_new (logdomain, loglevel, msg2);
     guint hdlr = g_log_set_handler (logdomain, loglevel,
                                     (GLogFunc)test_list_handler, NULL);
+    qof_log_set_level (logdomain, QOF_LOG_INFO);
     test_add_error (check1);
     test_add_error (check2);
 
@@ -1417,7 +1408,6 @@ test_do_destroy (GainsFixture *fixture, gconstpointer pData)
 {
     Fixture *base = &(fixture->base);
     auto base_split =  static_cast<Split*>(g_list_nth_data (base->txn->splits, 1));
-    QofBook *book = qof_instance_get_book (base->txn);
     TestSignal sig = test_signal_new (QOF_INSTANCE (base->txn),
                                       QOF_EVENT_DESTROY, NULL);
     g_object_add_weak_pointer (G_OBJECT (base->txn->splits->data),
@@ -1425,6 +1415,9 @@ test_do_destroy (GainsFixture *fixture, gconstpointer pData)
     g_object_ref (base->txn);
     g_object_ref (fixture->gains_txn);
 
+   /* Protect against recursive calls to do_destroy from xaccTransCommitEdit */
+    xaccTransBeginEdit(base->txn);
+
     base->func->do_destroy (base->txn);
     g_assert_cmpint (test_signal_return_hits (sig), ==, 1);
     g_assert (base->txn->description == NULL);
@@ -1592,8 +1585,8 @@ test_xaccTransCommitEdit (void)
     xaccAccountSetCommodity (acc1, comm);
     xaccAccountSetCommodity (acc2, curr);
     txn->date_posted = posted;
-    split1->memo = static_cast<char*>(CACHE_INSERT ("foo"));
-    split1->action = static_cast<char*>(CACHE_INSERT ("bar"));
+    split1->memo = CACHE_INSERT ("foo");
+    split1->action = CACHE_INSERT ("bar");
     split1->amount = gnc_numeric_create (100000, 1000);
     split1->value = gnc_numeric_create (3200, 240);
     /* Note, deliberately imblanced to force xaccTransScrubImbalance
@@ -1603,8 +1596,8 @@ test_xaccTransCommitEdit (void)
     split2->value = gnc_numeric_create (-3000, 240);
     split1->acc = acc1;
     split2->acc = acc2;
-    txn->num = static_cast<char*>(CACHE_INSERT ("123"));
-    txn->description = static_cast<char*>(CACHE_INSERT ("Waldo Pepper"));
+    txn->num = CACHE_INSERT ("123");
+    txn->description = CACHE_INSERT ("Waldo Pepper");
     xaccTransBeginEdit (txn);
     {
         xaccTransSetCurrency (txn, curr);
@@ -1664,8 +1657,8 @@ test_xaccTransRollbackEdit (Fixture *fixture, gconstpointer pData)
     orig = txn->orig;
     base_frame = orig->inst.kvp_data; /* DupeTransaction copies the kvp_frame */
     g_object_ref (orig); /* Keep rollback from actually freeing it */
-    txn->num = static_cast<char*>(CACHE_INSERT("321"));
-    txn->description = static_cast<char*>(CACHE_INSERT("salt peanuts"));
+    txn->num = CACHE_INSERT("321");
+    txn->description = CACHE_INSERT("salt peanuts");
     txn->common_currency = NULL;
     txn->inst.kvp_data = NULL;
     txn->date_entered = new_entered;
@@ -1757,20 +1750,49 @@ test_xaccTransOrder_num_action (Fixture *fixture, gconstpointer pData)
     g_assert_cmpint (xaccTransOrder_num_action (NULL, NULL, NULL, NULL), ==, 0);
     g_assert_cmpint (xaccTransOrder_num_action (txnA, NULL, txnB, NULL), ==,
                      qof_instance_guid_compare (txnA, txnB));
-    txnB->description = static_cast<char*>(CACHE_INSERT ("Salt Peanuts"));
+    txnB->description = CACHE_INSERT ("Salt Peanuts");
     g_assert_cmpint (xaccTransOrder_num_action (txnA, NULL, txnB, NULL), >=, 1);
     txnB->date_entered += 1;
     g_assert_cmpint (xaccTransOrder_num_action (txnA, NULL, txnB, NULL), ==, -1);
-    txnB->num = static_cast<char*>(CACHE_INSERT ("101"));
+    txnB->num = CACHE_INSERT ("101");
     g_assert_cmpint (xaccTransOrder_num_action (txnA, NULL, txnB, NULL), ==, 1);
-    txnB->num = static_cast<char*>(CACHE_INSERT ("one-oh-one"));
+    txnA->num = CACHE_INSERT ("12a");
+    g_assert_cmpint (xaccTransOrder_num_action (txnA, NULL, txnB, NULL), ==, -1);
+    txnB->num = CACHE_INSERT ("12c");
+    g_assert_cmpint (xaccTransOrder_num_action (txnA, NULL, txnB, NULL), ==, -1);
+    txnB->num = CACHE_INSERT ("12");
     g_assert_cmpint (xaccTransOrder_num_action (txnA, NULL, txnB, NULL), ==, 1);
+    txnB->num = CACHE_INSERT ("one-oh-one");
+    g_assert_cmpint (xaccTransOrder_num_action (txnA, NULL, txnB, NULL), ==, -1);
     g_assert_cmpint (xaccTransOrder_num_action (txnA, "24", txnB, "42"), ==, -1);
     txnB->date_posted -= 1;
     g_assert_cmpint (xaccTransOrder_num_action (txnA, "24", txnB, "42"), ==, 1);
 
     fixture->func->xaccFreeTransaction (txnB);
 }
+
+static void
+test_xaccTransGetReadOnly (Fixture *fixture, gconstpointer pData)
+{
+    auto txn = fixture->txn;
+    g_assert_cmpstr (xaccTransGetReadOnly (txn), ==, nullptr);
+
+    xaccTransSetReadOnly (txn, "RO");
+    g_assert_cmpstr (xaccTransGetReadOnly (txn), ==, "RO");
+
+    xaccTransSetReadOnly (txn, nullptr); // reason being nullptr is a NOP
+    g_assert_cmpstr (xaccTransGetReadOnly (txn), ==, "RO");
+
+    xaccTransClearReadOnly (txn);
+    g_assert_cmpstr (xaccTransGetReadOnly (txn), ==, nullptr);
+
+    xaccTransSetReadOnly (txn, "");
+    g_assert_cmpstr (xaccTransGetReadOnly (txn), ==, "");
+
+    xaccTransClearReadOnly (txn);
+}
+
+
 /* xaccTransSetDateInternal Local: 7:0:0
  * set_gains_date_dirty Local: 4:0:0
  * xaccTransSetDatePostedSecs C: 17 in 13  Local: 0:0:0
@@ -1807,13 +1829,12 @@ test_xaccTransOrder_num_action (Fixture *fixture, gconstpointer pData)
 static void
 test_xaccTransGetTxnType (Fixture *fixture, gconstpointer pData)
 {
-    const char i = 'I';
-    const char p = 'P';
+    // note this will only test TXN_TYPE_NONE, because TxnType is derived
+    // from split data. Testing for TXN_TYPE_INVOICE TXN_TYPE_PAYMENT
+    // will require creating invoices, so, they are tested in
+    // utest-Invoice.c
     auto txn = fixture->txn;
-    xaccTransSetTxnType(txn, i);
-    g_assert_cmpint (i, ==, xaccTransGetTxnType(txn));
-    xaccTransSetTxnType(txn, p);
-    g_assert_cmpint (p, ==, xaccTransGetTxnType(txn));
+    g_assert_cmpint (TXN_TYPE_NONE, ==, xaccTransGetTxnType(txn));
 }
 
 /* xaccTransGetReadOnly C: 7 in 5  Local: 1:0:0
@@ -1837,6 +1858,26 @@ void
 xaccTransUnvoid (Transaction *trans)// C: 1  Local: 0:0:0
 */
 
+static void
+test_xaccTransSetDocLink (Fixture *fixture, gconstpointer pData)
+{
+    auto trans = fixture->txn;
+
+    g_assert_cmpstr (xaccTransGetDocLink (trans), ==, NULL);
+
+    xaccTransSetDocLink (trans, "doclink");
+    g_assert_cmpstr (xaccTransGetDocLink (trans), ==, "doclink");
+
+    xaccTransSetDocLink (trans, "unset");
+    g_assert_cmpstr (xaccTransGetDocLink (trans), ==, "unset");
+
+    xaccTransSetDocLink (trans, "");
+    g_assert_cmpstr (xaccTransGetDocLink (trans), ==, NULL);
+
+    xaccTransSetDocLink (trans, NULL);
+    g_assert_cmpstr (xaccTransGetDocLink (trans), ==, NULL);
+}
+
 static void
 test_xaccTransVoid (Fixture *fixture, gconstpointer pData)
 {
@@ -2039,6 +2080,8 @@ test_suite_transaction (void)
     GNC_TEST_ADD (suitename, "xaccTransRollbackEdit - Backend Errors", Fixture, NULL, setup, test_xaccTransRollbackEdit_BackendErrors, teardown);
     GNC_TEST_ADD (suitename, "xaccTransOrder_num_action", Fixture, NULL, setup, test_xaccTransOrder_num_action, teardown);
     GNC_TEST_ADD (suitename, "xaccTransGetTxnType", Fixture, NULL, setup, test_xaccTransGetTxnType, teardown);
+    GNC_TEST_ADD (suitename, "xaccTransGetreadOnly", Fixture, NULL, setup, test_xaccTransGetReadOnly, teardown);
+    GNC_TEST_ADD (suitename, "xaccTransSetDocLink", Fixture, NULL, setup, test_xaccTransSetDocLink, teardown);
     GNC_TEST_ADD (suitename, "xaccTransVoid", Fixture, NULL, setup, test_xaccTransVoid, teardown);
     GNC_TEST_ADD (suitename, "xaccTransReverse", Fixture, NULL, setup, test_xaccTransReverse, teardown);
     GNC_TEST_ADD (suitename, "xaccTransScrubGainsDate_no_dirty", GainsFixture, NULL, setup_with_gains, test_xaccTransScrubGainsDate_no_dirty, teardown_with_gains);
